@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -101,7 +101,7 @@ static void
 AppUtilCollectNamedIcons(GPtrArray *pixbufs,   // IN/OUT
                          const char *iconName) // IN
 {
-   char *myIconName;
+   char *myIconName = NULL;
    char *baseIconName;
    /*
     * Use the GtkIconTheme to track down any available icons for this app.
