@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -111,7 +111,7 @@ FileIO_MsgError(FileIOResult status) // IN
        * because you don't want to display error messages after a user has
        * cancelled an operation.
        */
-      result = MSGID(fileio.cancel) "The operation was canceled by the user";
+      result = MSGID(fileio.cancel) "The operation was cancelled by the user";
       break;
 
    case FILEIO_ERROR:
@@ -198,8 +198,8 @@ FileIO_MsgError(FileIOResult status) // IN
  */
 
 void
-FileIO_Init(FileIODescriptor *fd,   // IN/OUT:
-            ConstUnicode pathName)  // IN:
+FileIO_Init(FileIODescriptor *fd,  // IN/OUT:
+            const char *pathName)  // IN:
 {
    ASSERT(fd);
    ASSERT(pathName);
@@ -231,7 +231,7 @@ FileIO_Cleanup(FileIODescriptor *fd)  // IN/OUT:
    ASSERT(fd);
 
    if (fd->fileName) {
-      Unicode_Free(fd->fileName);
+      free(fd->fileName);
       fd->fileName = NULL;
    }
 }
@@ -341,7 +341,7 @@ FileIO_Lock(FileIODescriptor *file,  // IN/OUT:
       if (file->lockToken == NULL) {
          /* Describe the lock not acquired situation in detail */
          Warning(LGPFX" %s on '%s' failed: %s\n",
-                 __FUNCTION__, UTF8(file->fileName),
+                 __FUNCTION__, file->fileName,
                  (err == 0) ? "Lock timed out" : strerror(err));
 
          /* Return a serious failure status if the locking code did */
@@ -400,7 +400,7 @@ FileIO_Unlock(FileIODescriptor *file)  // IN/OUT:
 
       if (!FileLock_Unlock(file->lockToken, &err, NULL)) {
          Warning(LGPFX" %s on '%s' failed: %s\n",
-                 __FUNCTION__, UTF8(file->fileName), strerror(err));
+                 __FUNCTION__, file->fileName, strerror(err));
 
          ret = FILEIO_ERROR;
       }
@@ -446,7 +446,7 @@ FileIO_GetSize(const FileIODescriptor *fd)  // IN:
  *
  * FileIO_GetSizeByPath --
  *
- *      Get size of a file specified by path. 
+ *      Get size of a file specified by path.
  *
  * Results:
  *      Size of file or -1.
@@ -458,7 +458,7 @@ FileIO_GetSize(const FileIODescriptor *fd)  // IN:
  */
 
 int64
-FileIO_GetSizeByPath(ConstUnicode pathName)  // IN:
+FileIO_GetSizeByPath(const char *pathName)  // IN:
 {
    int64 logicalBytes;
 
@@ -485,7 +485,7 @@ FileIO_GetSizeByPath(ConstUnicode pathName)  // IN:
  *----------------------------------------------------------------------
  */
 
-ConstUnicode
+const char *
 FileIO_Filename(FileIODescriptor *fd)  // IN:
 {
    ASSERT(fd);
@@ -502,8 +502,9 @@ FileIO_Filename(FileIODescriptor *fd)  // IN:
  *      Closes and unlinks the file associated with a FileIODescriptor.
  *
  * Results:
- *      TRUE: An error occurred.
- *      FALSE: The file was closed and unlinked.
+ *      FILEIO_SUCCESS: The file was closed and unlinked. The FileIODescriptor
+ *                      is no longer valid.
+ *      FILEIO_ERROR: An error occurred.
  *
  * Side effects:
  *      File is probably closed and unlinked.
@@ -511,18 +512,25 @@ FileIO_Filename(FileIODescriptor *fd)  // IN:
  *----------------------------------------------------------------------
  */
 
-Bool
+FileIOResult
 FileIO_CloseAndUnlink(FileIODescriptor *fd)  // IN:
 {
-   Unicode path;
-   Bool ret;
+   char *path;
+   FileIOResult ret;
 
    ASSERT(fd);
    ASSERT(FileIO_IsValid(fd));
 
    path = Unicode_Duplicate(fd->fileName);
-   ret = FileIO_Close(fd) || File_Unlink(path);
-   Unicode_Free(path);
+
+   ret = FileIO_Close(fd);
+   if (FileIO_IsSuccess(ret)) {
+      if (File_UnlinkIfExists(path) == -1) {
+         ret = FILEIO_ERROR;
+      }
+   }
+
+   free(path);
 
    return ret;
 }
@@ -641,7 +649,7 @@ FileIO_IsSuccess(FileIOResult res)  // IN:
  *      The caller must free the path when done.
  *
  * Results:
- *      Unicode path if successful, NULL on failure.
+ *      UTF8 path if successful, NULL on failure.
  *
  * Side effects:
  *      None.
@@ -649,11 +657,11 @@ FileIO_IsSuccess(FileIOResult res)  // IN:
  *-----------------------------------------------------------------------------
  */
 
-Unicode
-FileIO_AtomicTempPath(ConstUnicode path)  // IN:
+char *
+FileIO_AtomicTempPath(const char *path)  // IN:
 {
-   Unicode srcPath;
-   Unicode retPath;
+   char *srcPath;
+   char *retPath;
 
    srcPath = File_FullPath(path);
    if (!srcPath) {
@@ -661,7 +669,7 @@ FileIO_AtomicTempPath(ConstUnicode path)  // IN:
       return NULL;
    }
    retPath = Unicode_Join(srcPath, "~", NULL);
-   Unicode_Free(srcPath);
+   free(srcPath);
 
    return retPath;
 }
@@ -672,7 +680,7 @@ FileIO_AtomicTempPath(ConstUnicode path)  // IN:
  *
  * FileIO_AtomicTempFile
  *
- *      Create a temp file in the same directory as the argument file. 
+ *      Create a temp file in the same directory as the argument file.
  *      On non-Windows attempts to create the temp file with the same
  *      permissions and owner/group as the argument file.
  *
@@ -689,7 +697,7 @@ FileIOResult
 FileIO_AtomicTempFile(FileIODescriptor *fileFD,  // IN:
                       FileIODescriptor *tempFD)  // OUT:
 {
-   Unicode tempPath = NULL;
+   char *tempPath = NULL;
    int permissions;
    FileIOResult status;
 #if !defined(_WIN32)
@@ -762,7 +770,7 @@ FileIO_AtomicTempFile(FileIODescriptor *fileFD,  // IN:
    }
 #endif
 
-   Unicode_Free(tempPath);
+   free(tempPath);
    return FILEIO_SUCCESS;
 
 bail:
@@ -780,7 +788,7 @@ bail:
       ASSERT(ret == 0);
 #endif
    }
-   Unicode_Free(tempPath);
+   free(tempPath);
    return status;
 }
 
@@ -797,19 +805,11 @@ bail:
  *      On hosted products, uses rename to swap files, so "new" becomes "curr",
  *      and path to "new" no longer exists on success.
  *
- *      On ESX on NFS:
- *
- *      If renameOnNFS is TRUE, use rename, like on hosted.
- *
- *      If renameOnNFS is FALSE, returns -1 rather than trying to use rename,
- *      to avoid various bugs in the vmkernel NFSv3 client.  Bug 839283,
- *      bug 862647, bug 841185, bug 856752.
- *
  *      On success the caller must call FileIO_IsValid on newFD to verify it
  *      is still open before using it again.
  *
  * Results:
- *      1 if successful, 0 on failure, -1 if not supported on this filesystem.
+ *      TRUE if successful, FALSE otherwise
  *      errno is preserved.
  *
  * Side effects:
@@ -818,19 +818,22 @@ bail:
  *-----------------------------------------------------------------------------
  */
 
-int
+Bool
 FileIO_AtomicUpdate(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
-                    FileIODescriptor *currFD,  // IN/OUT: file IO descriptor
-                    Bool renameOnNFS)          // IN: fall back to rename on NFS
+                    FileIODescriptor *currFD)  // IN/OUT: file IO descriptor
 {
-   char *currPath;
-   char *newPath;
+   char *currPath = NULL;
+   char *newPath = NULL;
+#if defined(_WIN32)
    uint32 currAccess;
    uint32 newAccess;
-   int ret = 0;
    FileIOResult status;
    FileIODescriptor tmpFD;
+#else
+   int fd;
+#endif
    int savedErrno = 0;
+   Bool ret = FALSE;
 
    ASSERT(FileIO_IsValid(newFD));
    ASSERT(FileIO_IsValid(currFD));
@@ -842,31 +845,40 @@ FileIO_AtomicUpdate(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
       char *fileName = NULL;
       char *dstDirName = NULL;
       char *dstFileName = NULL;
-      int fd;
 
       currPath = File_FullPath(FileIO_Filename(currFD));
-      newPath = File_FullPath(FileIO_Filename(newFD));
+      if (!currPath) {
+         savedErrno = errno;
+         Log("%s: File_FullPath of '%s' failed.\n", __FUNCTION__,
+             FileIO_Filename(currFD));
+         goto swapdone;
+      }
 
-      ASSERT(currPath);
-      ASSERT(newPath);
+      newPath = File_FullPath(FileIO_Filename(newFD));
+      if (!newPath) {
+         savedErrno = errno;
+         Log("%s: File_FullPath of '%s' failed.\n", __FUNCTION__,
+             FileIO_Filename(newFD));
+         goto swapdone;
+      }
 
       File_GetPathName(newPath, &dirName, &fileName);
       File_GetPathName(currPath, &dstDirName, &dstFileName);
 
-      ASSERT(dirName && *dirName);
+      ASSERT(dirName);
       ASSERT(fileName && *fileName);
-      ASSERT(dstDirName && *dstDirName);
+      ASSERT(dstDirName);
       ASSERT(dstFileName && *dstFileName);
-      ASSERT(!strcmp(dirName, dstDirName));
+      ASSERT(File_IsSameFile(dirName, dstDirName));
 
-      args = (FS_SwapFilesArgs *) Util_SafeCalloc(1, sizeof(*args));
-      if (Str_Snprintf(args->srcFile, sizeof(args->srcFile), "%s",
+      args = Util_SafeCalloc(1, sizeof *args);
+      if (Str_Snprintf(args->srcFile, sizeof args->srcFile, "%s",
                        fileName) < 0) {
          Log("%s: Path too long \"%s\".\n", __FUNCTION__, fileName);
          savedErrno = ENAMETOOLONG;
          goto swapdone;
       }
-      if (Str_Snprintf(args->dstFilePath, sizeof(args->dstFilePath), "%s/%s",
+      if (Str_Snprintf(args->dstFilePath, sizeof args->dstFilePath, "%s/%s",
                        dstDirName, dstFileName) < 0) {
          Log("%s: Path too long \"%s\".\n", __FUNCTION__, dstFileName);
          savedErrno = ENAMETOOLONG;
@@ -878,11 +890,16 @@ FileIO_AtomicUpdate(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
        * because the file could be open.
        */
 
+      if (!*dirName) {
+         /* need a proper root directory string for Posix_Open() */
+         free(dirName);
+         dirName = Util_SafeStrdup("/");
+      }
+
       fd = Posix_Open(dirName, O_RDONLY);
       if (fd < 0) {
-         Log("%s: Open failed \"%s\" %d.\n", __FUNCTION__, dirName,
-             errno);
-         ASSERT_BUG_DEBUGONLY(615124, errno != EBUSY);
+         Log("%s: Open failed \"%s\" %d.\n", __FUNCTION__, dirName, errno);
+         ASSERT(errno != EBUSY);   /* #615124. */
          savedErrno = errno;
          goto swapdone;
       }
@@ -891,10 +908,10 @@ FileIO_AtomicUpdate(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
          savedErrno = errno;
          if (errno != ENOSYS && errno != ENOTTY) {
             Log("%s: ioctl failed %d.\n", __FUNCTION__, errno);
-            ASSERT_BUG_DEBUGONLY(615124, errno != EBUSY);
+            ASSERT(errno != EBUSY);   /* #615124. */
          }
       } else {
-         ret = 1;
+         ret = TRUE;
       }
 
       close(fd);
@@ -907,35 +924,31 @@ FileIO_AtomicUpdate(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
        * Check for both ENOSYS and ENOTTY. PR 957695
        */
       if (savedErrno == ENOSYS || savedErrno == ENOTTY) {
-         if (renameOnNFS) {
-            /*
-             * NFS allows renames of locked files, even if both files
-             * are locked.  The file lock follows the file handle, not
-             * the name, so after the rename we can swap the underlying
-             * file descriptors instead of closing and reopening the
-             * target file.
-             *
-             * This is different than the hosted path below because
-             * ESX uses native file locks and hosted does not.
-             *
-             * We assume that all ESX file systems that support rename
-             * have the same file lock semantics as NFS.
-             */
+         /*
+          * NFS allows renames of locked files, even if both files
+          * are locked.  The file lock follows the file handle, not
+          * the name, so after the rename we can swap the underlying
+          * file descriptors instead of closing and reopening the
+          * target file.
+          *
+          * This is different than the hosted path below because
+          * ESX uses native file locks and hosted does not.
+          *
+          * We assume that all ESX file systems that support rename
+          * have the same file lock semantics as NFS.
+          */
 
-            if (File_Rename(newPath, currPath)) {
-               Log("%s: rename of '%s' to '%s' failed %d.\n",
-                   __FUNCTION__, newPath, currPath, errno);
-               savedErrno = errno;
-               goto swapdone;
-            }
-            ret = 1;
-            fd = newFD->posix;
-            newFD->posix = currFD->posix;
-            currFD->posix = fd;
-            FileIO_Close(newFD);
-         } else {
-            ret = -1;
+         if (File_Rename(newPath, currPath)) {
+            Log("%s: rename of '%s' to '%s' failed %d.\n",
+                __FUNCTION__, newPath, currPath, errno);
+            savedErrno = errno;
+            goto swapdone;
          }
+         ret = TRUE;
+         fd = newFD->posix;
+         newFD->posix = currFD->posix;
+         currFD->posix = fd;
+         FileIO_Close(newFD);
       }
 
 swapdone:
@@ -953,7 +966,7 @@ swapdone:
       NOT_REACHED();
 #endif
    }
-
+#if defined(_WIN32)
    currPath = Unicode_Duplicate(FileIO_Filename(currFD));
    newPath = Unicode_Duplicate(FileIO_Filename(newFD));
 
@@ -971,13 +984,8 @@ swapdone:
     * middle of transferring ownership.
     */
 
-#if defined(_WIN32)
    CloseHandle(currFD->win32);
    currFD->win32 = INVALID_HANDLE_VALUE;
-#else
-   close(currFD->posix);
-   currFD->posix = -1;
-#endif
    if (File_RenameRetry(newPath, currPath, 10) == 0) {
       ret = TRUE;
    } else {
@@ -1003,16 +1011,32 @@ swapdone:
    }
    ASSERT(tmpFD.lockToken == NULL);
 
-#if defined(_WIN32)
    currFD->win32 = tmpFD.win32;
-#else
-   currFD->posix = tmpFD.posix;
-#endif
 
    FileIO_Cleanup(&tmpFD);
-   Unicode_Free(currPath);
-   Unicode_Free(newPath);
+   free(currPath);
+   free(newPath);
    errno = savedErrno;
 
    return ret;
+#else
+   currPath = (char *)FileIO_Filename(currFD);
+   newPath = (char *)FileIO_Filename(newFD);
+
+   if (File_Rename(newPath, currPath)) {
+      Log("%s: rename of '%s' to '%s' failed %d.\n",
+          __FUNCTION__, newPath, currPath, errno);
+          savedErrno = errno;
+   } else {
+      ret = TRUE;
+      fd = newFD->posix;
+      newFD->posix = currFD->posix;
+      currFD->posix = fd;
+      FileIO_Close(newFD);
+   }
+
+   errno = savedErrno;
+
+   return ret;
+#endif
 }
