@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -51,7 +51,7 @@ typedef struct HgfsGuestChannelCBTable {
 extern HgfsGuestChannelCBTable gGuestBackdoorOps;
 
 /* For use by HgfsServerManager. */
-Bool HgfsChannelGuest_Init(HgfsServerMgrData *data);
+Bool HgfsChannelGuest_Init(HgfsServerMgrData *data, HgfsServerMgrCallbacks *cb);
 void HgfsChannelGuest_Exit(HgfsServerMgrData *data);
 Bool HgfsChannelGuest_Receive(HgfsServerMgrData *data,
                               char const *packetIn,
