@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,6 +26,10 @@
 #define _APP_UTIL_H_
 
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif // __cplusplus
@@ -85,7 +89,11 @@ HICON AppUtil_GetWindowIcon(HWND hwnd,
 
 void AppUtil_BuildGlobalApplicationList(void);
 
+wchar_t* AppUtil_SanitizeCommandLine(const wchar_t *commandLineUtf16);
 char *AppUtil_ActionURIForCommandLine(const WCHAR *commandLineUtf16);
+Bool  AppUtil_CommandLineForShellCommandURI(const char *shellCommandURI,
+                                            char **executablePath,
+                                            char **commandLine);
 
 Bool AppUtil_GetLinkIconData(const TCHAR *path,
                              AppUtilIconInfo *iconInfo,
@@ -109,6 +117,8 @@ Bool AppUtil_GetIconIndexAndLocationForShortcut(const TCHAR *shortcut,
                                                 TCHAR *iconFile,
                                                 int *iconIndex);
 
+PISECURITY_DESCRIPTOR AppUtil_AllocateLowIntegritySD(void);
+
 LPSTR  AppUtil_ToLowerUtf8(LPCSTR s);
 LPWSTR AppUtil_ToLowerUtf16(LPCWSTR s);
 
