@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2004 VMware, Inc. All rights reserved.
+ * Copyright (C) 2004-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,6 +30,7 @@
 #define INCLUDE_ALLOW_VMCORE
 #include "includeCheck.h"
 
+#include "vm_basic_defs.h"
 #include "vm_atomic.h"
 
 
@@ -57,85 +58,84 @@ typedef int (*HashTableForEachCallback)(const char *key,
 #define HASH_FLAG_COPYKEY       0x10    // copy string key
 
 HashTable *
-HashTable_Alloc(uint32               numEntries,
-                int                  keyType,
-                HashTableFreeEntryFn fn);
+HashTable_Alloc(uint32               numEntries,  // IN:
+                int                  keyType,     // IN:
+                HashTableFreeEntryFn fn);         // IN/OPT:
 
 HashTable *
-HashTable_AllocOnce(Atomic_Ptr          *var,
-                    uint32               numEntries,
-                    int                  keyType,
-                    HashTableFreeEntryFn fn);
+HashTable_AllocOnce(Atomic_Ptr          *var,         // IN/OUT:
+                    uint32               numEntries,  // IN:
+                    int                  keyType,     // IN:
+                    HashTableFreeEntryFn fn);         // IN/OPT:
 
 void
-HashTable_Free(HashTable *hashTable);
+HashTable_Free(HashTable *hashTable);  // IN/OUT:
 
 void
-HashTable_FreeUnsafe(HashTable *hashTable);
+HashTable_FreeUnsafe(HashTable *hashTable);  // IN/OUT:
 
 Bool
-HashTable_Insert(HashTable  *hashTable,
-                 const void *keyStr,
-                 void       *clientData);
+HashTable_Insert(HashTable  *hashTable,    // IN/OUT:
+                 const void *keyStr,       // IN:
+                 void       *clientData);  // IN/OPT:
 
 Bool
-HashTable_Lookup(HashTable  *hashTable,
-                 const void *keyStr,
-                 void      **clientData);
+HashTable_Lookup(HashTable  *hashTable,    // IN:
+                 const void *keyStr,       // IN:
+                 void      **clientData);  // OUT/OPT:
 
 void *
-HashTable_LookupOrInsert(HashTable  *hashTable,
-                         const void *keyStr,
-                         void       *clientData);
+HashTable_LookupOrInsert(HashTable  *hashTable,    // IN/OUT:
+                         const void *keyStr,       // IN:
+                         void       *clientData);  // IN/OPT:
 
 Bool
-HashTable_ReplaceOrInsert(HashTable  *hashTable,
-                          const void *keyStr,
-                          void       *clientData);
+HashTable_ReplaceOrInsert(HashTable  *hashTable,    // IN/OUT:
+                          const void *keyStr,       // IN:
+                          void       *clientData);  // IN/OPT
 
 Bool
-HashTable_ReplaceIfEqual(HashTable  *hashTable,
-                         const void *keyStr,
-                         void       *oldClientData,
-                         void       *newClientData);
+HashTable_ReplaceIfEqual(HashTable  *hashTable,       // IN/OUT:
+                         const void *keyStr,          // IN:
+                         void       *oldClientData,   // IN/OPT
+                         void       *newClientData);  // IN/OPT
 
 Bool
-HashTable_Delete(HashTable  *hashTable,
-                 const void *keyStr);
+HashTable_Delete(HashTable  *hashTable,  // IN/OUT:
+                 const void *keyStr);    // IN:
 
 Bool
-HashTable_LookupAndDelete(HashTable  *hashTable,
-                          const void *keyStr,
-                          void      **clientData);
+HashTable_LookupAndDelete(HashTable  *hashTable,    // IN/OUT:
+                          const void *keyStr,       // IN:
+                          void      **clientData);  // OUT:
 
 void
-HashTable_Clear(HashTable *ht);
+HashTable_Clear(HashTable *ht);  // IN/OUT:
 
 void
-HashTable_ToArray(const HashTable   *ht,
-                  void            ***clientDatas,
-                  size_t            *size);
+HashTable_ToArray(const HashTable   *ht,           // IN:
+                  void            ***clientDatas,  // OUT:
+                  size_t            *size);        // OUT:
 
 void
-HashTable_KeyArray(const HashTable   *ht,
-                   const void      ***keys,
-                   size_t            *size);
+HashTable_KeyArray(const HashTable   *ht,     // IN:
+                   const void      ***keys,   // OUT:
+                   size_t            *size);  // OUT:
 
 size_t
-HashTable_GetNumElements(const HashTable *ht);
+HashTable_GetNumElements(const HashTable *ht);  // IN:
 
 int
-HashTable_ForEach(const HashTable          *ht,
-                  HashTableForEachCallback  cb,
-                  void                     *clientData);
+HashTable_ForEach(const HashTable          *ht,           // IN:
+                  HashTableForEachCallback  cb,           // IN:
+                  void                     *clientData);  // IN:
 
 /*
  * Specialize hash table that uses the callers data structure as its
  * hash entry as well, the hash key being an address that must be unique.
  */
 
-typedef struct PtrHashEntry
-{
+typedef struct PtrHashEntry {
    struct PtrHashEntry  *next;
    void                 *ptr;
 } PtrHashEntry;
@@ -148,7 +148,7 @@ typedef struct PtrHashEntry
  */
 
 #define PTRHASH_CONTAINER(ptr, type, member) \
-   ((type *)((char *)(ptr) - offsetof(type, member)))
+   ((type *)((char *)(ptr) - vmw_offsetof(type, member)))
 
 typedef int (*PtrHashForEachCallback)(PtrHashEntry *entry,
                                       const void *clientData);
