@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,59 +22,39 @@
 #include "hgfs.h"             /* for HGFS_PACKET_MAX */
 #include "dbllnklst.h"
 
-/*
- * Function used for sending updates of server state to the manager.
- * Passed by the caller at session connect time.
- */
-typedef void
-HgfsServerStateLoggerFunc(void *data,     // IN
-                          uint64 cookie); // IN
-
-typedef struct HgfsServerStateLogger {
-   HgfsServerStateLoggerFunc  *logger;       // logger callback
-   void                       *loggerData;   // logger callback private data
-} HgfsServerStateLogger;
-
-typedef
-struct HgfsVmxIov {
+typedef struct HgfsVmxIov {
    void *va;           /* Virtual addr */
    uint64 pa;          /* Physical address passed by the guest */
    uint32 len;         /* length of data; should be <= PAGE_SIZE for VMCI; arbitrary for backdoor */
-   char *token;        /* Token for physMem_ APIs */
+   void *context;      /* Mapping context */
 } HgfsVmxIov;
 
-typedef
-struct HgfsVaIov {
-   void *va;
-   uint32 len;
-}HgfsVaIov;
-
 typedef enum {
    BUF_READABLE,      /* Establish readable mappings */
    BUF_WRITEABLE,     /* Establish writeable mappings */
    BUF_READWRITEABLE, /* Establish read-writeable mappings */
 } MappingType;
 
-typedef
-struct HgfsPacket {
+typedef uint64 HgfsStateFlags;
+#define HGFS_STATE_CLIENT_REQUEST         (1 << 0)
+#define HGFS_STATE_ASYNC_REQUEST          (1 << 1)
+typedef struct HgfsPacket {
    uint64 id;
 
-   /* Does the transport support Async operations ? */
-   Bool supportsAsync;
-
-   /* Does transport need to send Async reply ? */
-   Bool processedAsync;
-
-   /* Is the packet guest initiated ? */
-   Bool guestInitiated;
+   HgfsStateFlags state;
 
    /* For metapacket we always establish writeable mappings */
    void *metaPacket;
    size_t metaPacketSize;
+   uint32 metaPacketMappedIov;
+   size_t metaPacketDataSize;
    Bool metaPacketIsAllocated;
+   MappingType metaMappingType;
 
    void *dataPacket;
    size_t dataPacketSize;
+   uint32 dataPacketMappedIov;
+   size_t dataPacketDataSize;
    uint32 dataPacketIovIndex;
    Bool dataPacketIsAllocated;
    /* What type of mapping was established - readable/ writeable ? */
@@ -82,8 +62,12 @@ struct HgfsPacket {
 
    void *replyPacket;
    size_t replyPacketSize;
+   size_t replyPacketDataSize;
    Bool replyPacketIsAllocated;
 
+   /* Iov for the packet private to the channel. */
+   HgfsVmxIov channelIov[2];
+
    uint32 iovCount;
    HgfsVmxIov iov[1];
 
@@ -113,67 +97,30 @@ typedef uint32 HgfsSendFlags;
 #define HGFS_SEND_NO_COMPLETE       (1 << 1)
 
 // Channel capability flags
+typedef uint32 HgfsChannelFlags;
 #define HGFS_CHANNEL_SHARED_MEM     (1 << 0)
 #define HGFS_CHANNEL_ASYNC          (1 << 1)
 
-typedef Bool
-HgfsSessionSendFunc(void *opaqueSession,  // IN
-                    char *buffer,         // IN
-                    size_t bufferLen,     // IN
-                    HgfsSendFlags flags); // IN
+typedef struct HgfsServerChannelData {
+   HgfsChannelFlags flags;
+   uint32 maxPacketSize;
+}HgfsServerChannelData;
 
-typedef struct HgfsServerChannelCallbacks {
-    void* (*getReadVa)(uint64 pa, uint32 size, char **token);
-    void* (*getWriteVa)(uint64 pa, uint32 size, char **token);
-    void (*putVa)(char **token);
-    Bool (*send)(void *opaqueSession, HgfsPacket *packet, char *buffer,
-                 size_t bufferLen, HgfsSendFlags flags);
-}HgfsServerChannelCallbacks;
 
-typedef struct HgfsServerSessionCallbacks {
-   Bool (*connect)(void *, HgfsServerChannelCallbacks *, uint32 ,void **);
-   void (*disconnect)(void *);
-   void (*close)(void *);
-   void (*receive)(HgfsPacket *packet, void *);
-   void (*invalidateObjects)(void *, DblLnkLst_Links *);
-   uint32 (*invalidateInactiveSessions)(void *);
-   void (*sendComplete)(HgfsPacket *, void *);
-} HgfsServerSessionCallbacks;
+/* Default maximum number of open nodes. */
+#define HGFS_MAX_CACHED_FILENODES   30
 
-Bool HgfsServer_InitState(HgfsServerSessionCallbacks **, HgfsServerStateLogger *);
-void HgfsServer_ExitState(void);
+typedef uint32 HgfsConfigFlags;
+#define HGFS_CONFIG_USE_HOST_TIME                    (1 << 0)
+#define HGFS_CONFIG_NOTIFY_ENABLED                   (1 << 1)
+#define HGFS_CONFIG_VOL_INFO_MIN                     (1 << 2)
+#define HGFS_CONFIG_OPLOCK_ENABLED                   (1 << 3)
+#define HGFS_CONFIG_SHARE_ALL_HOST_DRIVES_ENABLED    (1 << 4)
 
-uint32 HgfsServer_GetHandleCounter(void);
-void HgfsServer_SetHandleCounter(uint32 newHandleCounter);
-
-/*
- * Function pointers used for getting names in HgfsServerGetDents
- *
- * Functions of this type are expected to return a NUL terminated
- * string and the length of that string.
- */
-typedef Bool
-HgfsGetNameFunc(void *data,        // IN
-                char const **name, // OUT
-                size_t *len,       // OUT
-                Bool *done);       // OUT
-
-/*
- * Associated setup and cleanup function types, which should be called
- * before and after (respectively) HgfsGetNameFunc.
- */
-typedef void *
-HgfsInitFunc(void);
-
-typedef Bool
-HgfsCleanupFunc(void *);  // IN
-
-/*
- * Function used for invalidating nodes and searches that fall outside of a
- * share when the list of shares changes.
- */
-typedef void
-HgfsInvalidateObjectsFunc(DblLnkLst_Links *shares); // IN
+typedef struct HgfsServerConfig {
+   HgfsConfigFlags flags;
+   uint32 maxCachedOpenNodes;
+}HgfsServerConfig;
 
 /*
  * Function used to notify HGFS server that a shared folder has been created or updated.
@@ -183,13 +130,79 @@ HgfsInvalidateObjectsFunc(DblLnkLst_Links *shares); // IN
 typedef uint32 HgfsSharedFolderHandle;
 #define HGFS_INVALID_FOLDER_HANDLE         ((HgfsSharedFolderHandle)~((HgfsSharedFolderHandle)0))
 
-typedef HgfsSharedFolderHandle
-HgfsRegisterSharedFolderFunc(const char *shareName,
-                             const char *sharePath,
-                             Bool addFolder);
-HgfsSharedFolderHandle HgfsServer_RegisterSharedFolder(const char *shareName,
-                                                       const char *sharePath,
-                                                       Bool addFolder);
+typedef HgfsSharedFolderHandle (*HgfsRegisterSharedFolderFunc)(const char *shareName,
+                                                               const char *sharePath,
+                                                               Bool addFolder);
+/*
+ * Callback functions to enumerate the share resources.
+ * Filled in by the HGFS server policy and passed in to the HGFS server
+ * so that it can call out to them to enumerate the shares.
+ */
+typedef void * (*HgfsServerResEnumInitFunc)(void);
+typedef Bool (*HgfsServerResEnumGetFunc)(void *data,
+                                         char const **name,
+                                         size_t *len,
+                                         Bool *done);
+typedef Bool (*HgfsServerResEnumExitFunc)(void *);
+
+typedef struct HgfsServerResEnumCallbacks {
+   HgfsServerResEnumInitFunc init;
+   HgfsServerResEnumGetFunc get;
+   HgfsServerResEnumExitFunc exit;
+} HgfsServerResEnumCallbacks;
+
+
+/*
+ * Server Manager callback functions to enumerate the share resources and state logging.
+ * Passed to the HGFS server on initialization.
+ */
+typedef struct HgfsServerMgrCallbacks {
+   HgfsServerResEnumCallbacks enumResources;
+} HgfsServerMgrCallbacks;
+
+/*
+ * Function used for invalidating nodes and searches that fall outside of a
+ * share when the list of shares changes.
+ */
+typedef void (*HgfsInvalidateObjectsFunc)(DblLnkLst_Links *shares);
+
+typedef Bool (*HgfsChannelSendFunc)(void *opaqueSession,
+                                    HgfsPacket *packet,
+                                    HgfsSendFlags flags);
+typedef void * (*HgfsChannelMapVirtAddrFunc)(uint64 pa, uint32 size, void **context);
+typedef void (*HgfsChannelUnmapVirtAddrFunc)(void **context);
+
+typedef struct HgfsServerChannelCallbacks {
+    HgfsChannelMapVirtAddrFunc getReadVa;
+    HgfsChannelMapVirtAddrFunc getWriteVa;
+    HgfsChannelUnmapVirtAddrFunc putVa;
+    HgfsChannelSendFunc send;
+}HgfsServerChannelCallbacks;
+
+typedef struct HgfsServerSessionCallbacks {
+   Bool (*connect)(void *, HgfsServerChannelCallbacks *, HgfsServerChannelData *,void **);
+   void (*disconnect)(void *);
+   void (*close)(void *);
+   void (*receive)(HgfsPacket *packet, void *);
+   void (*invalidateObjects)(void *, DblLnkLst_Links *);
+   uint32 (*invalidateInactiveSessions)(void *);
+   void (*sendComplete)(HgfsPacket *, void *);
+} HgfsServerSessionCallbacks;
+
+typedef struct HgfsServerCallbacks {
+   HgfsServerSessionCallbacks session;
+   HgfsRegisterSharedFolderFunc registerShare;
+} HgfsServerCallbacks;
+
+Bool HgfsServer_InitState(HgfsServerCallbacks **,
+                          HgfsServerConfig *,
+                          HgfsServerMgrCallbacks *);
+void HgfsServer_ExitState(void);
+
+uint32 HgfsServer_GetHandleCounter(void);
+void HgfsServer_SetHandleCounter(uint32 newHandleCounter);
+
+
 void HgfsServer_Quiesce(Bool freeze);
 
 #endif // _HGFS_SERVER_H_
