@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2003 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -57,6 +57,7 @@
    LOGLEVEL_VAR(serial), \
    LOGLEVEL_VAR(parallel), \
    LOGLEVEL_VAR(chipset), \
+   LOGLEVEL_VAR(smram), \
    LOGLEVEL_VAR(smc), \
    LOGLEVEL_VAR(ich7m), \
    LOGLEVEL_VAR(hpet), \
@@ -78,9 +79,8 @@
    LOGLEVEL_VAR(vusbhid), \
    LOGLEVEL_VAR(vusbtablet), \
    LOGLEVEL_VAR(vusbaudio), \
+   LOGLEVEL_VAR(vusbvideo),\
    LOGLEVEL_VAR(hidQueue), \
-   LOGLEVEL_VAR(pci_1394), \
-   LOGLEVEL_VAR(1394), \
    LOGLEVEL_VAR(pci_vlance), \
    LOGLEVEL_VAR(pci_svga), \
    LOGLEVEL_VAR(pci_e1000), \
@@ -98,7 +98,10 @@
    LOGLEVEL_VAR(vcpuhotplug), \
    LOGLEVEL_VAR(vcpuNUMA), \
    LOGLEVEL_VAR(heci), \
-   LOGLEVEL_VAR(vmiopluginlib), \
+   LOGLEVEL_VAR(pciplugin), \
+   LOGLEVEL_VAR(vsock), \
+   LOGLEVEL_VAR(vrdma), \
+   LOGLEVEL_VAR(nvdimm), \
    \
    /* user/disk */ \
    LOGLEVEL_VAR(aioMgr), \
@@ -120,6 +123,7 @@
    LOGLEVEL_VAR(gui), \
    LOGLEVEL_VAR(guiWin32), \
    LOGLEVEL_VAR(mks), \
+   LOGLEVEL_VAR(mksInput), \
    LOGLEVEL_VAR(mksSWB), \
    LOGLEVEL_VAR(mksClient), \
    LOGLEVEL_VAR(mksServer), \
@@ -128,12 +132,22 @@
    LOGLEVEL_VAR(mksMouse), \
    LOGLEVEL_VAR(mksHostCursor), \
    LOGLEVEL_VAR(mksCursorPosition), \
-   LOGLEVEL_VAR(mksHostOps), \
+   LOGLEVEL_VAR(mksBasicOps), \
+   LOGLEVEL_VAR(mksRenderOps), \
+   LOGLEVEL_VAR(mksGLBasic), \
    LOGLEVEL_VAR(mksGLManager), \
    LOGLEVEL_VAR(mksGLFBO), \
    LOGLEVEL_VAR(mksGLShader), \
    LOGLEVEL_VAR(mksGLState), \
    LOGLEVEL_VAR(mksGLWindow), \
+   LOGLEVEL_VAR(mksGLContextMux), \
+   LOGLEVEL_VAR(mksGLDraw), \
+   LOGLEVEL_VAR(mksGLQuery), \
+   LOGLEVEL_VAR(mksGLTextureView), \
+   LOGLEVEL_VAR(mksWinBSOD), \
+   LOGLEVEL_VAR(mksDX11Renderer), \
+   LOGLEVEL_VAR(mksDX11ResourceView), \
+   LOGLEVEL_VAR(mksDX11ShimOps), \
    LOGLEVEL_VAR(vdpPlugin), \
    \
    /* user/sound */ \
@@ -142,6 +156,9 @@
    LOGLEVEL_VAR(pci_hdaudio), \
    LOGLEVEL_VAR(hdaudio_alsa), \
    \
+   /* user video */ \
+   LOGLEVEL_VAR(AVCapture), \
+   \
    /* user/disklib */ \
    LOGLEVEL_VAR(disklib), \
    LOGLEVEL_VAR(dmg), \
@@ -160,6 +177,7 @@
    LOGLEVEL_VAR(macbw), \
    LOGLEVEL_VAR(macfi), \
    LOGLEVEL_VAR(vmkcfg), \
+   LOGLEVEL_VAR(policy), \
    LOGLEVEL_VAR(poll), \
    LOGLEVEL_VAR(barrier), \
    LOGLEVEL_VAR(mstat), \
@@ -221,8 +239,8 @@
    LOGLEVEL_VAR(automation), \
    LOGLEVEL_VAR(oemDevice), \
    LOGLEVEL_VAR(cptOps), \
-   LOGLEVEL_VAR(VProbeExec), \
-   LOGLEVEL_VAR(VP), \
+   LOGLEVEL_VAR(vprobe), \
+   LOGLEVEL_VAR(VProbeClient), \
    LOGLEVEL_VAR(device), \
    LOGLEVEL_VAR(devicePowerOn), \
    LOGLEVEL_VAR(vmxvmdbCallbacks), \
@@ -249,18 +267,14 @@
    LOGLEVEL_VAR(inputdevtap), \
    LOGLEVEL_VAR(objlib), \
    LOGLEVEL_VAR(vsanobj), \
+   LOGLEVEL_VAR(vvolbe), \
    LOGLEVEL_VAR(svgadevtap), \
    LOGLEVEL_VAR(masReceipt), /* lib/masReceipt */ \
    LOGLEVEL_VAR(serviceImpl), /* lib/serviceImpl */ \
    LOGLEVEL_VAR(serviceUser), /* lib/serviceUser */ \
    LOGLEVEL_VAR(ssl), \
-   LOGLEVEL_VAR(vmrc), \
    LOGLEVEL_VAR(namespaceDb), \
    LOGLEVEL_VAR(namespaceMgr), \
-   LOGLEVEL_VAR(blobMgr), \
-   LOGLEVEL_VAR(vblobbe), \
-   LOGLEVEL_VAR(blobFileBE), \
-   LOGLEVEL_VAR(blobPythonBE), \
    LOGLEVEL_VAR(grainTrack), \
    LOGLEVEL_VAR(shim3D), \
    LOGLEVEL_VAR(crc32), \
@@ -269,6 +283,16 @@
    LOGLEVEL_VAR(vva),  /* apps/rde/vva */ \
    LOGLEVEL_VAR(ftConfig), /*lib/ftConfig */ \
    LOGLEVEL_VAR(vmname),  /* lib/vmname */ \
+   LOGLEVEL_VAR(gpumgmt), \
+   LOGLEVEL_VAR(unityMsg),  /* mks/remote/vdpUnityVmdb */ \
+   LOGLEVEL_VAR(sharedFolderMgr),  /* mks/remote/vdpFolderSharedMgrVmdb */ \
+   LOGLEVEL_VAR(crtbora),  /* apps/crtbora */ \
+   LOGLEVEL_VAR(mirror), \
+   LOGLEVEL_VAR(filtlib), \
+   LOGLEVEL_VAR(epd), \
+   LOGLEVEL_VAR(ddecomd), \
+   LOGLEVEL_VAR(hostctl), \
+   LOGLEVEL_VAR(pmemobj), \
    /* end of list */
 
 LOGLEVEL_EXTENSION_DECLARE(LOGLEVEL_USER);
