@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -114,7 +114,7 @@ VMX86_EXTERN_DATA Msg_String const Msg_RetryCancelButtons[];
 VMX86_EXTERN_DATA Msg_String const Msg_OKCancelButtons[];
 VMX86_EXTERN_DATA Msg_String const Msg_RetryAbortButtons[];
 
-extern Msg_String const Msg_Severities[];
+VMX86_EXTERN_DATA Msg_String const Msg_Severities[];
 
 
 /*
