@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -35,15 +35,6 @@
  * (Keep all of the below offsets in hex).
  */
 
-/*
- * workerLib default completion lock
- *
- * Used for workerLib callers who don't provide their own lock. Held
- * around arbitrary completion callbacks so it probably makes sense to
- * be of a low rank.
- */
-#define RANK_workerLibCmplLock      RANK_libLockBase
-
 /*
  * hostDeviceInfo HAL lock
  *
@@ -69,16 +60,43 @@
 #define RANK_hgfsSearchArrayLock     (RANK_libLockBase + 0x4060)
 #define RANK_hgfsNodeArrayLock       (RANK_libLockBase + 0x4070)
 
-/*
- * SLPv2 global lock
- */
-#define RANK_slpv2GlobalLock         (RANK_libLockBase + 0x4305)
-
 /*
  * vigor (must be < VMDB range and < disklib, see bug 741290)
  */
-#define RANK_vigorClientLock         (RANK_libLockBase + 0x4400)
-#define RANK_vigorOfflineClientLock  (RANK_libLockBase + 0x4410)
+#define RANK_vigorOnlineLock         (RANK_libLockBase + 0x4400)
+#define RANK_vigorOfflineLock        (RANK_libLockBase + 0x4410)
+
+/*
+ * filtlib (must be > vigor and < disklib and workercmlp, PR 1340298)
+ */
+#define RANK_filtLibPollLock         (RANK_vigorOfflineLock + 1)
+
+/*
+ * filtib lock which protects a disk's allocation bitmap state.
+ * Must be > filtLibPollLock, as it could be acquire with the poll lock held.
+ * And as evidenced by PR1437159, it must also be lower than workerLibCmplLock.
+ */
+#define RANK_filtLibAllocBitmapLock (RANK_filtLibPollLock + 1)
+
+/*
+ * workerLib default completion lock
+ *
+ * Used for workerLib callers who don't provide their own lock. Held
+ * around arbitrary completion callbacks so it makes sense to be of
+ * a low rank.
+ *
+ * Must be > RANK_vigorOfflineLock because we may queue work in Vigor
+ * offline.
+ *
+ * Must be < RANK_nfcLibLock, because NFC uses AIO Generic to perform
+ * async writes to the virtual disk.
+ *
+ * Must be > RANK_filtLibPollLock so that filtlib timers can wait
+ * for queued work.
+ *
+ * Must be > RANK_filtLibAllocBitmapLock due to PR1437159.
+ */
+#define RANK_workerLibCmplLock       (RANK_filtLibAllocBitmapLock + 1)
 
 /*
  * NFC lib lock
@@ -86,9 +104,9 @@
 #define RANK_nfcLibLock              (RANK_libLockBase + 0x4505)
 
 /*
- * policy ops pending list lock
+ * Policy lib lock
  */
-#define RANK_popPendingListLock      (RANK_libLockBase + 0x4605)
+#define RANK_policyLibLock           (RANK_libLockBase + 0x4605)
 
 /*
  * disklib and I/O related locks
@@ -105,6 +123,14 @@
 #define RANK_namespaceLock           (RANK_libLockBase + 0x5080)
 #define RANK_vvolLibLock             (RANK_libLockBase + 0x5090)
 
+/*
+ * Persistent-memory logical and hardware  management locks
+ */
+/* The nvdimm layer is the hardware layer */
+#define RANK_nvdHandleLock          (RANK_libLockBase + 0x5300)
+/* The pmem layer is the logical layer */
+#define RANK_pmemHandleLock         (RANK_libLockBase + 0x5310)
+
 /*
  * VMDB range:
  * (RANK_libLockBase + 0x5500, RANK_libLockBase + 0x5600)
@@ -128,6 +154,7 @@
 #define RANK_usbEnumGlobalLock       (RANK_libLockBase + 0x6506)
 #define RANK_usbArbLibAsockLock      (RANK_libLockBase + 0x6507)
 #define RANK_usbEnumBackendLock      (RANK_libLockBase + 0x6508)
+#define RANK_sensorQueueLock         (RANK_libLockBase + 0x6509)
 
 /*
  * misc locks
@@ -159,4 +186,5 @@
 #define RANK_licenseCheckLock        (RANK_libLockBase + 0x7090)
 #define RANK_preferenceLock          (RANK_libLockBase + 0x7100)
 
+
 #endif /* _LIBMUTEXRANK_H */
