@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -31,45 +31,13 @@
 extern "C" {
 #endif
 
-#if defined(VMTOOLS_USE_GLIB)
-#  include <glib.h>
-#else
-#  include "dbllnklst.h"
-#endif
-
-/* Helper macro for porting old callbacks that currently use RpcIn_SetRetVals. */
-#define RPCIN_SETRETVALS(data, val, retVal)                                \
-   RpcIn_SetRetVals((char const **) &(data)->result, &(data)->resultLen,   \
-                    (val), (retVal))
-
 typedef void RpcIn_ErrorFunc(void *clientData, char const *status);
 
 typedef struct RpcIn RpcIn;
 
-/* Data passed to new-style RpcIn callbacks. */
-typedef struct RpcInData {
-   /* Data from the host's RPC request. */
-   const char *name;
-   const char *args;
-   size_t argsSize;
-   /* Data to be returned to the host. */
-   char *result;
-   size_t resultLen;
-   Bool freeResult;
-   /* Client data. */
-   void *appCtx;
-   void *clientData;
-} RpcInData;
+#if defined(VMTOOLS_USE_GLIB) /* { */
 
-
-/*
- * Type for RpcIn callbacks. The callback function is responsible for
- * allocating memory for the result string.
- */
-typedef Bool (*RpcIn_Callback)(RpcInData *data);
-
-
-#if defined(VMTOOLS_USE_GLIB)
+#include "vmware/tools/guestrpc.h"
 
 RpcIn *RpcIn_Construct(GMainContext *mainCtx,
                        RpcIn_Callback dispatch,
@@ -78,19 +46,21 @@ RpcIn *RpcIn_Construct(GMainContext *mainCtx,
 Bool RpcIn_start(RpcIn *in, unsigned int delay,
                  RpcIn_ErrorFunc *errorFunc, void *errorData);
 
-#else
+#else /* } { */
+
+#include "dbllnklst.h"
 
 /*
  * Type for old RpcIn callbacks. Don't use this anymore - this is here
  * for backwards compatibility.
  */
 typedef Bool
-(*RpcIn_CallbackOld)(char const **result,     // OUT
-                     size_t *resultLen,       // OUT
-                     const char *name,        // IN
-                     const char *args,        // IN
-                     size_t argsSize,         // IN
-                     void *clientData);       // IN
+(*RpcIn_Callback)(char const **result,     // OUT
+                  size_t *resultLen,       // OUT
+                  const char *name,        // IN
+                  const char *args,        // IN
+                  size_t argsSize,         // IN
+                  void *clientData);       // IN
 
 RpcIn *RpcIn_Construct(DblLnkLst_Links *eventQueue);
 
@@ -103,21 +73,18 @@ Bool RpcIn_start(RpcIn *in, unsigned int delay,
  * Use RpcIn_RegisterCallbackEx() instead.
  */
 void RpcIn_RegisterCallback(RpcIn *in, const char *name,
-                            RpcIn_CallbackOld callback, void *clientData);
+                            RpcIn_Callback callback, void *clientData);
 
-void RpcIn_RegisterCallbackEx(RpcIn *in, const char *name,
-                              RpcIn_Callback callback, void *clientData);
 void RpcIn_UnregisterCallback(RpcIn *in, const char *name);
 
-#endif
-
-void RpcIn_Destruct(RpcIn *in);
-Bool RpcIn_restart(RpcIn *in);
-Bool RpcIn_stop(RpcIn *in);
-
 unsigned int RpcIn_SetRetVals(char const **result, size_t *resultLen,
                               const char *resultVal, Bool retVal);
 
+#endif /* } */
+
+void RpcIn_Destruct(RpcIn *in);
+void RpcIn_stop(RpcIn *in);
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
