@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -125,6 +125,23 @@
 #define SC_R13(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r13)
 #define SC_R14(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r14)
 #define SC_R15(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r15)
+#elif defined(__arm__)
+#define SC_R0(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[0])
+#define SC_R1(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[1])
+#define SC_R2(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[2])
+#define SC_R3(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[3])
+#define SC_R4(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[4])
+#define SC_R5(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[5])
+#define SC_R6(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[6])
+#define SC_R7(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[7])
+#define SC_R8(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[8])
+#define SC_R9(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[9])
+#define SC_R10(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r[10])
+#define SC_FP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[11])
+#define SC_IP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[12])
+#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__sp)
+#define SC_LR(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__lr)
+#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__pc)
 #else
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__eax)
 #define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__ebx)
@@ -253,6 +270,10 @@
 #define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_sp)
 #define SC_LR(uc)  ((unsigned long) (uc)->uc_mcontext.arm_lr)
 #define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext.arm_pc)
+#elif defined(__aarch64__)
+#define SC_X(uc,n) ((unsigned long) (uc)->uc_mcontext.regs[n])
+#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext.sp)
+#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext.pc)
 #else
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EAX])
 #define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EBX])
