@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -61,7 +61,8 @@
  * This does not mean it has bsd_vsnwprintf().
  */
 #if (defined _WIN32 && !defined STR_NO_WIN32_LIBS) || \
-    (defined __linux__ && !defined __UCLIBC__) || defined __APPLE__
+    (defined __linux__ && !defined __UCLIBC__) || \
+    defined __APPLE__
 #define HAS_BSD_PRINTF 1
 #endif
 
@@ -81,37 +82,60 @@
  * NOTE: All size_t arguments and integer returns values are in bytes.
  *
  * NOTE: Str_Asprintf/Str_Vasprintf return NULL on failure, while
- * Str_SafeAsprintf/Str_SafeVasprintf ASSERT_NOT_IMPLEMENTED.
+ * Str_SafeAsprintf/Str_SafeVasprintf VERIFY.
  *
  * NOTE: "%s" refers to strings of "char" units, while "%S" refers to
  * strings of "wchar_t" units, regardless of platform.
  */
 
 #ifdef HAS_BSD_PRINTF
-int Str_Sprintf_C_Locale(char *buf, size_t max,
-                         const char *fmt, ...) PRINTF_DECL(3, 4);
+int Str_Sprintf_C_Locale(char *buf,               // OUT:
+                         size_t max,              // IN:
+                         const char *fmt,         // IN:
+                         ...) PRINTF_DECL(3, 4);  // IN:
 #endif
 
-int Str_Sprintf(char *buf, size_t max,
-                const char *fmt, ...) PRINTF_DECL(3, 4);
-int Str_Snprintf(char *buf, size_t len,
-                 const char *fmt, ...) PRINTF_DECL(3, 4);
-int Str_Vsnprintf(char *buf, size_t len,
-                  const char *fmt, va_list args);
-size_t Str_Strlen(const char *src, size_t maxLen);
-char *Str_Strnstr(const char *src, const char *sub, size_t n);
-char *Str_Strcpy(char *dst, const char *src, size_t maxLen);
-char *Str_Strcat(char *dst, const char *src, size_t maxLen);
-char *Str_Strncat(char *buf, size_t bufSize, const char *src, size_t n);
+int Str_Sprintf(char *buf,               // OUT:
+                size_t max,              // IN:
+                const char *fmt,         // IN:
+                ...) PRINTF_DECL(3, 4);  // IN:
+int Str_Snprintf(char *buf,               // OUT:
+                 size_t len,              // IN:
+                 const char *fmt,         // IN:
+                 ...) PRINTF_DECL(3, 4);  // IN:
+int Str_Vsnprintf(char *buf,        // OUT:
+                  size_t len,       // IN:
+                  const char *fmt,  // IN:
+                  va_list args);    // IN:
 
-char *Str_Asprintf(size_t *length,
-                   const char *format, ...) PRINTF_DECL(2, 3);
-char *Str_Vasprintf(size_t *length, const char *format,
-                    va_list arguments);
-char *Str_SafeAsprintf(size_t *length,
-                       const char *format, ...) PRINTF_DECL(2, 3);
-char *Str_SafeVasprintf(size_t *length, const char *format,
-                        va_list arguments);
+size_t Str_Strlen(const char *src,  // IN:
+                  size_t maxLen);   // IN:
+char *Str_Strnstr(const char *src,  // IN:
+                  const char *sub,  // IN:
+                  size_t n);        // IN:
+char *Str_Strcpy(char *dst,        // OUT:
+                 const char *src,  // IN:
+                 size_t maxLen);   // IN:
+char *Str_Strcat(char *dst,        // IN/OUT:
+                 const char *src,  // IN:
+                 size_t maxLen);   // IN:
+char *Str_Strncat(char *buf,        // IN/OUT:
+                  size_t bufSize,   // IN:
+                  const char *src,  // IN:
+                  size_t n);        // IN:
+
+char *Str_Asprintf(size_t *length,          // OUT/OPT:
+                   const char *format,      // IN:
+                   ...) PRINTF_DECL(2, 3);  // IN:
+char *Str_Vasprintf(size_t *length,      // OUT/OPT:
+                    const char *format,  // IN:
+                    va_list arguments);  // IN:
+char *Str_SafeAsprintf(size_t *length,          // OUT/OPT:
+                       const char *format,      // IN:
+                       ...) PRINTF_DECL(2, 3);  // IN:
+char *Str_SafeVasprintf(size_t *length,      // OUT/OPT:
+                        const char *format,  // IN:
+                        va_list arguments);  // IN:
 
 #if defined(_WIN32) || defined(__linux__) // {
 
@@ -122,36 +146,54 @@ char *Str_SafeVasprintf(size_t *length, const char *format,
  * wchar_ts, not bytes.
  *
  * NOTE: Str_Aswprintf/Str_Vaswprintf return NULL on failure, while
- * Str_SafeAswprintf/Str_SafeVaswprintf ASSERT_NOT_IMPLEMENTED.
+ * Str_SafeAswprintf/Str_SafeVaswprintf VERIFY.
  *
  * NOTE: "%s" refers to strings of "char" units, while "%S" refers to
  * strings of "wchar_t" units, regardless of platform.
  */
 
-int Str_Swprintf(wchar_t *buf, size_t max,
-                 const wchar_t *fmt, ...);
-int Str_Snwprintf(wchar_t *buf, size_t len,
-                  const wchar_t *fmt, ...);
-int Str_Vsnwprintf(wchar_t *buf, size_t len,
-                   const wchar_t *fmt, va_list args);
-wchar_t *Str_Wcscpy(wchar_t *dst, const wchar_t *src, size_t maxLen);
-wchar_t *Str_Wcscat(wchar_t *dst, const wchar_t *src, size_t maxLen);
-wchar_t *Str_Wcsncat(wchar_t *buf, size_t bufSize, const wchar_t *src,
-                     size_t n);
+int Str_Swprintf(wchar_t *buf,        // OUT:
+                 size_t max,          // IN:
+                 const wchar_t *fmt,  // IN:
+                 ...);
+int Str_Snwprintf(wchar_t *buf,        // OUT:
+                  size_t len,          // IN:
+                  const wchar_t *fmt,  // IN:
+                  ...);
+int Str_Vsnwprintf(wchar_t *buf,        // OUT:
+                   size_t len,          // IN:
+                   const wchar_t *fmt,  // IN:
+                   va_list args);
+wchar_t *Str_Wcscpy(wchar_t *dst,        // OUT:
+                    const wchar_t *src,  // IN:
+                    size_t maxLen);      // IN:
+wchar_t *Str_Wcscat(wchar_t *dst,        // IN/OUT:
+                    const wchar_t *src,  // IN:
+                    size_t maxLen);      // IN:
+wchar_t *Str_Wcsncat(wchar_t *buf,        // IN/OUT:
+                     size_t bufSize,      // IN:
+                     const wchar_t *src,  // IN:
+                     size_t n);           // IN:
 
-wchar_t *Str_Aswprintf(size_t *length,
-                       const wchar_t *format, ...);
-wchar_t *Str_Vaswprintf(size_t *length, const wchar_t *format,
-                        va_list arguments);
-wchar_t *Str_SafeAswprintf(size_t *length,
-                           const wchar_t *format, ...);
-wchar_t *Str_SafeVaswprintf(size_t *length, const wchar_t *format,
-                            va_list arguments);
+wchar_t *Str_Aswprintf(size_t *length,         // OUT/OPT:
+                       const wchar_t *format,  // IN:
+                       ...);                   // IN:
+wchar_t *Str_Vaswprintf(size_t *length,         // OUT/OPT:
+                        const wchar_t *format,  // IN:
+                        va_list arguments);     // IN:
+wchar_t *Str_SafeAswprintf(size_t *length,         // OUT/OPT:
+                           const wchar_t *format,  // IN:
+                           ...);                   // IN:
+wchar_t *Str_SafeVaswprintf(size_t *length,         // OUT/OPT:
+                            const wchar_t *format,  // IN:
+                            va_list arguments);     // IN:
 
-unsigned char *Str_Mbscpy(char *buf, const char *src,
-                          size_t maxSize);
-unsigned char *Str_Mbscat(char *buf, const char *src,
-                          size_t maxSize);
+unsigned char *Str_Mbscpy(char *buf,        // OUT:
+                          const char *src,  // IN:
+                          size_t maxSize);  // IN:
+unsigned char *Str_Mbscat(char *buf,        // IN/OUT:
+                          const char *src,  // IN:
+                          size_t maxSize);  // IN:
 
 /*
  * These are handly for Windows programmers.  They are like
