@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -36,7 +36,7 @@
 
 uint64 System_GetTimeMonotonic(void);
 uint64 System_Uptime(void);
-Unicode System_GetTimeAsString(void);
+char *System_GetTimeAsString(void);
 void System_Shutdown(Bool reboot);
 Bool System_GetNodeName(size_t outBufSize, char *outBuf);
 
@@ -78,6 +78,8 @@ DWORD System_GetServiceState(LPCWSTR szServiceName);
 DblLnkLst_Links *System_GetMonitors();
 void System_SetFocusedWindow(HWND windowToFocus, Bool force);
 Bool System_EnableDesktopComposition(BOOL enabled);
+LPWSTR System_GetImageFilePathForWindow(HWND hwnd);
+HANDLE System_OpenProcessForHWND(DWORD mask, BOOL inherit, HWND hwnd);
 #endif
 
 
