@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2012 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -32,7 +32,9 @@
 
 #include <stdarg.h>
 #include <string.h>
+#ifndef VMKBOOT
 #include <stdlib.h>
+#endif
 
 #ifdef _WIN32
    #ifdef USERLEVEL
@@ -40,10 +42,12 @@
       #include <windows.h> /* for definition of HANDLE */
    #endif
 #else
+   #include <unistd.h>
    #include <sys/types.h>
    #include "errno.h"
 #endif
 #include "vm_assert.h"
+#include "vm_basic_defs.h"
 #include "unicodeTypes.h"
 
 
@@ -52,14 +56,19 @@
  */
 #if defined(_WIN32)
    typedef DWORD Util_ThreadID;
+#elif defined(__linux__) || defined(__ANDROID__)
+   typedef pid_t Util_ThreadID;
+#elif defined(__APPLE__)
+#  include <pthread.h>
+   typedef mach_port_t Util_ThreadID;
+#elif defined(__sun__)
+#  include <thread.h>
+   typedef thread_t Util_ThreadID;
+#elif defined(__FreeBSD__)
+#  include <pthread.h>
+   typedef pthread_t Util_ThreadID;
 #else
-   #include <unistd.h>
-   #if defined(__APPLE__) || defined(__FreeBSD__)
-      #include <pthread.h>
-      typedef pthread_t Util_ThreadID;
-   #else // Linux et al
-      typedef pid_t Util_ThreadID;
-   #endif
+#  error "Need typedef for Util_ThreadID"
 #endif
 
 uint32 CRC_Compute(const uint8 *buf, int len);
@@ -67,7 +76,7 @@ uint32 Util_Checksum32(const uint32 *buf, int len);
 uint32 Util_Checksum(const uint8 *buf, int len);
 uint32 Util_Checksumv(void *iov, int numEntries);
 uint32 Util_HashString(const char *str);
-Unicode Util_ExpandString(ConstUnicode fileName);
+char *Util_ExpandString(const char *fileName);
 void Util_ExitThread(int);
 NORETURN void Util_ExitProcessAbruptly(int);
 int Util_HasAdminPriv(void);
@@ -100,11 +109,20 @@ typedef struct UtilSingleUseResource UtilSingleUseResource;
 UtilSingleUseResource *Util_SingleUseAcquire(const char *name);
 void Util_SingleUseRelease(UtilSingleUseResource *res);
 
+#ifndef _WIN32
+Bool Util_IPv4AddrValid(const char *addr);
+Bool Util_IPv6AddrValid(const char *addr);
+Bool Util_IPAddrValid(const char *addr);
+#endif
 
 #if defined(__linux__) || defined(__FreeBSD__) || defined(sun)
 Bool Util_GetProcessName(pid_t pid, char *bufOut, size_t bufOutSize);
 #endif
 
+#if defined __linux__ && !defined VMX86_SERVER
+Bool Util_IsPhysicalSSD(const char* device);
+#endif
+
 // backtrace functions and utilities
 
 #define UTIL_BACKTRACE_LINE_LEN (511)
@@ -133,14 +151,15 @@ int Util_CompareDotted(const char *s1, const char *s2);
  * This enum defines how Util_GetOpt should handle non-option arguments:
  *
  * UTIL_NONOPT_PERMUTE: Permute argv so that all non-options are at the end.
- * UTIL_NONOPT_STOP:    Stop when first non-option argument is seen.
+ * UTIL_NONOPT_STOP:    Stop when first non-option argument is seen. (This is
+ *                      the standard POSIX behavior.)
  * UTIL_NONOPT_ALL:     Return each non-option argument as if it were
  *                      an option with character code 1.
  */
 typedef enum { UTIL_NONOPT_PERMUTE, UTIL_NONOPT_STOP, UTIL_NONOPT_ALL } Util_NonOptMode;
 struct option;
 int Util_GetOpt(int argc, char * const *argv, const struct option *opts,
-                Util_NonOptMode mode);
+                Util_NonOptMode mode, Bool manualErrorHandling);
 
 
 #if defined(VMX86_STATS)
@@ -152,9 +171,10 @@ Bool Util_QueryCStResidency(uint32 *numCpus, uint32 *numCStates,
 /*
  * In util_shared.h
  */
+#define UTIL_FASTRAND_SEED_MAX (0x7fffffff)
 Bool Util_Throttle(uint32 count);
 uint32 Util_FastRand(uint32 seed);
-
+uint64 Util_CheckSum64(uint32 *data, unsigned numWords);
 
 // Not thread safe!
 void Util_OverrideHomeDir(const char *path);
@@ -270,12 +290,16 @@ Bool Util_MakeSureDirExistsAndAccessible(char const *path,
 #   define DIRSEPC_W	      L'\\'
 #   define VALID_DIRSEPS      "\\/"
 #   define VALID_DIRSEPS_W    L"\\/"
+#   define CUR_DIRS_W         L"."
+#   define CUR_DIRC_W         L'.'
 #else
 #   define DIRSEPS	      "/"
 #   define DIRSEPC	      '/'
 #   define VALID_DIRSEPS      DIRSEPS
 #endif
 
+#define CURR_DIRS             "."
+#define CURR_DIRC             '.'
 
 /*
  *-----------------------------------------------------------------------
@@ -332,7 +356,7 @@ char *UtilSafeStrndup0(const char *s, size_t n);
 char *UtilSafeStrndup1(const char *s, size_t n,
                       int bugNumber, const char *file, int lineno);
 
-/* 
+/*
  * Debug builds carry extra arguments into the allocation functions for
  * better error reporting. Non-debug builds don't pay this extra overhead.
  */
@@ -435,6 +459,12 @@ Util_Zero(void *buf,       // OUT
       SecureZeroMemory(buf, bufSize);
 #else
       memset(buf, 0, bufSize);
+#if !defined _WIN32
+      /*
+       * Memset calls before free might be optimized out.  See PR1248269.
+       */
+      __asm__ __volatile__("" : : "r"(&buf) : "memory");
+#endif
 #endif
    }
 }
@@ -457,7 +487,7 @@ Util_Zero(void *buf,       // OUT
  */
 
 static INLINE void
-Util_ZeroString(char *str)  // IN/OUT
+Util_ZeroString(char *str)  // IN/OUT/OPT
 {
    if (str != NULL) {
       Util_Zero(str, strlen(str));
@@ -465,6 +495,7 @@ Util_ZeroString(char *str)  // IN/OUT
 }
 
 
+#ifndef VMKBOOT
 /*
  *-----------------------------------------------------------------------------
  *
@@ -483,7 +514,7 @@ Util_ZeroString(char *str)  // IN/OUT
  */
 
 static INLINE void
-Util_ZeroFree(void *buf,       // OUT
+Util_ZeroFree(void *buf,       // OUT/OPT
               size_t bufSize)  // IN
 {
    if (buf != NULL) {
@@ -511,7 +542,7 @@ Util_ZeroFree(void *buf,       // OUT
  */
 
 static INLINE void
-Util_ZeroFreeString(char *str)  // IN
+Util_ZeroFreeString(char *str)  // IN/OUT/OPT
 {
    if (str != NULL) {
       Util_ZeroString(str);
@@ -539,7 +570,7 @@ Util_ZeroFreeString(char *str)  // IN
  */
 
 static INLINE void
-Util_ZeroFreeStringW(wchar_t *str)  // IN
+Util_ZeroFreeStringW(wchar_t *str)  // IN/OUT/OPT
 {
    if (str != NULL) {
       Util_Zero(str, wcslen(str) * sizeof *str);
@@ -573,7 +604,7 @@ Util_ZeroFreeStringW(wchar_t *str)  // IN
  */
 
 static INLINE void
-Util_FreeList(void **list,      // IN/OUT: the list to free
+Util_FreeList(void **list,      // IN/OUT/OPT: the list to free
               ssize_t length)   // IN: the length
 {
    if (list == NULL) {
@@ -600,35 +631,54 @@ Util_FreeList(void **list,      // IN/OUT: the list to free
 }
 
 static INLINE void
-Util_FreeStringList(char **list,      // IN/OUT: the list to free
+Util_FreeStringList(char **list,      // IN/OUT/OPT: the list to free
                     ssize_t length)   // IN: the length
 {
    Util_FreeList((void **) list, length);
 }
+#endif
+
 
-#ifndef _WIN32
 /*
  *-----------------------------------------------------------------------------
  *
- * Util_IsFileDescriptorOpen --
+ * Util_Memcpy32 --
  *
- *      Check if given file descriptor is open.
+ *      Special purpose version of memcpy that requires nbytes be a
+ *      multiple of 4.  This assumption lets us have a very small,
+ *      inlineable implementation.
  *
  * Results:
- *      TRUE if fd is open.
+ *      dst
  *
  * Side effects:
- *      Clobbers errno.
+ *      See above.
  *
  *-----------------------------------------------------------------------------
  */
 
-static INLINE Bool
-Util_IsFileDescriptorOpen(int fd)   // IN
+static INLINE void *
+Util_Memcpy32(void *dst,
+              const void *src,
+              size_t nbytes)
 {
-   lseek(fd, 0L, SEEK_CUR);
-   return errno != EBADF;
+   ASSERT((nbytes % 4) == 0);
+#if defined __GNUC__ && (defined(__i386__) || defined(__x86_64__))
+   do {
+      int dummy0, dummy1, dummy2;
+      __asm__ __volatile__(
+           "cld \n\t"
+           "rep ; movsl"  "\n\t"
+        : "=&c" (dummy0), "=&D" (dummy1), "=&S" (dummy2)
+        : "0" (nbytes / 4), "1" ((long) dst), "2" ((long) src)
+        : "memory", "cc"
+      );
+      return dst;
+   } while (0);
+#else
+   return memcpy(dst, src, nbytes);
+#endif
 }
-#endif /* !_WIN32 */
+
 
 #endif /* UTIL_H */
