@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2003-2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2003-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,9 +26,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of VMware Inc. nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission of VMware Inc.
  *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -68,7 +65,6 @@
 #define _VM_BASIC_DEFS_H_
 
 #define INCLUDE_ALLOW_USERLEVEL
-
 #define INCLUDE_ALLOW_MODULE
 #define INCLUDE_ALLOW_VMMON
 #define INCLUDE_ALLOW_VMKERNEL
@@ -76,7 +72,6 @@
 #define INCLUDE_ALLOW_VMK_MODULE
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 #include "vm_basic_types.h" // For INLINE.
 
@@ -98,23 +93,42 @@
  * Simple macros
  */
 
+#ifndef vmw_offsetof
+#define vmw_offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
+#endif
+
 #if (defined __APPLE__ || defined __FreeBSD__) && \
     (!defined KERNEL && !defined _KERNEL && !defined VMKERNEL && !defined __KERNEL__)
 #   include <stddef.h>
 #else
-// XXX the _WIN32 one matches that of VC++, to prevent redefinition warning
-// XXX the other one matches that of gcc3.3.3/glibc2.2.4 to prevent redefinition warnings
 #ifndef offsetof
+#define VMW_DEFINED_OFFSETOF
+
+/*
+ * XXX While the _WIN32 implementation appears to be identical to vmw_offsetof
+ * in terms of behavior, they need to be separate to match verbatim the
+ * definition used by the respective compilers, to avoid a redefinition warning.
+ *
+ * This is necessary until we eliminate the inclusion of <windows.h> above.
+ */
 #ifdef _WIN32
 #define offsetof(s,m)   (size_t)&(((s *)0)->m)
+/*
+ * We use the builtin offset for gcc/clang, except when we're running under the
+ * vmkernel's GDB macro preprocessor, since gdb doesn't understand
+ * __builtin_offsetof.
+ */
+#elif defined __GNUC__ && !defined VMKERNEL_GDB_MACRO_BUILDER
+#define offsetof __builtin_offsetof
 #else
-#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
-#endif
+#define offsetof vmw_offsetof
 #endif
+
+#endif // offsetof
 #endif // __APPLE__
 
 #define VMW_CONTAINER_OF(ptr, type, member) \
-   ((type *)((char *)(ptr) - offsetof(type, member)))
+   ((type *)((char *)(ptr) - vmw_offsetof(type, member)))
 
 #ifndef ARRAYSIZE
 #define ARRAYSIZE(a) (sizeof (a) / sizeof *(a))
@@ -234,6 +248,8 @@ Max(int a, int b)
    #define PAGE_SHIFT    12
 #elif defined __APPLE__
    #define PAGE_SHIFT    12
+#elif defined VM_ARM_64
+   #define PAGE_SHIFT    12
 #elif defined __arm__
    #define PAGE_SHIFT    12
 #else
@@ -242,7 +258,7 @@ Max(int a, int b)
 #endif // }
 
 #ifndef PAGE_SIZE
-#define PAGE_SIZE     (1<<PAGE_SHIFT)
+#define PAGE_SIZE     (1 << PAGE_SHIFT)
 #endif
 
 #ifndef PAGE_MASK
@@ -250,11 +266,15 @@ Max(int a, int b)
 #endif
 
 #ifndef PAGE_OFFSET
-#define PAGE_OFFSET(_addr)  ((uintptr_t)(_addr)&(PAGE_SIZE-1))
+#define PAGE_OFFSET(_addr)  ((uintptr_t)(_addr) & (PAGE_SIZE - 1))
+#endif
+
+#ifndef PAGE_NUMBER
+#define PAGE_NUMBER(_addr)  ((uintptr_t)(_addr) / PAGE_SIZE)
 #endif
 
 #ifndef VM_PAGE_BASE
-#define VM_PAGE_BASE(_addr)  ((_addr)&~(PAGE_SIZE-1))
+#define VM_PAGE_BASE(_addr)  ((_addr) & ~(PAGE_SIZE - 1))
 #endif
 
 #ifndef VM_PAGES_SPANNED
@@ -294,9 +314,17 @@ Max(int a, int b)
 #define MBYTES_2_BYTES(_nbytes) ((uint64)(_nbytes) << 20)
 #endif
 
+#ifndef BYTES_2_GBYTES
+#define BYTES_2_GBYTES(_nbytes) ((_nbytes) >> 30)
+#endif
+
+#ifndef GBYTES_2_BYTES
+#define GBYTES_2_BYTES(_nbytes) ((uint64)(_nbytes) << 30)
+#endif
+
 #ifndef VM_PAE_LARGE_PAGE_SHIFT
 #define VM_PAE_LARGE_PAGE_SHIFT 21
-#endif 
+#endif
 
 #ifndef VM_PAE_LARGE_PAGE_SIZE
 #define VM_PAE_LARGE_PAGE_SIZE (1 << VM_PAE_LARGE_PAGE_SHIFT)
@@ -310,10 +338,6 @@ Max(int a, int b)
 #define VM_PAE_LARGE_2_SMALL_PAGES (BYTES_2_PAGES(VM_PAE_LARGE_PAGE_SIZE))
 #endif
 
-#ifndef NR_MPNS_PER_PAGE
-#define NR_MPNS_PER_PAGE (PAGE_SIZE / sizeof(MPN))
-#endif
-
 /*
  * Word operations
  */
@@ -332,8 +356,12 @@ Max(int a, int b)
 #define HIBYTE(_w)    (((_w) >> 8) & 0xff)
 #endif
 
+#ifndef HIDWORD
 #define HIDWORD(_qw)   ((uint32)((_qw) >> 32))
+#endif
+#ifndef LODWORD
 #define LODWORD(_qw)   ((uint32)(_qw))
+#endif
 #define QWORD(_hi, _lo)   ((((uint64)(_hi)) << 32) | ((uint32)(_lo)))
 
 
@@ -368,39 +396,13 @@ void *_ReturnAddress(void);
 #ifdef __GNUC__
 #ifndef sun
 
-static INLINE_SINGLE_CALLER uintptr_t
-GetFrameAddr(void)
-{
-   uintptr_t bp;
-#if  !(__GNUC__ == 4 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1))
-   bp = (uintptr_t)__builtin_frame_address(0);
-#else
-   /*
-    * We use this assembly hack due to a bug discovered in gcc 4.1.1.
-    * The bug was fixed in 4.2.0; assume it originated with 4.0.
-    * PR147638, PR554369.
-    */
-     __asm__ __volatile__(
-#  if defined(VM_X86_64)
-                          "movq %%rbp, %0\n"
-#  else
-                          "movl %%ebp, %0\n"
-#  endif
-                          : "=g" (bp));
-#endif
-   return bp;
-}
-
-
 /*
- * Returns the frame pointer of the calling function.
- * Equivalent to __builtin_frame_address(1).
+ * A bug in __builtin_frame_address was discovered in gcc 4.1.1, and
+ * fixed in 4.2.0; assume it originated in 4.0. PR 147638 and 554369.
  */
-static INLINE_SINGLE_CALLER uintptr_t
-GetCallerFrameAddr(void)
-{
-   return *(uintptr_t*)GetFrameAddr();
-}
+#if  !(__GNUC__ == 4 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1))
+#define GetFrameAddr() __builtin_frame_address(0)
+#endif
 
 #endif // sun
 #endif // __GNUC__
@@ -573,10 +575,32 @@ typedef int pid_t;
 #endif
 #endif
 
+/*
+ * Convenience definitions of unicode characters.
+ */
+
+#ifndef UTF8_ELLIPSIS
+#define UTF8_ELLIPSIS "\xe2\x80\xa6"
+#endif
+
 /*
  * Convenience macros and definitions. Can often be used instead of #ifdef.
  */
 
+#undef ARM64_ONLY
+#ifdef VM_ARM_64
+#define ARM64_ONLY(x)    x
+#else
+#define ARM64_ONLY(x)
+#endif
+
+#undef X86_ONLY
+#ifdef VM_X86_ANY
+#define X86_ONLY(x)      x
+#else
+#define X86_ONLY(x)
+#endif
+
 #undef DEBUG_ONLY
 #ifdef VMX86_DEBUG
 #define vmx86_debug      1
@@ -754,4 +778,48 @@ typedef int pid_t;
 #define VISIBILITY_HIDDEN /* nothing */
 #endif
 
+
+/*
+ * Bitfield extraction.
+ */
+
+#define EXTRACT_BITSLICE32(_val , _lsb, _msb)  \
+   (((uint32)(_val) << (31 - (_msb))) >> ((31 - (_msb)) + (_lsb)))
+#define EXTRACT_BITFIELD32(_val, _pos, _len) \
+   EXTRACT_BITSLICE32((_val), (_pos), ((_pos) + (_len) - 1))
+#define EXTRACT_BITSLICE64(_val, _lsb, _msb) \
+   (((uint64)(_val) << (63 - (_msb))) >> ((63 - (_msb)) + (_lsb)))
+#define EXTRACT_BITFIELD64(_val, _pos, _len) \
+   EXTRACT_BITSLICE64((_val), (_pos), ((_pos) + (_len) - 1))
+
+/*
+ * Typical cache line size.  Use this for aligning structures to cache
+ * lines for performance, but do not rely on it for correctness.
+ *
+ * On x86, all current processors newer than P4 have 64-byte lines,
+ * but P4 had 128.
+ *
+ * On ARM, the line size can vary between cores.  64-byte lines are
+ * common, but either larger or smaller powers of two are possible.
+ */
+#define CACHELINE_SIZE             64
+#define CACHELINE_SHIFT            6
+#define CACHELINE_ALIGNMENT_MASK   (CACHELINE_SIZE - 1)
+
+
+/*
+ * Bits to bytes sizes.
+ */
+
+#define SIZE_8BIT   1
+#define SIZE_16BIT  2
+#define SIZE_24BIT  3
+#define SIZE_32BIT  4
+#define SIZE_48BIT  6
+#define SIZE_64BIT  8
+#define SIZE_80BIT  10
+#define SIZE_128BIT 16
+#define SIZE_256BIT 32
+
+
 #endif // ifndef _VM_BASIC_DEFS_H_
