@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,9 +26,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of VMware Inc. nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission of VMware Inc.
  *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -78,7 +75,6 @@
 #define INCLUDE_ALLOW_VMK_MODULE
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 
 /* STRICT ANSI means the Xserver build and X defines Bool differently. */
@@ -108,18 +104,62 @@ typedef char           Bool;
 #  define __i386__
 #endif
 
+/*
+ * Setup a bunch of defines for instruction set architecture (ISA) related
+ * properties.
+ *
+ * For compiler types/size:
+ *
+ * - VM_32BIT for a 32-bit ISA (with the same C compiler types/sizes as 32-bit
+ *   x86/ARM).
+ * - VM_64BIT for a 64-bit ISA (with the same C compiler types/sizes as 64-bit
+ *   x86/ARM).
+ *
+ * For a given <arch> in {X86, ARM}:
+ *
+ * - VM_<arch>_32 for the 32-bit variant.
+ * - VM_<arch>_64 for the 64-bit variant.
+ * - VM_<arch>_ANY for any variant of <arch>.
+ *
+ * VM_X86_ANY is synonymous with the confusing and deprecated VM_I386 (which
+ * should really be VM_X86_32).
+ */
+
 #ifdef __i386__
+/* VM_I386 is historically synonymous with VM_X86_ANY in bora, but misleading,
+ * since it is confused with the __i386__ gcc but defined for both 32- and
+ * 64-bit x86. We retain it here for legacy compatibility.
+ */
 #define VM_I386
+#define VM_X86_32
+#define VM_X86_ANY
+#define VM_32BIT
 #endif
 
 #ifdef __x86_64__
 #define VM_X86_64
 #define VM_I386
+#define VM_X86_ANY
+#define VM_64BIT
 #define vm_x86_64 (1)
 #else
 #define vm_x86_64 (0)
 #endif
 
+#ifdef __arm__
+#define VM_ARM_32
+#define VM_ARM_ANY
+#define VM_32BIT
+#endif
+
+#ifdef __aarch64__
+#define VM_ARM_64
+#define VM_ARM_ANY
+#define VM_64BIT
+#define vm_arm_64 (1)
+#else
+#define vm_arm_64 (0)
+#endif
 
 #ifdef _MSC_VER
 
@@ -134,7 +174,31 @@ typedef char           Bool;
 
 #endif
 
-#if defined(__APPLE__) || defined(HAVE_STDINT_H)
+#if defined(__linux__) && defined(__cplusplus) && __cplusplus >= 201103L
+
+/*
+ * We're using stdint.h instead of cstdint below because of libstdcpp.cpp.
+ * It looks like a C++ file. When being preprocessed all the C++ specific
+ * defines(e.g. __cplusplus) are set, but the C++ include paths are not.
+ */
+#include <stdint.h>
+
+typedef char          Bool;
+
+typedef uint64_t    uint64;
+typedef  int64_t     int64;
+typedef uint32_t    uint32;
+typedef  int32_t     int32;
+typedef uint16_t    uint16;
+typedef  int16_t     int16;
+typedef  uint8_t     uint8;
+typedef   int8_t      int8;
+
+typedef uint64 BA;
+typedef uint64 MA;
+typedef uint32 MPN32;
+
+#elif defined(__APPLE__) || defined(HAVE_STDINT_H)
 
 /*
  * TODO: This is a C99 standard header.  We should be able to test for
@@ -173,7 +237,7 @@ typedef signed __int64 int64;
 #elif __GNUC__
 /* The Xserver source compiles with -ansi -pendantic */
 #   if !defined(__STRICT_ANSI__) || defined(__FreeBSD__)
-#      if defined(VM_X86_64)
+#      if defined(VM_X86_64) || defined(VM_ARM_64)
 typedef unsigned long uint64;
 typedef long int64;
 #      else
@@ -255,24 +319,30 @@ typedef signed char        int8;
 #   else
 #      if !defined(__intptr_t_defined) && !defined(intptr_t)
 #         ifdef VM_I386
-#         define __intptr_t_defined
-#            ifdef VM_X86_64
+#            define __intptr_t_defined
+#            if defined(VM_X86_64)
 typedef int64     intptr_t;
 #            else
 typedef int32     intptr_t;
 #            endif
+#         elif defined(VM_ARM_64)
+#            define __intptr_t_defined
+typedef int64     intptr_t;
 #         elif defined(__arm__)
+#            define __intptr_t_defined
 typedef int32     intptr_t;
 #         endif
 #      endif
 
 #      ifndef _STDINT_H
 #         ifdef VM_I386
-#            ifdef VM_X86_64
+#            if defined(VM_X86_64)
 typedef uint64    uintptr_t;
 #            else
 typedef uint32    uintptr_t;
 #            endif
+#         elif defined(VM_ARM_64)
+typedef uint64    uintptr_t;
 #         elif defined(__arm__)
 typedef uint32    uintptr_t;
 #         endif
@@ -293,7 +363,8 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
 /*
  * Printf format specifiers for size_t and 64-bit number.
  * Use them like this:
- *    printf("%"FMT64"d\n", big);
+ *    printf("%" FMT64 "d\n", big);
+ * The spaces are important for C++11 compatibility.
  *
  * FMTH is for handles/fds.
  */
@@ -329,7 +400,7 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
        * ((__FreeBSD__ + 0) < 5).  No, we can't remove "+ 0" from
        * ((__FreeBSD__ + 0) < 5).
        */
-      #ifdef VM_X86_64
+      #if defined(VM_X86_64) || defined(VM_ARM_64)
          #define FMTSZ  "l"
          #define FMTPD  "l"
       #else
@@ -343,7 +414,7 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
       /* BSD, Linux */
       #define FMTSZ     "z"
 
-      #if defined(VM_X86_64)
+      #if defined(VM_X86_64) || defined(VM_ARM_64)
          #define FMTPD  "l"
       #else
          #define FMTPD  ""
@@ -351,13 +422,13 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
    #else
       /* Systems with a pre-C99 libc */
       #define FMTSZ     "Z"
-      #ifdef VM_X86_64
+      #if defined(VM_X86_64) || defined(VM_ARM_64)
          #define FMTPD  "l"
       #else
          #define FMTPD  ""
       #endif
    #endif
-   #ifdef VM_X86_64
+   #if defined(VM_X86_64) || defined(VM_ARM_64)
       #define FMT64     "l"
    #elif defined(sun) || defined(__FreeBSD__)
       #define FMT64     "ll"
@@ -388,7 +459,7 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
 #define CONST64(c) c##LL
 #define CONST64U(c) c##uLL
 #elif __GNUC__
-#ifdef VM_X86_64
+#if defined(VM_X86_64) || defined(VM_ARM_64)
 #define CONST64(c) c##L
 #define CONST64U(c) c##uL
 #else
@@ -408,7 +479,7 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
  * more than 31 on a 64-bit compile.
  */
 
-#ifdef VM_X86_64
+#if defined(VM_X86_64) || defined(VM_ARM_64)
     #define CONST3264(a) CONST64(a)
     #define CONST3264U(a) CONST64U(a)
 #else
@@ -455,6 +526,9 @@ typedef uintptr_t VPN;
 typedef uint64    PA;
 typedef uint32    PPN;
 
+typedef uint64    TPA;
+typedef uint32    TPPN;
+
 typedef uint64    PhysMemOff;
 typedef uint64    PhysMemSize;
 
@@ -462,7 +536,16 @@ typedef uint64    PhysMemSize;
 #ifndef __STRICT_ANSI__
 typedef uint64    BA;
 #endif
-typedef uint32    BPN;
+
+#ifdef VMKERNEL
+typedef void     *BPN;
+#else
+typedef uint64    BPN;
+#endif
+
+#define UINT64_2_BPN(u) ((BPN)(u))
+#define BPN_2_UINT64(b) ((uint64)(b))
+
 typedef uint32    PageNum;
 typedef unsigned      MemHandle;
 typedef unsigned int  IoHandle;
@@ -489,10 +572,25 @@ typedef User_CartelID User_CartelGroupID;
 typedef uint32 Worldlet_ID;
 #define INVALID_WORLDLET_ID ((Worldlet_ID)-1)
 
+typedef  int8    Reg8;
+typedef  int16   Reg16;
+typedef  int32   Reg32;
+typedef  int64   Reg64;
+
+typedef uint8   UReg8;
+typedef uint16  UReg16;
+typedef uint32  UReg32;
+typedef uint64  UReg64;
+
+#if defined(VMM) || defined(COREQUERY) || defined(EXTDECODER) ||  \
+    defined (VMKERNEL) || defined (VMKBOOT)
+typedef  Reg64  Reg;
+typedef UReg64 UReg;
+#endif
 /* The Xserver source compiles with -ansi -pendantic */
 #ifndef __STRICT_ANSI__
 typedef uint64 MA;
-typedef uint32 MPN;
+typedef uint32 MPN32;
 #endif
 
 /*
@@ -533,8 +631,6 @@ typedef uint32 LA32;
 typedef uint32 LPN32;
 typedef uint32 PA32;
 typedef uint32 PPN32;
-typedef uint32 MA32;
-typedef uint32 MPN32;
 
 /*
  * On 64 bit platform, the following types are the same as the
@@ -547,7 +643,14 @@ typedef uint64 LPN64;
 typedef uint64 PA64;
 typedef uint64 PPN64;
 typedef uint64 MA64;
-typedef uint64 MPN64;
+typedef uint64 MPN;
+
+/*
+ * IO device DMA virtual address and page number (translated by IOMMU to
+ * MA/MPN). IOPN can be in the inclusive range 0 -> MAX_IOPN.
+ */
+typedef uint64 IOA;
+typedef uint64 IOPN;
 
 /*
  * VA typedefs for user world apps.
@@ -564,39 +667,37 @@ typedef void * UserVA;
 #endif
 
 
-/*
- * Maximal possible PPN value (errors too) that PhysMem can handle.
- * Must be at least as large as MAX_PPN which is the maximum PPN
- * for any region other than buserror.
- */
-#define PHYSMEM_MAX_PPN   ((PPN)0xffffffff)
-#define MAX_PPN           ((PPN)0x3fffffff) /* Maximal observable PPN value. */
+#define MAX_PPN_BITS      31
+#define MAX_PPN           (((PPN)1 << MAX_PPN_BITS) - 1) /* Maximal observable PPN value. */
 #define INVALID_PPN       ((PPN)0xffffffff)
 #define APIC_INVALID_PPN  ((PPN)0xfffffffe)
 
-#define INVALID_BPN       ((BPN)0x3fffffff)
+#define INVALID_BPN       ((BPN)0x000000ffffffffffull)
 
-#define RESERVED_MPN      ((MPN) 0)
-#define INVALID_MPN       ((MPN)-1)
-#define MEMREF_MPN        ((MPN)-2)
-#define RELEASED_MPN      ((MPN)-3)
+#define MPN38_MASK        ((1ull << 38) - 1)
 
-/* 0xfffffffc to account for special MPNs defined above. */
-#define MAX_MPN           ((MPN)0xfffffffc)  /* 44 bits of address space. */
+#define RESERVED_MPN      ((MPN)0)
+#define INVALID_MPN       ((MPN)MPN38_MASK)
+#define MEMREF_MPN        ((MPN)MPN38_MASK - 1)
+#define RELEASED_MPN      ((MPN)MPN38_MASK - 2)
+
+/* account for special MPNs defined above */
+#define MAX_MPN           ((MPN)MPN38_MASK - 3) /* 50 bits of address space */
+
+#define INVALID_IOPN      ((IOPN)-1)
+#define MAX_IOPN          (INVALID_IOPN - 1)
 
 #define INVALID_LPN       ((LPN)-1)
 #define INVALID_VPN       ((VPN)-1)
 #define INVALID_LPN64     ((LPN64)-1)
 #define INVALID_PAGENUM   ((PageNum)-1)
 
-#define INVALID_MPN64     ((MPN64)-1)
-
 /*
  * Format modifier for printing VA, LA, and VPN.
- * Use them like this: Log("%#"FMTLA"x\n", laddr)
+ * Use them like this: Log("%#" FMTLA "x\n", laddr)
  */
 
-#if defined(VMM) || defined(FROBOS64) || vm_x86_64 || defined __APPLE__
+#if defined(VMM) || defined(FROBOS64) || vm_x86_64 || vm_arm_64 || defined __APPLE__
 #   define FMTLA "l"
 #   define FMTVA "l"
 #   define FMTVPN "l"
@@ -631,12 +732,21 @@ typedef void * UserVA;
 #  define VMX86_EXTERN_DATA       extern
 #endif
 
-#if defined(_WIN32) && !defined(VMX86_NO_THREADS)
-#define THREADSPECIFIC __declspec(thread)
+#ifdef _WIN32
+
+/* under windows, __declspec(thread) is supported since VS 2003 */
+#define __thread __declspec(thread)
+
 #else
-#define THREADSPECIFIC
+
+/*
+ * under other platforms instead, __thread is supported by gcc since
+ * version 3.3.1 and by clang since version 3.x
+ */
+
 #endif
 
+
 /*
  * Due to the wonderful "registry redirection" feature introduced in
  * 64-bit Windows, if you access any key under HKLM\Software in 64-bit
@@ -662,7 +772,7 @@ typedef void * UserVA;
  * The various linux kernel modules may use older (gcc-3.3) compilers.
  */
 #if defined __GNUC__ && (__GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ < 3))
-#error "gcc version is to old to compile assembly, need gcc-3.3 or better"
+#error "gcc version is too old to compile assembly, need gcc-3.3 or better"
 #endif
 
 
@@ -698,15 +808,17 @@ typedef void * UserVA;
  * that gcc will not do inlining.
  */
 
-#if defined(__GNUC__) && (defined(VMM) || defined (VMKERNEL) || defined (VMKBOOT))
+#if defined(__GNUC__)
 #define ABSOLUTELY_NOINLINE __attribute__((__noinline__))
+#elif defined(_MSC_VER)
+#define ABSOLUTELY_NOINLINE __declspec(noinline)
 #endif
 
 /*
  * Used when a function has no effects except the return value and the
  * return value depends only on the parameters and/or global variables
  * Such a function can be subject to common subexpression elimination
- * and loop optimization just as an arithmetic operator would be. 
+ * and loop optimization just as an arithmetic operator would be.
  */
 
 #if defined(__GNUC__) && (defined(VMM) || defined (VMKERNEL))
@@ -715,6 +827,19 @@ typedef void * UserVA;
 #define SIDE_EFFECT_FREE
 #endif
 
+/*
+ * Used when a function exmaines no input other than its arguments and
+ * has no side effects other than its return value.  Stronger than
+ * SIDE_EFFECT_FREE as the function is not allowed to read from global
+ * memory.
+ */
+
+#if defined(__GNUC__) && (defined(VMM) || defined (VMKERNEL))
+#define CONST_FUNCTION __attribute__((__const__))
+#else
+#define CONST_FUNCTION
+#endif
+
 /*
  * Attributes placed on function declarations to tell the compiler
  * that the function never returns.
@@ -806,6 +931,11 @@ typedef void * UserVA;
 # endif
 #endif
 
+#ifndef UNUSED_TYPE
+// XXX _Pragma would better but doesn't always work right now.
+#  define UNUSED_TYPE(_parm) UNUSED_PARAM(_parm)
+#endif
+
 #ifndef UNUSED_VARIABLE
 // XXX is there a better way?
 #  define UNUSED_VARIABLE(_var) (void)_var
@@ -874,6 +1004,9 @@ typedef void * UserVA;
 #         else
              typedef uint32 size_t;
 #         endif
+#      elif defined(VM_ARM_64)
+#         define _SIZE_T
+          typedef uint64 size_t;
 #      elif defined(__arm__)
 #         define _SIZE_T
           typedef uint32 size_t;
@@ -888,15 +1021,23 @@ typedef void * UserVA;
 #         define _SSIZE_T
 #         define __ssize_t_defined
 #         define _SSIZE_T_DECLARED
+#         define _SSIZE_T_DEFINED_
 #         ifdef VM_X86_64
              typedef int64 ssize_t;
 #         else
              typedef int32 ssize_t;
 #         endif
+#      elif defined(VM_ARM_64)
+#         define _SSIZE_T
+#         define __ssize_t_defined
+#         define _SSIZE_T_DECLARED
+#         define _SSIZE_T_DEFINED_
+          typedef int64 ssize_t;
 #      elif defined(__arm__)
 #         define _SSIZE_T
 #         define __ssize_t_defined
 #         define _SSIZE_T_DECLARED
+#         define _SSIZE_T_DEFINED_
              typedef int32 ssize_t;
 #      endif
 #   endif
@@ -906,7 +1047,7 @@ typedef void * UserVA;
 /*
  * Format modifier for printing pid_t.  On sun the pid_t is a ulong, but on
  * Linux it's an int.
- * Use this like this: printf("The pid is %"FMTPID".\n", pid);
+ * Use this like this: printf("The pid is %" FMTPID ".\n", pid);
  */
 #ifdef sun
 #   ifdef VM_X86_64
@@ -921,7 +1062,7 @@ typedef void * UserVA;
 /*
  * Format modifier for printing uid_t.  On Solaris 10 and earlier, uid_t
  * is a ulong, but on other platforms it's an unsigned int.
- * Use this like this: printf("The uid is %"FMTUID".\n", uid);
+ * Use this like this: printf("The uid is %" FMTUID ".\n", uid);
  */
 #if defined(sun) && !defined(SOL11)
 #   ifdef VM_X86_64
@@ -936,7 +1077,7 @@ typedef void * UserVA;
 /*
  * Format modifier for printing mode_t.  On sun the mode_t is a ulong, but on
  * Linux it's an int.
- * Use this like this: printf("The mode is %"FMTMODE".\n", mode);
+ * Use this like this: printf("The mode is %" FMTMODE ".\n", mode);
  */
 #ifdef sun
 #   ifdef VM_X86_64
@@ -953,7 +1094,7 @@ typedef void * UserVA;
  * a long int, but on FreeBSD (as of 5.0, it seems), the time_t is a signed
  * size quantity. Refer to the definition of FMTSZ to see why we need silly
  * preprocessor arithmetic.
- * Use this like this: printf("The mode is %"FMTTIME".\n", time);
+ * Use this like this: printf("The mode is %" FMTTIME ".\n", time);
  */
 #if defined(__FreeBSD__) && (__FreeBSD__ + 0) && ((__FreeBSD__ + 0) >= 5)
 #   define FMTTIME FMTSZ"d"
