@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2005 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,7 +30,6 @@
 #define INCLUDE_ALLOW_VMCORE
 #include "includeCheck.h"
 
-#include "buildNumber.h"
 #include "vm_basic_types.h"
 #endif /* RC_INVOKED */
 
@@ -64,6 +63,16 @@
 #ifndef RC_INVOKED
 
 typedef uint32 ToolsVersion;
+typedef enum {
+  TOOLS_TYPE_UNSET = -1,
+  TOOLS_TYPE_UNKNOWN = 0,
+  TOOLS_TYPE_MSI,
+  TOOLS_TYPE_TARBALL,
+  TOOLS_TYPE_OSP,
+  TOOLS_TYPE_OVT,
+  TOOLS_TYPE_MAX,
+} ToolsInstallType;
+
 typedef struct {
    uint8 major;
    uint8 minor;
@@ -92,6 +101,9 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
  */
 #define TOOLS_VERSION_UNMANAGED 0x7fffffff
 
+/* Historically -1 indicates tools version is not available. */
+#define TOOLS_VERSION_INVALID   0xffffffff
+
 #define STRINGER(MJR, MNR, BASE)     #MJR "." #MNR "." #BASE
 #define TOOLS_VERSION_STRINGER(MJR, MNR, BASE)    STRINGER(MJR, MNR, BASE)
 
@@ -1146,6 +1158,22 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_HOSTED12_UPDATE1_V_MNR  2
 #define   TOOLS_VERSION_HOSTED12_UPDATE1_V_BASE 1
 
+/* Workstation 9.0.1, Fusion 5.0.2 (hosted12-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_BASE 2
+
+/* Workstation 9.0.2 (hosted12-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_BASE 3
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_OP_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_BETA1_V)
 #endif /* RC_INVOKED */
@@ -1175,22 +1203,159 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_OP_RELEASE_V_BASE 0
 
 #ifndef RC_INVOKED
-#define   TOOLS_VERSION_OP_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_UPDATE1_V)
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V)
 #endif /* RC_INVOKED */
-#define   TOOLS_VERSION_OP_UPDATE1_V_MJR  9
-#define   TOOLS_VERSION_OP_UPDATE1_V_MNR  4
-#define   TOOLS_VERSION_OP_UPDATE1_V_BASE 5
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_MNR  5
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_BASE 1
 
 #ifndef RC_INVOKED
-#define   TOOLS_VERSION_OP_PATCH02PS TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_PATCH02PS_V)
+#define   TOOLS_VERSION_HOSTED13_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_RC_V)
 #endif /* RC_INVOKED */
-#define   TOOLS_VERSION_OP_PATCH02PS_V_MJR  9
-#define   TOOLS_VERSION_OP_PATCH02PS_V_MNR  4
-#define   TOOLS_VERSION_OP_PATCH02PS_V_BASE 6
+#define   TOOLS_VERSION_HOSTED13_RC_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_RC_V_MNR  5
+#define   TOOLS_VERSION_HOSTED13_RC_V_BASE 2
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_OP_PATCH02PS
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_OP_PATCH02PS)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_OP_PATCH02PS)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED13_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_MNR  6
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED13_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED13_UPDATE1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_UPDATE1_V_MNR  6
+#define   TOOLS_VERSION_HOSTED13_UPDATE1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_2015_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_2015_BETA1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_2015_BETA1_V_MJR  9
+#define   TOOLS_VERSION_2015_BETA1_V_MNR  7
+#define   TOOLS_VERSION_2015_BETA1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_2015_BETA2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_2015_BETA2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_2015_BETA2_V_MJR  9
+#define   TOOLS_VERSION_2015_BETA2_V_MNR  7
+#define   TOOLS_VERSION_2015_BETA2_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_2015_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_2015_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_2015_RC_V_MJR  9
+#define   TOOLS_VERSION_2015_RC_V_MNR  7
+#define   TOOLS_VERSION_2015_RC_V_BASE 3
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED14_TECHPREVIEW1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW1_V_MNR  8
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED14_TECHPREVIEW2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW2_V_MJR  9
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW2_V_MNR  8
+#define   TOOLS_VERSION_HOSTED14_TECHPREVIEW2_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_FUSION70_RELEASE     TOOLS_VERSION_TO_UINT(TOOLS_VERSION_FUSION70_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_FUSION70_RELEASE_V_MJR     9
+#define   TOOLS_VERSION_FUSION70_RELEASE_V_MNR     8
+#define   TOOLS_VERSION_FUSION70_RELEASE_V_BASE    3
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_FUSION70_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_FUSION70_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_FUSION70_UPDATE1_V_MJR  9
+#define   TOOLS_VERSION_FUSION70_UPDATE1_V_MNR  8
+#define   TOOLS_VERSION_FUSION70_UPDATE1_V_BASE 4
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED14_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED14_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED14_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_HOSTED14_RELEASE_V_MNR  9
+#define   TOOLS_VERSION_HOSTED14_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED14_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED14_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED14_UPDATE1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED14_UPDATE1_V_MNR  9
+#define   TOOLS_VERSION_HOSTED14_UPDATE1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED14_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED14_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED14_UPDATE2_V_MJR  9
+#define   TOOLS_VERSION_HOSTED14_UPDATE2_V_MNR  9
+#define   TOOLS_VERSION_HOSTED14_UPDATE2_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_2015_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_2015_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_2015_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_2015_RELEASE_V_MNR  10
+#define   TOOLS_VERSION_2015_RELEASE_V_BASE 0
+
+/*
+ * Patch01OVT - open-vm-tools release from vsphere60p01tools - 9.10.2
+ */
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_2015_PATCH01OVT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_2015_PATCH01OVT_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_2015_PATCH01OVT_V_MJR  9
+#define   TOOLS_VERSION_2015_PATCH01OVT_V_MNR  10
+#define   TOOLS_VERSION_2015_PATCH01OVT_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_FC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_FC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_FC_V_MJR  9
+#define   TOOLS_VERSION_TORQUE_WRENCH_FC_V_MNR  12
+#define   TOOLS_VERSION_TORQUE_WRENCH_FC_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_RC_V_MJR  9
+#define   TOOLS_VERSION_TORQUE_WRENCH_RC_V_MNR  12
+#define   TOOLS_VERSION_TORQUE_WRENCH_RC_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_RELEASE_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_RELEASE_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_BASE 7
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_TORQUE_WRENCH_UPDATE3
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3)
 
 /*
  * The extended Tools version is the current Tools version with the
