@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,6 +16,45 @@
  *
  *********************************************************/
 
+/*********************************************************
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ *
+ *********************************************************/
+
+/*********************************************************
+ * The contents of this file are subject to the terms of the Common
+ * Development and Distribution License (the "License") version 1.0
+ * and no later version.  You may not use this file except in
+ * compliance with the License.
+ *
+ * You can obtain a copy of the License at
+ *         http://www.opensource.org/licenses/cddl1.php
+ *
+ * See the License for the specific language governing permissions
+ * and limitations under the License.
+ *
+ *********************************************************/
+
 /*
  * vmblock.h --
  *
@@ -73,7 +112,7 @@
 #define VMBLOCK_FUSE_READ_RESPONSE     "I am VMBLOCK-FUSE"
 
 #define VMBLOCK_FUSE_FS_NAME           "fuse.vmware-vmblock"
-#define VMBLOCK_FUSE_MOUNT_POINT       "/tmp/vmblock-fuse"
+#define VMBLOCK_FUSE_MOUNT_POINT       "/var/run/vmblock-fuse"
 #define VMBLOCK_FUSE_CTRL_MNTPNT       "blockdir"
 #define VMBLOCK_FUSE_FS_ROOT           VMBLOCK_FUSE_MOUNT_POINT "/" VMBLOCK_FUSE_CTRL_MNTPNT
 #define VMBLOCK_FUSE_DEVICE_NAME       "dev"
@@ -101,6 +140,7 @@
 # define VMBLOCK_FS_NAME               VMBLOCK_FUSE_FS_NAME
 # define VMBLOCK_DEVICE_NAME           VMBLOCK_FUSE_DEVICE_NAME
 # define VMBLOCK_CONTROL_MOUNTPOINT    VMBLOCK_FUSE_CTRL_MNTPNT
+# define VMBLOCK_FS_ROOT               VMBLOCK_FUSE_FS_ROOT
 # define VMBLOCK_DEVICE                VMBLOCK_FUSE_DEVICE
 # define VMBLOCK_DEVICE_MODE           VMBLOCK_FUSE_DEVICE_MODE
 # define VMBLOCK_MOUNT_POINT           VMBLOCK_FUSE_MOUNT_POINT
