@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2012 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -19,7 +19,7 @@
 /*
  * vmci_sockets.h --
  *
- *    VMCI sockets public constants and types.
+ *    vSockets public constants and types.
  */
 
 #ifndef _VMCI_SOCKETS_H_
@@ -39,7 +39,9 @@
 #  if defined(__APPLE__)
 #    include <sys/socket.h>
 #    include <string.h>
-#  endif // __APPLE__
+#  elif defined(__FreeBSD__)
+#     include <sys/socket.h>
+#  endif // __FreeBSD__
 #endif // linux && !VMKERNEL
 #endif
 
@@ -381,9 +383,9 @@
  */
 
 struct sockaddr_vm {
-#if defined(__APPLE__)
+#if defined(__APPLE__) || defined(__FreeBSD__)
    unsigned char svm_len;
-#endif // __APPLE__
+#endif // __APPLE__ || __FreeBSD__
 
    /** \brief Address family. \see VMCISock_GetAFValueFd() */
    sa_family_t svm_family;
@@ -559,9 +561,9 @@ struct uuid_2_cid {
       int fd;
       unsigned int version;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMCI_SOCKETS_INVALID_VERSION;
          }
@@ -581,9 +583,9 @@ struct uuid_2_cid {
     *
     * \brief Retrieve the address family value for vSockets.
     *
-    * Returns the value to be used for the VMCI Sockets address family.
+    * Returns the value to be used for the vSockets address family.
     * This value should be used as the domain argument to \c socket(2) (when
-    * you might otherwise use \c AF_INET).  For VMCI Socket-specific options,
+    * you might otherwise use \c AF_INET).  For vSocket-specific options,
     * this value should also be used for the level argument to
     * \c setsockopt(2) (when you might otherwise use \c SOL_TCP).
     *
@@ -619,11 +621,34 @@ struct uuid_2_cid {
    static inline int VMCISock_GetAFValueFd(int *outFd)
    {
       int fd;
-      int family;
+      int family = -1;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+#if defined(linux)
+      /*
+       * vSockets is now in mainline kernel with address family 40.  As part
+       * of upstreaming, we removed the IOCTL we use below to determine the
+       * address family.  So to handle both a new and old kernel we do this:
+       * 1. Check if our family already exists by making a socket with it.
+       *    Some weird kernel might claim this too, but it's very unlikely
+       *    (Linus' tree has us at 40, and that's what we care about).
+       * 2. If that fails, try the normal IOCTL path, since it's probably an
+       *    older kernel with vSockets from Tools.
+       * 3. If that fails, then vSockets really isn't available.
+       */
+#define AF_VSOCK_LOCAL 40
+      {
+         int s = socket(AF_VSOCK_LOCAL, SOCK_DGRAM, 0);
+         if (s != -1) {
+            close(s);
+            return AF_VSOCK_LOCAL;
+         }
+      }
+#undef AF_VSOCK_LOCAL
+#endif // linux
+
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return -1;
          }
@@ -649,9 +674,9 @@ struct uuid_2_cid {
     *
     * \brief Retrieve the address family value for vSockets.
     *
-    * Returns the value to be used for the VMCI Sockets address family.
+    * Returns the value to be used for the vSockets address family.
     * This value should be used as the domain argument to \c socket(2) (when
-    * you might otherwise use \c AF_INET).  For VMCI Socket-specific options,
+    * you might otherwise use \c AF_INET).  For vSocket-specific options,
     * this value should also be used for the level argument to
     * \c setsockopt(2) (when you might otherwise use \c SOL_TCP).
     *
@@ -742,9 +767,9 @@ struct uuid_2_cid {
       int fd;
       unsigned int contextId;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMADDR_CID_ANY;
          }
@@ -795,9 +820,9 @@ struct uuid_2_cid {
       int fd;
       struct uuid_2_cid io;
 
-      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDONLY);
       if (fd < 0) {
-         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDONLY);
          if (fd < 0) {
             return VMADDR_CID_ANY;
          }
@@ -812,7 +837,19 @@ struct uuid_2_cid {
       return io.u2c_context_id;
    }
 #  endif // __KERNEL__
-#endif // linux && !VMKERNEL
+#elif defined(__FreeBSD__)
+   /*
+    * No FreeBSD support yet, but it might appear in the future. Just define
+    * some stubs that return errors - that way a client doesn't have to ifdef
+    * the calls (assuming it can handle the failures).
+    */
+#  define VMCISock_Version()                    VMCI_SOCKETS_INVALID_VERSION
+#  define VMCISock_GetAFValueFd(outFd)          (-1)
+#  define VMCISock_GetAFValue()                 VMCISock_GetAFValueFd(NULL)
+#  define VMCISock_ReleaseAFValueFd(fd)         do { } while (0)
+#  define VMCISock_GetLocalCID()                VMADDR_CID_ANY
+#  define VMCISock_Uuid2ContextId(uuidString)   VMADDR_CID_ANY
+#endif // __FreeBSD__
 #endif // _WIN32
 
 
