@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -47,12 +47,12 @@ typedef enum AutorunState {
 /* Defines */
 #define VMX_SHUTDOWN_ORDER    0x100    // Application reserved last shutdown range.
 #define UI_SHUTDOWN_ORDER     0x300    // Application reserved first shutdown range.
-#define TOOLS_SHUTDOWN_ORDER  0x100    // Application reserved last shutdown range 
+#define TOOLS_SHUTDOWN_ORDER  0x100    // Application reserved last shutdown range
 
 /* Function declarations */
 
-Unicode W32Util_GetInstallPath(void);
-Unicode W32Util_GetInstallPath64(void);
+char * W32Util_GetInstallPath(void);
+char * W32Util_GetInstallPath64(void);
 
 /*
  * The string returned is allocated on the heap and must be freed by the
@@ -60,18 +60,18 @@ Unicode W32Util_GetInstallPath64(void);
  */
 
 
-Unicode W32Util_GetAppDataFilePath(ConstUnicode fileName);
-Unicode W32Util_GetLocalAppDataFilePath(ConstUnicode fileName);
+char *W32Util_GetAppDataFilePath(const char *fileName);
+char *W32Util_GetLocalAppDataFilePath(const char *fileName);
 
-Unicode W32Util_GetCommonAppDataFilePath(ConstUnicode fileName);
-Unicode W32Util_GetVmwareCommonAppDataFilePath(ConstUnicode fileName);
+char *W32Util_GetCommonAppDataFilePath(const char *fileName);
+char *W32Util_GetVmwareCommonAppDataFilePath(const char *fileName);
 
-Unicode W32Util_GetMyDocumentPath(void);
-Unicode W32Util_GetMyVideoPath(BOOL myDocumentsOnFail);
+char *W32Util_GetMyDocumentPath(void);
+char *W32Util_GetMyVideoPath(BOOL myDocumentsOnFail);
 
-Unicode W32Util_GetDefaultVMPath(ConstUnicode pref);
-Unicode W32Util_GetInstalledFilePath(ConstUnicode fileName);
-Unicode W32Util_GetInstalledFilePath64(ConstUnicode fileName);
+char *W32Util_GetDefaultVMPath(const char *pref);
+char *W32Util_GetInstalledFilePath(const char *fileName);
+char *W32Util_GetInstalledFilePath64(const char *fileName);
 
 HKEY W32Util_OpenProductRegKey(REGSAM access);
 HKEY W32Util_OpenUserRegKey(REGSAM access);
@@ -79,21 +79,21 @@ HKEY W32Util_OpenUserRegKey(REGSAM access);
 LPTOP_LEVEL_EXCEPTION_FILTER W32Util_SetUnhandledExceptionFilter(void);
 
 
-Bool W32Util_CreateProcessArgv(ConstUnicode lpApplicationName,            
-                               ConstUnicode *argv,                        
-                               LPSECURITY_ATTRIBUTES lpProcessAttributes, 
-                               LPSECURITY_ATTRIBUTES lpThreadAttributes,  
-                               BOOL bInheritHandles,                      
-                               DWORD dwCreationFlags,                     
-                               LPVOID lpEnvironment,                      
-                               ConstUnicode lpCurrentDirectory,           
-                               LPSTARTUPINFOA lpStartupInfo,              
+Bool W32Util_CreateProcessArgv(const char *lpApplicationName,
+                               const char **argv,
+                               LPSECURITY_ATTRIBUTES lpProcessAttributes,
+                               LPSECURITY_ATTRIBUTES lpThreadAttributes,
+                               BOOL bInheritHandles,
+                               DWORD dwCreationFlags,
+                               LPVOID lpEnvironment,
+                               const char *lpCurrentDirectory,
+                               LPSTARTUPINFOA lpStartupInfo,
                                LPPROCESS_INFORMATION lpProcessInformation);
 
 void W32Util_SplitCommandLine(char *commandLine,
                               int maxArgs,
                               char *progName,
-                              int *argc,   
+                              int *argc,
                               char **argv);
 
 BOOL W32Util_ReadFileTimeout(HANDLE hFile,                // handle of file to read
@@ -105,29 +105,29 @@ BOOL W32Util_ReadFileTimeout(HANDLE hFile,                // handle of file to r
 BOOL W32Util_WriteFileTimeout(HANDLE hFile,                    // handle to file to write to
                               LPCVOID lpBuffer,                // pointer to data to write to file
                               DWORD nNumberOfBytesToWrite,     // number of bytes to write
-                              LPDWORD lpNumberOfBytesWritten,  // pointer to number of bytes written  
+                              LPDWORD lpNumberOfBytesWritten,  // pointer to number of bytes written
                               DWORD msTimeout);                // timeout in milliseconds
 
-Unicode W32Util_RealPath(ConstUnicode path);
+char *W32Util_RealPath(const char *path);
 
 BOOL W32Util_CheckForPrivilegeHeld(HANDLE token, LPCTSTR priv);
 
 typedef BOOL (*SidFilterFunction)(PSID psid, void *cbData);
-BOOL W32Util_GetEffectiveRightsForName(ConstUnicode user,
-                                       ConstUnicode path,
+BOOL W32Util_GetEffectiveRightsForName(const char *user,
+                                       const char *path,
                                        DWORD *rights);
 BOOL W32Util_GetEffectiveRightsForSid(PSID psid,
-                                      ConstUnicode path,
+                                      const char *path,
                                       DWORD *rights);
 
 BOOL W32Util_ModifyRights(PSID psid,
-                          ConstUnicode path,
+                          const char *path,
                           DWORD rights,
                           BOOL isAllow);
 void W32Util_FreeSids(PSID *sidList);
-BOOL W32Util_GetMatchingSids(ConstUnicode path, 
-                             PSID **psidList, 
-                             SidFilterFunction matchCb, 
+BOOL W32Util_GetMatchingSids(const char *path,
+                             PSID **psidList,
+                             SidFilterFunction matchCb,
                              void *cbData);
 
 LPSTR W32Util_WideStrToAsciiStr(LPCWSTR wideStr);
@@ -139,23 +139,23 @@ BOOL W32Util_WinSockAddReference(void);
 BOOL W32Util_WinSockDereference(void);
 
 Bool W32Util_RegisterService(Bool bRegister,
-                             ConstUnicode name,
-			     ConstUnicode displayName,
-                             ConstUnicode description,
-			     ConstUnicode binaryPath,
-                             Unicode *errString);
+                             const char *name,
+                             const char *displayName,
+                             const char *description,
+                             const char *binaryPath,
+                             char **errString);
 
-Bool W32Util_DebugService(ConstUnicode dbgFile);
+Bool W32Util_DebugService(const char *dbgFile);
 
-Bool W32Util_RegisterEventLog(ConstUnicode serviceName,
+Bool W32Util_RegisterEventLog(const char *serviceName,
                               Bool registryVolatile,
                               DWORD typesSupported,
-                              ConstUnicode eventMsgFile,
-                              ConstUnicode categoryMsgFile,
+                              const char *eventMsgFile,
+                              const char *categoryMsgFile,
                               DWORD categoryCount,
-                              ConstUnicode paramMsgFile);
+                              const char *paramMsgFile);
 
-Bool W32Util_UnregisterEventLog(ConstUnicode serviceName);
+Bool W32Util_UnregisterEventLog(const char *serviceName);
 
 typedef enum SetSDPrivsAccounts {
    SDPRIV_GROUP_ADMIN = 0x1,
@@ -165,22 +165,22 @@ typedef enum SetSDPrivsAccounts {
 
 Bool W32Util_SetSDPrivs(PSECURITY_DESCRIPTOR pSecurityDescriptor,
                         DWORD accessType, SetSDPrivsAccounts accounts,
-                        PACL *pAcl, Unicode *errString);
+                        PACL *pAcl, char **errString);
 
 BOOL W32Util_SetSecurityDescriptorW(PSECURITY_DESCRIPTOR pSecurityDescriptor,
-                                    ConstUnicode Owner, PACL *pAcl);
+                                    const char *Owner, PACL *pAcl);
 BOOL W32Util_SetSecurityDescriptorSid(PSECURITY_DESCRIPTOR sd, PSID sid,
                                       PACL *pAcl);
 BOOL W32Util_GetThreadHandle(HANDLE *handle);
 
-Bool W32Util_AccessCheck(HANDLE token, 
-                         PSECURITY_DESCRIPTOR pSecurityDescriptor, 
+Bool W32Util_AccessCheck(HANDLE token,
+                         PSECURITY_DESCRIPTOR pSecurityDescriptor,
                          int desiredAccess);
 
-Bool W32Util_HasAccessToFile(ConstUnicode filename, ACCESS_MASK desiredAccess,
-			     HANDLE token);
+Bool W32Util_HasAccessToFile(const char *filename, ACCESS_MASK desiredAccess,
+                             HANDLE token);
 
-Bool W32Util_GetSecurityDescriptor(ConstUnicode path, 
+Bool W32Util_GetSecurityDescriptor(const char *path,
                                    PSECURITY_DESCRIPTOR *ppSecurityDescriptor);
 
 void W32Util_InitStdioConsole(void);
@@ -193,17 +193,17 @@ Bool W32Util_CreateWellKnownSid(WELL_KNOWN_SID_TYPE wsdType,
 Bool W32Util_GetCurrentUserSid(PSID *pSid);
 Bool W32Util_GetLocalAdminGroupSid(PSID *pSid);
 Bool W32Util_GetVMwareGroupSid(PSID *pSid);
-Bool W32Util_MakeSafeDirectory(ConstUnicode path);
-Bool W32Util_IsDirectorySafe(ConstUnicode path);
-Bool W32Util_DoesVolumeSupportAcls(ConstUnicode path);
+Bool W32Util_MakeSafeDirectory(const char *path);
+Bool W32Util_IsDirectorySafe(const char *path);
+Bool W32Util_DoesVolumeSupportAcls(const char *path);
 
 Bool W32Util_GetRegistryAutorun(AutorunState* state);
 Bool W32Util_SetRegistryAutorun(const AutorunState state);
 
 Bool W32Util_AllowAdminCOM(void);
 
-Unicode W32Util_GetAppDataPath(void);
-Unicode W32Util_RobustGetLongPath(ConstUnicode path);
+char * W32Util_GetAppDataPath(void);
+char * W32Util_RobustGetLongPath(const char *path);
 
 typedef enum SecureObjectType {
    SecureObject_Process,
@@ -219,11 +219,11 @@ HMODULE W32Util_GetModuleByAddress(const void *addr);
 
 Bool W32Util_VerifyXPModeHostLicense(void);
 
-Unicode W32Util_GetPipeNameFromFilePath(ConstUnicode fileName);
+char * W32Util_GetPipeNameFromFilePath(const char *fileName);
 
 Bool W32Util_CheckGroupMembership(HANDLE hToken,
                                   BOOL respectUAC,
-                                  Unicode *errString,
+                                  char * *errString,
                                   BOOL *bMember);
 
 Bool W32Util_DismountVolumes(uint16 drive,
