@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,6 +26,7 @@
 #include "random.h"
 
 static Bool mxInPanic = FALSE;  // track when involved in a panic
+static Bool mxUserCollectLockingTree = FALSE;
 
 Bool (*MXUserTryAcquireForceFail)() = NULL;
 
@@ -35,6 +36,58 @@ void (*MXUserMX_LockRec)(struct MX_MutexRec *lock) = NULL;
 void (*MXUserMX_UnlockRec)(struct MX_MutexRec *lock) = NULL;
 Bool (*MXUserMX_TryLockRec)(struct MX_MutexRec *lock) = NULL;
 Bool (*MXUserMX_IsLockedByCurThreadRec)(const struct MX_MutexRec *lock) = NULL;
+char *(*MXUserMX_NameRec)(const struct MX_MutexRec *lock) = NULL;
+static void (*MXUserMX_SetInPanic)(void) = NULL;
+static Bool (*MXUserMX_InPanic)(void) = NULL;
+
+#define	MXUSER_MAX_LOOP 5
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_IsLockingTreeAvailable
+ *
+ *      Is the lock tracking tree available for reporting?
+ *
+ * Results:
+ *      TRUE  MXuser lock tree tracking is enabled
+ *      FALSE MXUser lock tree tracking is disabled
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_IsLockingTreeAvailable(void)
+{
+   return mxUserCollectLockingTree;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_LockingTreeCollection
+ *
+ *      Enable or disable locking tree data collection.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUser_LockingTreeCollection(Bool enabled)  // IN:
+{
+   mxUserCollectLockingTree = vmx86_devel && vmx86_debug && enabled;
+}
 
 
 /*
@@ -60,10 +113,10 @@ Bool (*MXUserMX_IsLockedByCurThreadRec)(const struct MX_MutexRec *lock) = NULL;
 MXRecLock *
 MXUserInternalSingleton(Atomic_Ptr *storage)  // IN:
 {
-   MXRecLock *lock = (MXRecLock *) Atomic_ReadPtr(storage);
+   MXRecLock *lock = Atomic_ReadPtr(storage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXRecLock *newLock = Util_SafeMalloc(sizeof(MXRecLock));
+      MXRecLock *newLock = Util_SafeMalloc(sizeof *newLock);
 
       if (MXRecLockInit(newLock)) {
          lock = Atomic_ReadIfEqualWritePtr(storage, NULL, (void *) newLock);
@@ -84,8 +137,6 @@ MXUserInternalSingleton(Atomic_Ptr *storage)  // IN:
 }
 
 
-#define MXUSER_SYNDROME
-#if defined(MXUSER_SYNDROME)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -146,7 +197,6 @@ MXUserSyndrome(void)
 
    return syndrome;
 }
-#endif
 
 
 /*
@@ -173,7 +223,6 @@ MXUserGetSignature(MXUserObjectType objectType)  // IN:
    ASSERT((objectType >= 0) && (objectType < 16) &&
           (objectType != MXUSER_TYPE_NEVER_USE));
 
-#if defined(MXUSER_SYNDROME)
    /*
     * Use a random syndrome combined with a unique bit pattern mapping
     * of objectType to bits in a nibble. The random portion of the signature
@@ -182,46 +231,6 @@ MXUserGetSignature(MXUserObjectType objectType)  // IN:
     */
 
    signature = (MXUserSyndrome() & 0x0FFFFFFF) | (objectType << 28);
-#else
-   /*
-    * Map the abstract objectType back to the bit patterns used in older
-    * versions of lib/lock. This provides absolute compatibility with
-    * these older libraries.
-    */
-
-   switch (objectType) {
-   case MXUSER_TYPE_RW:
-      signature = 0x57524B4C;
-      break;
-
-   case MXUSER_TYPE_REC:
-      signature = 0x43524B4C;
-      break;
-
-   case MXUSER_TYPE_RANK:
-      signature = 0x4E4B5241;
-      break;
-
-   case MXUSER_TYPE_EXCL:
-      signature = 0x58454B4C;
-      break;
-
-   case MXUSER_TYPE_SEMA:
-      signature = 0x414D4553;
-      break;
-
-   case MXUSER_TYPE_CONDVAR:
-      signature = 0x444E4F43;
-      break;
-
-   case MXUSER_TYPE_BARRIER:
-      signature = 0x52524142;
-      break;
-
-   default:
-      Panic("%s: unknown objectType %d\n", __FUNCTION__, objectType);
-   }
-#endif
 
    ASSERT(signature);
 
@@ -229,6 +238,34 @@ MXUserGetSignature(MXUserObjectType objectType)  // IN:
 }
 
 
+/*
+ *---------------------------------------------------------------------
+ *
+ *  MXUser_SetInPanic --
+ *
+ *     Notify the locking system that a panic is occurring.
+ *
+ *  Results:
+ *     Set the "in a panic" state in userland locks and, when possible,
+ *     MX locks.
+ *
+ *  Side effects:
+ *     None
+ *
+ *---------------------------------------------------------------------
+ */
+
+void
+MXUser_SetInPanic(void)
+{
+   mxInPanic = TRUE;
+
+   if (MXUserMX_SetInPanic != NULL) {
+      MXUserMX_SetInPanic();
+   }
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -252,9 +289,19 @@ MXUserDumpAndPanic(MXUserHeader *header,  // IN:
 {
    char *msg;
    va_list ap;
+   static uint32 loopCounter = 0;  // Is panic looping through there?
 
    ASSERT((header != NULL) && (header->dumpFunc != NULL));
 
+   if (++loopCounter > MXUSER_MAX_LOOP) {
+      /*
+       * Panic is looping through MXUser to here - no progress is being made.
+       * Switch to panic mode in the hopes that this will allow some progress.
+       */
+
+      MXUser_SetInPanic();
+   }
+
    (*header->dumpFunc)(header);
 
    va_start(ap, fmt);
@@ -265,39 +312,12 @@ MXUserDumpAndPanic(MXUserHeader *header,  // IN:
 }
 
 
-/*
- *---------------------------------------------------------------------
- *
- *  MXUser_SetInPanic --
- *	Notify the locking system that a panic is occurring.
- *
- *      This is the "out of the monitor" - userland - implementation. The "in
- *      the monitor" implementation lives in mutex.c.
- *
- *  Results:
- *     Set the internal "in a panic" global variable.
- *
- *  Side effects:
- *     None
- *
- *---------------------------------------------------------------------
- */
-
-void
-MXUser_SetInPanic(void)
-{
-   mxInPanic = TRUE;
-}
-
-
 /*
  *---------------------------------------------------------------------
  *
  *  MXUser_InPanic --
- *	Is the caller in the midst of a panic?
  *
- *      This is the "out of the monitor" - userland - implementation. The "in
- *      the monitor" implementation lives in mutex.c.
+ *     Is the caller in the midst of a panic?
  *
  *  Results:
  *     TRUE   Yes
@@ -312,7 +332,7 @@ MXUser_SetInPanic(void)
 Bool
 MXUser_InPanic(void)
 {
-   return mxInPanic;
+   return mxInPanic || (MXUserMX_InPanic != NULL && MXUserMX_InPanic());
 }
 
 
@@ -337,10 +357,13 @@ MXUser_InPanic(void)
 void
 MXUserInstallMxHooks(void (*theLockListFunc)(void),
                      MX_Rank (*theRankFunc)(void),
-                     void (*theLockFunc)(struct MX_MutexRec *lock),
-                     void (*theUnlockFunc)(struct MX_MutexRec *lock),
-                     Bool (*theTryLockFunc)(struct MX_MutexRec *lock),
-                     Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock))
+                     void  (*theLockFunc)(struct MX_MutexRec *lock),
+                     void  (*theUnlockFunc)(struct MX_MutexRec *lock),
+                     Bool  (*theTryLockFunc)(struct MX_MutexRec *lock),
+                     Bool  (*theIsLockedFunc)(const struct MX_MutexRec *lock),
+                     char *(*theNameFunc)(const struct MX_MutexRec *lock),
+                     void  (*theSetInPanicFunc)(void),
+                     Bool  (*theInPanicFunc)(void))
 {
    /*
     * This function can be called more than once but the second and later
@@ -353,20 +376,30 @@ MXUserInstallMxHooks(void (*theLockListFunc)(void),
        (MXUserMX_LockRec == NULL) &&
        (MXUserMX_UnlockRec == NULL) &&
        (MXUserMX_TryLockRec == NULL) &&
-       (MXUserMX_IsLockedByCurThreadRec == NULL)) {
+       (MXUserMX_IsLockedByCurThreadRec == NULL) &&
+       (MXUserMX_NameRec == NULL) &&
+       (MXUserMX_SetInPanic == NULL) &&
+       (MXUserMX_InPanic == NULL)
+       ) {
       MXUserMxLockLister = theLockListFunc;
       MXUserMxCheckRank = theRankFunc;
       MXUserMX_LockRec = theLockFunc;
       MXUserMX_UnlockRec = theUnlockFunc;
       MXUserMX_TryLockRec = theTryLockFunc;
       MXUserMX_IsLockedByCurThreadRec = theIsLockedFunc;
+      MXUserMX_NameRec = theNameFunc;
+      MXUserMX_SetInPanic = theSetInPanicFunc;
+      MXUserMX_InPanic = theInPanicFunc;
    } else {
       ASSERT((MXUserMxLockLister == theLockListFunc) &&
              (MXUserMxCheckRank == theRankFunc) &&
              (MXUserMX_LockRec == theLockFunc) &&
              (MXUserMX_UnlockRec == theUnlockFunc) &&
              (MXUserMX_TryLockRec == theTryLockFunc) &&
-             (MXUserMX_IsLockedByCurThreadRec == theIsLockedFunc)
+             (MXUserMX_IsLockedByCurThreadRec == theIsLockedFunc) &&
+             (MXUserMX_NameRec == theNameFunc) &&
+             (MXUserMX_SetInPanic == theSetInPanicFunc) &&
+             (MXUserMX_InPanic == theInPanicFunc)
             );
    }
 }
@@ -385,7 +418,6 @@ static MXUserPerThread *perThreadFreeList = NULL;
 
 static Atomic_Ptr hashTableMem;
 
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -471,8 +503,10 @@ MXUserFreePerThread(MXUserPerThread *perThread)  // IN:
 
    MXRecLockAcquire(perThreadLock,
                     NULL);          // non-stats
+
    perThread->next = perThreadFreeList;
    perThreadFreeList = perThread;
+
    MXRecLockRelease(perThreadLock);
 }
 
@@ -545,10 +579,10 @@ MXUserGetPerThread(Bool mayAlloc)  // IN: alloc perThread if not present?
  *      if no locks have been taken.
  *
  * Results:
- *      The list is printed
+ *      None
  *
  * Side effects:
- *      None
+ *      The list is printed.
  *
  *-----------------------------------------------------------------------------
  */
@@ -702,7 +736,7 @@ MXUserAcquisitionTracking(MXUserHeader *header,  // IN:
 {
    MXUserPerThread *perThread = MXUserGetPerThread(TRUE);
 
-   ASSERT_NOT_IMPLEMENTED(perThread->locksHeld < MXUSER_MAX_LOCKS_PER_THREAD);
+   VERIFY(perThread->locksHeld < MXUSER_MAX_LOCKS_PER_THREAD);
 
    /*
     * Rank checking anyone?
@@ -754,8 +788,28 @@ MXUserAcquisitionTracking(MXUserHeader *header,  // IN:
       }
    }
 
-   /* Add lock instance to the calling threads perThread information */
+   /* Add a lock instance to the calling threads perThread information */
    perThread->lockArray[perThread->locksHeld++] = header;
+
+   /*
+    * Maintain the lock tracking tree when approporiate.
+    */
+
+   if (vmx86_devel && vmx86_debug && mxUserCollectLockingTree) {
+      uint32 i;
+      MXUserLockTreeNode *node = NULL;
+
+      MXUserLockTreeAcquire();
+
+      for (i = 0; i < perThread->locksHeld; i++) {
+         header = perThread->lockArray[i];
+
+         node = MXUserLockTreeAdd(node, header->name,
+                                  header->bits.serialNumber, header->rank);
+      }
+
+      MXUserLockTreeRelease();
+   }
 }
 
 
@@ -807,7 +861,11 @@ MXUserReleaseTracking(MXUserHeader *header)  // IN: lock, via its header
    lastEntry = perThread->locksHeld - 1;
 
    if (i < lastEntry) {
-      perThread->lockArray[i] = perThread->lockArray[lastEntry];
+      uint32 j;
+
+      for (j = i + 1; j < perThread->locksHeld; j++) {
+         perThread->lockArray[i++] = perThread->lockArray[j];
+      }
    }
 
    perThread->lockArray[lastEntry] = NULL;  // tidy up memory
@@ -829,7 +887,7 @@ MXUserReleaseTracking(MXUserHeader *header)  // IN: lock, via its header
  *      Unknown
  *
  * Side effects:
- *      Always entertaining...
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
@@ -846,14 +904,14 @@ MXUser_TryAcquireFailureControl(Bool (*func)(const char *name))  // IN:
  *
  * MXUserValidateHeader --
  *
- *      Validate an MXUser object header
+ *      Validate an MXUser object header.
  *
  * Results:
  *      Return  All is well
  *      Panic   All is NOT well
  *
  * Side effects:
- *      Always entertaining...
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
@@ -864,14 +922,24 @@ MXUserValidateHeader(MXUserHeader *header,         // IN:
 {
    uint32 expected = MXUserGetSignature(objectType);
 
+   if (header->bits.badHeader == 1) {
+      return; // No need to panic on a bad header repeatedly...
+   }
+
    if (header->signature != expected) {
+      header->bits.badHeader = 1;
+
       MXUserDumpAndPanic(header,
                         "%s: signature failure! expected 0x%X observed 0x%X\n",
                          __FUNCTION__, expected, header->signature);
+
    }
 
-   if (header->serialNumber == 0) {
-      MXUserDumpAndPanic(header, "%s: Invalid serial number!", __FUNCTION__);
+   if (header->bits.serialNumber == 0) {
+      header->bits.badHeader = 1;
+
+      MXUserDumpAndPanic(header, "%s: Invalid serial number!\n",
+                         __FUNCTION__);
    }
 }
 #endif
