@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,20 +22,6 @@
 #include "userlock.h"
 #include "hostinfo.h"
 #include "ulInt.h"
-#include "vm_atomic.h"
-
-typedef struct
-{
-   MXUserAcquisitionStats  data;
-   Atomic_Ptr              histo;
-} MXUserAcquireStats;
-
-typedef struct
-{
-   VmTimeType        holdStart;
-   MXUserBasicStats  data;
-   Atomic_Ptr        histo;
-} MXUserHeldStats;
 
 struct MXUserExclLock
 {
@@ -99,16 +85,14 @@ MXUserStatsActionExcl(MXUserHeader *header)  // IN:
       MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
 
       if (UNLIKELY(isHot)) {
-         MXUserForceHisto(&acquireStats->histo,
-                          MXUSER_STAT_CLASS_ACQUISITION,
-                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                          MXUSER_DEFAULT_HISTO_DECADES);
+         MXUserForceAcquisitionHisto(&lock->acquireStatsMem,
+                                     MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                                     MXUSER_DEFAULT_HISTO_DECADES);
 
          if (UNLIKELY(heldStats != NULL)) {
-            MXUserForceHisto(&heldStats->histo,
-                             MXUSER_STAT_CLASS_HELD,
-                             MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                             MXUSER_DEFAULT_HISTO_DECADES);
+            MXUserForceHeldHisto(&lock->heldStatsMem,
+                                 MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                                 MXUSER_DEFAULT_HISTO_DECADES);
          }
 
          if (doLog) {
@@ -123,144 +107,170 @@ MXUserStatsActionExcl(MXUserHeader *header)  // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * MXUser_ControlExclLock --
+ * MXUser_EnableStatsExclLock
  *
- *      Perform the specified command on the specified lock.
+ *      Enable basic stats on the specified lock.
  *
  * Results:
  *      TRUE    succeeded
  *      FALSE   failed
  *
  * Side effects:
- *      Depends on the command, no?
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
 
 Bool
-MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
-                       uint32 command,        // IN:
-                       ...)                   // IN:
+MXUser_EnableStatsExclLock(MXUserExclLock *lock,       // IN/OUT:
+                           Bool trackAcquisitionTime,  // IN:
+                           Bool trackHeldTime)         // IN:
+{
+   ASSERT(lock);
+   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+
+   if (vmx86_stats) {
+      MXUserEnableStats(trackAcquisitionTime ? &lock->acquireStatsMem : NULL,
+                        trackHeldTime        ? &lock->heldStatsMem    : NULL);
+   }
+
+   return vmx86_stats;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_DisableStatsExclLock --
+ *
+ *      Disable basic stats on the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_DisableStatsExclLock(MXUserExclLock *lock)  // IN/OUT:
+{
+   ASSERT(lock);
+   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+
+   if (vmx86_stats) {
+      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
+   }
+
+   return vmx86_stats;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_SetContentionRatioFloorExclLock --
+ *
+ *      Set the contention ratio floor for the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_SetContentionRatioFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
+                                       double ratio)          // IN:
 {
    Bool result;
 
    ASSERT(lock);
    MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
-   switch (command) {
-   case MXUSER_CONTROL_ACQUISITION_HISTO: {
-      if (vmx86_stats) {
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (acquireStats == NULL) {
-            result = FALSE;
-         } else {
-            va_list a;
-            uint32 decades;
-            uint64 minValue;
-
-            va_start(a, command);
-            minValue = va_arg(a, uint64);
-            decades = va_arg(a, uint32);
-            va_end(a);
-
-            MXUserForceHisto(&acquireStats->histo,
-                             MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
-
-            result = TRUE;
-         }
-
-      } else {
-         result = FALSE;
-      }
-
-      break;
+   if (vmx86_stats) {
+      result = MXUserSetContentionRatioFloor(&lock->acquireStatsMem, ratio);
+   } else {
+      result = FALSE;
    }
 
-   case MXUSER_CONTROL_HELD_HISTO: {
-      if (vmx86_stats) {
-         MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+   return result;
+}
 
-         if (heldStats == NULL) {
-            result = FALSE;
-         } else {
-            va_list a;
-            uint32 decades;
-            uint32 minValue;
 
-            va_start(a, command);
-            minValue = va_arg(a, uint64);
-            decades = va_arg(a, uint32);
-            va_end(a);
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_SetContentionCountFloorExclLock --
+ *
+ *      Set the contention count floor for the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
 
-            MXUserForceHisto(&heldStats->histo, MXUSER_STAT_CLASS_HELD,
-                             minValue, decades);
+Bool
+MXUser_SetContentionCountFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
+                                       uint64 count)          // IN:
+{
+   Bool result;
 
-            result = TRUE;
-         }
-      } else {
-         result = FALSE;
-      }
+   ASSERT(lock);
+   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
-      break;
+   if (vmx86_stats) {
+      result = MXUserSetContentionCountFloor(&lock->acquireStatsMem, count);
+   } else {
+      result = FALSE;
    }
 
-   case MXUSER_CONTROL_ENABLE_STATS: {
-      if (vmx86_stats) {
-         va_list a;
-         Bool trackHeldTimes;
-         MXUserHeldStats *heldStats;
-         MXUserAcquireStats *acquireStats;
+   return result;
+}
 
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-         if (LIKELY(acquireStats == NULL)) {
-            MXUserAcquireStats *before;
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_SetContentionDurationFloorExclLock --
+ *
+ *      Set the contention count floor for the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
 
-            acquireStats = Util_SafeCalloc(1, sizeof(*acquireStats));
-            MXUserAcquisitionStatsSetUp(&acquireStats->data);
+Bool
+MXUser_SetContentionDurationFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
+                                          uint64 duration)       // IN:
+{
+   Bool result;
 
-            before = Atomic_ReadIfEqualWritePtr(&lock->acquireStatsMem, NULL,
-                                                (void *) acquireStats);
+   ASSERT(lock);
+   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
-            if (before) {
-               free(acquireStats);
-            }
-         }
-
-         va_start(a, command);
-         trackHeldTimes = va_arg(a, int);
-         va_end(a);
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if ((heldStats == NULL) && trackHeldTimes) {
-            MXUserHeldStats *before;
-
-            heldStats = Util_SafeCalloc(1, sizeof(*heldStats));
-            MXUserBasicStatsSetUp(&heldStats->data, MXUSER_STAT_CLASS_HELD);
-
-            before = Atomic_ReadIfEqualWritePtr(&lock->heldStatsMem, NULL,
-                                                (void *) heldStats);
-
-            if (before) {
-               free(heldStats);
-            }
-         }
-
-         lock->header.statsFunc = MXUserStatsActionExcl;
-
-         result = TRUE;
-      } else {
-         result = FALSE;
-      }
-
-      break;
-   }
-
-   default:
+   if (vmx86_stats) {
+      result = MXUserSetContentionDurationFloor(&lock->acquireStatsMem,
+                                                duration);
+   } else {
       result = FALSE;
    }
 
@@ -294,7 +304,7 @@ MXUserDumpExclLock(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
-   Warning("\tserial number %u\n", lock->header.serialNumber);
+   Warning("\tserial number %u\n", lock->header.bits.serialNumber);
 
    Warning("\tlock count %d\n", MXRecLockCount(&lock->recursiveLock));
 
@@ -326,9 +336,7 @@ MXUser_CreateExclLock(const char *userName,  // IN:
 {
    uint32 statsMode;
    char *properName;
-   MXUserExclLock *lock;
-
-   lock = Util_SafeCalloc(1, sizeof(*lock));
+   MXUserExclLock *lock = Util_SafeCalloc(1, sizeof *lock);
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "X-%p", GetReturnAddress());
@@ -346,24 +354,25 @@ MXUser_CreateExclLock(const char *userName,  // IN:
    lock->header.signature = MXUserGetSignature(MXUSER_TYPE_EXCL);
    lock->header.name = properName;
    lock->header.rank = rank;
-   lock->header.serialNumber = MXUserAllocSerialNumber();
+   lock->header.bits.serialNumber = MXUserAllocSerialNumber();
    lock->header.dumpFunc = MXUserDumpExclLock;
 
    statsMode = MXUserStatsMode();
 
-   switch (statsMode) {
+   switch (MXUserStatsMode()) {
    case 0:
+      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
       lock->header.statsFunc = NULL;
-      Atomic_WritePtr(&lock->acquireStatsMem, NULL);
-      Atomic_WritePtr(&lock->heldStatsMem, NULL);
       break;
 
    case 1:
-      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS, FALSE);
+      MXUserEnableStats(&lock->acquireStatsMem, NULL);
+      lock->header.statsFunc = MXUserStatsActionExcl;
       break;
 
    case 2:
-      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS, TRUE);
+      MXUserEnableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
+      lock->header.statsFunc = MXUserStatsActionExcl;
       break;
 
    default:
@@ -404,35 +413,15 @@ MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
                             __FUNCTION__);
       }
 
-      lock->header.signature = 0;  // just in case...
-
       MXRecLockDestroy(&lock->recursiveLock);
 
       MXUserRemoveFromList(&lock->header);
 
       if (vmx86_stats) {
-         MXUserHeldStats *heldStats;
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (LIKELY(acquireStats != NULL)) {
-            MXUserAcquisitionStatsTearDown(&acquireStats->data);
-            MXUserHistoTearDown(Atomic_ReadPtr(&acquireStats->histo));
-
-            free(acquireStats);
-         }
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            MXUserBasicStatsTearDown(&heldStats->data);
-            MXUserHistoTearDown(Atomic_ReadPtr(&heldStats->histo));
-
-            free(heldStats);
-         }
+         MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
       }
 
+      lock->header.signature = 0;  // just in case...
       free(lock->header.name);
       lock->header.name = NULL;
       free(lock);
@@ -477,7 +466,8 @@ MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
       if (LIKELY(acquireStats != NULL)) {
          MXUserHisto *histo;
 
-         MXUserAcquisitionSample(&acquireStats->data, TRUE, value != 0,
+         MXUserAcquisitionSample(&acquireStats->data, TRUE,
+                            value > acquireStats->data.contentionDurationFloor,
                                  value);
 
          histo = Atomic_ReadPtr(&acquireStats->histo);
