@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -20,9 +20,7 @@
 #include <windows.h>
 #else
 #if defined(__APPLE__)
-#include <mach/mach_init.h>
-#include <mach/task.h>
-#include <mach/semaphore.h>
+#include <dispatch/dispatch.h>
 #else
 #if (_XOPEN_SOURCE < 600) && !defined(__FreeBSD__) && !defined(sun)
 #undef _XOPEN_SOURCE
@@ -49,18 +47,18 @@
 typedef HANDLE NativeSemaphore;
 #else
 #if defined(__APPLE__)
-typedef semaphore_t NativeSemaphore;
+/*
+ * The Mac OS implementation uses dispatch_semaphore_t instead of
+ * semaphore_t due to better performance in the uncontended case, by
+ * avoiding a system call.  It, also, avoids weird error cases that
+ * resulted in bug 916600.
+ */
+typedef dispatch_semaphore_t NativeSemaphore;
 #else
 typedef sem_t NativeSemaphore;
 #endif
 #endif
 
-typedef struct
-{
-   MXUserAcquisitionStats  data;
-   Atomic_Ptr              histo;
-} MXUserAcquireStats;
-
 struct MXUserSemaphore
 {
    MXUserHeader     header;
@@ -101,7 +99,7 @@ struct MXUserSemaphore
 
 #if defined(_WIN32)
 static int
-MXUserInit(NativeSemaphore *sema)  // IN:
+MXUserInit(NativeSemaphore *sema)  // OUT:
 {
    *sema = Win32U_CreateSemaphore(NULL, 0, INT_MAX, NULL);
 
@@ -185,15 +183,17 @@ MXUserUp(NativeSemaphore *sema)  // IN:
 
 #elif defined(__APPLE__)
 static int
-MXUserInit(NativeSemaphore *sema)  // IN:
+MXUserInit(NativeSemaphore *sema)  // OUT:
 {
-   return semaphore_create(mach_task_self(), sema, SYNC_POLICY_FIFO, 0);
+   *sema = dispatch_semaphore_create(0);
+   return *sema == NULL;
 }
 
 static int
 MXUserDestroy(NativeSemaphore *sema)  // IN:
 {
-   return semaphore_destroy(mach_task_self(), *sema);
+   dispatch_release(*sema);
+   return 0;
 }
 
 static int
@@ -201,63 +201,17 @@ MXUserTimedDown(NativeSemaphore *sema,  // IN:
                 uint32 msecWait,        // IN:
                 Bool *downOccurred)     // OUT:
 {
-   uint64 nsecWait;
-   VmTimeType before;
-   kern_return_t err;
-
-   ASSERT_ON_COMPILE(KERN_SUCCESS == 0);
-
-   /*
-    * Work in nanoseconds. Time the semaphore_timedwait operation in case
-    * it is interrupted (KERN_ABORT). If it is, determine how much time is
-    * necessary to fulfill the specified wait time and retry with a new
-    * and appropriate timeout.
-    */
-
-   nsecWait = 1000000ULL * (uint64) msecWait;
-   before = Hostinfo_SystemTimerNS();
-
-   do {
-      VmTimeType after;
-      mach_timespec_t ts;
-
-      ts.tv_sec = nsecWait / MXUSER_A_BILLION;
-      ts.tv_nsec = nsecWait % MXUSER_A_BILLION;
-
-      err = semaphore_timedwait(*sema, ts);
-      after = Hostinfo_SystemTimerNS();
-
-      if (err == KERN_SUCCESS) {
-         *downOccurred = TRUE;
-      } else {
-         *downOccurred = FALSE;
-
-         if (err == KERN_OPERATION_TIMED_OUT) {
-            /* Really timed out; no down occurred, no error */
-            err = KERN_SUCCESS;
-         } else {
-            if (err == KERN_ABORTED) {
-               VmTimeType duration = after - before;
-
-               if (duration < nsecWait) {
-                  nsecWait -= duration;
-
-                  before = after;
-               } else {
-                  err = KERN_SUCCESS;  // "timed out" anyway... no error
-               }
-            }
-         }
-      }
-   } while (nsecWait && (err == KERN_ABORTED));
-
-   return err;
+   int64 nsecWait = 1000000LL * (int64)msecWait;
+   dispatch_time_t deadline = dispatch_time(DISPATCH_TIME_NOW, nsecWait);
+   *downOccurred = dispatch_semaphore_wait(*sema, deadline) == 0;
+   return 0;
 }
 
 static int
 MXUserDown(NativeSemaphore *sema)  // IN:
 {
-   return semaphore_wait(*sema);
+   dispatch_semaphore_wait(*sema, DISPATCH_TIME_FOREVER);
+   return 0;
 }
 
 static int
@@ -265,24 +219,22 @@ MXUserTryDown(NativeSemaphore *sema,  // IN:
               Bool *downOccurred)     // OUT:
 {
    /*
-    * Use a wait for zero time to implement the try operation. This timed
-    * down will either succeed immediately (down occurred), fail (something
-    * terrible happened) or time out immediately (the down could not be
-    * performed and that is OK).
+    * Provide 'try' semantics by requesting an immediate timeout.
     */
-
-   return MXUserTimedDown(sema, 0, downOccurred);
+   *downOccurred = dispatch_semaphore_wait(*sema, DISPATCH_TIME_NOW) == 0;
+   return 0;
 }
 
 static int
 MXUserUp(NativeSemaphore *sema)  // IN:
 {
-    return semaphore_signal(*sema);
+    dispatch_semaphore_signal(*sema);
+    return 0;
 }
 
 #else
 static int
-MXUserInit(NativeSemaphore *sema)  // IN:
+MXUserInit(NativeSemaphore *sema)  // OUT:
 {
    return (sem_init(sema, 0, 0) == -1) ? errno : 0;
 }
@@ -360,7 +312,7 @@ MXUserTryDown(NativeSemaphore *sema,  // IN:
       *downOccurred = FALSE;
 
       /*
-       * If the error that occured indicates that the try operation cannot
+       * If the error that occurred indicates that the try operation cannot
        * succeed (EAGAIN) or was interrupted (EINTR) suppress the error
        * indicator as these are considered "normal", non-error cases.
        *
@@ -427,10 +379,9 @@ MXUserStatsActionSema(MXUserHeader *header)  // IN:
       MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
 
       if (isHot) {
-         MXUserForceHisto(&acquireStats->histo,
-                          MXUSER_STAT_CLASS_ACQUISITION,
-                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                          MXUSER_DEFAULT_HISTO_DECADES);
+         MXUserForceAcquisitionHisto(&sema->acquireStatsMem,
+                                     MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                                     MXUSER_DEFAULT_HISTO_DECADES);
 
          if (doLog) {
             Log("HOT SEMAPHORE (%s); contention ratio %f\n",
@@ -467,7 +418,7 @@ MXUserDumpSemaphore(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", sema->header.signature);
    Warning("\tname %s\n", sema->header.name);
    Warning("\trank 0x%X\n", sema->header.rank);
-   Warning("\tserial number %u\n", sema->header.serialNumber);
+   Warning("\tserial number %u\n", sema->header.bits.serialNumber);
 
    Warning("\treference count %u\n", Atomic_Read(&sema->activeUserCount));
    Warning("\taddress of native semaphore %p\n", &sema->nativeSemaphore);
@@ -499,9 +450,7 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
                        MX_Rank rank)          // IN:
 {
    char *properName;
-   MXUserSemaphore *sema;
-
-   sema = Util_SafeCalloc(1, sizeof(*sema));
+   MXUserSemaphore *sema = Util_SafeCalloc(1, sizeof *sema);
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "Sema-%p", GetReturnAddress());
@@ -510,24 +459,30 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
    }
 
    if (LIKELY(MXUserInit(&sema->nativeSemaphore) == 0)) {
+      uint32 statsMode;
+
       sema->header.signature = MXUserGetSignature(MXUSER_TYPE_SEMA);
       sema->header.name = properName;
       sema->header.rank = rank;
-      sema->header.serialNumber = MXUserAllocSerialNumber();
+      sema->header.bits.serialNumber = MXUserAllocSerialNumber();
       sema->header.dumpFunc = MXUserDumpSemaphore;
 
-      if (MXUserStatsMode() == 0) {
+      statsMode = MXUserStatsMode();
+
+      switch (MXUserStatsMode()) {
+      case 0:
+         MXUserDisableStats(&sema->acquireStatsMem, NULL);
          sema->header.statsFunc = NULL;
-         Atomic_WritePtr(&sema->acquireStatsMem, NULL);
-      } else {
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Util_SafeCalloc(1, sizeof(*acquireStats));
-
-         MXUserAcquisitionStatsSetUp(&acquireStats->data);
+         break;
 
+      case 1:
+      case 2:
+         MXUserEnableStats(&sema->acquireStatsMem, NULL);
          sema->header.statsFunc = MXUserStatsActionSema;
-         Atomic_WritePtr(&sema->acquireStatsMem, acquireStats);
+         break;
+
+      default:
+         Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
       }
 
       MXUserAddToList(&sema->header);
@@ -558,7 +513,7 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
  */
 
 void
-MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
+MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    if (LIKELY(sema != NULL)) {
       int err;
@@ -611,7 +566,7 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
  *      on a semaphore.
  *
  * Results:
- *      The count will be decremented; a sleep may occur until the decement
+ *      The count will be decremented; a sleep may occur until the decrement
  *      is possible.
  *
  * Side effects:
@@ -782,7 +737,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
  *
  * NOTE:
  *      A "TryAcquire" does not rank check should the down operation succeed.
- *      This duplicates the behavor of MX semaphores.
+ *      This duplicates the behavior of MX semaphores.
  *
  *-----------------------------------------------------------------------------
  */
@@ -847,7 +802,14 @@ MXUser_UpSemaphore(MXUserSemaphore *sema)  // IN/OUT:
    ASSERT(sema);
    MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
-   Atomic_Inc(&sema->activeUserCount);
+   /*
+    * The purpose of the activeUserCount tracking is to help catch potentially
+    * fatal cases of destroying an active semaphore; it is not expected to be
+    * (nor it can be) perfect (with low overhead). In this case the time spent
+    * in up is tiny and a decrement at the bottom might not be reached before
+    * another thread comes out of down and does a destroy - so no
+    * activeUserCount tracking here.
+    */
 
    err = MXUserUp(&sema->nativeSemaphore);
 
@@ -855,8 +817,6 @@ MXUser_UpSemaphore(MXUserSemaphore *sema)  // IN/OUT:
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
                          __FUNCTION__, err);
    }
-
-   Atomic_Dec(&sema->activeUserCount);
 }
 
 
