@@ -1,5 +1,5 @@
 /* **********************************************************
- * Copyright 1998 VMware, Inc.  All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc.  All rights reserved.
  * **********************************************************/
 
 /*
@@ -248,7 +248,7 @@ CodeSetGetModulePath(uint32 priv)
 
 #else
 #if defined(VMX86_SERVER)
-   if (HostType_OSIsPureVMK()) {
+   if (HostType_OSIsVMK()) {
       goto exit;
    }
 #endif
@@ -604,15 +604,14 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
    } else {
       /* Use a default ICU data dir. */
 #   if defined __APPLE__
-      Location_GetLibrary_Type *Location_GetLibrary =
-         Location_GetLibrary_Addr();
+      Location_Get_Type *Location_Get = Location_Get_Addr();
 
-      if (Location_GetLibrary) {
-         char *libDir = Location_GetLibrary();
-         Bool success =    libDir
-                        && DynBuf_Append(&dbpath, libDir, strlen(libDir));
+      if (Location_Get) {
+         char *icuDir = Location_Get("icuDir");
+         Bool success =    icuDir
+                        && DynBuf_Append(&dbpath, icuDir, strlen(icuDir));
 
-         free(libDir);
+         free(icuDir);
          if (!success) {
             goto exit;
          }
@@ -620,14 +619,11 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
 #   endif
 
       {
-         if (!DynBuf_Append(&dbpath, POSIX_ICU_DIR, strlen(POSIX_ICU_DIR))) {
+         if (!DynBuf_Append(&dbpath, POSIX_ICU_DIR, strlen(POSIX_ICU_DIR)) ||
+             !DynBuf_Append(&dbpath, "/icu", strlen("/icu"))) {
             goto exit;
          }
       }
-
-      if (!DynBuf_Append(&dbpath, "/icu", strlen("/icu"))) {
-         goto exit;
-      }
    }
    if (!DynBuf_Append(&dbpath, DIRSEPS, strlen(DIRSEPS)) ||
        !DynBuf_Append(&dbpath, ICU_DATA_FILE, strlen(ICU_DATA_FILE)) ||
@@ -1102,7 +1098,7 @@ CodeSet_GenericToGeneric(const char *codeIn,  // IN
                          const char *codeOut, // IN
                          unsigned int flags,  // IN
                          char **bufOut,       // OUT
-                         size_t *sizeOut)     // OUT
+                         size_t *sizeOut)     // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1180,7 +1176,7 @@ Bool
 CodeSet_Utf8ToCurrent(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT
+                      size_t *sizeOut)    // OUT/OPT
 {
 #if !defined(CURRENT_IS_UTF8)
    DynBuf db;
@@ -1229,7 +1225,7 @@ Bool
 CodeSet_CurrentToUtf8(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT
+                      size_t *sizeOut)    // OUT/OPT
 {
 #if !defined(CURRENT_IS_UTF8)
    DynBuf db;
@@ -1315,7 +1311,7 @@ Bool
 CodeSet_Utf16leToUtf8(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT
+                      size_t *sizeOut)    // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1359,7 +1355,7 @@ Bool
 CodeSet_Utf8ToUtf16le(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT
+                      size_t *sizeOut)    // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1403,7 +1399,7 @@ Bool
 CodeSet_Utf8FormDToUtf8FormC(const char *bufIn,     // IN
                              size_t sizeIn,         // IN
                              char **bufOut,         // OUT
-                             size_t *sizeOut)       // OUT
+                             size_t *sizeOut)       // OUT/OPT
 {
    /*
     * Fallback if necessary.
@@ -1452,7 +1448,7 @@ Bool
 CodeSet_Utf8FormCToUtf8FormD(const char *bufIn,     // IN
                              size_t sizeIn,         // IN
                              char **bufOut,         // OUT
-                             size_t *sizeOut)       // OUT
+                             size_t *sizeOut)       // OUT/OPT
 {
    /*
     * Fallback if necessary.
@@ -1499,7 +1495,7 @@ Bool
 CodeSet_CurrentToUtf16le(const char *bufIn, // IN
                          size_t sizeIn,     // IN
                          char **bufOut,     // OUT
-                         size_t *sizeOut)   // OUT
+                         size_t *sizeOut)   // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1542,7 +1538,7 @@ Bool
 CodeSet_Utf16leToCurrent(const char *bufIn,  // IN
                          size_t sizeIn,      // IN
                          char **bufOut,      // OUT
-                         size_t *sizeOut)    // OUT
+                         size_t *sizeOut)    // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1585,7 +1581,7 @@ Bool
 CodeSet_Utf16beToCurrent(const char *bufIn,  // IN
                          size_t sizeIn,      // IN
                          char **bufOut,      // OUT
-                         size_t *sizeOut)    // OUT
+                         size_t *sizeOut)    // OUT/OPT
 {
    DynBuf db;
    Bool ok;
@@ -1682,7 +1678,7 @@ CodeSet_Validate(const char *buf,   // IN: the string
    UErrorCode uerr;
 
    // ucnv_toUChars takes 32-bit int size
-   ASSERT_NOT_IMPLEMENTED(size <= (size_t) MAX_INT32);
+   VERIFY(size <= (size_t) MAX_INT32);
 
    if (size == 0) {
       return TRUE;
@@ -1705,9 +1701,9 @@ CodeSet_Validate(const char *buf,   // IN: the string
 
    uerr = U_ZERO_ERROR;
    cv = ucnv_open(code, &uerr);
-   ASSERT_NOT_IMPLEMENTED(uerr == U_ZERO_ERROR);
+   VERIFY(uerr == U_ZERO_ERROR);
    ucnv_setToUCallBack(cv, UCNV_TO_U_CALLBACK_STOP, NULL, NULL, NULL, &uerr);
-   ASSERT_NOT_IMPLEMENTED(uerr == U_ZERO_ERROR);
+   VERIFY(uerr == U_ZERO_ERROR);
    ucnv_toUChars(cv, NULL, 0, buf, size, &uerr);
    ucnv_close(cv);
 
