@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -75,7 +75,11 @@
 #include <sys/vfs.h>
 #endif
 #if !defined(sun) && !defined __ANDROID__ && (!defined(USING_AUTOCONF) || (defined(HAVE_SYS_IO_H) && defined(HAVE_SYS_SYSINFO_H)))
+#if defined(__i386__) || defined(__x86_64__)
 #include <sys/io.h>
+#else
+#define NO_IOPL
+#endif
 #include <sys/sysinfo.h>
 #ifndef HAVE_SYSINFO
 #define HAVE_SYSINFO 1
@@ -100,7 +104,6 @@
 #include "hostinfo.h"
 #include "hostinfoInt.h"
 #include "safetime.h"
-#include "vm_version.h"
 #include "str.h"
 #include "err.h"
 #include "msg.h"
@@ -122,6 +125,10 @@
 #include "dynbuf.h"
 #include "strutil.h"
 
+#if defined(__APPLE__)
+#include "utilMacos.h"
+#endif
+
 #if defined(VMX86_SERVER)
 #include "uwvmkAPI.h"
 #include "uwvmk.h"
@@ -148,12 +155,12 @@ static Atomic_Ptr hostinfoOSVersion;
 
 #define DISTRO_BUF_SIZE 255
 
+#if !defined __APPLE__ && !defined USERWORLD
 typedef struct lsb_distro_info {
    char *name;
    char *scanstring;
 } LSBDistroInfo;
 
-
 static const LSBDistroInfo lsbFields[] = {
    {"DISTRIB_ID=",          "DISTRIB_ID=%s"},
    {"DISTRIB_RELEASE=",     "DISTRIB_RELEASE=%s"},
@@ -162,30 +169,13 @@ static const LSBDistroInfo lsbFields[] = {
    {NULL, NULL},
 };
 
-
 typedef struct distro_info {
    char *name;
    char *filename;
 } DistroInfo;
 
+/* KEEP SORTED! (sort -d) */
 static const DistroInfo distroArray[] = {
-   {"OracleLinux",        "/etc/oracle-release"},
-   {"RedHat",             "/etc/redhat-release"},
-   {"RedHat",             "/etc/redhat_version"},
-   {"Sun",                "/etc/sun-release"},
-   {"SuSE",               "/etc/SuSE-release"},
-   {"SuSE",               "/etc/novell-release"},
-   {"SuSE",               "/etc/sles-release"},
-   {"Debian",             "/etc/debian_version"},
-   {"Debian",             "/etc/debian_release"},
-   {"Mandrake",           "/etc/mandrake-release"},
-   {"Mandriva",           "/etc/mandriva-release"},
-   {"Mandrake",           "/etc/mandrakelinux-release"},
-   {"TurboLinux",         "/etc/turbolinux-release"},
-   {"Fedora Core",        "/etc/fedora-release"},
-   {"Gentoo",             "/etc/gentoo-release"},
-   {"Novell",             "/etc/nld-release"},
-   {"Ubuntu",             "/etc/lsb-release"},
    {"Annvix",             "/etc/annvix-release"},
    {"Arch",               "/etc/arch-release"},
    {"Arklinux",           "/etc/arklinux-release"},
@@ -193,23 +183,42 @@ static const DistroInfo distroArray[] = {
    {"BlackCat",           "/etc/blackcat-release"},
    {"Cobalt",             "/etc/cobalt-release"},
    {"Conectiva",          "/etc/conectiva-release"},
+   {"Debian",             "/etc/debian_release"},
+   {"Debian",             "/etc/debian_version"},
+   {"Fedora Core",        "/etc/fedora-release"},
+   {"Gentoo",             "/etc/gentoo-release"},
    {"Immunix",            "/etc/immunix-release"},
    {"Knoppix",            "/etc/knoppix_version"},
    {"Linux-From-Scratch", "/etc/lfs-release"},
    {"Linux-PPC",          "/etc/linuxppc-release"},
+   {"Mandrake",           "/etc/mandrakelinux-release"},
+   {"Mandrake",           "/etc/mandrake-release"},
+   {"Mandriva",           "/etc/mandriva-release"},
    {"MkLinux",            "/etc/mklinux-release"},
+   {"Novell",             "/etc/nld-release"},
+   {"OracleLinux",        "/etc/oracle-release"},
+   {"Photon",             "/etc/lsb-release"},
    {"PLD",                "/etc/pld-release"},
-   {"Slackware",          "/etc/slackware-version"},
+   {"RedHat",             "/etc/redhat-release"},
+   {"RedHat",             "/etc/redhat_version"},
    {"Slackware",          "/etc/slackware-release"},
+   {"Slackware",          "/etc/slackware-version"},
    {"SMEServer",          "/etc/e-smith-release"},
    {"Solaris",            "/etc/release"},
+   {"Sun",                "/etc/sun-release"},
+   {"SuSE",               "/etc/novell-release"},
+   {"SuSE",               "/etc/sles-release"},
+   {"SuSE",               "/etc/SuSE-release"},
    {"Tiny Sofa",          "/etc/tinysofa-release"},
+   {"TurboLinux",         "/etc/turbolinux-release"},
+   {"Ubuntu",             "/etc/lsb-release"},
    {"UltraPenguin",       "/etc/ultrapenguin-release"},
    {"UnitedLinux",        "/etc/UnitedLinux-release"},
    {"VALinux",            "/etc/va-release"},
    {"Yellow Dog",         "/etc/yellowdog-release"},
    {NULL, NULL},
 };
+#endif
 
 #if defined __ANDROID__
 /*
@@ -454,7 +463,7 @@ Hostinfo_GetSystemBitness(void)
 }
 
 
-#if !defined __APPLE__
+#if !defined __APPLE__ && !defined USERWORLD
 /*
  *-----------------------------------------------------------------------------
  *
@@ -561,6 +570,9 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
    } else if (strstr(distroLower, "asianux server 4") ||
               strstr(distroLower, "asianux client 4")) {
       Str_Strcpy(distroShort, STR_OS_ASIANUX_4, distroShortSize);
+   } else if (strstr(distroLower, "asianux server 5") ||
+              strstr(distroLower, "asianux client 5")) {
+      Str_Strcpy(distroShort, STR_OS_ASIANUX_5, distroShortSize);
    } else if (strstr(distroLower, "aurox")) {
       Str_Strcpy(distroShort, STR_OS_AUROX, distroShortSize);
    } else if (strstr(distroLower, "black cat")) {
@@ -578,10 +590,10 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
          Str_Strcpy(distroShort, STR_OS_DEBIAN_5, distroShortSize);
       } else if (strstr(distroLower, "6.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_6, distroShortSize);
-      } else if (strstr(distroLower, "7.0")) {
+      } else if (strstr(distroLower, "7.")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_7, distroShortSize);
-      } else if (strstr(distroLower, "7.1")) {
-         Str_Strcpy(distroShort, STR_OS_DEBIAN_7_1, distroShortSize);
+      } else if (strstr(distroLower, "8.")) {
+         Str_Strcpy(distroShort, STR_OS_DEBIAN_8, distroShortSize);
       }
    } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
               StrUtil_StartsWith(distroLower, "oracle")) {
@@ -742,7 +754,6 @@ out:
 
    return ret;
 }
-#endif
 
 
 /*
@@ -764,17 +775,27 @@ out:
 static char *
 HostinfoGetCmdOutput(const char *cmd)  // IN:
 {
+   Bool isSuperUser = FALSE;
    DynBuf db;
    FILE *stream;
    char *out = NULL;
 
+   /*
+    * Attempt to lower privs, because we use popen and an attacker
+    * may control $PATH.
+    */
+   if (vmx86_linux && Id_IsSuperUser()) {
+      Id_EndSuperUser(getuid());
+      isSuperUser = TRUE;
+   }
+
    DynBuf_Init(&db);
 
    stream = Posix_Popen(cmd, "r");
    if (stream == NULL) {
       Warning("Unable to get output of command \"%s\"\n", cmd);
 
-      return NULL;
+      goto exit;
    }
 
    for (;;) {
@@ -793,7 +814,7 @@ HostinfoGetCmdOutput(const char *cmd)  // IN:
          break;
 
       default:
-         ASSERT_NOT_IMPLEMENTED(FALSE);
+         NOT_IMPLEMENTED();
       }
 
       if (line == NULL) {
@@ -808,13 +829,19 @@ HostinfoGetCmdOutput(const char *cmd)  // IN:
    if (DynBuf_Get(&db)) {
       out = (char *) DynBuf_AllocGet(&db);
    }
- closeIt:
-   DynBuf_Destroy(&db);
 
+ closeIt:
    pclose(stream);
 
+ exit:
+   DynBuf_Destroy(&db);
+
+   if (isSuperUser) {
+      Id_BeginSuperUser();
+   }
    return out;
 }
+#endif // !defined __APPLE__ && !defined USERWORLD
 
 
 /*
@@ -863,35 +890,57 @@ HostinfoOSData(void)
    Str_Sprintf(osNameFull, sizeof osNameFull, "%s %s", buf.sysname,
                buf.release);
 
-#if defined __APPLE__
+#if defined USERWORLD
+   if (buf.release[0] <= '4') {
+      Str_Strcpy(osName, "vmkernel", sizeof osName);
+   } else {
+      Str_Strcpy(osName, "vmkernel5", sizeof osName);
+   }
+   Str_Snprintf(osNameFull, sizeof osNameFull, "VMware ESXi %c.x",
+                buf.release[0]);
+#elif defined __APPLE__
    {
       /*
-       * The easiest way is to invoke "system_profiler" and hope that the
-       * format of its unlocalized output will never change.
+       * Read the version info from ServerVersion.plist or SystemVersion.plist.
+       * Mac OS Server (10.6 and earlier) has both files, and the product
+       * name in ServerVersion.plist is "Mac OS X Server". Client versions
+       * of Mac OS only have SystemVersion.plist with the name "Mac OS X".
        *
-       * Alternatively, we could do what system_profiler does and use the
-       * CFPropertyList/CFDIctionary APIs to parse
-       * /System/Library/CoreServices/{Server,System}Version.plist.
-       *
-       * On a MacBookPro4,1 (and possibly other models), invoking
-       * "system_profiler" can take several seconds: it seems to spin up the CD
-       * drive as a side-effect. So use "system_profiler SPSoftwareDataType"
-       * instead.
+       * This is better than executing system_profiler or sw_vers, or using
+       * the deprecated Gestalt() function (which only gets version numbers).
+       * All of those methods just read the same plist files anyway.
        */
-      char *sysname = HostinfoGetCmdOutput(
-                         "/usr/sbin/system_profiler SPSoftwareDataType"
-                      " | /usr/bin/grep 'System Version:'"
-                      " | /usr/bin/cut -d : -f 2");
+      static char const *versionPlists[] = {
+         "/System/Library/CoreServices/ServerVersion.plist",
+         "/System/Library/CoreServices/SystemVersion.plist"
+      };
+      unsigned int i;
+      char *productName;
+      char *productVersion;
+      char *productBuildVersion;
+      Bool haveVersion = FALSE;
 
-      if (sysname) {
-         char *trimmed = Unicode_Trim(sysname);
+      for (i = 0; !haveVersion && i < ARRAYSIZE(versionPlists); i++) {
+         CFDictionaryRef versionDict =
+            UtilMacos_CreateCFDictionaryWithContentsOfFile(versionPlists[i]);
+         if (versionDict != NULL) {
+            haveVersion = UtilMacos_ReadSystemVersion(versionDict,
+                                                      &productName,
+                                                      &productVersion,
+                                                      &productBuildVersion);
+            CFRelease(versionDict);
+         }
+      }
 
-         ASSERT_MEM_ALLOC(trimmed);
-         free(sysname);
-         Str_Snprintf(osNameFull, sizeof osNameFull, "%s", trimmed);
-         free(trimmed);
+      if (haveVersion) {
+         Str_Sprintf(osNameFull, sizeof osNameFull, "%s %s (%s) %s %s",
+                     productName, productVersion, productBuildVersion,
+                     buf.sysname, buf.release);
+         free(productName);
+         free(productVersion);
+         free(productBuildVersion);
       } else {
-         Log("%s: Failed to get output of system_profiler.\n", __FUNCTION__);
+         Log("%s: Failed to read system version plist.\n", __FUNCTION__);
          /* Fall back to returning the original osNameFull. */
       }
 
@@ -931,7 +980,7 @@ HostinfoOSData(void)
        * Try to get OS detailed information from the lsb_release command.
        */
 
-      lsbOutput = HostinfoGetCmdOutput("lsb_release -sd 2>/dev/null");
+      lsbOutput = HostinfoGetCmdOutput("/usr/bin/lsb_release -sd 2>/dev/null");
       if (!lsbOutput) {
          int i;
 
@@ -1038,6 +1087,7 @@ HostinfoOSData(void)
    }
 #endif
 
+#ifndef USERWORLD
    if (Hostinfo_GetSystemBitness() == 64) {
       if (strlen(osName) + sizeof STR_OS_64BIT_SUFFIX > sizeof osName) {
          Warning("%s: Error: buffer too small\n", __FUNCTION__);
@@ -1046,6 +1096,7 @@ HostinfoOSData(void)
       }
       Str_Strcat(osName, STR_OS_64BIT_SUFFIX, sizeof osName);
    }
+#endif
 
    /*
     * Before returning, truncate the \n character at the end of the full name.
@@ -1349,24 +1400,24 @@ Hostinfo_OSIsSMP(void)
  *-----------------------------------------------------------------------------
  */
 
-Unicode
+char *
 Hostinfo_NameGet(void)
 {
-   Unicode result;
+   char *result;
 
    static Atomic_Ptr state; /* Implicitly initialized to NULL. --hpreg */
 
    result = Atomic_ReadPtr(&state);
 
    if (UNLIKELY(result == NULL)) {
-      Unicode before;
+      char *before;
 
       result = Hostinfo_HostName();
 
       before = Atomic_ReadIfEqualWritePtr(&state, NULL, result);
 
       if (before) {
-         Unicode_Free(result);
+         free(result);
 
          result = before;
       }
@@ -1393,14 +1444,14 @@ Hostinfo_NameGet(void)
  *-----------------------------------------------------------------------------
  */
 
-Unicode
+char *
 Hostinfo_GetUser(void)
 {
    char buffer[BUFSIZ];
    struct passwd pw;
    struct passwd *ppw = &pw;
-   Unicode env = NULL;
-   Unicode name = NULL;
+   char *env = NULL;
+   char *name = NULL;
 
    if ((Posix_Getpwuid_r(getuid(), &pw, buffer, sizeof buffer, &ppw) == 0) &&
        (ppw != NULL)) {
@@ -1771,7 +1822,7 @@ HostinfoSystemTimerPosix(VmTimeType *result)  // OUT
 VmTimeType
 Hostinfo_SystemTimerNS(void)
 {
-   VmTimeType result;
+   VmTimeType result = 0;  // = 0 silence compiler warning
 
    if ((vmx86_apple && HostinfoSystemTimerMach(&result)) ||
        (vmx86_posix && HostinfoSystemTimerPosix(&result))) {
@@ -1912,7 +1963,7 @@ Hostinfo_ResetProcessState(const int *keepFds, // IN:
       err = iopl(0);
 #endif
       Id_SetEUid(euid);
-      ASSERT_NOT_IMPLEMENTED(err == 0);
+      VERIFY(err == 0);
    }
 #endif
 }
@@ -2397,7 +2448,7 @@ HostinfoGetCpuInfo(int nCpu,    // IN:
          /* Free previous value */
          free(value);
          value = strdup(s);
-         ASSERT_MEM_ALLOC(value);
+         VERIFY(value);
 
          cpu++;
       }
@@ -2565,6 +2616,9 @@ Hostinfo_GetCpuDescription(uint32 cpuNumber)  // IN:
    return HostinfoGetSysctlStringAlloc("hw.model");
 #else
 #ifdef VMX86_SERVER
+#ifdef VM_ARM_64
+   return strdup("armv8 unknown");
+#else
    if (HostType_OSIsVMK()) {
       char mName[48];
 
@@ -2579,7 +2633,8 @@ Hostinfo_GetCpuDescription(uint32 cpuNumber)  // IN:
 
       return NULL;
    }
-#endif
+#endif // VM_ARM_64
+#endif // VMX86_SERVER
 
    return HostinfoGetCpuInfo(cpuNumber, "model name");
 #endif
@@ -2816,7 +2871,7 @@ HostinfoGetKernelZoneElemSizeZprint(char const *name) // IN: Kernel zone name
       pid = waitpid(child, &status, 0);
    } while ((pid == -1) && (errno == EINTR));
 
-   ASSERT_NOT_IMPLEMENTED(pid == child);
+   VERIFY(pid == child);
 
    retval = shared->retval;
    munmap((void *)shared, sizeof *shared);
@@ -2975,7 +3030,7 @@ Hostinfo_SystemUpTime(void)
     * /proc/uptime does not exist on Visor.  Use syscall instead.
     * Discovering Visor is a run-time check with a compile-time hint.
     */
-   if (vmx86_server && HostType_OSIsPureVMK()) {
+   if (vmx86_server && HostType_OSIsVMK()) {
       uint64 uptime;
 #ifdef VMX86_SERVER
       if (UNLIKELY(VMKernel_GetUptimeUS(&uptime) != VMK_OK)) {
@@ -3463,10 +3518,10 @@ Hostinfo_GetMemoryInfoInPages(unsigned int *minSize,      // OUT:
  *-----------------------------------------------------------------------------
  */
 
-Unicode
+char *
 Hostinfo_GetModulePath(uint32 priv)  // IN:
 {
-   Unicode path;
+   char *path;
 
 #if defined(__APPLE__)
    uint32_t pathSize = FILE_MAXPATH;
