@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -110,7 +110,7 @@ IOV_Zero(VMIOVec *iov)   // IN
       size_t c = MIN(numBytesLeft, iov->entries[i].iov_len);
       void *buf;
 
-      ASSERT_NOT_IMPLEMENTED(i < iov->numEntries);
+      VERIFY(i < iov->numEntries);
       buf = iov->entries[i].iov_base;
       ASSERT(buf && buf != LAZY_ALLOC_MAGIC);
       memset(buf, 0, c);
@@ -460,7 +460,7 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
  *
  *      This function takes an iov and a buffer as input and writes the content
  *      of the buffers pointed to by the iov into buf.
- * 
+ *
  * Result:
  *      None.
  *
@@ -471,10 +471,10 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
  */
 
 void
-IOV_WriteIovToBuf(struct iovec* entries,   // IN
-                  int numEntries,          // IN
-                  uint8* bufOut,           // OUT
-                  size_t bufSize)          // IN
+IOV_WriteIovToBuf(struct iovec const *entries, // IN
+                  int numEntries,              // IN
+                  uint8 *bufOut,               // OUT
+                  size_t bufSize)              // IN
 {
    size_t count = 0;
    int i;
@@ -497,7 +497,7 @@ IOV_WriteIovToBuf(struct iovec* entries,   // IN
          return;
       }
 
-      ASSERT_NOT_IMPLEMENTED(count <= bufSize);
+      VERIFY(count <= bufSize);
    }
 }
 
@@ -570,7 +570,7 @@ IOV_Free(VMIOVec* iov)     // IN
  *
  *      This function copies the content of bufIn into the buffer pointed to by
  *      entries. It basically does the opposite of IOV_WriteIovToBuf.
- * 
+ *
  * Result:
  *      None.
  *
@@ -581,16 +581,16 @@ IOV_Free(VMIOVec* iov)     // IN
  */
 
 void
-IOV_WriteBufToIov(const uint8* bufIn,           // IN
-                  size_t bufSize,               // IN
-                  struct iovec* entries,        // OUT
-                  int numEntries)               // IN
+IOV_WriteBufToIov(const uint8 *bufIn,          // IN
+                  size_t bufSize,              // IN
+                  struct iovec const *entries, // OUT
+                  int numEntries)              // IN
 {
    size_t count = 0;
    int i;
 
    ASSERT(entries);
-   ASSERT_BUG(29009, bufIn);
+   VERIFY_BUG(29009, bufIn);
 
    for (i = 0; i < numEntries; i++) {
       size_t numBytes;
@@ -605,7 +605,7 @@ IOV_WriteBufToIov(const uint8* bufIn,           // IN
       if (count >= bufSize) {
          return;
       }
-      ASSERT_NOT_IMPLEMENTED(count <= bufSize);
+      VERIFY(count <= bufSize);
    }
 }
 
@@ -697,7 +697,7 @@ IOV_WriteIovToBufPlus(struct iovec* entries,    // IN
    size_t count = bufSize;
    int i;
 
-   ASSERT_BUG(29009, bufOut);
+   VERIFY_BUG(29009, bufOut);
 
    i = IOVFindFirstEntryOffset(entries, numEntries, iovOffset, &entryOffset);
 
@@ -756,7 +756,7 @@ IOV_WriteBufToIovPlus(uint8* bufIn,             // IN
    size_t count = bufSize;
    int i;
 
-   ASSERT_BUG(29009, bufIn);
+   VERIFY_BUG(29009, bufIn);
 
    i = IOVFindFirstEntryOffset(entries, numEntries, iovOffset, &entryOffset);
 
@@ -764,7 +764,7 @@ IOV_WriteBufToIovPlus(uint8* bufIn,             // IN
       char *base = (char *)(entries[i].iov_base) + entryOffset;
       size_t iov_len = entries[i].iov_len;
 
-      ASSERT_BUG(33859, entries[i].iov_base || entries[i].iov_len == 0);
+      VERIFY_BUG(33859, entries[i].iov_base || entries[i].iov_len == 0);
       ASSERT(entries[i].iov_base != LAZY_ALLOC_MAGIC);
 
       if (iov_len <= 0) {
