@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2011 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -194,9 +194,9 @@ MsgList_AppendStr(MsgList **list,  // IN reference to existing list
  */
 
 void
-MsgList_VAppend(MsgList **list,     // IN reference to existing list
-                const char *idFmt,  // IN message ID and English message
-                va_list args)       // IN args
+MsgList_VAppend(MsgList **list,     // IN/OUT/OPT: reference to existing list
+                const char *idFmt,  // IN: message ID and English message
+                va_list args)       // IN: args
 {
    ASSERT(idFmt != NULL);
 
@@ -251,9 +251,9 @@ MsgList_VAppend(MsgList **list,     // IN reference to existing list
  */
 
 void
-MsgList_Append(MsgList **list,     // IN reference to existing list
-               const char *idFmt,  // IN message ID and English message
-               ...)                // IN args
+MsgList_Append(MsgList **list,     // IN/OUT/OPT: reference to existing list
+               const char *idFmt,  // IN: message ID and English message
+               ...)                // IN: args
 {
    va_list args;
 
