@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -38,6 +38,7 @@
 #endif
 
 #include "vmware.h"
+#include "log.h"
 #include "random.h"
 #include "util.h"
 
@@ -57,7 +58,7 @@
  *
  *-----------------------------------------------------------------------------
  */
- 
+
 static Bool
 RandomBytesWin32(size_t size,   // IN:
                  void *buffer)  // OUT:
@@ -65,20 +66,24 @@ RandomBytesWin32(size_t size,   // IN:
    HCRYPTPROV csp;
 
    if (size != (DWORD) size) {
+      Log("%s: Overflow: %"FMTSZ"d\n", __FUNCTION__, size);
       return FALSE;
    }
 
    if (CryptAcquireContext(&csp, NULL, NULL, PROV_RSA_FULL,
                            CRYPT_VERIFYCONTEXT) == FALSE) {
+      Log("%s: CryptAcquireContext failed: %d\n", __FUNCTION__, GetLastError());
       return FALSE;
    }
 
    if (CryptGenRandom(csp, (DWORD) size, buffer) == FALSE) {
+      Log("%s: CryptGenRandom failed: %d\n", __FUNCTION__, GetLastError());
       CryptReleaseContext(csp, 0);
       return FALSE;
    }
 
    if (CryptReleaseContext(csp, 0) == FALSE) {
+      Log("%s: CryptReleaseContext failed: %d\n", __FUNCTION__, GetLastError());
       return FALSE;
    }
 
@@ -229,7 +234,8 @@ struct rqContext {
 rqContext *
 Random_QuickSeed(uint32 seed)  // IN:
 {
-   struct rqContext *rs;
+   uint32 i;
+   struct rqContext *rs = Util_SafeMalloc(sizeof *rs);
 
    const uint32 xx[N] = {
       0x95F24DAB, 0x0B685215, 0xE76CCAE7, 0xAF3EC239, 0x715FAD23,
@@ -239,19 +245,14 @@ Random_QuickSeed(uint32 seed)  // IN:
       0x512C0C03, 0xEA857CCD, 0x4CC1D30F, 0x8891A8A1, 0xA6B7AADB
    };
 
-   rs = (struct rqContext *) Util_SafeMalloc(sizeof *rs);
 
-   if (rs != NULL) {
-      uint32 i;
-
-      for (i = 0; i < N; i++) {
-         rs->x[i] = xx[i] ^ seed;
-      }
-
-      rs->p = N - 1;
-      rs->q = N - M - 1;
+   for (i = 0; i < N; i++) {
+      rs->x[i] = xx[i] ^ seed;
    }
 
+   rs->p = N - 1;
+   rs->q = N - M - 1;
+
    return rs;
 }
 
