@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,11 +30,45 @@
 #include <stdlib.h>
 
 #include "vm_assert.h"
+#include "util.h"
+
+#ifdef __APPLE__
+#include <TargetConditionals.h>
+#if !defined TARGET_OS_IPHONE
+#define TARGET_OS_IPHONE 0
+#endif
+#endif
+
+static NORETURN void UtilAllocationFailure0(void);
+static NORETURN void UtilAllocationFailure1(int bugNumber, 
+                                            const char *file, int lineno);
+
+
+static void
+UtilAllocationFailure0(void)
+{
+   Panic("Unrecoverable memory allocation failure\n");
+}
+
+
+static void
+UtilAllocationFailure1(int bugNumber, const char *file, int lineno)
+{
+   if (bugNumber == -1) {
+      Panic("Unrecoverable memory allocation failure at %s:%d\n",
+            file, lineno);
+   } else {
+      Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
+            "number: %d\n", file, lineno, bugNumber);
+   }
+}
+
 
 /*
  *-----------------------------------------------------------------------------
  *
- * Util_SafeInternalMalloc --
+ * UtilSafeMalloc0 --
+ * UtilSafeMalloc1 --
  *      Helper function for malloc
  *
  * Results:
@@ -47,21 +81,25 @@
  */
 
 void *
-Util_SafeInternalMalloc(int bugNumber,        // IN:
-                        size_t size,          // IN:
-			const char *file,     // IN:
-                        int lineno)           // IN:
+UtilSafeMalloc0(size_t size)            // IN:
 {
    void *result = malloc(size);
-
    if (result == NULL && size != 0) {
-      if (bugNumber == -1) {
-         Panic("Unrecoverable memory allocation failure at %s:%d\n",
-               file, lineno);
-      } else {
-         Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
-               "number: %d\n", file, lineno, bugNumber);
-      }
+      UtilAllocationFailure0();
+   }
+   return result;
+}
+
+
+void *
+UtilSafeMalloc1(size_t size,            // IN:
+                int bugNumber,          // IN:
+                const char *file,       // IN:
+                int lineno)             // IN:
+{
+   void *result = malloc(size);
+   if (result == NULL && size != 0) {
+      UtilAllocationFailure1(bugNumber, file, lineno);
    }
    return result;
 }
@@ -70,7 +108,8 @@ Util_SafeInternalMalloc(int bugNumber,        // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * Util_SafeInternalRealloc --
+ * UtilSafeRealloc0 --
+ * UtilSafeRealloc1 --
  *      Helper function for realloc
  *
  * Results:
@@ -83,22 +122,27 @@ Util_SafeInternalMalloc(int bugNumber,        // IN:
  */
 
 void *
-Util_SafeInternalRealloc(int bugNumber,        // IN:
-                         void *ptr,            // IN:
-                         size_t size,          // IN:
-                         const char *file,     // IN:
-                         int lineno)           // IN:
+UtilSafeRealloc0(void *ptr,            // IN:
+                 size_t size)          // IN:
 {
    void *result = realloc(ptr, size);
-
    if (result == NULL && size != 0) {
-      if (bugNumber == -1) {
-         Panic("Unrecoverable memory allocation failure at %s:%d\n",
-               file, lineno);
-      } else {
-         Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
-               "number: %d\n", file, lineno, bugNumber);
-      }
+      UtilAllocationFailure0();
+   }
+   return result;
+}
+
+
+void *
+UtilSafeRealloc1(void *ptr,            // IN:
+                 size_t size,          // IN:
+                 int bugNumber,        // IN:
+                 const char *file,     // IN:
+                 int lineno)           // IN:
+{
+   void *result = realloc(ptr, size);
+   if (result == NULL && size != 0) {
+      UtilAllocationFailure1(bugNumber, file, lineno);
    }
    return result;
 }
@@ -107,7 +151,8 @@ Util_SafeInternalRealloc(int bugNumber,        // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * Util_SafeInternalCalloc --
+ * UtilSafeCalloc0 --
+ * UtilSafeCalloc1 --
  *      Helper function for calloc
  *
  * Results:
@@ -120,22 +165,27 @@ Util_SafeInternalRealloc(int bugNumber,        // IN:
  */
 
 void *
-Util_SafeInternalCalloc(int bugNumber,        // IN:
-                        size_t nmemb,         // IN:
-                        size_t size,          // IN:
-			const char *file,     // IN:
-                        int lineno)           // IN:
+UtilSafeCalloc0(size_t nmemb,         // IN:
+                size_t size)          // IN:
 {
    void *result = calloc(nmemb, size);
-
    if (result == NULL && nmemb != 0 && size != 0) {
-      if (bugNumber == -1) {
-         Panic("Unrecoverable memory allocation failure at %s:%d\n",
-               file, lineno);
-      } else {
-         Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
-               "number: %d\n", file, lineno, bugNumber);
-      }
+      UtilAllocationFailure0();
+   }
+   return result;
+}
+
+
+void *
+UtilSafeCalloc1(size_t nmemb,         // IN:
+                size_t size,          // IN:
+                int bugNumber,        // IN:
+                const char *file,     // IN:
+                int lineno)           // IN:
+{
+   void *result = calloc(nmemb, size);
+   if (result == NULL && nmemb != 0 && size != 0) {
+      UtilAllocationFailure1(bugNumber, file, lineno);
    }
    return result;
 }
@@ -144,7 +194,7 @@ Util_SafeInternalCalloc(int bugNumber,        // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * Util_SafeInternalStrdup --
+ * Util_SafeStrdup --
  *      Helper function for strdup
  *
  * Results:
@@ -157,31 +207,40 @@ Util_SafeInternalCalloc(int bugNumber,        // IN:
  */
 
 char *
-Util_SafeInternalStrdup(int bugNumber,        // IN:
-                        const char *s,        // IN:
-                        const char *file,     // IN:
-                        int lineno)           // IN:
+UtilSafeStrdup0(const char *s)        // IN:
 {
    char *result;
-
    if (s == NULL) {
       return NULL;
    }
-
 #if defined(_WIN32)
    if ((result = _strdup(s)) == NULL) {
 #else
    if ((result = strdup(s)) == NULL) {
 #endif
-      if (bugNumber == -1) {
-         Panic("Unrecoverable memory allocation failure at %s:%d\n",
-               file, lineno);
-      } else {
-         Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
-               "number: %d\n", file, lineno, bugNumber);
-      }
+      UtilAllocationFailure0();
    }
+   return result;
+}
 
+
+char *
+UtilSafeStrdup1(const char *s,        // IN:
+                int bugNumber,        // IN:
+                const char *file,     // IN:
+                int lineno)           // IN:
+{
+   char *result;
+   if (s == NULL) {
+      return NULL;
+   }
+#if defined(_WIN32)
+   if ((result = _strdup(s)) == NULL) {
+#else
+   if ((result = strdup(s)) == NULL) {
+#endif
+      UtilAllocationFailure1(bugNumber, file, lineno);
+   }
    return result;
 }
 
@@ -189,7 +248,7 @@ Util_SafeInternalStrdup(int bugNumber,        // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * UtilSafeStrndupInternal --
+ * Util_SafeStrndup --
  *
  *      Returns a string consisting of first n characters of 's' if 's' has
  *      length >= 'n', otherwise returns a string duplicate of 's'.
@@ -204,32 +263,29 @@ Util_SafeInternalStrdup(int bugNumber,        // IN:
  */
 
 char *
-Util_SafeInternalStrndup(int bugNumber,        // IN:
-                         const char *s,        // IN:
-                         size_t n,             // IN:
-                         const char *file,     // IN:
-                         int lineno)           // IN:
+UtilSafeStrndup0(const char *s,        // IN:
+                 size_t n)             // IN:
 {
    size_t size;
    char *copy;
    const char *null;
+   size_t newSize;
 
    if (s == NULL) {
       return NULL;
    }
 
-   null = (char *) memchr(s, '\0', n);
-   size = null ? null - s: n;
-   copy = (char *) malloc(size + 1);
+   null = memchr(s, '\0', n);
+   size = null ? (size_t)(null - s) : n;
+   newSize = size + 1;
+   if (newSize < size) {  // Prevent integer overflow
+      copy = NULL;
+   } else {
+      copy = malloc(newSize);
+   }
 
    if (copy == NULL) {
-      if (bugNumber == -1) {
-         Panic("Unrecoverable memory allocation failure at %s:%d\n",
-               file, lineno);
-      } else {
-         Panic("Unrecoverable memory allocation failure at %s:%d.  Bug "
-               "number: %d\n", file, lineno, bugNumber);
-      }
+      UtilAllocationFailure0();
    }
 
    copy[size] = '\0';
@@ -237,11 +293,48 @@ Util_SafeInternalStrndup(int bugNumber,        // IN:
    return (char *) memcpy(copy, s, size);
 }
 
+
+char *
+UtilSafeStrndup1(const char *s,        // IN:
+                 size_t n,             // IN:
+                 int bugNumber,        // IN:
+                 const char *file,     // IN:
+                 int lineno)           // IN:
+{
+   size_t size;
+   char *copy;
+   const char *null;
+   size_t newSize;
+
+   if (s == NULL) {
+      return NULL;
+   }
+
+   null = memchr(s, '\0', n);
+   size = null ? (size_t)(null - s) : n;
+   newSize = size + 1;
+   if (newSize < size) {  // Prevent integer overflow
+      copy = NULL;
+   } else {
+      copy = malloc(newSize);
+   }
+
+   if (copy == NULL) {
+      UtilAllocationFailure1(bugNumber, file, lineno);
+   }
+
+   copy[size] = '\0';
+
+   return (char *) memcpy(copy, s, size);
+}
+
+
 void *
 Util_Memcpy(void *dest,
             const void *src,
             size_t count)
 {
+#if defined(__x86_64__) || defined(__i386__)
    uintptr_t align = ((uintptr_t)dest | (uintptr_t)src | count);
 
 #if defined __GNUC__
@@ -313,6 +406,7 @@ Util_Memcpy(void *dest,
    #endif
 
 
+#endif
 #endif
 
    memcpy(dest, src, count);
