@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -46,18 +46,25 @@
 #  endif
 #  include <unistd.h>
 #  include <pwd.h>
+#  include <sys/socket.h>    // for AF_INET[6]
+#  include <arpa/inet.h>     // for inet_pton
+#  include <netinet/in.h>    // for INET6_ADDRSTRLEN
 #endif
 
 #if defined(__APPLE__) || defined(__FreeBSD__)
 #include <pthread.h>
+#elif defined(__sun__)
+#include <thread.h>
+#endif
+
+#if defined(__APPLE__)
+#include <CoreFoundation/CoreFoundation.h>
 #endif
 
 #include "vmware.h"
 #include "msg.h"
 #include "util.h"
 #include "str.h"
-/* For HARD_EXPIRE --hpreg */
-#include "vm_version.h"
 #include "su.h"
 #include "escape.h"
 #include "posix.h"
@@ -67,6 +74,10 @@
 #include "win32u.h"
 #include "win32util.h"
 #endif
+#if defined(__APPLE__)
+#include "utilMacos.h"
+#endif
+
 /*
  * ESX with userworld VMX
  */
@@ -196,7 +207,7 @@ Util_GetCanonicalPathForHash(const char *path)  // IN: UTF-8
  *-----------------------------------------------------------------------------
  */
 
-static char*
+static char *
 UtilGetLegacyEncodedString(const char *path)  // IN: UTF-8
 {
    char *ret = NULL;
@@ -323,10 +334,10 @@ Util_CanonicalPathsIdentical(const char *path1,  // IN:
 Bool
 Util_IsAbsolutePath(const char *path)  // IN: path to check
 {
-#if defined(__linux__) || defined(__APPLE__)
+#if defined __linux__ || defined __APPLE__ || defined __FreeBSD__ || defined sun
    // path[0] is valid even for the empty string.
    return path && path[0] == DIRSEPC;
-#elif defined(_WIN32)
+#elif defined _WIN32
    // if the length is 2, path[2] will be valid because of the null terminator.
    if (!path || strlen(path) < 2) {
       return FALSE;
@@ -376,7 +387,7 @@ Util_GetPrime(unsigned n0)  // IN:
     * There is no 32-bit prime larger than 4294967291.
     */
 
-   ASSERT_NOT_IMPLEMENTED(n0 <= 4294967291U);
+   VERIFY(n0 <= 4294967291U);
    if (n0 <= 2) {
       return 2;
    }
@@ -405,43 +416,6 @@ Util_GetPrime(unsigned n0)  // IN:
 }
 
 
-#if defined(linux) && !defined(__ANDROID__)
-/*
- * Android has its own gettid. gettid has been declared in
- * <sys/linux-unistd.h> and defined as an extern function in <unistd.h>.
- */
-
-/*
- *-----------------------------------------------------------------------------
- *
- * gettid --
- *
- *      Retrieve unique thread identification suitable for kill or setpriority.
- *	Do not call this function directly, use Util_GetCurrentThreadId()
- *      instead.
- *
- * Results:
- *      Unique thread identification on success.
- *      (pid_t)-1 on error, errno set (when kernel does not support this call)
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE
-pid_t gettid(void)
-{
-#if defined(SYS_gettid)
-   return (pid_t)syscall(SYS_gettid);
-#else
-   return -1;
-#endif
-}
-#endif
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -450,6 +424,8 @@ pid_t gettid(void)
  *      Retrieves a unique thread identification suitable to identify a thread
  *      to kill it or change its scheduling priority.
  *
+ *      The tid is NOT guaranteed to be correct across fork().
+ *
  * Results:
  *      Unique thread identification on success.
  *	ASSERTs on failure (should not happen).
@@ -463,48 +439,53 @@ pid_t gettid(void)
 Util_ThreadID
 Util_GetCurrentThreadId(void)
 {
-#if defined(linux) || defined __ANDROID__
+#if defined(__linux__) && !defined(__ANDROID__)
    /*
-    * It is possible that two threads can enter gettid() path simultaneously,
-    * both eventually clearing useTid to zero. It does not matter - only
-    * problem which can happen is that useTid will be set to zero twice.
-    * And it has no impact on useTid value...
+    * Linux does not declare gettid, but the raw syscall
+    * works fine. We must supply our own TLS caching.
     */
-
-   static int useTid = 1;
-#if defined(VMX86_SERVER) && defined(GLIBC_VERSION_25)
-   static __thread pid_t tid = -1;
-#else
-   pid_t tid;
-#endif
-
-
-   if (useTid) {
-#if defined(VMX86_SERVER) && defined(GLIBC_VERSION_25)
-      if (tid != -1) {
-         return tid;
-      }
-#endif
-      tid = gettid();
-      if (tid != (pid_t)-1) {
-         return tid;
-      }
-      ASSERT(errno == ENOSYS);
-      useTid = 0;
+#if defined(GLIBC_VERSION_23)
+   static __thread pid_t tid;
+   if (UNLIKELY(tid == 0)) {
+      tid = (pid_t)syscall(SYS_gettid);
+      ASSERT(tid != -1);  // All kernels that support TLS also implement gettid
    }
-   tid = getpid();
-   ASSERT(tid != (pid_t)-1);
-
    return tid;
-#elif defined(sun)
-   pid_t tid;
+#else
+   /*
+    * Always a syscall, but glibc-2.2 doesn't have __thread TLS so we couldn't
+    * cache anyway. We don't support kernels from before 2.4.11 when SYS_gettid
+    * was introduced, so no need to check the return value.
+    */
+   return (pid_t)syscall(SYS_gettid);
+#endif
 
-   tid = getpid();
-   ASSERT(tid != (pid_t)-1);
-
-   return tid;
-#elif defined(__APPLE__) || defined(__FreeBSD__)
-   ASSERT_ON_COMPILE(sizeof(Util_ThreadID) == sizeof(pthread_t));
+#elif defined(__ANDROID__)
+   /*
+    * Bionic supplies a gettid implementation in <unistd.h> that
+    * natively uses TLS.
+    */
+   return gettid();
+#elif defined(__sun__)
+   /*
+    * The old thr_ API returns an integer thread identifier. It is
+    * still available with Solaris pthreads.
+    */
+   return thr_self();
+#elif defined(__APPLE__)
+   /*
+    * NB: do not use mach_thread_self here. mach_thread_self returns
+    * a reference and requires a matching mach_port_deallocate, which
+    * would take two syscalls instead of zero.
+    */
+   return pthread_mach_thread_np(pthread_self());
+#elif defined(__FreeBSD__)
+   /*
+    * These OSes do not implement OS-native thread IDs. You probably
+    * didn't need one anyway, but guess that pthread_self works
+    * well enough.
+    */
+   ASSERT_ON_COMPILE(sizeof(Util_ThreadID) >= sizeof(pthread_t));
 
    return pthread_self();
 #elif defined(_WIN32)
@@ -621,10 +602,10 @@ UtilGetLoginName(struct passwd *pwd) // IN/OPT: user passwd struct
  *----------------------------------------------------------------------
  */
 
-static Unicode
-UtilDoTildeSubst(ConstUnicode user)  // IN: name of user
+static char *
+UtilDoTildeSubst(const char *user)  // IN: name of user
 {
-   Unicode str = NULL;
+   char *str = NULL;
    struct passwd *pwd = NULL;
 
    if (gHomeDirOverride) {
@@ -700,11 +681,11 @@ UtilDoTildeSubst(ConstUnicode user)  // IN: name of user
 
 #define UTIL_MAX_PATH_CHUNKS 100
 
-Unicode
-Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
+char *
+Util_ExpandString(const char *fileName) // IN  file path to expand
 {
-   Unicode copy = NULL;
-   Unicode result = NULL;
+   char *copy = NULL;
+   char *result = NULL;
    int nchunk = 0;
    char *chunks[UTIL_MAX_PATH_CHUNKS];
    int chunkSize[UTIL_MAX_PATH_CHUNKS];
@@ -719,8 +700,7 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
    /*
     * quick exit
     */
-   if (!Unicode_StartsWith(fileName, "~") &&
-       Unicode_Find(fileName, "$") == UNICODE_INDEX_NOT_FOUND) {
+   if ((*fileName != '~') && (strchr(fileName, '$') == NULL)) {
       return copy;
    }
 
@@ -728,8 +708,8 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
     * XXX Because the rest part of code depends pretty heavily from
     *     character pointer operations we want to leave it as-is and
     *     don't want to re-work it with using unicode library. However
-    *     it's acceptable only until our Unicode type is utf-8 and
-    *     until code below works correctly with utf-8.
+    *     it's acceptable only until our Unicode type is UTF-8 and
+    *     until code below works correctly with UTF-8.
     */
 
    /*
@@ -755,7 +735,7 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
       }
       if (nchunk >= UTIL_MAX_PATH_CHUNKS) {
          Log("%s: Filename \"%s\" has too many chunks.\n", __FUNCTION__,
-             UTF8(fileName));
+             fileName);
 	 goto out;
       }
       chunks[nchunk] = cp;
@@ -793,7 +773,7 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
 
    for (i = 0; i < nchunk; i++) {
       char save;
-      Unicode expand = NULL;
+      char *expand = NULL;
       char buf[100];
 #if defined(_WIN32)
       utf16_t bufW[100];
@@ -874,7 +854,7 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
       chunks[i] = expand;
       if (chunks[i] == NULL) {
 	 Log("%s: Cannot allocate memory to expand \"%s\" in \"%s\".\n",
-             __FUNCTION__, expand, UTF8(fileName));
+             __FUNCTION__, expand, fileName);
 	 goto out;
       }
       chunkSize[i] = strlen(expand);
@@ -894,7 +874,7 @@ Util_ExpandString(ConstUnicode fileName) // IN  file path to expand
    }
    if (result == NULL) {
       Log("%s: Cannot allocate memory for the expansion of \"%s\".\n",
-          __FUNCTION__, UTF8(fileName));
+          __FUNCTION__, fileName);
       goto out;
    }
    cp = result;
@@ -949,3 +929,309 @@ Util_OverrideHomeDir(const char *path) // IN
    ASSERT(!gHomeDirOverride);
    gHomeDirOverride = Util_SafeStrdup(path);
 }
+
+
+#if defined(__APPLE__)
+
+/*
+ * XXX TODO: move these to utilMacos.c after it gets split up to avoid
+ * dependencies on IOKit and DiskArbitration.
+ */
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Util_CFStringToUTF8CString --
+ *
+ *      Convert a CFString into a UTF-8 encoded C string.
+ *
+ *      Amazingly, CFString does not provide this functionality, so everybody
+ *      (including Apple, see smb-217.18/lib/smb/charsets.c in darwinsource)
+ *      ends up re-implementing it this way...
+ *
+ * Results:
+ *      On success: Allocated, UTF8-encoded, NUL-terminated string.
+ *      On failure: NULL.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+char *
+Util_CFStringToUTF8CString(CFStringRef s) // IN
+{
+   static CFStringEncoding const encoding = kCFStringEncodingUTF8;
+   char const *fast;
+   char *result;
+
+   ASSERT(s);
+
+   fast = CFStringGetCStringPtr(s, encoding);
+   if (fast) {
+      result = strdup(fast);
+   } else {
+      size_t maxSize;
+
+      maxSize = CFStringGetMaximumSizeForEncoding(CFStringGetLength(s),
+                                                  encoding) + 1;
+      result = malloc(maxSize);
+      if (result) {
+         if (CFStringGetCString(s, result, maxSize, encoding)) {
+            /*
+             * It is likely that less than 'maxSize' bytes have actually been
+             * written into 'result'. If that becomes a problem in the future,
+             * we can always trim the buffer here.
+             */
+         } else {
+            free(result);
+            result = NULL;
+         }
+      }
+   }
+
+   if (!result) {
+      Log("Failed to get C string from CFString.\n");
+   }
+
+   return result;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * UtilMacos_CreateCFDictionaryWithContentsOfFile --
+ *
+ *      Creates and returns a dictionary with the contents of the specified
+ *      property list file. The file can be either XML or binary.
+ *
+ *      This is equivalent to +[NSDictionary dictionaryWithContentsOfFile:],
+ *      unfortunately Apple did not provide a similar CoreFoundation function.
+ *
+ * Results:
+ *      Dictionary with contents of the property list file, or NULL on failure.
+ *      Caller must release the dictionary with CFRelease.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+CFDictionaryRef
+UtilMacos_CreateCFDictionaryWithContentsOfFile(const char *path) // IN
+{
+   struct stat s;
+   CFURLRef url;
+   CFReadStreamRef stream = NULL;
+   CFPropertyListRef plist = NULL;
+   CFPropertyListFormat unusedFormat;
+   CFDictionaryRef result = NULL;
+
+   /*
+    * Avoid creating the unnecessary CFURL and CFReadStream objects if the
+    * file does not exist. Trying to read a non-existent file will only
+    * fail in CFReadStreamOpen().
+    */
+   if (Posix_Stat(path, &s) != 0) {
+      return NULL;
+   }
+
+   url = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault,
+                                                 (const UInt8 *)path,
+                                                 strlen(path),
+                                                 false);
+   if (   url != NULL
+       && (stream = CFReadStreamCreateWithFile(kCFAllocatorDefault, url))
+       && CFReadStreamOpen(stream)
+       && (plist = CFPropertyListCreateFromStream(kCFAllocatorDefault,
+                                                  stream,
+                                                  0,
+                                                  kCFPropertyListImmutable,
+                                                  &unusedFormat,
+                                                  NULL))
+       && (CFGetTypeID(plist) == CFDictionaryGetTypeID())) {
+      result = (CFDictionaryRef)plist;
+      plist = NULL;
+   }
+
+   if (plist) {
+      CFRelease(plist);
+   }
+
+   if (stream) {
+      CFReadStreamClose(stream);
+      CFRelease(stream);
+   }
+
+   if (url) {
+      CFRelease(url);
+   }
+
+   return result;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * UtilMacos_ReadSystemVersion --
+ *
+ *      Reads the Mac OS system version from the provided dictionary, and
+ *      returns one or more of the requested values.
+ *
+ * Results:
+ *      TRUE if the all the values were read successfully, or FALSE on failure.
+ *      Caller must free the values.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+UtilMacos_ReadSystemVersion(CFDictionaryRef versionDict, // IN
+                            char **productName,          // OUT/OPT
+                            char **productVersion,       // OUT/OPT
+                            char **productBuildVersion)  // OUT/OPT
+{
+   struct {
+      const CFStringRef key;
+      char **outValue;
+   } keyedValues[] = {
+      { CFSTR("ProductName"),         productName         },
+      { CFSTR("ProductVersion"),      productVersion      },
+      { CFSTR("ProductBuildVersion"), productBuildVersion },
+   };
+   int i;
+
+   for (i = 0; i < ARRAYSIZE(keyedValues); i++) {
+      CFStringRef curVal;
+
+      if (keyedValues[i].outValue == NULL) {
+         continue;
+      }
+
+      curVal = CFDictionaryGetValue(versionDict, keyedValues[i].key);
+      if (   curVal != NULL
+          && CFGetTypeID(curVal) == CFStringGetTypeID()
+          && (*keyedValues[i].outValue = Util_CFStringToUTF8CString(curVal))) {
+         continue;
+      }
+
+      /*
+       * Error retrieving one of the values. Clean up any previously saved
+       * values and return with failure.
+       */
+      while (--i >= 0) {
+         if (keyedValues[i].outValue != NULL) {
+            free(*keyedValues[i].outValue);
+            *keyedValues[i].outValue = NULL;
+         }
+      }
+
+      return FALSE;
+   }
+
+   return TRUE;
+}
+
+#endif // defined(__APPLE__)
+
+/*
+ * These are generic functions which might be useful on any platform
+ * (ESX/Linux/BSD/Apple/Windows). Currently the Windows version have
+ * not been defined as that is not very straightforward and I do not
+ * know how to fully test it. f.e  we have to handle char* to wchar_t*
+ * conversion, also it appears to cause linking problems for some Windows
+ * tools.
+ * If you want to use these for Windows, please feel free to fix these
+ * and remove the ifdef macro below and in util.h.
+ */
+
+#if !defined(_WIN32)
+/*
+ *----------------------------------------------------------------------
+ *
+ * Util_IPv4AddrValid --
+ *
+ * Results:
+ *      TRUE if the passed-in string represents a valid IPv4 address in
+ *      dotted decimal notation, FALSE otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+Util_IPv4AddrValid(const char *addr)  // IN: IPv4 address string
+{
+   struct in_addr dummyInAddr;
+
+   return inet_pton(AF_INET, addr, &dummyInAddr) == 1;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Util_IPv6AddrValid --
+ *
+ * Results:
+ *      TRUE if the passed-in string represents a valid IPv6 address,
+ *      FALSE otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+Util_IPv6AddrValid(const char *addr)  // IN: IPv6 address string
+{
+   struct in6_addr dummyInAddr;
+   char   ipv6AddrStr[INET6_ADDRSTRLEN + 1];
+
+   /*
+    * IPv6 link-local addresses can have a suffix of the form
+    * %{ifname}, e.g. %vmk0, %eth0.
+    * We cannot pass it as-is to inet_pton(), need to strip off the
+    * suffix before passing to inet_pton().
+    */
+   if (sscanf(addr, "%"XSTR(INET6_ADDRSTRLEN)"[^%]",
+              ipv6AddrStr) != 1) {
+      return FALSE;
+   }
+   return inet_pton(AF_INET6, ipv6AddrStr, &dummyInAddr) == 1;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Util_IPAddrValid --
+ *
+ * Results:
+ *      TRUE if the passed-in string represents a valid IPv4 or IPv6 address,
+ *      FALSE otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+Util_IPAddrValid(const char *addr)  // IN: IPv4 or IPv6 address string
+{
+   return Util_IPv4AddrValid(addr) || Util_IPv6AddrValid(addr);
+}
+
+#endif   /* ! _WIN32 */
