@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -70,7 +70,7 @@
 #include "util.h"
 #include "str.h"
 
-#define MAX_IFACES      4
+#define MAX_IFACES      64
 #define LOOPBACK        "lo"    // XXX: We would have a problem with something like "loa0".
 #ifndef INET_ADDRSTRLEN
 #define INET_ADDRSTRLEN 16
@@ -348,87 +348,6 @@ NetUtil_GetIfName(int ifIndex)  // IN: interface index
 }
 
 
-#   else // ifdef DUMMY_NETUTIL
-/*
- *-----------------------------------------------------------------------------
- *
- * NetUtil_GetIfIndex --
- *
- *      Given an interface name, return its index.
- *
- * Results:
- *      Returns a valid interface index on success or -1 on failure.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-NetUtil_GetIfIndex(const char *ifName)  // IN: interface name
-{
-   struct ifreq ifreq;
-   int fd = -1;
-   int ifIndex = -1;
-
-   memset(&ifreq, 0, sizeof ifreq);
-   if (Str_Snprintf(ifreq.ifr_name, sizeof ifreq.ifr_name, "%s", ifName) == -1) {
-      return -1;
-   }
-
-   if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
-      return -1;
-   }
-
-   if (ioctl(fd, SIOCGIFINDEX, &ifreq) == 0) {
-      ifIndex = ifreq.ifr_ifindex;
-   }
-
-   close(fd);
-
-   return ifIndex;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * NetUtil_GetIfName --
- *
- *      Given an interface index, return its name.
- *
- * Results:
- *      Returns a valid interface name on success, NULL on failure.
- *
- * Side effects:
- *      Caller is responsible for freeing the returned string.
- *
- *-----------------------------------------------------------------------------
- */
-
-char *
-NetUtil_GetIfName(int ifIndex)  // IN: interface index
-{
-   struct ifreq ifreq;
-   char *ifName = NULL;
-   int fd = -1;
-
-   memset(&ifreq, 0, sizeof ifreq);
-   ifreq.ifr_ifindex = ifIndex;
-
-   if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
-      return NULL;
-   }
-
-   if (ioctl(fd, SIOCGIFNAME, &ifreq) == 0) {
-      ifName = Util_SafeStrdup(ifreq.ifr_name);
-   }
-
-   close(fd);
-
-   return ifName;
-}
 #   endif // ifdef DUMMY_NETUTIL
 
 
