@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -24,10 +24,6 @@
  *
  */
 
-#ifndef VMX86_DEVEL
-
-#endif
-
 // pull in setresuid()/setresgid() if possible
 #define  _GNU_SOURCE
 #include <unistd.h>
@@ -81,10 +77,15 @@
 #include "dynarray.h"
 #include "su.h"
 #include "str.h"
-#include "fileIO.h"
+#include "strutil.h"
 #include "codeset.h"
 #include "unicode.h"
 
+#ifdef USERWORLD
+#include <vm_basic_types.h>
+#include <vmkuserstatus.h>
+#include <vmkusercompat.h>
+#endif
 
 /*
  * All signals that:
@@ -107,7 +108,7 @@ static int const cSignals[] = {
 struct ProcMgr_AsyncProc {
    pid_t waiterPid;          // pid of the waiter process
    pid_t resultPid;          // pid of the process created for the client
-   FileIODescriptor fd;      // fd to write to when the child is done
+   int fd;                   // fd to write to when the child is done
    Bool validExitCode;
    int exitCode;
 };
@@ -126,11 +127,14 @@ static Bool ProcMgrKill(pid_t pid,
 
 #if defined(__APPLE__)
 static int ProcMgrGetCommandLineArgs(long pid,
-                                     DynBuf *argsBuf);
+                                     DynBuf *argsBuf,
+                                     char **procCmdName);
 #endif
 
 #ifdef sun
-#define  SOLARIS_BASH_PATH "/usr/bin/bash"
+#define  BASH_PATH "/usr/bin/bash"
+#else
+#define  BASH_PATH "/bin/bash"
 #endif
 
 #if defined(linux) && !defined(GLIBC_VERSION_23) && !defined(__UCLIBC__)
@@ -190,7 +194,7 @@ setresgid(gid_t ruid,
  *----------------------------------------------------------------------
  */
 
-#if !defined(sun) && !defined(__FreeBSD__) && !defined(__APPLE__)
+#if defined(linux)
 int
 ProcMgr_ReadProcFile(int fd,                       // IN
                      char **contents)              // OUT
@@ -244,7 +248,6 @@ done:
    return size;
 }
 
-#endif   // !sun && !FreeBSD && !APPLE
 
 /*
  *----------------------------------------------------------------------
@@ -264,7 +267,6 @@ done:
  *----------------------------------------------------------------------
  */
 
-#if defined(linux)
 ProcMgrProcInfoArray *
 ProcMgr_ListProcesses(void)
 {
@@ -279,7 +281,8 @@ ProcMgr_ListProcesses(void)
 
    procList = Util_SafeCalloc(1, sizeof *procList);
    ProcMgrProcInfoArray_Init(procList, 0);
-   procInfo.procCmd = NULL;
+   procInfo.procCmdName = NULL;
+   procInfo.procCmdLine = NULL;
    procInfo.procOwner = NULL;
 
    /*
@@ -353,6 +356,8 @@ ProcMgr_ListProcesses(void)
       unsigned long long dummy;
       unsigned long long relativeStartTime;
       char *stringBegin;
+      char *cmdNameBegin;
+      Bool cmdNameLookup = TRUE;
 
       /*
        * We only care about dirs that look like processes.
@@ -404,6 +409,24 @@ ProcMgr_ListProcesses(void)
           */
          for (replaceLoop = 0 ; replaceLoop < (numRead - 1) ; replaceLoop++) {
             if ('\0' == cmdLineTemp[replaceLoop]) {
+               if (cmdNameLookup) {
+                  /*
+                   * Store the command name.
+                   * Find the last path separator, to get the cmd name.
+                   * If no separator is found, then use the whole name.
+                   */
+                  cmdNameBegin = strrchr(cmdLineTemp, '/');
+                  if (NULL == cmdNameBegin) {
+                     cmdNameBegin = cmdLineTemp;
+                  } else {
+                     /*
+                      * Skip over the last separator.
+                      */
+                     cmdNameBegin++;
+                  }
+                  procInfo.procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+                  cmdNameLookup = FALSE;
+               }
                cmdLineTemp[replaceLoop] = ' ';
             }
          }
@@ -454,6 +477,10 @@ ProcMgr_ListProcesses(void)
                *(copyItr++) = *(nameStart++);
             }
             *copyItr = '\0';
+            /*
+             * Store the command name.
+             */
+            procInfo.procCmdName = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
          }
       }
 
@@ -527,9 +554,9 @@ ProcMgr_ListProcesses(void)
        * Store the command line string pointer in dynbuf.
        */
       if (cmdLineTemp) {
-         procInfo.procCmd = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
+         procInfo.procCmdLine = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
       } else {
-         procInfo.procCmd = Unicode_Alloc("", STRING_ENCODING_UTF8);
+         procInfo.procCmdLine = Unicode_Alloc("", STRING_ENCODING_UTF8);
       }
 
       /*
@@ -558,7 +585,8 @@ ProcMgr_ListProcesses(void)
                  __FUNCTION__);
          goto abort;
       }
-      procInfo.procCmd = NULL;
+      procInfo.procCmdName = NULL;
+      procInfo.procCmdLine = NULL;
       procInfo.procOwner = NULL;
 
 next_entry:
@@ -573,7 +601,8 @@ next_entry:
 abort:
    closedir(dir);
 
-   free(procInfo.procCmd);
+   free(procInfo.procCmdName);
+   free(procInfo.procCmdLine);
    free(procInfo.procOwner);
 
    if (failed) {
@@ -619,7 +648,8 @@ ProcMgr_ListProcesses(void)
    int flag=0;
 
    procList = Util_SafeCalloc(1, sizeof *procList);
-   procInfo.procCmd = NULL;
+   procInfo.procCmdName = NULL;
+   procInfo.procCmdLine = NULL;
    procInfo.procOwner = NULL;
 
    /*
@@ -656,9 +686,11 @@ ProcMgr_ListProcesses(void)
    for (i = 0; i < nentries; ++i, ++kp) {
       struct passwd *pwd;
       char **cmdLineTemp = NULL;
+      char *cmdNameBegin = NULL;
+      Bool cmdNameLookup = TRUE;
 
       /*
-       * Store the pid of the process
+       * Store the pid of the process.
        */
       procInfo.procId = kp->ki_pid;
 
@@ -671,11 +703,23 @@ ProcMgr_ListProcesses(void)
                            : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
 
       /*
-       * Store the command line string of the process
+       * If the command name in the kinfo_proc struct is strictly less than the
+       * maximum allowed size, then we can save it right now. Else we shall
+       * need to try and parse it from the entire command line.
+       */
+      if (strlen(kp->ki_comm) + 1 < sizeof kp->ki_comm) {
+         procInfo.procCmdName = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
+      }
+
+      /*
+       * Store the command line string of the process.
        */
       cmdLineTemp = kvm_getargv(kd, kp, 0);
       if (cmdLineTemp != NULL) {
-         // flatten the argument list to get cmd & all params
+         /*
+          * Flatten the argument list to get cmd & all params.
+          */
          DynBuf dbuf;
 
          DynBuf_Init(&dbuf);
@@ -685,9 +729,29 @@ ProcMgr_ListProcesses(void)
                        __FUNCTION__);
                goto abort;
             }
+            if (cmdNameLookup) {
+               /*
+                * Store the command name of the process.
+                * Find the last path separator, to get the cmd name.
+                * If no separator is found, then use the whole name.
+                */
+               cmdNameBegin = strrchr(*cmdLineTemp, '/');
+               if (NULL == cmdNameBegin) {
+                  cmdNameBegin = *cmdLineTemp;
+               } else {
+                  /*
+                   * Skip over the last separator.
+                   */
+                  cmdNameBegin++;
+               }
+               procInfo.procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+               cmdNameLookup = FALSE;
+            }
             cmdLineTemp++;
             if (*cmdLineTemp != NULL) {
-               // add the whitespace between arguments
+               /*
+                * Add the whitespace between arguments.
+                */
                if (!DynBuf_Append(&dbuf, " ", 1)) {
                   Warning("%s: failed to append ' ' in DynBuf - no memory\n",
                           __FUNCTION__);
@@ -695,17 +759,23 @@ ProcMgr_ListProcesses(void)
                }
             }
          }
-         // add the NUL term
+         /*
+          * Add the NUL term.
+          */
          if (!DynBuf_Append(&dbuf, "", 1)) {
             Warning("%s: failed to append NUL in DynBuf - out of memory\n",
                     __FUNCTION__);
             goto abort;
          }
          DynBuf_Trim(&dbuf);
-         procInfo.procCmd = DynBuf_Detach(&dbuf);
+         procInfo.procCmdLine = DynBuf_Detach(&dbuf);
          DynBuf_Destroy(&dbuf);
       } else {
-         procInfo.procCmd = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+         procInfo.procCmdLine = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+         if (cmdNameLookup) {
+            procInfo.procCmdName = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
       }
 
       /*
@@ -717,7 +787,8 @@ ProcMgr_ListProcesses(void)
        * Store the process info pointer into a list buffer.
        */
       *ProcMgrProcInfoArray_AddressOf(procList, i) = procInfo;
-      procInfo.procCmd = NULL;
+      procInfo.procCmdLine = NULL;
+      procInfo.procCmdName = NULL;
       procInfo.procOwner = NULL;
 
    } // for nentries
@@ -729,7 +800,8 @@ abort:
       kvm_close(kd);
    }
 
-   free(procInfo.procCmd);
+   free(procInfo.procCmdLine);
+   free(procInfo.procCmdName);
    free(procInfo.procOwner);
 
    if (failed) {
@@ -754,6 +826,7 @@ abort:
  * Results:
  *      Number of arguments retrieved.
  *      Buffer is returned with argument names, if any.
+ *      Command name is returned.
  *
  * Side effects:
  *
@@ -762,11 +835,13 @@ abort:
 
 static int
 ProcMgrGetCommandLineArgs(long pid,               // IN:  process id
-                          DynBuf *argsBuf)        // OUT: Buffer with arguments
+                          DynBuf *argsBuf,        // OUT: Buffer with arguments
+                          char **procCmdName)     // OUT: Command name string
 {
    int argCount = 0;
    int argNum;
    char *argUnicode = NULL;
+   char *cmdNameBegin = NULL;
    char *cmdLineRaw = NULL;
    char *cmdLineTemp;
    char *cmdLineEnd;
@@ -774,6 +849,12 @@ ProcMgrGetCommandLineArgs(long pid,               // IN:  process id
    size_t maxargsSize;
    int maxargsName[] = {CTL_KERN, KERN_ARGMAX};
    int argName[] = {CTL_KERN, KERN_PROCARGS2, pid};
+   Bool cmdNameLookup = TRUE;
+   Bool failed = TRUE;
+
+   if (NULL != procCmdName) {
+      *procCmdName = NULL;
+   }
 
    /*
     * Get the sysctl kern argmax.
@@ -874,6 +955,27 @@ ProcMgrGetCommandLineArgs(long pid,               // IN:  process id
             break;
          }
          ++argCount;
+         /*
+          * If this is the command name, retrieve it.
+          */
+         if (NULL != procCmdName && cmdNameLookup) {
+            /*
+             * Store the command name of the process.
+             * Find the last path separator, to get the cmd name.
+             * If no separator is found, then use the whole name.
+             */
+            cmdNameBegin = strrchr(cmdLineTemp, '/');
+            if (NULL == cmdNameBegin) {
+               cmdNameBegin = cmdLineTemp;
+            } else {
+               /*
+                * Skip over the last separator.
+                */
+               cmdNameBegin++;
+            }
+            *procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
       }
       /*
        * Skip over the current arg that we just saved.
@@ -893,9 +995,17 @@ ProcMgrGetCommandLineArgs(long pid,               // IN:  process id
    }
    DynBuf_Trim(argsBuf);
 
+   failed = FALSE;
 abort:
    free(cmdLineRaw);
    free(argUnicode);
+   if (failed) {
+      if (NULL != procCmdName) {
+         free(*procCmdName);
+         *procCmdName = NULL;
+      }
+      argCount = 0;
+   }
 
    return argCount;
 }
@@ -932,7 +1042,8 @@ ProcMgr_ListProcesses(void)
    int procName[] = {CTL_KERN, KERN_PROC, KERN_PROC_ALL};
 
    procList = Util_SafeCalloc(1, sizeof *procList);
-   procInfo.procCmd = NULL;
+   procInfo.procCmdLine = NULL;
+   procInfo.procCmdName = NULL;
    procInfo.procOwner = NULL;
 
    /*
@@ -944,6 +1055,7 @@ ProcMgr_ListProcesses(void)
       goto abort;
    }
    nentries = (int)(procsize / sizeof *kp);
+
    /*
     * Get the list of process info structs.
     */
@@ -953,12 +1065,14 @@ ProcMgr_ListProcesses(void)
                __FUNCTION__, errno);
       goto abort;
    }
+
    /*
     * Recalculate the number of entries as they may have changed.
     */
    if (0 >= (nentries = (int)(procsize / sizeof *kp))) {
       goto abort;
    }
+
    /*
     * Pre-allocate the dynamic array of required size.
     */
@@ -967,6 +1081,7 @@ ProcMgr_ListProcesses(void)
               __FUNCTION__);
       goto abort;
    }
+
    /*
     * Iterate through the list of process entries
     */
@@ -976,11 +1091,14 @@ ProcMgr_ListProcesses(void)
       struct passwd pw;
       struct passwd *ppw = &pw;
       int error;
+      int argCount = 0;
+      Bool cmdNameLookup = TRUE;
 
       /*
        * Store the pid of the process
        */
       procInfo.procId = kptmp->kp_proc.p_pid;
+
       /*
        * Store the owner of the process.
        */
@@ -990,26 +1108,53 @@ ProcMgr_ListProcesses(void)
                            ? Str_SafeAsprintf(NULL, "%d", (int) kptmp->kp_eproc.e_pcred.p_ruid)
                            : Unicode_Alloc(ppw->pw_name, STRING_ENCODING_DEFAULT);
 
+      /*
+       * If the command name in the kinfo_proc struct is strictly less than the
+       * maximum allowed size, then we can save it right now. Else we shall
+       * need to try and parse it from the entire command line.
+       */
+      if (strlen(kptmp->kp_proc.p_comm) + 1 < sizeof kptmp->kp_proc.p_comm) {
+         procInfo.procCmdName = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
+      }
+
       /*
        * Store the command line arguments of the process.
        * If no arguments are found, use the full command name.
        */
       DynBuf_Init(&argsBuf);
-      if (ProcMgrGetCommandLineArgs(kptmp->kp_proc.p_pid, &argsBuf) > 0) {
-         procInfo.procCmd = DynBuf_Detach(&argsBuf);
+      if (cmdNameLookup) {
+         argCount = ProcMgrGetCommandLineArgs(kptmp->kp_proc.p_pid, &argsBuf, &procInfo.procCmdName);
       } else {
-         procInfo.procCmd = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+         argCount = ProcMgrGetCommandLineArgs(kptmp->kp_proc.p_pid, &argsBuf, NULL);
+      }
+      if (0 < argCount) {
+         procInfo.procCmdLine = DynBuf_Detach(&argsBuf);
+         /*
+          * cmdName would have been filled up by the function ProcMgrGetCommandLineArgs().
+          */
+         cmdNameLookup = FALSE;
+      } else {
+         procInfo.procCmdLine = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+         if (cmdNameLookup) {
+            procInfo.procCmdName = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
       }
       DynBuf_Destroy(&argsBuf);
+
       /*
        * Store the start time of the process
        */
       procInfo.procStartTime = kptmp->kp_proc.p_starttime.tv_sec;
+
       /*
        * Store the process info pointer into a list buffer.
        */
       *ProcMgrProcInfoArray_AddressOf(procList, i) = procInfo;
-      procInfo.procCmd = NULL;
+
+      procInfo.procCmdLine = NULL;
+      procInfo.procCmdName = NULL;
       procInfo.procOwner = NULL;
 
    } // nentries
@@ -1018,7 +1163,8 @@ ProcMgr_ListProcesses(void)
 
 abort:
    free(kp);
-   free(procInfo.procCmd);
+   free(procInfo.procCmdLine);
+   free(procInfo.procCmdName);
    free(procInfo.procOwner);
 
    if (failed) {
@@ -1058,10 +1204,9 @@ ProcMgr_FreeProcList(ProcMgrProcInfoArray *procList)
 
    procCount = ProcMgrProcInfoArray_Count(procList);
    for (i = 0; i < procCount; i++) {
-      ProcMgrProcInfo *procInfo;
-
-      procInfo = ProcMgrProcInfoArray_AddressOf(procList, i);
-      free(procInfo->procCmd);
+      ProcMgrProcInfo *procInfo = ProcMgrProcInfoArray_AddressOf(procList, i);
+      free(procInfo->procCmdName);
+      free(procInfo->procCmdLine);
       free(procInfo->procOwner);
    }
 
@@ -1158,45 +1303,73 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
       envpCurrent = Unicode_GetAllocList(envp, -1, STRING_ENCODING_DEFAULT);
    }
 
+#ifdef USERWORLD
+   do {
+      static const char filePath[] = "/bin/sh";
+      char * const argv[] = { "sh", "++group=host/vim/tmp",
+                              "-c", cmdCurrent, NULL };
+      int initFds[] = { STDIN_FILENO, STDOUT_FILENO, STDERR_FILENO };
+      int workingDirFd;
+      VmkuserStatus_Code status;
+      int outPid;
+
+      workingDirFd = open(workingDir != NULL ? workingDir : "/tmp", O_RDONLY);
+      status = VmkuserCompat_ForkExec(filePath,
+                                      argv,
+                                      envpCurrent,
+                                      workingDirFd,
+                                      initFds,
+                                      ARRAYSIZE(initFds),
+                                      geteuid(),
+                                      getegid(),
+                                      0,
+                                      &outPid);
+      if (workingDirFd >= 0) {
+         close(workingDirFd);
+      }
+      if (VmkuserStatus_IsOK(status)) {
+         pid = (pid_t)outPid;
+      } else {
+         VmkuserStatus_CodeToErrno(status, &errno);
+         pid = -1;
+      }
+   } while (FALSE);
+#else
    pid = fork();
 
    if (pid == -1) {
       Warning("Unable to fork: %s.\n\n", strerror(errno));
    } else if (pid == 0) {
-#ifdef sun
-      /*
-       * On Solaris, /bin/sh is the Bourne shell, and it
-       * doesn't appear to have the optimization that bash does -- when
-       * called with -c, bash appears to just use exec() to replace itself.
-       * Bourne shell does a fork & exec, so 2 processes are started.
-       * This is bad for us because we then see the PID of the shell, not the
-       * app that it starts.  When this PID is returned to a user to
-       * watch, they'll watch the wrong process.
-       *
-       * So for Solaris, use bash instead if possible.  We support
-       * Solaris 10 and better; it contains bash, but not in its
-       * minimal 'core' package, so it may not exist.
-       */
-      static const char bashShellPath[] = SOLARIS_BASH_PATH;
+      static const char bashShellPath[] = BASH_PATH;
       char *bashArgs[] = { "bash", "-c", cmdCurrent, NULL };
       static const char bourneShellPath[] = "/bin/sh";
       char *bourneArgs[] = { "sh", "-c", cmdCurrent, NULL };
       const char *shellPath;
       char **args;
-#else
-      static const char shellPath[] = "/bin/sh";
-      char *args[] = { "sh", "-c", cmdCurrent, NULL };
-#endif
 
-#ifdef sun
-      if (File_Exists(SOLARIS_BASH_PATH)) {
+      /*
+       * Check bug 772203. To start the program, we start the shell
+       * and specify the program using the option '-c'. We should return the
+       * PID of the app that gets started.
+       *
+       * When the option '-c' is specified,
+       * - bash shell just uses exec() to replace itself. So, 'bash' returns
+       * the PID of the new application that is started.
+       *
+       * - bourne shell does a fork & exec. So two processes are started. We
+       * see the PID of the shell and not the app that it starts. When the PID
+       * is returned to a user to watch, they'll watch the wrong process.
+       *
+       * In order to return the proper PID, use bash if possible. If bash
+       * is not available, then use the bourne shell.
+       */
+      if (File_Exists(bashShellPath)) {
          shellPath = bashShellPath;
          args = bashArgs;
       } else {
          shellPath = bourneShellPath;
          args = bourneArgs;
       }
-#endif
 
       /*
        * Child
@@ -1219,6 +1392,7 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
       Panic("Unable to execute the \"%s\" shell command: %s.\n\n",
             cmd, strerror(errno));
    }
+#endif
 
    /*
     * Parent
@@ -1226,7 +1400,7 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
 
    free(cmdCurrent);
    free(workDir);
-   Unicode_FreeList(envpCurrent, -1);
+   Util_FreeStringList(envpCurrent, -1);
    return pid;
 }
 
@@ -1244,12 +1418,12 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
  *      FALSE on failure or if an error occurred (detail is displayed)
  *
  * Side effects:
- *	Prevents zombification of the process.
+ *      Prevents zombification of the process.
  *
  *----------------------------------------------------------------------
  */
 
-static Bool 
+static Bool
 ProcMgrWaitForProcCompletion(pid_t pid,                 // IN
                              Bool *validExitCode,       // OUT: Optional
                              int *exitCode)             // OUT: Optional
@@ -1323,8 +1497,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
    Bool validExitCode;
    int exitCode;
    pid_t resultPid;
-   FileIODescriptor readFd;
-   FileIODescriptor writeFd;
+   int readFd, writeFd;
 
    Debug("Executing async command: '%s' in working dir '%s'\n",
          cmd, (userArgs && userArgs->workingDirectory) ? userArgs->workingDirectory : "");
@@ -1334,8 +1507,8 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
       return NULL;
    }
 
-   readFd = FileIO_CreateFDPosix(fds[0], O_RDONLY);
-   writeFd = FileIO_CreateFDPosix(fds[1], O_WRONLY);
+   readFd = fds[0];
+   writeFd = fds[1];
 
    pid = fork();
 
@@ -1362,7 +1535,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
        */
       maxfd = sysconf(_SC_OPEN_MAX);
       for (i = STDERR_FILENO + 1; i < maxfd; i++) {
-         if (i != readFd.posix && i != writeFd.posix) {
+         if (i != readFd && i != writeFd) {
             close(i);
          }
       }
@@ -1377,7 +1550,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
          status = FALSE;
       }
 
-      FileIO_Close(&readFd);
+      close(readFd);
 
       /*
        * Only run the program if we have not already experienced a failure.
@@ -1393,8 +1566,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
        * Send the child's pid back immediately, so that the caller can
        * report the result pid back synchronously.
        */
-      if (FileIO_Write(&writeFd, &childPid, sizeof childPid, NULL) !=
-          FILEIO_SUCCESS) {
+      if (write(writeFd, &childPid, sizeof childPid) == -1) {
          Warning("Waiter unable to write back to parent.\n");
          
          /*
@@ -1419,9 +1591,8 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
        * block waiting for data we'll never send.
        */
       Debug("Writing the command %s a success to fd %x\n", 
-            status ? "was" : "was not", writeFd.posix);
-      if (FileIO_Write(&writeFd, &status, sizeof status, NULL) !=
-          FILEIO_SUCCESS) {
+            status ? "was" : "was not", writeFd);
+      if (write(writeFd, &status, sizeof status) == -1) {
          Warning("Waiter unable to write back to parent\n");
          
          /*
@@ -1432,8 +1603,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
          exit(-1);
       }
 
-      if (FileIO_Write(&writeFd, &exitCode, sizeof exitCode , NULL) != 
-          FILEIO_SUCCESS) {
+      if (write(writeFd, &exitCode, sizeof exitCode) == -1) {
          Warning("Waiter unable to write back to parent\n");
          
          /*
@@ -1444,7 +1614,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
          exit(-1);
       }
 
-      FileIO_Close(&writeFd);
+      close(writeFd);
 
       if (status &&
           Signal_ResetGroupHandler(cSignals, olds, ARRAYSIZE(cSignals)) == 0) {
@@ -1464,13 +1634,13 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
     * Parent
     */
 
-   FileIO_Close(&writeFd);
+   close(writeFd);
+   writeFd = -1;
 
    /*
     * Read the pid of the child's child from the pipe.
     */
-   if (FileIO_Read(&readFd, &resultPid, sizeof resultPid , NULL) !=
-       FILEIO_SUCCESS) {
+   if (read(readFd, &resultPid, sizeof resultPid) != sizeof resultPid) {
       Warning("Unable to read result pid from the pipe.\n");
       
       /*
@@ -1495,18 +1665,18 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
 
    asyncProc = Util_SafeMalloc(sizeof *asyncProc);
    asyncProc->fd = readFd;
-   FileIO_Invalidate(&readFd);
+   readFd = -1;
    asyncProc->waiterPid = pid;
    asyncProc->validExitCode = FALSE;
    asyncProc->exitCode = -1;
    asyncProc->resultPid = resultPid;
 
  abort:
-   if (FileIO_IsValid(&readFd)) {
-      FileIO_Close(&readFd);
+   if (readFd != -1) {
+      close(readFd);
    }
-   if (FileIO_IsValid(&writeFd)) {
-      FileIO_Close(&writeFd);
+   if (writeFd != -1) {
+      close(writeFd);
    }
        
    return asyncProc;
@@ -1730,7 +1900,7 @@ ProcMgr_GetAsyncProcSelectable(ProcMgr_AsyncProc *asyncProc)
 {
    ASSERT(asyncProc);
    
-   return asyncProc->fd.posix;
+   return asyncProc->fd;
 }
 
 
@@ -1788,15 +1958,13 @@ ProcMgr_GetExitCode(ProcMgr_AsyncProc *asyncProc,  // IN
    if (asyncProc->waiterPid != -1) {
       Bool status;
 
-      if (FileIO_Read(&(asyncProc->fd), &status, sizeof status, NULL) !=
-          FILEIO_SUCCESS) {
+      if (read(asyncProc->fd, &status, sizeof status) != sizeof status) {
          Warning("Error reading async process status.\n");
          goto exit;
       }
 
-      if (FileIO_Read(&(asyncProc->fd), &(asyncProc->exitCode),
-                      sizeof asyncProc->exitCode, NULL) !=
-          FILEIO_SUCCESS) {
+      if (read(asyncProc->fd, &asyncProc->exitCode,
+               sizeof asyncProc->exitCode) != sizeof asyncProc->exitCode) {
          Warning("Error reading async process status.\n");
          goto exit;
       }
@@ -1804,7 +1972,7 @@ ProcMgr_GetExitCode(ProcMgr_AsyncProc *asyncProc,  // IN
       asyncProc->validExitCode = TRUE;
 
       Debug("Child w/ fd %x exited with code=%d\n",
-            asyncProc->fd.posix, asyncProc->exitCode);
+            asyncProc->fd, asyncProc->exitCode);
    }
 
    *exitCode = asyncProc->exitCode;
@@ -1858,8 +2026,8 @@ ProcMgr_Free(ProcMgr_AsyncProc *asyncProc) // IN
    }
 #endif 
 
-   if ((asyncProc != NULL) && FileIO_IsValid(&(asyncProc->fd))) {
-      FileIO_Close(&(asyncProc->fd));
+   if (asyncProc != NULL && asyncProc->fd != -1) {
+      close(asyncProc->fd);
    }
    
    free(asyncProc);
@@ -1936,7 +2104,9 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
    }
 
    // first change group
-#if defined(__APPLE__)
+#if defined(USERWORLD)
+   ret = Id_SetREGid(ppw->pw_gid, ppw->pw_gid);
+#elif defined(__APPLE__)
    ret = setregid(ppw->pw_gid, ppw->pw_gid);
 #else
    ret = setresgid(ppw->pw_gid, ppw->pw_gid, root_gid);
@@ -1945,13 +2115,17 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
       Warning("Failed to set gid for user %s\n", user);
       return FALSE;
    }
+#ifndef USERWORLD
    ret = initgroups(ppw->pw_name, ppw->pw_gid);
    if (ret < 0) {
       Warning("Failed to initgroups() for user %s\n", user);
       goto failure;
    }
+#endif
    // now user
-#if defined(__APPLE__)
+#if defined(USERWORLD)
+   ret = Id_SetREUid(ppw->pw_uid, ppw->pw_uid);
+#elif defined(__APPLE__)
    ret = setreuid(ppw->pw_uid, ppw->pw_uid);
 #else
    ret = setresuid(ppw->pw_uid, ppw->pw_uid, 0);
@@ -2011,7 +2185,9 @@ ProcMgr_ImpersonateUserStop(void)
    }
 
    // first change back user
-#if defined(__APPLE__)
+#if defined(USERWORLD)
+   ret = Id_SetREUid(ppw->pw_uid, ppw->pw_uid);
+#elif defined(__APPLE__)
    ret = setreuid(ppw->pw_uid, ppw->pw_uid);
 #else
    ret = setresuid(ppw->pw_uid, ppw->pw_uid, 0);
@@ -2022,7 +2198,9 @@ ProcMgr_ImpersonateUserStop(void)
    }
 
    // now group
-#if defined(__APPLE__)
+#if defined(USERWORLD)
+   ret = Id_SetREGid(ppw->pw_gid, ppw->pw_gid);
+#elif defined(__APPLE__)
    ret = setregid(ppw->pw_gid, ppw->pw_gid);
 #else
    ret = setresgid(ppw->pw_gid, ppw->pw_gid, ppw->pw_gid);
@@ -2031,11 +2209,13 @@ ProcMgr_ImpersonateUserStop(void)
       Warning("Failed to set gid for root\n");
       return FALSE;
    }
+#ifndef USERWORLD
    ret = initgroups(ppw->pw_name, ppw->pw_gid);
    if (ret < 0) {
       Warning("Failed to initgroups() for root\n");
       return FALSE;
    }
+#endif
 
    // set env
    setenv("USER", ppw->pw_name, 1);
