@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -57,7 +57,7 @@
 extern int vasprintf(char **ptr, const char *f, va_list arg);
 /*
  * Declare vswprintf on platforms where it's not known to exist.  We know
- * it's available on glibc >= 2.2, FreeBSD >= 5.0, and all versions of 
+ * it's available on glibc >= 2.2, FreeBSD >= 5.0, and all versions of
  * Solaris.
  * (Re: Solaris, vswprintf has been present since Solaris 8, and we only
  * support Solaris 9 and above, since that was the first release available
@@ -80,21 +80,21 @@ extern int vswprintf(wchar_t *wcs, size_t maxlen, const wchar_t *format, va_list
  *
  * Str_Vsnprintf --
  *
- *	Compatibility wrapper b/w different libc versions
+ *      Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *	int - number of bytes stored in 'str' (not including NUL
- *	terminate character), -1 on overflow (insufficient space for
- *	NUL terminate is considered overflow)
+ *      int - number of bytes stored in 'str' (not including NUL
+ *      terminate character), -1 on overflow (insufficient space for
+ *      NUL terminate is considered overflow)
  *
- *	NB: on overflow the buffer WILL be NUL terminated at the last
- *	UTF-8 code point boundary within the buffer's bounds.
+ *      NB: on overflow the buffer WILL be NUL terminated at the last
+ *      UTF-8 code point boundary within the buffer's bounds.
  *
  * WARNING: See warning at the top of this file.
  *
  * Side effects:
- *	None
+ *      None
  *
  *----------------------------------------------------------------------
  */
@@ -110,7 +110,7 @@ Str_Vsnprintf(char *str,          // OUT
    ASSERT(str != NULL);
    ASSERT(format != NULL);
 
-#ifdef HAS_BSD_PRINTF
+#if defined HAS_BSD_PRINTF && !defined __ANDROID__
    retval = bsd_vsnprintf(&str, size, format, ap);
 #else
    retval = vsnprintf(str, size, format, ap);
@@ -165,13 +165,12 @@ Str_Sprintf_C_Locale(char *buf,        // OUT:
                      const char *fmt,  // IN:
                      ...)              // IN:
 {
-   uint32 *stack = (uint32*) &buf;
    va_list args;
    int retval;
 
    ASSERT(buf);
    ASSERT(fmt);
-   
+
    va_start(args, fmt);
    retval = bsd_vsnprintf_c_locale(&buf, maxSize, fmt, args);
    va_end(args);
@@ -186,7 +185,7 @@ Str_Sprintf_C_Locale(char *buf,        // OUT:
    }
 
    if (retval >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
 
    return retval;
@@ -216,15 +215,14 @@ Str_Sprintf(char *buf,       // OUT
             const char *fmt, // IN
             ...)             // IN
 {
-   uint32 *stack = (uint32*) &buf;
    va_list args;
    int i;
-   
+
    va_start(args, fmt);
    i = Str_Vsnprintf(buf, maxSize, fmt, args);
    va_end(args);
    if (i < 0) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
    return i;
 }
@@ -235,23 +233,23 @@ Str_Sprintf(char *buf,       // OUT
  *
  * Str_Snprintf --
  *
- *	Compatibility wrapper b/w different libc versions
+ *      Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *	int - number of bytes stored in 'str' (not including NUL
- *	terminate character), -1 on overflow (insufficient space for
- *	NUL terminate is considered overflow)
+ *      int - number of bytes stored in 'str' (not including NUL
+ *      terminate character), -1 on overflow (insufficient space for
+ *      NUL terminate is considered overflow)
  *
- *	NB: on overflow the buffer WILL be NUL terminated
+ *      NB: on overflow the buffer WILL be NUL terminated
  *
  * Side effects:
- *	None
+ *      None
  *
  *----------------------------------------------------------------------
  */
 
-int      
+int
 Str_Snprintf(char *str,          // OUT
              size_t size,        // IN
              const char *format, // IN
@@ -292,7 +290,6 @@ Str_Strcpy(char *buf,       // OUT
            const char *src, // IN
            size_t maxSize)  // IN
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
    ASSERT(buf != NULL);
@@ -300,8 +297,7 @@ Str_Strcpy(char *buf,       // OUT
 
    len = strlen(src);
    if (len >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
-      ASSERT_BUG(5686, FALSE);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
    return memcpy(buf, src, len + 1);
 }
@@ -315,7 +311,7 @@ Str_Strcpy(char *buf,       // OUT
  *      Calculate length of the string.
  *
  * Results:
- *      Length of s not including the terminating '\0' character. 
+ *      Length of s not including the terminating '\0' character.
  *      If there is no '\0' for first maxLen bytes, then it
  *      returns maxLen.
  *
@@ -326,8 +322,8 @@ Str_Strcpy(char *buf,       // OUT
  */
 
 size_t
-Str_Strlen(const char *s,  	// IN 
-	   size_t maxLen)     	// IN
+Str_Strlen(const char *s,  // IN:
+           size_t maxLen)  // IN:
 
 {
    const char *end;
@@ -336,7 +332,7 @@ Str_Strlen(const char *s,  	// IN
 
    if ((end = memchr(s, '\0', maxLen)) == NULL) {
       return maxLen;
-   } 
+   }
    return end - s;
 }
 
@@ -346,11 +342,11 @@ Str_Strlen(const char *s,  	// IN
  *
  * Str_Strnstr --
  *
- *	Find a substring within a string of length at most n. 'sub' must be
+ *      Find a substring within a string of length at most n. 'sub' must be
  *      NUL-terminated. 'n' is interpreted as an unsigned int.
  *
  * Results:
- *	A pointer to the beginning of the substring, or NULL if not found.
+ *      A pointer to the beginning of the substring, or NULL if not found.
  *
  * Side Effects:
  *      None
@@ -359,10 +355,9 @@ Str_Strlen(const char *s,  	// IN
  */
 
 char *
-Str_Strnstr(const char *src,  	// IN 
-            const char *sub,	// IN
-            size_t n)         	// IN
-
+Str_Strnstr(const char *src,  // IN:
+            const char *sub,  // IN:
+            size_t n)         // IN:
 {
    size_t subLen;
    const char *end;
@@ -381,9 +376,9 @@ Str_Strnstr(const char *src,  	// IN
       return NULL;
    }
    for (;
-	(src = memchr(src, sub[0], end - src)) != NULL &&
-	memcmp(src, sub, subLen) != 0;
-	src++) {
+       (src = memchr(src, sub[0], end - src)) != NULL &&
+        memcmp(src, sub, subLen) != 0;
+        src++) {
    }
    return (char *) src;
 }
@@ -406,11 +401,10 @@ Str_Strnstr(const char *src,  	// IN
  */
 
 char *
-Str_Strcat(char *buf,       // IN-OUT
+Str_Strcat(char *buf,       // IN/OUT
            const char *src, // IN
            size_t maxSize)  // IN
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
@@ -422,7 +416,7 @@ Str_Strcat(char *buf,       // IN-OUT
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
 
    memcpy(buf + bufLen, src, srcLen + 1);
@@ -451,20 +445,16 @@ Str_Strcat(char *buf,       // IN-OUT
  */
 
 char *
-Str_Strncat(char *buf,       // IN-OUT
+Str_Strncat(char *buf,       // IN/OUT
             size_t bufSize,  // IN: Size of buf
             const char *src, // IN: String to append
             size_t n)        // IN: Max chars of src to append
 {
-   uint32 *stack; 
-   size_t bufLen; 
+   size_t bufLen;
 
    ASSERT(buf != NULL);
    ASSERT(src != NULL);
 
-   stack = (uint32 *)&buf;
-   bufLen = strlen(buf);
-
    /*
     * Check bufLen + n first so we can avoid the second call to strlen
     * if possible.
@@ -477,9 +467,11 @@ Str_Strncat(char *buf,       // IN-OUT
     * happen by adding the == case to the Panic test.
     */
 
+   bufLen = strlen(buf);
+
    if (bufLen + n >= bufSize &&
        bufLen + strlen(src) >= bufSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__,__LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__,__LINE__);
    }
 
    /*
@@ -508,7 +500,7 @@ Str_Strncat(char *buf,       // IN-OUT
  */
 
 char *
-Str_Asprintf(size_t *length,       // OUT
+Str_Asprintf(size_t *length,       // OUT/OPT
              const char *format,   // IN
              ...)                  // IN
 {
@@ -540,13 +532,13 @@ Str_Asprintf(size_t *length,       // OUT
  */
 
 char *
-Str_SafeAsprintf(size_t *length,       // OUT
+Str_SafeAsprintf(size_t *length,       // OUT/OPT
                  const char *format,   // IN
                  ...)                  // IN
 {
    va_list arguments;
    char *result;
-   
+
    va_start(arguments, format);
    result = Str_SafeVasprintf(length, format, arguments);
    va_end(arguments);
@@ -580,7 +572,7 @@ Str_SafeAsprintf(size_t *length,       // OUT
  */
 
 static char *
-StrVasprintfInternal(size_t *length,       // OUT:
+StrVasprintfInternal(size_t *length,       // OUT/OPT:
                      const char *format,   // IN:
                      va_list arguments,    // IN:
                      Bool assertOnFailure) // IN:
@@ -588,7 +580,7 @@ StrVasprintfInternal(size_t *length,       // OUT:
    char *buf = NULL;
    int ret;
 
-#ifdef HAS_BSD_PRINTF
+#if defined HAS_BSD_PRINTF && !defined __ANDROID__
    ret = bsd_vsnprintf(&buf, 0, format, arguments);
 
 #elif !defined sun && !defined STR_NO_WIN32_LIBS
@@ -638,7 +630,7 @@ StrVasprintfInternal(size_t *length,       // OUT:
 
   exit:
    if (assertOnFailure) {
-      ASSERT_NOT_IMPLEMENTED(buf);
+      VERIFY(buf);
    }
    return buf;
 }
@@ -662,7 +654,7 @@ StrVasprintfInternal(size_t *length,       // OUT:
  */
 
 char *
-Str_Vasprintf(size_t *length,       // OUT
+Str_Vasprintf(size_t *length,       // OUT/OPT
               const char *format,   // IN
               va_list arguments)    // IN
 {
@@ -678,7 +670,7 @@ Str_Vasprintf(size_t *length,       // OUT
  *    See StrVasprintfInternal.
  *
  * Results:
- *    Calls ASSERT_NOT_IMPLEMENTED on failure.
+ *    Calls VERIFY on failure.
  *
  * WARNING: See warning at the top of this file.
  *
@@ -689,7 +681,7 @@ Str_Vasprintf(size_t *length,       // OUT
  */
 
 char *
-Str_SafeVasprintf(size_t *length,       // OUT
+Str_SafeVasprintf(size_t *length,       // OUT/OPT
                   const char *format,   // IN
                   va_list arguments)    // IN
 {
@@ -720,15 +712,14 @@ Str_Swprintf(wchar_t *buf,       // OUT
              const wchar_t *fmt, // IN
              ...)                // IN
 {
-   uint32 *stack = (uint32*) &buf;
    va_list args;
    int i;
-   
+
    va_start(args,fmt);
    i = Str_Vsnwprintf(buf, maxSize, fmt, args);
    va_end(args);
    if (i < 0) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
    return i;
 }
@@ -739,20 +730,20 @@ Str_Swprintf(wchar_t *buf,       // OUT
  *
  * Str_Vsnwprintf --
  *
- *	Compatibility wrapper b/w different libc versions
+ *      Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *	int - number of wchar_ts stored in 'str' (not including NUL
- *	terminate character), -1 on overflow (insufficient space for
- *	NUL terminate is considered overflow)
+ *      int - number of wchar_ts stored in 'str' (not including NUL
+ *      terminate character), -1 on overflow (insufficient space for
+ *      NUL terminate is considered overflow)
  *
- *	NB: on overflow the buffer WILL be NUL terminated
+ *      NB: on overflow the buffer WILL be NUL terminated
  *
  * WARNING: See warning at the top of this file.
  *
  * Side effects:
- *	None
+ *      None
  *
  *----------------------------------------------------------------------
  */
@@ -802,23 +793,23 @@ Str_Vsnwprintf(wchar_t *str,          // OUT
  *
  * Str_Snwprintf --
  *
- *	Compatibility wrapper b/w different libc versions
+ *      Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *	int - number of wchar_ts stored in 'str' (not including NUL
- *	terminate character), -1 on overflow (insufficient space for
- *	NUL terminate is considered overflow)
+ *      int - number of wchar_ts stored in 'str' (not including NUL
+ *      terminate character), -1 on overflow (insufficient space for
+ *      NUL terminate is considered overflow)
  *
- *	NB: on overflow the buffer WILL be NUL terminated
+ *      NB: on overflow the buffer WILL be NUL terminated
  *
  * Side effects:
- *	None
+ *      None
  *
  *----------------------------------------------------------------------
  */
 
-int      
+int
 Str_Snwprintf(wchar_t *str,          // OUT
               size_t size,           // IN: Size of str, in wide-characters.
               const wchar_t *format, // IN
@@ -855,13 +846,11 @@ Str_Wcscpy(wchar_t *buf,       // OUT
            const wchar_t *src, // IN
            size_t maxSize)     // IN: Size of buf, in wide-characters.
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
    len = wcslen(src);
-   if (len >= maxSize) { 
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
-      ASSERT_BUG(5686, FALSE); 
+   if (len >= maxSize) {
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
    return memcpy(buf, src, (len + 1)*sizeof(wchar_t));
 }
@@ -884,11 +873,10 @@ Str_Wcscpy(wchar_t *buf,       // OUT
  */
 
 wchar_t *
-Str_Wcscat(wchar_t *buf,       // IN-OUT
+Str_Wcscat(wchar_t *buf,       // IN/OUT
            const wchar_t *src, // IN
            size_t maxSize)     // IN: Size of buf, in wide-characters.
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
@@ -897,7 +885,7 @@ Str_Wcscat(wchar_t *buf,       // IN-OUT
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
 
    memcpy(buf + bufLen, src, (srcLen + 1)*sizeof(wchar_t));
@@ -926,12 +914,11 @@ Str_Wcscat(wchar_t *buf,       // IN-OUT
  */
 
 wchar_t *
-Str_Wcsncat(wchar_t *buf,       // IN-OUT
+Str_Wcsncat(wchar_t *buf,       // IN/OUT
             size_t bufSize,     // IN: Size of buf, in wide-characters.
             const wchar_t *src, // IN: String to append
             size_t n)           // IN: Max chars of src to append
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t bufLen = wcslen(buf);
 
    /*
@@ -948,7 +935,7 @@ Str_Wcsncat(wchar_t *buf,       // IN-OUT
 
    if (bufLen + n >= bufSize &&
        bufLen + wcslen(src) >= bufSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__,__LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__,__LINE__);
    }
 
    /*
@@ -977,16 +964,15 @@ Str_Wcsncat(wchar_t *buf,       // IN-OUT
  */
 
 unsigned char *
-Str_Mbscpy(char *buf,                // OUT
-           const char *src,          // IN
-           size_t maxSize)           // IN
+Str_Mbscpy(char *buf,        // OUT
+           const char *src,  // IN
+           size_t maxSize)   // IN
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
-   len = strlen((const char *) src);
+   len = strlen(src);
    if (len >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
    return memcpy(buf, src, len + 1);
 }
@@ -1012,20 +998,19 @@ Str_Mbscpy(char *buf,                // OUT
  */
 
 unsigned char *
-Str_Mbscat(char *buf,                // IN-OUT
-           const char *src,          // IN
-           size_t maxSize)           // IN
+Str_Mbscat(char *buf,        // IN/OUT
+           const char *src,  // IN
+           size_t maxSize)   // IN
 {
-   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
-   bufLen = strlen((const char *) buf);
-   srcLen = strlen((const char *) src);
+   bufLen = strlen(buf);
+   srcLen = strlen(src);
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
    }
 
    memcpy(buf + bufLen, src, srcLen + 1);
@@ -1057,7 +1042,7 @@ Str_Mbscat(char *buf,                // IN-OUT
  */
 
 static wchar_t *
-StrVaswprintfInternal(size_t *length,         // OUT:
+StrVaswprintfInternal(size_t *length,         // OUT/OPT:
                       const wchar_t *format,  // IN:
                       va_list arguments,      // IN
                       Bool assertOnFailure)   // IN
@@ -1103,7 +1088,7 @@ StrVaswprintfInternal(size_t *length,         // OUT:
 
   exit:
    if (assertOnFailure) {
-      ASSERT_NOT_IMPLEMENTED(buf);
+      VERIFY(buf);
    }
    return buf;
 }
@@ -1126,13 +1111,13 @@ StrVaswprintfInternal(size_t *length,         // OUT:
  */
 
 wchar_t *
-Str_Aswprintf(size_t *length,         // OUT
+Str_Aswprintf(size_t *length,         // OUT/OPT
               const wchar_t *format,  // IN
               ...)                    // IN
 {
    va_list arguments;
    wchar_t *result;
-   
+
    va_start(arguments, format);
    result = Str_Vaswprintf(length, format, arguments);
    va_end(arguments);
@@ -1160,7 +1145,7 @@ Str_Aswprintf(size_t *length,         // OUT
  */
 
 wchar_t *
-Str_Vaswprintf(size_t *length,         // OUT
+Str_Vaswprintf(size_t *length,         // OUT/OPT
                const wchar_t *format,  // IN
                va_list arguments)      // IN
 {
@@ -1185,13 +1170,13 @@ Str_Vaswprintf(size_t *length,         // OUT
  */
 
 wchar_t *
-Str_SafeAswprintf(size_t *length,         // OUT
+Str_SafeAswprintf(size_t *length,         // OUT/OPT
                   const wchar_t *format,  // IN
                   ...)                    // IN
 {
    va_list arguments;
    wchar_t *result;
-   
+
    va_start(arguments, format);
    result = Str_SafeVaswprintf(length, format, arguments);
    va_end(arguments);
@@ -1208,7 +1193,7 @@ Str_SafeAswprintf(size_t *length,         // OUT
  *    See StrVaswprintfInternal.
  *
  * Results:
- *    Calls ASSERT_NOT_IMPLEMENTED on failure.
+ *    Calls VERIFY on failure.
  *
  * WARNING: See warning at the top of this file.
  *
@@ -1219,7 +1204,7 @@ Str_SafeAswprintf(size_t *length,         // OUT
  */
 
 wchar_t *
-Str_SafeVaswprintf(size_t *length,         // OUT
+Str_SafeVaswprintf(size_t *length,         // OUT/OPT
                    const wchar_t *format,  // IN
                    va_list arguments)      // IN
 {
@@ -1317,7 +1302,7 @@ PrintAndCheck(char *fmt,  // IN:
    char buf1[1024], buf2[1024];
    int count;
    va_list args;
-   
+
    va_start(args, fmt);
    count = Str_Vsnprintf(buf1, sizeof buf1, fmt, args);
    va_end(args);
@@ -1357,7 +1342,7 @@ PrintAndCheckW(wchar_t *fmt, ...)
    wchar_t buf1[1024], buf2[1024];
    int count;
    va_list args;
-   
+
    va_start(args, fmt);
    count = Str_Vsnwprintf(buf1, sizeof buf1, fmt, args);
    va_end(args);
