@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -34,7 +34,7 @@
 #include "mntinfo.h"
 
 static SyncFreezeFn gBackends[] = {
-#if defined(linux)
+#if defined(__linux__) && !defined(USERWORLD)
    LinuxDriver_Freeze,
    VmSync_Freeze,
    NullDriver_Freeze,
@@ -42,10 +42,11 @@ static SyncFreezeFn gBackends[] = {
 };
 
 static const char *gRemoteFSTypes[] = {
+   "autofs",
+   "cifs",
    "nfs",
    "nfs4",
    "smbfs",
-   "cifs",
    "vmhgfs"
 };
 
@@ -72,8 +73,7 @@ SyncDriverIsRemoteFSType(const char *fsType)
    size_t i;
 
    for (i = 0; i < ARRAYSIZE(gRemoteFSTypes); i++) {
-      if (Str_Strncmp(fsType, gRemoteFSTypes[i],
-                      sizeof gRemoteFSTypes[i]) == 0) {
+      if (Str_Strcmp(gRemoteFSTypes[i], fsType) == 0) {
          return TRUE;
       }
    }
@@ -197,7 +197,7 @@ SyncDriver_Init(void)
  *    follow the order in the "gBackends" array, and keep on trying different
  *    backends while SD_UNAVAILABLE is returned. If all backends are
  *    unavailable (unlikely given the "null" backend), the the function returns
- *    error.
+ *    error. NullDriver will be tried only if enableNullDriver is TRUE.
  *
  * Results:
  *    TRUE on success
@@ -211,6 +211,7 @@ SyncDriver_Init(void)
 
 Bool
 SyncDriver_Freeze(const char *userPaths,     // IN
+                  Bool enableNullDriver,     // IN
                   SyncDriverHandle *handle)  // OUT
 {
    char *paths = NULL;
@@ -220,8 +221,15 @@ SyncDriver_Freeze(const char *userPaths,     // IN
    /*
     * First, convert the given path list to something the backends will
     * understand: a colon-separated list of paths.
+    *
+    * NOTE: Ignore disk UUIDs. We ignore the userPaths if it does
+    * not start with '/' because all paths are absolute and it is
+    * possible only when we get diskUUID as userPaths. So, all
+    * mount points are considered instead of the userPaths provided.
     */
-   if (userPaths == NULL || Str_Strncmp(userPaths, "all", sizeof "all") == 0) {
+   if (userPaths == NULL ||
+       Str_Strncmp(userPaths, "all", sizeof "all") == 0 ||
+       *userPaths != '/') {
       paths = SyncDriverListMounts();
       if (paths == NULL) {
          Debug(LGPFX "Failed to list mount points.\n");
@@ -242,7 +250,16 @@ SyncDriver_Freeze(const char *userPaths,     // IN
    }
 
    while (err == SD_UNAVAILABLE && i < ARRAYSIZE(gBackends)) {
-      err = gBackends[i++](paths, handle);
+      SyncFreezeFn freezeFn = gBackends[i];
+      Debug(LGPFX "Calling backend %d.\n", (int) i);
+      i++;
+#if defined(__linux__) && !defined(USERWORLD)
+      if (!enableNullDriver && (freezeFn == NullDriver_Freeze)) {
+         Debug(LGPFX "Skipping nullDriver backend.\n");
+         continue;
+      }
+#endif
+      err = freezeFn(paths, handle);
    }
 
    free(paths);
