@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -63,6 +63,12 @@
 #include "ifaddrs.h"
 #endif
 
+#ifdef USERWORLD
+#include <vm_basic_types.h>
+#include <vmkuserstatus.h>
+#include <vmkuseruptime.h>
+#endif
+
 #include "vm_assert.h"
 #include "system.h"
 #include "debug.h"
@@ -183,7 +189,16 @@ System_Uptime(void)
 {
    uint64 uptime = -1;
 
-#ifdef __linux__
+#ifdef USERWORLD
+   {
+      VmkuserStatus_Code status;
+      uint64 sysUptime;
+      status = VmkuserUptime_GetUptime(&sysUptime);
+      if (VmkuserStatus_IsOK(status)) {
+         uptime = sysUptime / 10000;
+      }
+   }
+#elif defined(__linux__)
    {
       FILE *procStream;
       char *buf = NULL;
@@ -266,7 +281,7 @@ System_Uptime(void)
  *-----------------------------------------------------------------------------
  */
 
-Unicode
+char *
 System_GetTimeAsString(void)
 {
    struct timeval tv;
@@ -275,8 +290,8 @@ System_GetTimeAsString(void)
    size_t charsWritten;
    size_t bufSize = 8; // Multiplied by 2 for the initial allocation.
    char *buf = NULL;
-   Unicode dateTime = NULL;
-   Unicode output = NULL;
+   char *dateTime = NULL;
+   char *output = NULL;
 
    if (gettimeofday(&tv, NULL)) {
       goto out;
@@ -316,7 +331,7 @@ System_GetTimeAsString(void)
 
   out:
    free(buf);
-   Unicode_Free(dateTime);
+   free(dateTime);
    return output;
 }
 
@@ -345,6 +360,8 @@ System_Shutdown(Bool reboot)  // IN: "reboot or shutdown" flag
    if (reboot) {
 #if defined(sun)
       cmd = "/usr/sbin/shutdown -g 0 -i 6 -y";
+#elif defined(USERWORLD)
+      cmd = "/bin/reboot";
 #else
       cmd = "/sbin/shutdown -r now";
 #endif
@@ -353,6 +370,8 @@ System_Shutdown(Bool reboot)  // IN: "reboot or shutdown" flag
       cmd = "/sbin/shutdown -p now";
 #elif defined(sun)
       cmd = "/usr/sbin/shutdown -g 0 -i 5 -y";
+#elif defined(USERWORLD)
+      cmd = "/bin/halt";
 #else
       cmd = "/sbin/shutdown -h now";
 #endif
