@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2012 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@
 #include "includeCheck.h"
 
 #include "vm_basic_types.h"
+#include "vm_basic_defs.h"
 #include "vm_atomic.h"
 #include "vm_assert.h"
 
@@ -117,36 +118,34 @@ typedef enum VMCIIntrType {
 typedef uint32 VMCI_Resource;
 
 /* VMCI reserved hypervisor datagram resource IDs. */
-#define VMCI_RESOURCES_QUERY      0
-#define VMCI_GET_CONTEXT_ID       1
-#define VMCI_SET_NOTIFY_BITMAP    2
-#define VMCI_DOORBELL_LINK        3
-#define VMCI_DOORBELL_UNLINK      4
-#define VMCI_DOORBELL_NOTIFY      5
+#define VMCI_RESOURCES_QUERY        0
+#define VMCI_GET_CONTEXT_ID         1
+#define VMCI_SET_NOTIFY_BITMAP      2
+#define VMCI_DOORBELL_LINK          3
+#define VMCI_DOORBELL_UNLINK        4
+#define VMCI_DOORBELL_NOTIFY        5
 /*
  * VMCI_DATAGRAM_REQUEST_MAP and VMCI_DATAGRAM_REMOVE_MAP are
  * obsoleted by the removal of VM to VM communication.
  */
-#define VMCI_DATAGRAM_REQUEST_MAP 6
-#define VMCI_DATAGRAM_REMOVE_MAP  7
-#define VMCI_EVENT_SUBSCRIBE      8
-#define VMCI_EVENT_UNSUBSCRIBE    9
-#define VMCI_QUEUEPAIR_ALLOC      10
-#define VMCI_QUEUEPAIR_DETACH     11
+#define VMCI_DATAGRAM_REQUEST_MAP   6
+#define VMCI_DATAGRAM_REMOVE_MAP    7
+#define VMCI_EVENT_SUBSCRIBE        8
+#define VMCI_EVENT_UNSUBSCRIBE      9
+#define VMCI_QUEUEPAIR_ALLOC        10
+#define VMCI_QUEUEPAIR_DETACH       11
 /*
  * VMCI_VSOCK_VMX_LOOKUP was assigned to 12 for Fusion 3.0/3.1,
  * WS 7.0/7.1 and ESX 4.1
  */
-#define VMCI_HGFS_TRANSPORT       13
-#define VMCI_UNITY_PBRPC_REGISTER 14
+#define VMCI_HGFS_TRANSPORT         13
+#define VMCI_UNITY_PBRPC_REGISTER   14
 /*
- * The next two resources are for RPC calls from guest Tools, to replace the
- * backdoor calls we used previously.  Privileged is for admin/root RPCs,
- * unprivileged is for RPCs from any user.
+ * This resource is used for VMCI socket control packets sent to the
+ * hypervisor (CID 0) because RID 1 is already reserved.
  */
-#define VMCI_RPC_PRIVILEGED       15
-#define VMCI_RPC_UNPRIVILEGED     16
-#define VMCI_RESOURCE_MAX         17
+#define VSOCK_PACKET_HYPERVISOR_RID 15
+#define VMCI_RESOURCE_MAX           16
 /*
  * The core VMCI device functionality only requires the resource IDs of
  * VMCI_QUEUEPAIR_DETACH and below.
@@ -358,13 +357,17 @@ typedef uint32 VMCI_Event;
 #define VMCI_EVENT_CTX_ID_UPDATE  0  // Only applicable to guest endpoints
 #define VMCI_EVENT_CTX_REMOVED    1  // Applicable to guest and host
 #define VMCI_EVENT_QP_RESUMED     2  // Only applicable to guest endpoints
-#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest and host
-#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest and host
+#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest, host and VMX
+#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest, host and VMX
 #define VMCI_EVENT_MEM_ACCESS_ON  5  // Applicable to VMX and vmk.  On vmk,
                                      // this event has the Context payload type.
 #define VMCI_EVENT_MEM_ACCESS_OFF 6  // Applicable to VMX and vmk.  Same as
                                      // above for the payload type.
-#define VMCI_EVENT_MAX            7
+#define VMCI_EVENT_GUEST_PAUSED   7  // Applicable to vmk. This event has the
+                                     // Context payload type.
+#define VMCI_EVENT_GUEST_UNPAUSED 8  // Applicable to vmk. Same as above for
+                                     // the payload type.
+#define VMCI_EVENT_MAX            9
 
 /*
  * Of the above events, a few are reserved for use in the VMX, and
@@ -374,14 +377,19 @@ typedef uint32 VMCI_Event;
  * endpoints.
  */
 
-#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_MEM_ACCESS_ON || \
+#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_QP_PEER_ATTACH || \
+                                      _event == VMCI_EVENT_QP_PEER_DETACH || \
+                                      _event == VMCI_EVENT_MEM_ACCESS_ON || \
                                       _event == VMCI_EVENT_MEM_ACCESS_OFF)
 
 #if defined(VMX86_SERVER)
 #define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX)
 #else // VMX86_SERVER
 #define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX && \
-                                  !VMCI_EVENT_VALID_VMX(_event))
+                                  _event != VMCI_EVENT_MEM_ACCESS_ON && \
+                                  _event != VMCI_EVENT_MEM_ACCESS_OFF && \
+                                  _event != VMCI_EVENT_GUEST_PAUSED && \
+                                  _event != VMCI_EVENT_GUEST_UNPAUSED)
 #endif // VMX86_SERVER
 
 /* Reserved guest datagram resource ids. */
@@ -408,7 +416,7 @@ typedef uint32 VMCIPrivilegeFlags;
 #define VMCI_PRIVILEGE_FLAG_RESTRICTED     0x01
 #define VMCI_PRIVILEGE_FLAG_TRUSTED        0x02
 #define VMCI_PRIVILEGE_ALL_FLAGS           (VMCI_PRIVILEGE_FLAG_RESTRICTED | \
-				            VMCI_PRIVILEGE_FLAG_TRUSTED)
+                                            VMCI_PRIVILEGE_FLAG_TRUSTED)
 #define VMCI_NO_PRIVILEGE_FLAGS            0x00
 #define VMCI_DEFAULT_PROC_PRIVILEGE_FLAGS  VMCI_NO_PRIVILEGE_FLAGS
 #define VMCI_LEAST_PRIVILEGE_FLAGS         VMCI_PRIVILEGE_FLAG_RESTRICTED
