@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2011 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2014 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -28,7 +28,7 @@
 #define _VMCI_KERNEL_IF_H_
 
 #if !defined(linux) && !defined(_WIN32) && !defined(__APPLE__) && \
-    !defined(VMKERNEL) && !defined(SOLARIS)
+    !defined(VMKERNEL)
 #  error "Platform not supported."
 #endif
 
@@ -55,21 +55,12 @@
 
 #ifdef VMKERNEL
 #  include "splock.h"
+#  include "splock_customRanks.h"
 #  include "semaphore_ext.h"
 #  include "vmkapi.h"
 #  include "world_dist.h"
 #endif
 
-#ifdef SOLARIS
-#  include <sys/ddi.h>
-#  include <sys/kmem.h>
-#  include <sys/mutex.h>
-#  include <sys/poll.h>
-#  include <sys/semaphore.h>
-#  include <sys/sunddi.h>
-#  include <sys/types.h>
-#endif
-
 #include "vm_basic_types.h"
 #include "vmci_defs.h"
 
@@ -97,7 +88,7 @@
 #endif
 
 #if defined(VMKERNEL)
-  typedef SP_SpinLock VMCILock;
+  typedef MCSLock VMCILock;
   typedef SP_IRQL VMCILockFlags;
   typedef Semaphore VMCIEvent;
   typedef Semaphore VMCIMutex;
@@ -110,7 +101,11 @@
   typedef wait_queue_head_t VMCIEvent;
   typedef struct semaphore VMCIMutex;
   typedef PPN *VMCIPpnList; /* List of PPNs in produce/consume queue. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+  typedef kuid_t VMCIHostUser;
+#else
   typedef uid_t VMCIHostUser;
+#endif
   typedef VA64 VMCIQPGuestMem;
 #elif defined(__APPLE__)
   typedef IOLock *VMCILock;
@@ -132,14 +127,6 @@
   typedef PMDL VMCIPpnList; /* MDL to map the produce/consume queue. */
   typedef PSID VMCIHostUser;
   typedef VA64 *VMCIQPGuestMem;
-#elif defined(SOLARIS)
-  typedef kmutex_t VMCILock;
-  typedef unsigned long VMCILockFlags;
-  typedef ksema_t VMCIEvent;
-  typedef kmutex_t VMCIMutex;
-  typedef PPN *VMCIPpnList; /* List of PPNs in produce/consume queue. */
-  typedef uid_t VMCIHostUser;
-  typedef VA64 VMCIQPGuestMem;
 #endif // VMKERNEL
 
 /* Callback needed for correctly waiting on events. */
@@ -158,8 +145,18 @@ typedef int (*VMCIEventReleaseCB)(void *clientData);
 
   #define VMCI_SEMA_RANK_QPHEADER       (SEMA_RANK_FS - 1)
 
-  #define VMCI_LOCK_RANK_MAX            (MIN(SP_RANK_WAIT, \
+  #define VMCI_LOCK_RANK_MAX_NONBLOCK   (MIN(SP_RANK_WAIT, \
                                              SP_RANK_HEAPLOCK_DYNAMIC) - 1)
+  #define VMCI_LOCK_RANK_MAX            (SP_RANK_BLOCKABLE_HIGHEST_MAJOR - 2)
+
+  /*
+   * Determines whether VMCI locks will be blockable or not. If blockable,
+   * all locks will be at or below VMCI_LOCK_RANK_MAX. If not, locks will
+   * instead use VMCI_LOCK_RANK_MAX_NONBLOCK as the maximum. The other
+   * VMCI_LOCK_RANK_XXX values will be rebased to be non-blocking as well
+   * in that case.
+   */
+  extern Bool vmciBlockableLock;
 #else
   typedef unsigned long VMCILockRank;
   typedef unsigned long VMCISemaRank;
@@ -205,11 +202,6 @@ typedef struct VMCIHost {
    KEVENT *callEvent; /* Ptr to userlevel event used when signalling
                        * new pending guestcalls in kernel.
                        */
-#elif defined(SOLARIS)
-   struct pollhead pollhead; /* Per datagram handle pollhead structure to
-                              * be treated as a black-box. None of its
-                              * fields should be referenced.
-                              */
 #endif
 } VMCIHost;
 
@@ -223,9 +215,6 @@ typedef struct VMCIHost {
 #elif defined(_WIN32)
    typedef PUCHAR VMCIIoPort;
    typedef int VMCIIoHandle;
-#elif defined(SOLARIS)
-   typedef uint8_t * VMCIIoPort;
-   typedef ddi_acc_handle_t VMCIIoHandle;
 #elif defined(__APPLE__)
    typedef unsigned short int VMCIIoPort;
    typedef void *VMCIIoHandle;
@@ -260,6 +249,8 @@ void VMCIHost_SetInactiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 uint32 VMCIHost_NumHnds(VMCIHost *hostContext);
 uintptr_t VMCIHost_GetActiveHnd(VMCIHost *hostContext);
 void VMCIHost_SignalBitmap(VMCIHost *hostContext);
+void VMCIHost_SignalBitmapAlways(VMCIHost *hostContext);
+void VMCIHost_SignalCallAlways(VMCIHost *hostContext);
 #endif
 
 #if defined(_WIN32)
@@ -296,7 +287,7 @@ Bool VMCI_WaitOnEventInterruptible(VMCIEvent *event,
 #endif
 
 #if !defined(VMKERNEL) && (defined(__linux__) || defined(_WIN32) || \
-                           defined(__APPLE__) || defined(SOLARIS))
+                           defined(__APPLE__))
 int VMCI_CopyFromUser(void *dst, VA64 src, size_t len);
 #endif
 
@@ -309,16 +300,16 @@ void VMCIMutex_Destroy(VMCIMutex *mutex);
 void VMCIMutex_Acquire(VMCIMutex *mutex);
 void VMCIMutex_Release(VMCIMutex *mutex);
 
-#if defined(SOLARIS) || defined(_WIN32) || defined(__APPLE__)
+#if defined(_WIN32) || defined(__APPLE__)
 int VMCIKernelIf_Init(void);
 void VMCIKernelIf_Exit(void);
 #if defined(_WIN32)
 void VMCIKernelIf_DrainDelayedWork(void);
 #endif // _WIN32
-#endif // SOLARIS || _WIN32 || __APPLE__ || VMKERNEL
+#endif // _WIN32 || __APPLE__
 
-#if !defined(VMKERNEL) && (defined(__linux__) || defined(_WIN32) || \
-                           defined(SOLARIS) || defined(__APPLE__))
+#if !defined(VMKERNEL) && \
+    (defined(__linux__) || defined(_WIN32) || defined(__APPLE__))
 void *VMCI_AllocQueue(uint64 size, uint32 flags);
 void VMCI_FreeQueue(void *q, uint64 size);
 typedef struct PPNSet {
@@ -351,19 +342,15 @@ typedef uint32 VMCIGuestMemID;
 #if defined(VMKERNEL) || defined(__linux__)  || defined(_WIN32) || \
     defined(__APPLE__)
   struct QueuePairPageStore;
-  int VMCIHost_RegisterUserMemory(unsigned int index,
-                                  struct QueuePairPageStore *pageStore,
+  int VMCIHost_RegisterUserMemory(struct QueuePairPageStore *pageStore,
                                   struct VMCIQueue *produceQ,
                                   struct VMCIQueue *consumeQ);
-  void VMCIHost_UnregisterUserMemory(unsigned int index,
-                                     struct VMCIQueue *produceQ,
+  void VMCIHost_UnregisterUserMemory(struct VMCIQueue *produceQ,
                                      struct VMCIQueue *consumeQ);
-  int VMCIHost_MapQueues(unsigned int index,
-                         struct VMCIQueue *produceQ,
+  int VMCIHost_MapQueues(struct VMCIQueue *produceQ,
                          struct VMCIQueue *consumeQ,
                          uint32 flags);
-  int VMCIHost_UnmapQueues(unsigned int index,
-                           VMCIGuestMemID gid,
+  int VMCIHost_UnmapQueues(VMCIGuestMemID gid,
                            struct VMCIQueue *produceQ,
                            struct VMCIQueue *consumeQ);
   void VMCI_InitQueueMutex(struct VMCIQueue *produceQ,
@@ -377,44 +364,40 @@ typedef uint32 VMCIGuestMemID;
 #  define VMCI_CleanupQueueMutex(_pq, _cq) do { } while (0)
 #  define VMCI_AcquireQueueMutex(_q, _cb) VMCI_SUCCESS
 #  define VMCI_ReleaseQueueMutex(_q) do { } while (0)
-#  define VMCIHost_RegisterUserMemory(_idx, _ps, _pq, _cq) VMCI_ERROR_UNAVAILABLE
-#  define VMCIHost_UnregisterUserMemory(_idx, _pq, _cq) do { } while (0)
-#  define VMCIHost_MapQueues(_idx, _pq, _cq, _f) VMCI_SUCCESS
-#  define VMCIHost_UnmapQueues(_idx, _gid, _pq, _cq) VMCI_SUCCESS
+#  define VMCIHost_RegisterUserMemory(_ps, _pq, _cq) VMCI_ERROR_UNAVAILABLE
+#  define VMCIHost_UnregisterUserMemory(_pq, _cq) do { } while (0)
+#  define VMCIHost_MapQueues(_pq, _cq, _f) VMCI_SUCCESS
+#  define VMCIHost_UnmapQueues(_gid, _pq, _cq) VMCI_SUCCESS
 #endif
 
 #if defined(VMKERNEL)
-  void VMCIHost_MarkQueuesAvailable(unsigned int index,
-                                    struct VMCIQueue *produceQ,
+  void VMCIHost_MarkQueuesAvailable(struct VMCIQueue *produceQ,
                                     struct VMCIQueue *consumeQ);
-  void VMCIHost_MarkQueuesUnavailable(unsigned int index,
-                                      struct VMCIQueue *produceQ,
+  void VMCIHost_MarkQueuesUnavailable(struct VMCIQueue *produceQ,
                                       struct VMCIQueue *consumeQ);
 #else
-#  define VMCIHost_MarkQueuesAvailable(_idx, _q, _p) do { } while (0)
-#  define VMCIHost_MarkQueuesUnavailable(_idx, _q, _p) do { } while(0)
+#  define VMCIHost_MarkQueuesAvailable(_q, _p) do { } while (0)
+#  define VMCIHost_MarkQueuesUnavailable(_q, _p) do { } while(0)
 #endif
 
 #if defined(VMKERNEL) || defined(__linux__)
    void VMCI_LockQueueHeader(struct VMCIQueue *queue);
    void VMCI_UnlockQueueHeader(struct VMCIQueue *queue);
 #else
-#  define VMCI_LockQueueHeader(_q) ASSERT_NOT_IMPLEMENTED(FALSE)
-#  define VMCI_UnlockQueueHeader(_q) ASSERT_NOT_IMPLEMENTED(FALSE)
+#  define VMCI_LockQueueHeader(_q) NOT_IMPLEMENTED()
+#  define VMCI_UnlockQueueHeader(_q) NOT_IMPLEMENTED()
 #endif
 
 #if (!defined(VMKERNEL) && defined(__linux__)) || defined(_WIN32) ||  \
-   defined(__APPLE__) || defined(SOLARIS)
-  int VMCIHost_GetUserMemory(unsigned int index,
-                             VA64 produceUVA, VA64 consumeUVA,
+    defined(__APPLE__)
+  int VMCIHost_GetUserMemory(VA64 produceUVA, VA64 consumeUVA,
                              struct VMCIQueue *produceQ,
                              struct VMCIQueue *consumeQ);
-  void VMCIHost_ReleaseUserMemory(unsigned int index,
-                                  struct VMCIQueue *produceQ,
+  void VMCIHost_ReleaseUserMemory(struct VMCIQueue *produceQ,
                                   struct VMCIQueue *consumeQ);
 #else
-#  define VMCIHost_GetUserMemory(_idx, _puva, _cuva, _pq, _cq) VMCI_ERROR_UNAVAILABLE
-#  define VMCIHost_ReleaseUserMemory(_idx, _pq, _cq) ASSERT_NOT_IMPLEMENTED(FALSE)
+#  define VMCIHost_GetUserMemory(_puva, _cuva, _pq, _cq) VMCI_ERROR_UNAVAILABLE
+#  define VMCIHost_ReleaseUserMemory(_pq, _cq) NOT_IMPLEMENTED()
 #endif
 
 #if defined(_WIN32)
