@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2012 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2012,2014-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -48,7 +48,8 @@ static int VMCIContextFireNotification(VMCIId contextID,
                                        VMCIPrivilegeFlags privFlags);
 #if defined(VMKERNEL)
 static void VMCIContextReleaseGuestMemLocked(VMCIContext *context,
-                                             VMCIGuestMemID gid);
+                                             VMCIGuestMemID gid,
+                                             Bool powerOff);
 static void VMCIContextInFilterCleanup(VMCIContext *context);
 #endif
 
@@ -577,8 +578,9 @@ VMCIContext_PendingDatagrams(VMCIId cid,      // IN
  */
 
 int
-VMCIContext_EnqueueDatagram(VMCIId cid,        // IN: Target VM
-                            VMCIDatagram *dg)  // IN:
+VMCIContext_EnqueueDatagram(VMCIId cid,         // IN: Target VM
+                            VMCIDatagram *dg,   // IN:
+                            Bool notify)        // IN:
 {
    DatagramQueueEntry *dqEntry;
    VMCIContext *context;
@@ -648,8 +650,12 @@ VMCIContext_EnqueueDatagram(VMCIId cid,        // IN: Target VM
    VMCIList_Insert(&dqEntry->listItem, &context->datagramQueue);
    context->pendingDatagrams++;
    context->datagramQueueSize += vmciDgSize;
-   VMCIContextSignalNotify(context);
-   VMCIHost_SignalCall(&context->hostContext);
+
+   if (notify) {
+      VMCIContextSignalNotify(context);
+      VMCIHost_SignalCall(&context->hostContext);
+   }
+
    VMCI_ReleaseLock(&context->lock, flags);
    VMCIContext_Release(context);
 
@@ -809,7 +815,7 @@ VMCIContext_Release(VMCIContext *context)  // IN
 {
    uint32 refCount;
    ASSERT(context);
-   refCount = Atomic_FetchAndDec(&context->refCount);
+   refCount = Atomic_ReadDec32(&context->refCount);
    if (refCount == 1) {
       VMCIContextFreeContext(context);
    }
@@ -1223,6 +1229,98 @@ VMCIContext_SetId(VMCIContext *context,         // IN
    context->cid = cid;
    VMCI_ReleaseLock(&context->lock, flags);
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContextGenerateEvent --
+ *
+ *      Generates a VMCI event that only takes context ID as event data.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+VMCIContextGenerateEvent(VMCIId cid,       // IN
+                         VMCI_Event event) // IN
+{
+   VMCIEventMsg *eMsg;
+   VMCIEventPayload_Context *ePayload;
+   /* buf is only 48 bytes. */
+   char buf[sizeof *eMsg + sizeof *ePayload];
+
+   eMsg = (VMCIEventMsg *)buf;
+   ePayload = VMCIEventMsgPayload(eMsg);
+
+   eMsg->hdr.dst = VMCI_MAKE_HANDLE(VMCI_HOST_CONTEXT_ID, VMCI_EVENT_HANDLER);
+   eMsg->hdr.src = VMCI_MAKE_HANDLE(VMCI_HYPERVISOR_CONTEXT_ID,
+                                    VMCI_CONTEXT_RESOURCE_ID);
+   eMsg->hdr.payloadSize = sizeof *eMsg + sizeof *ePayload - sizeof eMsg->hdr;
+   eMsg->eventData.event = event;
+   ePayload->contextID = cid;
+
+   (void)VMCIEvent_Dispatch((VMCIDatagram *)eMsg);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_NotifyGuestPaused --
+ *
+ *      Notify subscribers of a execution state change of the VM
+ *      with the given context ID. This will happen when a VM is
+ *      quiesced/unquiesced.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+VMCIContext_NotifyGuestPaused(VMCIId cid,  // IN
+                              Bool paused) // IN
+{
+   VMCIContextGenerateEvent(cid, paused ? VMCI_EVENT_GUEST_PAUSED :
+                                          VMCI_EVENT_GUEST_UNPAUSED);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_NotifyMemoryAccess --
+ *
+ *      Notify subscribers of a memory access change to the device.
+ *      This can occur when the device is enabled/disabled/reset.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+VMCIContext_NotifyMemoryAccess(VMCIId cid, // IN
+                               Bool on)    // IN
+{
+   VMCIContextGenerateEvent(cid, on ? VMCI_EVENT_MEM_ACCESS_ON :
+                                      VMCI_EVENT_MEM_ACCESS_OFF);
+}
 #endif
 
 
@@ -1489,6 +1587,143 @@ VMCIContextFireNotification(VMCIId contextID,             // IN
    return VMCI_SUCCESS;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContextDgHypervisorSaveStateSize --
+ *
+ *      Calculate the size for the hypervisor datagram checkpoint
+ *      save data.
+ *
+ *      The format is as follows:
+ *
+ *      uint32 count - number of entries
+ *      uint32 size  - size of first entry
+ *      char bytes[] - contents of first entry
+ *      uint32 size  - size of second entry
+ *      char bytes[] - contents of second entry
+ *      ...
+ *
+ * Results:
+ *      VMCI_SUCCESS on success, error code otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+VMCIContextDgHypervisorSaveStateSize(VMCIContext *context,  // IN
+                                     uint32 *bufSize,       // IN/OUT
+                                     char **cptBufPtr)      // UNUSED
+{
+   uint32 total;
+   VMCIListItem *iter;
+
+   *bufSize = total = 0;
+
+   VMCIList_Scan(iter, &context->datagramQueue) {
+      DatagramQueueEntry *dqEntry =
+         VMCIList_Entry(iter, DatagramQueueEntry, listItem);
+
+      if (dqEntry->dg->src.context == VMCI_HYPERVISOR_CONTEXT_ID) {
+         /* Size of the datagram followed by the contents of the datagram. */
+         total += sizeof(uint32) + dqEntry->dgSize;
+      }
+   }
+
+   if (total > 0) {
+      /* Don't forget the datagram count. */
+      *bufSize = total + sizeof(uint32);
+   }
+
+   return VMCI_SUCCESS;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContextDgHypervisorSaveState --
+ *
+ *      Get the hypervisor datagram checkpoint save data.
+ *
+ * Results:
+ *      VMCI_SUCCESS on success, error code otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+VMCIContextDgHypervisorSaveState(VMCIContext *context,   // IN
+                                 uint32 *bufSize,        // IN/OUT
+                                 char **cptBufPtr)       // OUT
+{
+   uint8 *p;
+   uint32 total;
+   uint32 count;
+   VMCIListItem *iter;
+
+   /* Need at least the count field and the size of one entry. */
+   if (*bufSize < sizeof(uint32) * 2) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   p = VMCI_AllocKernelMem(*bufSize, VMCI_MEMORY_NORMAL);
+   if (p == NULL) {
+      return VMCI_ERROR_NO_MEM;
+   }
+
+   *cptBufPtr = p;
+
+   /* Leave space for the datagram count at the start. */
+   total  = sizeof(uint32);
+   p     += sizeof(uint32);
+
+   count = 0;
+   VMCIList_Scan(iter, &context->datagramQueue) {
+      DatagramQueueEntry *dqEntry =
+         VMCIList_Entry(iter, DatagramQueueEntry, listItem);
+
+      if (dqEntry->dg->src.context == VMCI_HYPERVISOR_CONTEXT_ID) {
+
+         /*
+          * VMX might have capped the amount of space we can use to checkpoint
+          * hypervisor datagrams. Respect that here. Those that are not written
+          * to the buffer will get dropped.
+          */
+
+         if (total + sizeof(uint32) + dqEntry->dgSize > *bufSize) {
+            break;
+         }
+
+         total += sizeof(uint32) + dqEntry->dgSize;
+
+         /*
+          * Write in the size of the datagram followed by the contents of the
+          * datagram itself.
+          */
+
+         *(uint32 *)p = dqEntry->dgSize;
+         p += sizeof(uint32);
+
+         memcpy(p, dqEntry->dg, dqEntry->dgSize);
+         p += dqEntry->dgSize;
+
+         count++;
+      }
+   }
+
+   /* Now rollback and write the count at the start of the block. */
+   *(uint32 *)(*cptBufPtr) = count;
+
+   return VMCI_SUCCESS;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -1546,6 +1781,12 @@ VMCIContext_GetCheckpointState(VMCIId contextID,    // IN:
       ASSERT(context->doorbellArray);
       array = context->doorbellArray;
       getContextID = FALSE;
+   } else if (cptType == VMCI_DG_HYPERVISOR_SAVE_STATE_SIZE) {
+      result = VMCIContextDgHypervisorSaveStateSize(context, bufSize, cptBufPtr);
+      goto release;
+   } else if (cptType == VMCI_DG_HYPERVISOR_SAVE_STATE) {
+      result = VMCIContextDgHypervisorSaveState(context, bufSize, cptBufPtr);
+      goto release;
    } else {
       VMCI_DEBUG_LOG(4, (LGPFX"Invalid cpt state (type=%d).\n", cptType));
       result = VMCI_ERROR_INVALID_ARGS;
@@ -1589,10 +1830,9 @@ VMCIContext_GetCheckpointState(VMCIId contextID,    // IN:
    }
    result = VMCI_SUCCESS;
 
-  release:
+release:
    VMCI_ReleaseLock(&context->lock, flags);
    VMCIContext_Release(context);
-
    return result;
 }
 
@@ -1977,8 +2217,7 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
    if (srcCID != handle.context) {
       VMCIPrivilegeFlags dstPrivFlags;
 
-      if (!vmkernel && VMCI_CONTEXT_IS_VM(srcCID) &&
-          VMCI_CONTEXT_IS_VM(handle.context)) {
+      if (VMCI_CONTEXT_IS_VM(srcCID) && VMCI_CONTEXT_IS_VM(handle.context)) {
          VMCI_DEBUG_LOG(4, (LGPFX"Doorbell notification from VM to VM not "
                             "supported (src=0x%x, dst=0x%x).\n",
                             srcCID, handle.context));
@@ -2082,7 +2321,7 @@ VMCIContext_SignalPendingDoorbells(VMCIId contextID)
    VMCI_ReleaseLock(&context->lock, flags);
 
    if (pending) {
-      VMCIHost_SignalBitmap(&context->hostContext);
+      VMCIHost_SignalBitmapAlways(&context->hostContext);
    }
 
    VMCIContext_Release(context);
@@ -2126,7 +2365,7 @@ VMCIContext_SignalPendingDatagrams(VMCIId contextID)
    VMCI_ReleaseLock(&context->lock, flags);
 
    if (pending) {
-      VMCIHost_SignalCall(&context->hostContext);
+      VMCIHost_SignalCallAlways(&context->hostContext);
    }
 
    VMCIContext_Release(context);
@@ -2208,7 +2447,7 @@ VMCI_EXPORT_SYMBOL(vmci_is_context_owner)
 #if defined(linux) && !defined(VMKERNEL)
 int
 vmci_is_context_owner(VMCIId contextID,   // IN
-                      uid_t uid)          // IN
+                      VMCIHostUser uid)          // IN
 {
    int isOwner = 0;
 
@@ -2216,7 +2455,7 @@ vmci_is_context_owner(VMCIId contextID,   // IN
       VMCIContext *context = VMCIContext_Get(contextID);
       if (context) {
          if (context->validUser) {
-            if (VMCIHost_CompareUser((VMCIHostUser *)&uid,
+            if (VMCIHost_CompareUser(&uid,
                                      &context->user) == VMCI_SUCCESS) {
                isOwner = 1;
             }
@@ -2443,7 +2682,8 @@ VMCIContext_RegisterGuestMem(VMCIContext *context, // IN: Context structure
           * execution of the source VMX following a failed FSR.
           */
 
-         VMCIContextReleaseGuestMemLocked(context, context->curGuestMemID);
+         VMCIContextReleaseGuestMemLocked(context, context->curGuestMemID,
+                                          FALSE);
       } else {
          /*
           * When unquiescing the device during a restore sync not part
@@ -2507,15 +2747,20 @@ out:
 
 static void
 VMCIContextReleaseGuestMemLocked(VMCIContext *context, // IN: Context structure
-                                 VMCIGuestMemID gid)   // IN: Reference to guest
+                                 VMCIGuestMemID gid,   // IN: Reference to guest
+                                 Bool powerOff)        // IN: Device going away
 {
    uint32 numQueuePairs;
    uint32 cur;
 
+   if (powerOff) {
+      VMCIContext_NotifyMemoryAccess(context->cid, FALSE);
+   }
+
    /*
     * It is safe to access the queue pair array here, since no changes
     * to the queuePairArray can take place when the the quiescing
-    * has been initiated.
+    * has been initiated, or when the device is being cleaned up.
     */
 
    numQueuePairs = VMCIHandleArray_GetSize(context->queuePairArray);
@@ -2558,7 +2803,8 @@ VMCIContextReleaseGuestMemLocked(VMCIContext *context, // IN: Context structure
 
 void
 VMCIContext_ReleaseGuestMem(VMCIContext *context, // IN: Context structure
-                            VMCIGuestMemID gid)   // IN: Reference to guest
+                            VMCIGuestMemID gid,   // IN: Reference to guest
+                            Bool powerOff)        // IN: Device is going away
 {
 #ifdef VMKERNEL
    VMCIMutex_Acquire(&context->guestMemMutex);
@@ -2573,7 +2819,7 @@ VMCIContext_ReleaseGuestMem(VMCIContext *context, // IN: Context structure
        * memory, if this is the same gid, that registered the memory.
        */
 
-      VMCIContextReleaseGuestMemLocked(context, gid);
+      VMCIContextReleaseGuestMemLocked(context, gid, powerOff);
       context->curGuestMemID = INVALID_VMCI_GUEST_MEM_ID;
    }
 
