@@ -67,14 +67,20 @@
  */
 
 struct VMCIQueueKernelIf {
-   struct page **page;
-   struct page **headerPage;
-   void *va;
-   VMCIMutex __mutex;
-   VMCIMutex *mutex;
-   Bool host;
-   Bool isDataMapped;
-   size_t numPages;
+   VMCIMutex __mutex;               /* Protects the queue. */
+   VMCIMutex *mutex;                /* Shared by producer/consumer queues. */
+   size_t numPages;                 /* Number of pages incl. header. */
+   Bool host;                       /* Host or guest? */
+   union {
+      struct {
+         dma_addr_t *pas;           /* Physical addresses. */
+         void **vas;                /* Virtual addresses. */
+      } g;                          /* Guest. */
+      struct {
+         struct page **headerPage;  /* Guest queue header pages. */
+         struct page **page;        /* Guest queue pages. */
+      } h;                          /* Host. */
+   } u;
 };
 
 typedef struct VMCIDelayedWorkInfo {
@@ -83,6 +89,8 @@ typedef struct VMCIDelayedWorkInfo {
    void *data;
 } VMCIDelayedWorkInfo;
 
+extern struct pci_dev *vmci_pdev;
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -420,11 +428,13 @@ int VMCIHost_CompareUser(VMCIHostUser *user1,
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   if (*user1 == *user2) {
-      return VMCI_SUCCESS;
-   } else {
-      return VMCI_ERROR_GENERIC;
-   }
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+#  define vmw_uid_eq(a, b) uid_eq(a, b)
+#else
+#  define vmw_uid_eq(a, b) ((a) == (b))
+#endif
+
+   return vmw_uid_eq(*user1, *user2) ? VMCI_SUCCESS : VMCI_ERROR_GENERIC;
 }
 
 
@@ -868,14 +878,9 @@ VMCIMutex_Release(VMCIMutex *mutex) // IN:
  *
  * VMCI_AllocQueue --
  *
- *      Allocates kernel VA space of specified size, plus space for the
- *      queue structure/kernel interface and the queue header.  Allocates
- *      physical pages for the queue data pages.
- *
- *      PAGE m:      VMCIQueueHeader (VMCIQueue->qHeader)
- *      PAGE m+1:    VMCIQueue
- *      PAGE m+1+q:  VMCIQueueKernelIf (VMCIQueue->kernelIf)
- *      PAGE n-size: Data pages (VMCIQueue->kernelIf->page[])
+ *      Allocates kernel queue pages of specified size with IOMMU mappings,
+ *      plus space for the queue structure/kernel interface and the queue
+ *      header.
  *
  * Results:
  *      Pointer to the queue on success, NULL otherwise.
@@ -890,14 +895,13 @@ void *
 VMCI_AllocQueue(uint64 size,  // IN: size of queue (not including header)
                 uint32 flags) // IN: queuepair flags
 {
-   uint64 i;
+   size_t i;
    VMCIQueue *queue;
-   VMCIQueueHeader *qHeader;
-   const uint64 numDataPages = CEILING(size, PAGE_SIZE);
-   const uint queueSize =
-      PAGE_SIZE +
-      sizeof *queue + sizeof *(queue->kernelIf) +
-      numDataPages * sizeof *(queue->kernelIf->page);
+   const size_t numPages = CEILING(size, PAGE_SIZE) + 1;
+   const size_t pasSize = numPages * sizeof *queue->kernelIf->u.g.pas;
+   const size_t vasSize = numPages * sizeof *queue->kernelIf->u.g.vas;
+   const size_t queueSize =
+      sizeof *queue + sizeof *(queue->kernelIf) + pasSize + vasSize;
 
    /*
     * Size should be enforced by VMCIQPair_Alloc(), double-check here.
@@ -911,55 +915,33 @@ VMCI_AllocQueue(uint64 size,  // IN: size of queue (not including header)
       return NULL;
    }
 
-   /*
-    * If pinning is requested then double-check the size of the queue.
-    * VMCIQPair_Alloc() will do this for the total queuepair size.
-    */
-
-   if ((flags & VMCI_QPFLAG_PINNED) && size > VMCI_MAX_PINNED_QP_MEMORY) {
+   queue = vmalloc(queueSize);
+   if (!queue) {
       return NULL;
    }
 
-   qHeader = (VMCIQueueHeader *)vmalloc(queueSize);
-   if (!qHeader) {
-      return NULL;
-   }
-
-   queue = (VMCIQueue *)((uint8 *)qHeader + PAGE_SIZE);
-   queue->qHeader = qHeader;
+   queue->qHeader = NULL;
    queue->savedHeader = NULL;
-   queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
-   queue->kernelIf->headerPage = NULL; // Unused in guest.
-   queue->kernelIf->page = (struct page **)((uint8 *)queue->kernelIf +
-                                            sizeof *(queue->kernelIf));
-   queue->kernelIf->va = NULL;
+   queue->kernelIf = (VMCIQueueKernelIf *)(queue + 1);
+   queue->kernelIf->mutex = NULL;
+   queue->kernelIf->numPages = numPages;
+   queue->kernelIf->u.g.pas = (dma_addr_t *)(queue->kernelIf + 1);
+   queue->kernelIf->u.g.vas =
+      (void **)((uint8 *)queue->kernelIf->u.g.pas + pasSize);
    queue->kernelIf->host = FALSE;
-   queue->kernelIf->isDataMapped = FALSE;
 
-   for (i = 0; i < numDataPages; i++) {
-      queue->kernelIf->page[i] = alloc_pages(GFP_KERNEL, 0);
-      if (!queue->kernelIf->page[i]) {
-         VMCI_FreeQueue(queue, i * PAGE_SIZE);
+   for (i = 0; i < numPages; i++) {
+      queue->kernelIf->u.g.vas[i] =
+         dma_alloc_coherent(&vmci_pdev->dev, PAGE_SIZE,
+                            &queue->kernelIf->u.g.pas[i], GFP_KERNEL);
+      if (!queue->kernelIf->u.g.vas[i]) {
+         VMCI_FreeQueue(queue, i * PAGE_SIZE); /* Size excl. the header. */
          return NULL;
       }
    }
 
-   /*
-    * alloc_pages() returns pinned PAs, but we need a permanent mapping to VA
-    * if the caller has requested pinned queuepairs.  Map all of them into
-    * kernel VA now, for the lifetime of the queue.  The page VAs will be
-    * contiguous.
-    */
-
-   if (flags & VMCI_QPFLAG_PINNED) {
-      queue->kernelIf->va = vmap(queue->kernelIf->page, numDataPages, VM_MAP,
-                                 PAGE_KERNEL);
-      if (NULL == queue->kernelIf->va) {
-         VMCI_FreeQueue(queue, numDataPages * PAGE_SIZE);
-         return NULL;
-      }
-      queue->kernelIf->isDataMapped = TRUE;
-   }
+   /* Queue header is the first page. */
+   queue->qHeader = queue->kernelIf->u.g.vas[0];
 
    return (void *)queue;
 }
@@ -991,18 +973,13 @@ VMCI_FreeQueue(void *q,     // IN:
    if (queue) {
       uint64 i;
 
-      if (queue->kernelIf->isDataMapped) {
-         ASSERT(queue->kernelIf->va);
-         vunmap(queue->kernelIf->va);
-         queue->kernelIf->va = NULL;
+      /* Given size does not include header, so add in a page here. */
+      for (i = 0; i < CEILING(size, PAGE_SIZE) + 1; i++) {
+         dma_free_coherent(&vmci_pdev->dev, PAGE_SIZE,
+                           queue->kernelIf->u.g.vas[i],
+                           queue->kernelIf->u.g.pas[i]);
       }
-
-      ASSERT(NULL == queue->kernelIf->va);
-
-      for (i = 0; i < CEILING(size, PAGE_SIZE); i++) {
-         __free_page(queue->kernelIf->page[i]);
-      }
-      vfree(queue->qHeader);
+      vfree(queue);
    }
 }
 
@@ -1063,33 +1040,29 @@ VMCI_AllocPPNSet(void *prodQ,            // IN:
       return VMCI_ERROR_NO_MEM;
    }
 
-   producePPNs[0] = page_to_pfn(vmalloc_to_page(produceQ->qHeader));
-   for (i = 1; i < numProducePages; i++) {
+   for (i = 0; i < numProducePages; i++) {
       unsigned long pfn;
 
-      producePPNs[i] = pfn = page_to_pfn(produceQ->kernelIf->page[i - 1]);
+      producePPNs[i] = pfn = produceQ->kernelIf->u.g.pas[i] >> PAGE_SHIFT;
 
       /*
        * Fail allocation if PFN isn't supported by hypervisor.
        */
 
-      if (sizeof pfn > sizeof *producePPNs &&
-          pfn != producePPNs[i]) {
+      if (sizeof pfn > sizeof *producePPNs && pfn != producePPNs[i]) {
          goto ppnError;
       }
    }
-   consumePPNs[0] = page_to_pfn(vmalloc_to_page(consumeQ->qHeader));
-   for (i = 1; i < numConsumePages; i++) {
+   for (i = 0; i < numConsumePages; i++) {
       unsigned long pfn;
 
-      consumePPNs[i] = pfn = page_to_pfn(consumeQ->kernelIf->page[i - 1]);
+      consumePPNs[i] = pfn = consumeQ->kernelIf->u.g.pas[i] >> PAGE_SHIFT;
 
       /*
        * Fail allocation if PFN isn't supported by hypervisor.
        */
 
-      if (sizeof pfn > sizeof *consumePPNs &&
-          pfn != consumePPNs[i]) {
+      if (sizeof pfn > sizeof *consumePPNs && pfn != consumePPNs[i]) {
          goto ppnError;
       }
    }
@@ -1205,15 +1178,15 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
    size_t bytesCopied = 0;
 
    while (bytesCopied < size) {
-      uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
-      size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
+      const uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
+      const size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
       void *va;
       size_t toCopy;
 
-      if (kernelIf->isDataMapped) {
-         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      if (kernelIf->host) {
+         va = kmap(kernelIf->u.h.page[pageIndex]);
       } else {
-         va = kmap(kernelIf->page[pageIndex]);
+         va = kernelIf->u.g.vas[pageIndex + 1]; /* Skip header. */
       }
 
       ASSERT(va);
@@ -1231,8 +1204,8 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_fromiovec((uint8 *)va + pageOffset, iov, toCopy);
          if (err != 0) {
-            if (!kernelIf->isDataMapped) {
-               kunmap(kernelIf->page[pageIndex]);
+            if (kernelIf->host) {
+               kunmap(kernelIf->u.h.page[pageIndex]);
             }
             return VMCI_ERROR_INVALID_ARGS;
          }
@@ -1241,8 +1214,8 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
       }
 
       bytesCopied += toCopy;
-      if (!kernelIf->isDataMapped) {
-         kunmap(kernelIf->page[pageIndex]);
+      if (kernelIf->host) {
+         kunmap(kernelIf->u.h.page[pageIndex]);
       }
    }
 
@@ -1280,15 +1253,15 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
    size_t bytesCopied = 0;
 
    while (bytesCopied < size) {
-      uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
-      size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
+      const uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
+      const size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
       void *va;
       size_t toCopy;
 
-      if (kernelIf->isDataMapped) {
-         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      if (kernelIf->host) {
+         va = kmap(kernelIf->u.h.page[pageIndex]);
       } else {
-         va = kmap(kernelIf->page[pageIndex]);
+         va = kernelIf->u.g.vas[pageIndex + 1]; /* Skip header. */
       }
 
       ASSERT(va);
@@ -1306,8 +1279,8 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_toiovec(iov, (uint8 *)va + pageOffset, toCopy);
          if (err != 0) {
-            if (!kernelIf->isDataMapped) {
-               kunmap(kernelIf->page[pageIndex]);
+            if (kernelIf->host) {
+               kunmap(kernelIf->u.h.page[pageIndex]);
             }
             return VMCI_ERROR_INVALID_ARGS;
          }
@@ -1316,8 +1289,8 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
       }
 
       bytesCopied += toCopy;
-      if (!kernelIf->isDataMapped) {
-         kunmap(kernelIf->page[pageIndex]);
+      if (kernelIf->host) {
+         kunmap(kernelIf->u.h.page[pageIndex]);
       }
    }
 
@@ -1584,22 +1557,22 @@ VMCIHost_AllocQueue(uint64 size) // IN:
    VMCIQueue *queue;
    const size_t numPages = CEILING(size, PAGE_SIZE) + 1;
    const size_t queueSize = sizeof *queue + sizeof *(queue->kernelIf);
-   const size_t queuePageSize = numPages * sizeof *queue->kernelIf->page;
+   const size_t queuePageSize = numPages * sizeof *queue->kernelIf->u.h.page;
 
    queue = VMCI_AllocKernelMem(queueSize + queuePageSize, VMCI_MEMORY_NORMAL);
    if (queue) {
       queue->qHeader = NULL;
       queue->savedHeader = NULL;
-      queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
+      queue->kernelIf = (VMCIQueueKernelIf *)(queue + 1);
       queue->kernelIf->host = TRUE;
       queue->kernelIf->mutex = NULL;
       queue->kernelIf->numPages = numPages;
-      queue->kernelIf->headerPage = (struct page **)((uint8*)queue + queueSize);
-      queue->kernelIf->page = &queue->kernelIf->headerPage[1];
-      memset(queue->kernelIf->headerPage, 0,
-             sizeof *queue->kernelIf->headerPage * queue->kernelIf->numPages);
-      queue->kernelIf->va = NULL;
-      queue->kernelIf->isDataMapped = FALSE;
+      queue->kernelIf->u.h.headerPage =
+         (struct page **)((uint8*)queue + queueSize);
+      queue->kernelIf->u.h.page = &queue->kernelIf->u.h.headerPage[1];
+      memset(queue->kernelIf->u.h.headerPage, 0,
+             (sizeof *queue->kernelIf->u.h.headerPage *
+              queue->kernelIf->numPages));
    }
 
    return queue;
@@ -1886,16 +1859,15 @@ VMCIReleasePages(struct page **pages,  // IN
  */
 
 int
-VMCIHost_RegisterUserMemory(unsigned int index,             // IN
-                            QueuePairPageStore *pageStore,  // IN
+VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
                             VMCIQueue *produceQ,            // OUT
                             VMCIQueue *consumeQ)            // OUT
 {
    VA64 produceUVA;
    VA64 consumeUVA;
 
-   ASSERT(index == 0);
-   ASSERT(produceQ->kernelIf->headerPage && consumeQ->kernelIf->headerPage);
+   ASSERT(produceQ->kernelIf->u.h.headerPage &&
+          consumeQ->kernelIf->u.h.headerPage);
 
    /*
     * The new style and the old style mapping only differs in that we either
@@ -1905,7 +1877,7 @@ VMCIHost_RegisterUserMemory(unsigned int index,             // IN
 
    produceUVA = pageStore->pages;
    consumeUVA = pageStore->pages + produceQ->kernelIf->numPages * PAGE_SIZE;
-   return VMCIHost_GetUserMemory(index, produceUVA, consumeUVA, produceQ, consumeQ);
+   return VMCIHost_GetUserMemory(produceUVA, consumeUVA, produceQ, consumeQ);
 }
 
 
@@ -1928,21 +1900,23 @@ VMCIHost_RegisterUserMemory(unsigned int index,             // IN
  */
 
 void
-VMCIHost_UnregisterUserMemory(unsigned int index,          // IN
-                              VMCIQueue *produceQ,         // IN/OUT
+VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
                               VMCIQueue *consumeQ)         // IN/OUT
 {
-   ASSERT(index == 0);
    ASSERT(produceQ->kernelIf);
    ASSERT(consumeQ->kernelIf);
    ASSERT(!produceQ->qHeader && !consumeQ->qHeader);
 
-   VMCIReleasePages(produceQ->kernelIf->headerPage, produceQ->kernelIf->numPages, TRUE);
-   memset(produceQ->kernelIf->headerPage, 0,
-          sizeof *produceQ->kernelIf->headerPage * produceQ->kernelIf->numPages);
-   VMCIReleasePages(consumeQ->kernelIf->headerPage, consumeQ->kernelIf->numPages, TRUE);
-   memset(consumeQ->kernelIf->headerPage, 0,
-          sizeof *consumeQ->kernelIf->headerPage * consumeQ->kernelIf->numPages);
+   VMCIReleasePages(produceQ->kernelIf->u.h.headerPage,
+                    produceQ->kernelIf->numPages, TRUE);
+   memset(produceQ->kernelIf->u.h.headerPage, 0,
+          (sizeof *produceQ->kernelIf->u.h.headerPage *
+           produceQ->kernelIf->numPages));
+   VMCIReleasePages(consumeQ->kernelIf->u.h.headerPage,
+                    consumeQ->kernelIf->numPages, TRUE);
+   memset(consumeQ->kernelIf->u.h.headerPage, 0,
+          (sizeof *consumeQ->kernelIf->u.h.headerPage *
+           consumeQ->kernelIf->numPages));
 }
 
 
@@ -1967,14 +1941,12 @@ VMCIHost_UnregisterUserMemory(unsigned int index,          // IN
  */
 
 int
-VMCIHost_MapQueues(unsigned int index,   // IN
-                   VMCIQueue *produceQ,  // IN/OUT
+VMCIHost_MapQueues(VMCIQueue *produceQ,  // IN/OUT
                    VMCIQueue *consumeQ,  // IN/OUT
                    uint32 flags)         // UNUSED
 {
    int result;
 
-   ASSERT(index == 0);
    if (!produceQ->qHeader || !consumeQ->qHeader) {
       struct page *headers[2];
 
@@ -1982,15 +1954,16 @@ VMCIHost_MapQueues(unsigned int index,   // IN
          return VMCI_ERROR_QUEUEPAIR_MISMATCH;
       }
 
-      if (produceQ->kernelIf->headerPage == NULL ||
-          *produceQ->kernelIf->headerPage == NULL) {
+      if (produceQ->kernelIf->u.h.headerPage == NULL ||
+          *produceQ->kernelIf->u.h.headerPage == NULL) {
          return VMCI_ERROR_UNAVAILABLE;
       }
 
-      ASSERT(*produceQ->kernelIf->headerPage && *consumeQ->kernelIf->headerPage);
+      ASSERT(*produceQ->kernelIf->u.h.headerPage &&
+             *consumeQ->kernelIf->u.h.headerPage);
 
-      headers[0] = *produceQ->kernelIf->headerPage;
-      headers[1] = *consumeQ->kernelIf->headerPage;
+      headers[0] = *produceQ->kernelIf->u.h.headerPage;
+      headers[1] = *consumeQ->kernelIf->u.h.headerPage;
 
       produceQ->qHeader = vmap(headers, 2, VM_MAP, PAGE_KERNEL);
       if (produceQ->qHeader != NULL) {
@@ -2026,12 +1999,10 @@ VMCIHost_MapQueues(unsigned int index,   // IN
  */
 
 int
-VMCIHost_UnmapQueues(unsigned int index,   // IN
-                     VMCIGuestMemID gid,   // IN
+VMCIHost_UnmapQueues(VMCIGuestMemID gid,   // IN
                      VMCIQueue *produceQ,  // IN/OUT
                      VMCIQueue *consumeQ)  // IN/OUT
 {
-   ASSERT(index == 0);
    if (produceQ->qHeader) {
       ASSERT(consumeQ->qHeader);
 
@@ -2068,8 +2039,7 @@ VMCIHost_UnmapQueues(unsigned int index,   // IN
  */
 
 int
-VMCIHost_GetUserMemory(unsigned int index,    // IN
-                       VA64 produceUVA,       // IN
+VMCIHost_GetUserMemory(VA64 produceUVA,       // IN
                        VA64 consumeUVA,       // IN
                        VMCIQueue *produceQ,   // OUT
                        VMCIQueue *consumeQ)   // OUT
@@ -2077,18 +2047,17 @@ VMCIHost_GetUserMemory(unsigned int index,    // IN
    int retval;
    int err = VMCI_SUCCESS;
 
-   ASSERT(index == 0);
    down_write(&current->mm->mmap_sem);
    retval = get_user_pages(current,
                            current->mm,
                            (VA)produceUVA,
                            produceQ->kernelIf->numPages,
                            1, 0,
-                           produceQ->kernelIf->headerPage,
+                           produceQ->kernelIf->u.h.headerPage,
                            NULL);
    if (retval < produceQ->kernelIf->numPages) {
       Log("get_user_pages(produce) failed (retval=%d)\n", retval);
-      VMCIReleasePages(produceQ->kernelIf->headerPage, retval, FALSE);
+      VMCIReleasePages(produceQ->kernelIf->u.h.headerPage, retval, FALSE);
       err = VMCI_ERROR_NO_MEM;
       goto out;
    }
@@ -2098,12 +2067,12 @@ VMCIHost_GetUserMemory(unsigned int index,    // IN
                            (VA)consumeUVA,
                            consumeQ->kernelIf->numPages,
                            1, 0,
-                           consumeQ->kernelIf->headerPage,
+                           consumeQ->kernelIf->u.h.headerPage,
                            NULL);
    if (retval < consumeQ->kernelIf->numPages) {
       Log("get_user_pages(consume) failed (retval=%d)\n", retval);
-      VMCIReleasePages(consumeQ->kernelIf->headerPage, retval, FALSE);
-      VMCIReleasePages(produceQ->kernelIf->headerPage,
+      VMCIReleasePages(consumeQ->kernelIf->u.h.headerPage, retval, FALSE);
+      VMCIReleasePages(produceQ->kernelIf->u.h.headerPage,
                        produceQ->kernelIf->numPages, FALSE);
       err = VMCI_ERROR_NO_MEM;
    }
@@ -2133,14 +2102,12 @@ out:
  */
 
 void
-VMCIHost_ReleaseUserMemory(unsigned int index,   // IN
-                           VMCIQueue *produceQ,  // IN/OUT
+VMCIHost_ReleaseUserMemory(VMCIQueue *produceQ,  // IN/OUT
                            VMCIQueue *consumeQ)  // IN/OUT
 {
-   ASSERT(index == 0);
-   ASSERT(produceQ->kernelIf->headerPage);
+   ASSERT(produceQ->kernelIf->u.h.headerPage);
 
-   VMCIHost_UnregisterUserMemory(index, produceQ, consumeQ);
+   VMCIHost_UnregisterUserMemory(produceQ, consumeQ);
 }
 
 
