@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007,2014 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -31,10 +31,17 @@
  * it is used in several different contexts. In particular it is called from
  * vsockAddr.c which gets compiled into both our kernel modules as well as
  * the user level vsock library. In the linux kernel we need different behavior
- * than external kernel modules using VMCI Sockets api inside the kernel.
+ * than external kernel modules using vSockets API inside the kernel.
  */
 
-#if defined _WIN32
+#if defined VMX86_VMX
+   /*
+    * The VMX is a very special case because hypervisor sockets do not go
+    * through the host kernel and thus do not require an address family.
+    */
+#  define VMCI_SOCKETS_AF_VALUE 0
+#  define VMCISockGetAFValueInt() VMCI_SOCKETS_AF_VALUE
+#elif defined _WIN32
 #  define VMCI_SOCKETS_AF_VALUE 28
 #  if defined WINNT_DDK
 #     define _WIN2K_COMPAT_SLIST_USAGE
@@ -44,7 +51,7 @@
       /* In the kernel we can't call into the provider. */
 #     define VMCISockGetAFValueInt() VMCI_SOCKETS_AF_VALUE
 #  else // WINNT_DDK
-      /* In userland, just use the normal exported userlevel api. */
+      /* In userland, just use the normal exported userlevel API. */
 #     define VMCISockGetAFValueInt() VMCISock_GetAFValue()
 #     include <windows.h>
 #  endif // WINNT_DDK
@@ -65,7 +72,7 @@
       extern int VSockVmci_GetAFValue(void);
 #     define VMCISockGetAFValueInt() VSockVmci_GetAFValue()
 #  else // __KERNEL__
-      /* In userland, just use the normal exported userlevel api. */
+      /* In userland, just use the normal exported userlevel API. */
 #     define VMCISockGetAFValueInt() VMCISock_GetAFValue()
 #  endif
 #elif defined __APPLE__
@@ -94,6 +101,20 @@
 #include "vsockSocketWrapper.h"
 
 
+/*
+ * Local VSocket control packet resource ID.
+ *
+ * Stream sockets to the hypervisor were added later so VSOCK_PACKET_RID was
+ * already assigned to another application. VSOCK_PACKET_HYPERVISOR_RID is
+ * used instead.
+ */
+#if defined VMX86_VMX
+#  define VSOCK_PACKET_LOCAL_RID  VSOCK_PACKET_HYPERVISOR_RID
+#else
+#  define VSOCK_PACKET_LOCAL_RID  VSOCK_PACKET_RID
+#endif
+
+
 /* Memory allocation flags. */
 #define VSOCK_MEMORY_NORMAL   0
 #define VSOCK_MEMORY_ATOMIC   (1 << 0)
@@ -119,7 +140,7 @@
 static INLINE void *
 VSockVA64ToPtr(VA64 va64) // IN
 {
-#ifdef VM_X86_64
+#ifdef VM_64BIT
    ASSERT_ON_COMPILE(sizeof (void *) == 8);
 #else
    ASSERT_ON_COMPILE(sizeof (void *) == 4);
@@ -168,7 +189,7 @@ __declspec(selectany) extern const WSAPROTOCOL_INFOW vsockProtocolInfos[] = {
       0,                         /* Assigned by Winsock. */
       { 1, 0 },                  /* Base provider. */
       0,                         /* Version 0. */
-      VMCI_SOCKETS_AF_VALUE,     /* VMCI sockets protocol. */
+      VMCI_SOCKETS_AF_VALUE,     /* vSockets protocol. */
       16,                        /* Maximum address length in bytes. */
       16,                        /* Minimum address length in bytes. */
       SOCK_DGRAM,                /* STREAM. */
@@ -178,7 +199,7 @@ __declspec(selectany) extern const WSAPROTOCOL_INFOW vsockProtocolInfos[] = {
       SECURITY_PROTOCOL_NONE,    /* No security. */
       0,                         /* Message size unimportant. */
       0,                         /* None. */
-      L"VMCI sockets DGRAM"      /* Protocol name. */
+      L"vSockets DGRAM"          /* Protocol name. */
    },
    {
       (XP1_GUARANTEED_DELIVERY | /* Guaranteed delivery. */
@@ -192,7 +213,7 @@ __declspec(selectany) extern const WSAPROTOCOL_INFOW vsockProtocolInfos[] = {
       0,                         /* Assigned by Winsock. */
       { 1, 0 },                  /* Base provider. */
       0,                         /* Version 0. */
-      VMCI_SOCKETS_AF_VALUE,     /* VMCI sockets protocol. */
+      VMCI_SOCKETS_AF_VALUE,     /* vSockets protocol. */
       16,                        /* Maximum address length in bytes. */
       16,                        /* Minimum address length in bytes. */
       SOCK_STREAM,               /* STREAM. */
@@ -202,7 +223,7 @@ __declspec(selectany) extern const WSAPROTOCOL_INFOW vsockProtocolInfos[] = {
       SECURITY_PROTOCOL_NONE,    /* No security. */
       0,                         /* Message size unimportant. */
       0,                         /* None. */
-      L"VMCI sockets STREAM"     /* Protocol name. */
+      L"vSockets STREAM"         /* Protocol name. */
    },
 };
 
