@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -80,48 +80,13 @@
  *-----------------------------------------------------------------------------
  */
 
-ConstUnicode
+const char *
 DnD_GetFileRoot(void)
 {
    return "/tmp/VMwareDnD/";
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * DnD_PrependFileRoot --
- *
- *    Given a buffer of '\0' delimited filenames, this prepends the file root
- *    to each one and uses '\0' delimiting for the output buffer.  The buffer
- *    pointed to by *src will be freed and *src will point to a new buffer
- *    containing the results.  *srcSize is set to the size of the new buffer,
- *    not including the NUL-terminator.
- *
- * Results:
- *    TRUE on success, FALSE on failure.
- *
- * Side effects:
- *    *src will be freed, and a new buffer will be allocated. This buffer must
- *    be freed by the caller.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-DnD_PrependFileRoot(const char *fileRoot,  // IN    : file root to append
-                    char **src,            // IN/OUT: NUL-delimited list of paths
-                    size_t *srcSize)       // IN/OUT: size of list
-{
-   ASSERT(fileRoot);
-   ASSERT(src);
-   ASSERT(*src);
-   ASSERT(srcSize);
-
-   return DnDPrependFileRoot(fileRoot, '\0', src, srcSize);
-}
-
-
 /*
  *----------------------------------------------------------------------------
  *
@@ -334,6 +299,8 @@ Bool
 DnD_AddBlockLegacy(int blockFd,                    // IN
                    const char *blockPath)          // IN
 {
+   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
+
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL(blockFd, VMBLOCK_ADD_FILEBLOCK, blockPath) != 0) {
@@ -367,6 +334,8 @@ Bool
 DnD_RemoveBlockLegacy(int blockFd,                    // IN
                       const char *blockedPath)        // IN
 {
+   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
+
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL(blockFd, VMBLOCK_DEL_FILEBLOCK, blockedPath) != 0) {
          Log("%s: Cannot delete block on %s (%s)\n",
@@ -429,6 +398,8 @@ Bool
 DnD_AddBlockFuse(int blockFd,                    // IN
                  const char *blockPath)          // IN
 {
+   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
+
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_ADD_FILEBLOCK,
@@ -463,6 +434,8 @@ Bool
 DnD_RemoveBlockFuse(int blockFd,                    // IN
                     const char *blockedPath)        // IN
 {
+   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
+
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_DEL_FILEBLOCK,
                                blockedPath) != 0) {
@@ -784,7 +757,7 @@ DnD_CompleteBlockInitialization(int fd,                     // IN
  */
 
 Bool
-DnDRootDirUsable(ConstUnicode pathName)  // IN:
+DnDRootDirUsable(const char *pathName)  // IN:
 {
    struct stat buf;
 
@@ -817,7 +790,7 @@ DnDRootDirUsable(ConstUnicode pathName)  // IN:
  */
 
 Bool
-DnDSetPermissionsOnRootDir(ConstUnicode pathName)  // IN:
+DnDSetPermissionsOnRootDir(const char *pathName)  // IN:
 {
    return Posix_Chmod(pathName, S_ISVTX | DND_ROOTDIR_PERMS) == 0;
 }
@@ -842,7 +815,7 @@ DnDSetPermissionsOnRootDir(ConstUnicode pathName)  // IN:
  */
 
 Bool
-DnDStagingDirectoryUsable(ConstUnicode pathName)  // IN:
+DnDStagingDirectoryUsable(const char *pathName)  // IN:
 {
    struct stat buf;
 
@@ -871,7 +844,7 @@ DnDStagingDirectoryUsable(ConstUnicode pathName)  // IN:
  */
 
 Bool
-DnDSetPermissionsOnStagingDir(ConstUnicode pathName)  // IN:
+DnDSetPermissionsOnStagingDir(const char *pathName)  // IN:
 {
    return Posix_Chmod(pathName, DND_STAGINGDIR_PERMS) == 0;
 }
