@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -60,19 +60,44 @@
 #include "vmware/tools/utils.h"
 #include "vmware/tools/vmbackup.h"
 
+#if defined(_WIN32)
+#include "guestStats.h"
+#include "win32/guestInfoWin32.h"
+#endif
+
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
 #endif
 
 /**
- * Default poll interval is 30s (in milliseconds).
+ * Default poll interval for guestInfo is 30s
  */
-#define GUESTINFO_TIME_INTERVAL_MSEC 30000
+#define GUESTINFO_POLL_INTERVAL 30
+
+/**
+ * Default poll interval for guestStats is 20s
+ */
+#define GUESTINFO_STATS_INTERVAL 20
 
 #define GUESTINFO_DEFAULT_DELIMITER ' '
 
+/*
+ * Define what guest info types and nic info versions could be sent
+ * to update nic info at VMX. The order defines a sequence of fallback
+ * paths to provide backward compatibility for ESX running with nic
+ * info version older than the guest OS.
+ */
+typedef enum NicInfoMethod {
+   NIC_INFO_V3_WITH_INFO_IPADDRESS_V3,
+   NIC_INFO_V3_WITH_INFO_IPADDRESS_V2,
+   NIC_INFO_V2_WITH_INFO_IPADDRESS_V2,
+   NIC_INFO_V1_WITH_INFO_IPADDRESS,
+   NIC_INFO_METHOD_MAX
+} NicInfoMethod;
+
 /*
  * Stores information about all guest information sent to the vmx.
  */
@@ -82,6 +107,7 @@ typedef struct _GuestInfoCache {
    char          *value[INFO_MAX];
    NicInfoV3     *nicInfo;
    GuestDiskInfo *diskInfo;
+   NicInfoMethod  method;
 } GuestInfoCache;
 
 
@@ -92,12 +118,22 @@ typedef struct _GuestInfoCache {
  */
 int guestInfoPollInterval = 0;
 
+/**
+ * Defines the current stats interval (in milliseconds).
+ *
+ * This value is controlled by the guestinfo.stats-interval config file option.
+ */
+int guestInfoStatsInterval = 0;
 
 /**
- * Gather loop timeout source.
+ * GuestInfo gather loop timeout source.
  */
-static GSource *gatherTimeoutSource = NULL;
+static GSource *gatherInfoTimeoutSource = NULL;
 
+/**
+ * GuestStats gather loop timeout source.
+ */
+static GSource *gatherStatsTimeoutSource = NULL;
 
 /* Local cache of the guest information that was last sent to vmx. */
 static GuestInfoCache gInfoCache;
@@ -116,14 +152,17 @@ static Bool vmResumed;
  */
 
 
-static Bool GuestInfoUpdateVmdb(ToolsAppCtx *ctx, GuestInfoType infoType, void *info);
+static Bool GuestInfoUpdateVmdb(ToolsAppCtx *ctx,
+                                GuestInfoType infoType,
+                                void *info,
+                                size_t infoSize);
 static Bool SetGuestInfo(ToolsAppCtx *ctx, GuestInfoType key,
                          const char *value);
 static void SendUptime(ToolsAppCtx *ctx);
 static Bool DiskInfoChanged(const GuestDiskInfo *diskInfo);
 static void GuestInfoClearCache(void);
 static GuestNicList *NicInfoV3ToV2(const NicInfoV3 *infoV3);
-static void TweakGatherLoop(ToolsAppCtx *ctx, gboolean enable);
+static void TweakGatherLoops(ToolsAppCtx *ctx, gboolean enable);
 
 
 /*
@@ -178,7 +217,7 @@ GuestInfoVMSupport(RpcInData *data)
     vmSupportProcArgs.lpThreadAttributes = &saThread;
     vmSupportProcArgs.dwCreationFlags = CREATE_NO_WINDOW;
 
-    g_debug("Starting vm-support script - %s\n", vmSupport);
+    g_message("Starting vm-support script - %s\n", vmSupport);
     vmSupportProc = ProcMgr_ExecAsync(vmSupport, &vmSupportProcArgs);
     g_free(vmSupport);
 
@@ -195,7 +234,7 @@ GuestInfoVMSupport(RpcInData *data)
 
      gchar *vmSupportCmdArgv[] = {"vm-support", "-u", NULL};
 
-     g_debug("Starting vm-support script - %s\n", vmSupportCmdArgv[0]);
+     g_message("Starting vm-support script - %s\n", vmSupportCmdArgv[0]);
      if (!g_spawn_async(NULL, vmSupportCmdArgv, NULL,
                         G_SPAWN_SEARCH_PATH |
                         G_SPAWN_STDOUT_TO_DEV_NULL |
@@ -231,19 +270,17 @@ GuestInfoGather(gpointer data)
    char name[256];  // Size is derived from the SUS2 specification
                     // "Host names are limited to 255 bytes"
    char *osString = NULL;
+#if !defined(USERWORLD)
    gboolean disableQueryDiskInfo;
-   NicInfoV3 *nicInfo = NULL;
    GuestDiskInfo *diskInfo = NULL;
-#if defined(_WIN32) || defined(linux)
-   GuestMemInfo vmStats = {0};
-   gboolean perfmonEnabled;
 #endif
+   NicInfoV3 *nicInfo = NULL;
    ToolsAppCtx *ctx = data;
 
    g_debug("Entered guest info gather.\n");
 
    /* Send tools version. */
-   if (!GuestInfoUpdateVmdb(ctx, INFO_BUILD_NUMBER, BUILD_NUMBER)) {
+   if (!GuestInfoUpdateVmdb(ctx, INFO_BUILD_NUMBER, BUILD_NUMBER, 0)) {
       /*
        * An older vmx talking to new tools wont be able to handle
        * this message. Continue, if thats the case.
@@ -257,7 +294,7 @@ GuestInfoGather(gpointer data)
    if (osString == NULL) {
       g_warning("Failed to get OS info.\n");
    } else {
-      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME_FULL, osString)) {
+      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME_FULL, osString, 0)) {
          g_warning("Failed to update VMDB\n");
       }
    }
@@ -267,12 +304,13 @@ GuestInfoGather(gpointer data)
    if (osString == NULL) {
       g_warning("Failed to get OS info.\n");
    } else {
-      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME, osString)) {
+      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME, osString, 0)) {
          g_warning("Failed to update VMDB\n");
       }
    }
    free(osString);
 
+#if !defined(USERWORLD)
    disableQueryDiskInfo =
       g_key_file_get_boolean(ctx->config, CONFGROUPNAME_GUESTINFO,
                              CONFNAME_GUESTINFO_DISABLEQUERYDISKINFO, NULL);
@@ -280,7 +318,7 @@ GuestInfoGather(gpointer data)
       if ((diskInfo = GuestInfo_GetDiskInfo()) == NULL) {
          g_warning("Failed to get disk info.\n");
       } else {
-         if (GuestInfoUpdateVmdb(ctx, INFO_DISK_FREE_SPACE, diskInfo)) {
+         if (GuestInfoUpdateVmdb(ctx, INFO_DISK_FREE_SPACE, diskInfo, 0)) {
             GuestInfo_FreeDiskInfo(gInfoCache.diskInfo);
             gInfoCache.diskInfo = diskInfo;
          } else {
@@ -289,10 +327,11 @@ GuestInfoGather(gpointer data)
          }
       }
    }
+#endif
 
    if (!System_GetNodeName(sizeof name, name)) {
       g_warning("Failed to get netbios name.\n");
-   } else if (!GuestInfoUpdateVmdb(ctx, INFO_DNS_NAME, name)) {
+   } else if (!GuestInfoUpdateVmdb(ctx, INFO_DNS_NAME, name, 0)) {
       g_warning("Failed to update VMDB.\n");
    }
 
@@ -308,7 +347,7 @@ GuestInfoGather(gpointer data)
    if (GuestInfo_IsEqual_NicInfoV3(nicInfo, gInfoCache.nicInfo)) {
       g_debug("Nic info not changed.\n");
       GuestInfo_FreeNicInfo(nicInfo);
-   } else if (GuestInfoUpdateVmdb(ctx, INFO_IPADDRESS, nicInfo)) {
+   } else if (GuestInfoUpdateVmdb(ctx, INFO_IPADDRESS, nicInfo, 0)) {
       /*
        * Since the update succeeded, free the old cached object, and assign
        * ours to the cache.
@@ -323,23 +362,59 @@ GuestInfoGather(gpointer data)
    /* Send the uptime to VMX so that it can detect soft resets. */
    SendUptime(ctx);
 
-#if defined(_WIN32) || defined(linux)
-   /* Send the vmstats to the VMX. */
-   perfmonEnabled = !g_key_file_get_boolean(ctx->config,
+   return TRUE;
+}
+
+
+/*
+ ******************************************************************************
+ * GuestInfoStatsGather --                                               */ /**
+ *
+ * Collects all the desired guest stats and updates the VMX.
+ *
+ * @param[in]  data     The application context.
+ *
+ * @return TRUE to indicate that the timer should be rescheduled.
+ *
+ ******************************************************************************
+ */
+
+static gboolean
+GuestInfoStatsGather(gpointer data)
+{
+#if (defined(__linux__) && !defined(USERWORLD)) || defined(_WIN32)
+   ToolsAppCtx *ctx = data;
+   DynBuf stats;
+#endif
+#if defined(_WIN32)
+   gboolean perfmonLogStats;
+#endif
+
+   g_debug("Entered guest info stats gather.\n");
+
+#if defined(_WIN32)
+   perfmonLogStats = g_key_file_get_boolean(ctx->config,
                                             CONFGROUPNAME_GUESTINFO,
-                                            CONFNAME_GUESTINFO_DISABLEPERFMON,
+                                            CONFNAME_GUESTINFO_ENABLESTATLOGGING,
                                             NULL);
 
-   if (perfmonEnabled) {
-      if (!GuestInfo_PerfMon(&vmStats)) {
-         g_warning("Failed to get vmstats.\n");
-      } else {
-         vmStats.version = 1;
-         if (!GuestInfoUpdateVmdb(ctx, INFO_MEMORY, &vmStats)) {
-            g_warning("Failed to send vmstats.\n");
-         }
+   GuestInfo_SetStatLogging(perfmonLogStats);
+#endif
+
+#if (defined(__linux__) && !defined(USERWORLD)) || defined(_WIN32)
+   /* Send the vmstats to the VMX. */
+   DynBuf_Init(&stats);
+
+   if (GuestInfo_PerfMon(&stats)) {
+      if (!GuestInfoUpdateVmdb(ctx, INFO_MEMORY, DynBuf_Get(&stats),
+                               DynBuf_GetSize(&stats))) {
+         g_warning("Failed to send vmstats.\n");
       }
+   } else {
+      g_warning("Failed to get vmstats.\n");
    }
+
+   DynBuf_Destroy(&stats);
 #endif
 
    return TRUE;
@@ -419,6 +494,326 @@ GuestInfoConvertNicInfoToNicInfoV1(NicInfoV3 *info,
 }
 
 
+/*
+ ******************************************************************************
+ * GuestInfoSendMemoryInfo --                                            */ /**
+ *
+ * Push memory informations about the guest to the vmx
+ *
+ * @param[in] ctx       Application context.
+ * @param[in] infoSize  Size of the struct to send
+ * @param[in] info      Struct that contains memory info
+ *
+ * @retval TRUE  Update sent successfully.
+ * @retval FALSE Had trouble with transmission.
+ *
+ ******************************************************************************
+ */
+
+static Bool
+GuestInfoSendMemoryInfo(ToolsAppCtx *ctx,  // IN: Application context
+                        uint64 infoSize,   // IN: Size of the following struct
+                        void* info)        // IN: Pointer to GuestMemInfoRPC
+{
+   char *request;
+   char header[32];
+   size_t headerLen;
+   size_t requestSize;
+   Bool success = FALSE;
+
+   Str_Sprintf(header, sizeof header, "%s  %d ", GUEST_INFO_COMMAND,
+               INFO_MEMORY);
+
+   headerLen = strlen(header);
+   requestSize = headerLen + infoSize;
+
+   request = g_malloc(requestSize);
+
+   if (request == NULL) {
+      g_warning("Failed to allocate GuestMemInfo memory.\n");
+   } else {
+      g_debug("Sending GuestMemInfo message.\n");
+
+      /* Yes, stomp on the NUL at the end of the header */
+      memcpy(request, header, headerLen);
+      memcpy(request + headerLen, info, infoSize);
+
+      /* Send all the information in the message. */
+      success = RpcChannel_Send(ctx->rpc, request, requestSize, NULL, NULL);
+
+      g_free(request);
+   }
+
+   if (success) {
+      g_debug("GuestMemInfo sent successfully.\n");
+   } else {
+      g_warning("Error sending GuestMemInfo.\n");
+   }
+
+   return success;
+}
+
+
+/*
+ ******************************************************************************
+ *
+ * NIC_INFO_V3_WITH_INFO_IPADDRESS_V3: Bump up the NICINFO_MAX_IPS to 2048
+ * NIC_INFO_V3_WITH_INFO_IPADDRESS_V2: NICINFO_MAX_IPS to 64
+ *
+ * The current fallback paths:
+ * +---------------+-------------------+-------------------+------------------+
+ * |               | INFO_IPADDRESS_V3 | INFO_IPADDRESS_V2 | INFO_IPADDRESS   |
+ * +---------------+-------------------+-------------------+------------------+
+ * |  NIC_INFO_V3  |        (0)        |        (1)        |                  |
+ * +---------------+-------------------+-------------------+------------------+
+ * |  NIC_INFO_V2  |                   |        (2)        |                  |
+ * +---------------+-------------------+-------------------+------------------+
+ * |  NIC_INFO_V1  |                   |                   |        (3)       |
+ * +---------------+-------------------+-------------------+------------------+
+ *
+ ******************************************************************************
+ */
+
+
+/*
+ ******************************************************************************
+ * GuestInfoSendNicInfoXdr --                                            */ /**
+ *
+ * Push updated nic info to VMX using XDR data format.
+ *
+ * @param[in] ctx      Application context.
+ * @param[in] message  The protocol for a 'nic info' message.
+ * @param[in] type     Guest information type.
+ *
+ * @retval TRUE  Update sent successfully.
+ * @retval FALSE Had trouble with serialization or transmission.
+ *
+ ******************************************************************************
+ */
+
+static Bool
+GuestInfoSendNicInfoXdr(ToolsAppCtx *ctx,          // IN
+                        GuestNicProto *message,    // IN
+                        GuestInfoType type)        // IN
+{
+   Bool status = FALSE;
+   XDR xdrs;
+   gchar *request;
+   char *reply = NULL;
+   size_t replyLen;
+
+   /* Add the RPC preamble: message name, and type. */
+   request = g_strdup_printf("%s  %d ", GUEST_INFO_COMMAND, type);
+
+   if (DynXdr_Create(&xdrs) == NULL) {
+      goto exit;
+   }
+
+   /* Write preamble and serialized nic info to XDR stream. */
+   if (!DynXdr_AppendRaw(&xdrs, request, strlen(request)) ||
+       !xdr_GuestNicProto(&xdrs, message)) {
+      g_warning("Error serializing nic info v%d data.", message->ver);
+   } else {
+      status = RpcChannel_Send(ctx->rpc, DynXdr_Get(&xdrs), xdr_getpos(&xdrs),
+                               &reply, &replyLen);
+      if (!status) {
+         g_warning("%s: update failed: request \"%s\", reply \"%s\".\n",
+                    __FUNCTION__, request, reply);
+      }
+      vm_free(reply);
+   }
+   DynXdr_Destroy(&xdrs, TRUE);
+
+exit:
+   g_free(request);
+   return status;
+}
+
+
+/*
+ ******************************************************************************
+ * GuestInfoSendData --                                                  */ /**
+ *
+ * Push GuestInfo information to the VMX. So far, it is mainly used to
+ * send the fixed nic info V1 data.
+ *
+ * @param[in] ctx         Application context.
+ * @param[in] info        Guest information data.
+ * @param[in] infoLength  Length of guest information.
+ * @param[in] type        Guest information type.
+ *
+ * @retval TRUE  Update sent successfully.
+ * @retval FALSE Had trouble with transmission.
+ *
+ ******************************************************************************
+ */
+
+static Bool
+GuestInfoSendData(ToolsAppCtx *ctx,                // IN
+                  void *info,                      // IN
+                  u_int infoLength,                // IN
+                  GuestInfoType type)              // IN
+{
+   Bool status;
+   gchar *request;
+   u_int msgLength = infoLength;
+   gchar *message = NULL;
+   char *reply = NULL;
+   size_t replyLen;
+
+   /* Add the RPC preamble: message name, and type. */
+   request = g_strdup_printf("%s  %d ", GUEST_INFO_COMMAND, type);
+
+   msgLength += strlen(request);
+   message = g_malloc(msgLength);
+
+   memcpy(message, request, strlen(request));
+   memcpy(message + strlen(request), info, infoLength);
+
+   status = RpcChannel_Send(ctx->rpc, message, msgLength, &reply, &replyLen);
+   if (!status) {
+      g_warning("%s: update failed: request \"%s\", reply \"%s\".\n",
+                __FUNCTION__, request, reply);
+   }
+   vm_free(reply);
+
+   g_free(message);
+   g_free(request);
+
+   return status;
+}
+
+
+/*
+ ******************************************************************************
+ * GuestNicInfoV3ToV3_64 --                                              */ /**
+ *
+ * New NicInfoV3 allows more IP addresses than the previous version. Preseve
+ * the backward compatibility such that NicInfoV3 with old limit could still
+ * go through without falling all the way to V2 or V1. It reduces the number
+ * of IP addresses at each interface to the previous max limit 64.
+ *
+ * @param[in] info  NicInfoV3 container.
+ *
+ * @retval Pointer to a shallow copy of NicInfoV3 with reduced max IPs.
+ *
+ ******************************************************************************
+ */
+
+static NicInfoV3*
+GuestNicInfoV3ToV3_64(NicInfoV3 *info)             // IN
+{
+   u_int idx;
+
+   NicInfoV3 *info64 = g_malloc(sizeof(struct NicInfoV3));
+
+   memcpy(info64, info, sizeof(struct NicInfoV3));
+   info64->nics.nics_val =
+      g_malloc(info64->nics.nics_len * sizeof(struct GuestNicV3));
+
+   for (idx = 0; idx < info64->nics.nics_len; idx++) {
+      memcpy(&info64->nics.nics_val[idx], &info->nics.nics_val[idx],
+             sizeof(struct GuestNicV3));
+
+      if (info64->nics.nics_val[idx].ips.ips_len > INFO_IPADDRESS_V2_MAX_IPS) {
+         info64->nics.nics_val[idx].ips.ips_len = INFO_IPADDRESS_V2_MAX_IPS;
+      }
+   }
+
+   return info64;
+}
+
+
+/*
+ ******************************************************************************
+ * GuestInfoSendNicInfo --                                               */ /**
+ *
+ * Push updated nic info to the VMX. Take care of failed transmissions or
+ * unknown guest information types. Use a fixed sequence of fallback paths
+ * to retry.
+ *
+ * @param[in] ctx   Application context.
+ * @param[in] info  NicInfoV3 container.
+ *
+ * @retval TRUE  Update sent successfully.
+ * @retval FALSE Had trouble with serialization or transmission.
+ *
+ ******************************************************************************
+ */
+
+static Bool
+GuestInfoSendNicInfo(ToolsAppCtx *ctx,             // IN
+                     NicInfoV3 *info)              // IN
+{
+   Bool status = FALSE;
+   GuestNicProto message = {0};
+   NicInfoV3 *info64 = NULL;
+
+   do {
+      switch (gInfoCache.method) {
+      case NIC_INFO_V3_WITH_INFO_IPADDRESS_V3:
+         message.ver = NIC_INFO_V3;
+         message.GuestNicProto_u.nicInfoV3 = info;
+         if (GuestInfoSendNicInfoXdr(ctx, &message, INFO_IPADDRESS_V3)) {
+            status = TRUE;
+         }
+         break;
+      case NIC_INFO_V3_WITH_INFO_IPADDRESS_V2:
+         info64 = GuestNicInfoV3ToV3_64(info);
+         message.ver = NIC_INFO_V3;
+         message.GuestNicProto_u.nicInfoV3 = info64;
+         if (GuestInfoSendNicInfoXdr(ctx, &message, INFO_IPADDRESS_V2)) {
+            status = TRUE;
+         }
+         break;
+      case NIC_INFO_V2_WITH_INFO_IPADDRESS_V2:
+         if (!info64) {
+            info64 = GuestNicInfoV3ToV3_64(info);
+         }
+         {
+            GuestNicList *nicList = NicInfoV3ToV2(info64);
+            message.ver = NIC_INFO_V2;
+            message.GuestNicProto_u.nicsV2 = nicList;
+            if (GuestInfoSendNicInfoXdr(ctx, &message, INFO_IPADDRESS_V2)) {
+               status = TRUE;
+            }
+            VMX_XDR_FREE(xdr_GuestNicList, nicList);
+            free(nicList);
+         }
+         break;
+      case NIC_INFO_V1_WITH_INFO_IPADDRESS:
+         {
+            GuestNicInfoV1 infoV1;
+            GuestInfoConvertNicInfoToNicInfoV1(info, &infoV1);
+            if (GuestInfoSendData(ctx, &infoV1, sizeof(infoV1),
+                                  INFO_IPADDRESS)) {
+               status = TRUE;
+            }
+         }
+         break;
+      default:
+         g_error("Invalid nicInfo send method: %d\n", gInfoCache.method);
+         break;
+      }
+   } while (!status && (++gInfoCache.method < NIC_INFO_METHOD_MAX));
+
+   if (info64) {
+      g_free(info64->nics.nics_val);
+      g_free(info64);
+   }
+
+   if (status) {
+      g_debug("Updating nicInfo successfully: method=%d\n", gInfoCache.method);
+   } else {
+      gInfoCache.method = NIC_INFO_V3_WITH_INFO_IPADDRESS_V3;
+      g_warning("Fail to send nicInfo: method=%d status=%d\n",
+                gInfoCache.method, status);
+   }
+
+   return status;
+}
+
+
 /*
  ******************************************************************************
  * GuestInfoUpdateVmdb --                                                */ /**
@@ -437,10 +832,11 @@ GuestInfoConvertNicInfoToNicInfoV1(NicInfoV3 *info,
  ******************************************************************************
  */
 
-Bool
+static Bool
 GuestInfoUpdateVmdb(ToolsAppCtx *ctx,       // IN: Application context
                     GuestInfoType infoType, // IN: guest information type
-                    void *info)             // IN: type specific information
+                    void *info,             // IN: type specific information
+                    size_t infoSize)        // IN: size of *info
 {
    ASSERT(info);
    g_debug("Entered update vmdb: %d.\n", infoType);
@@ -480,166 +876,21 @@ GuestInfoUpdateVmdb(ToolsAppCtx *ctx,       // IN: Application context
 
    case INFO_IPADDRESS:
       {
-         static NicInfoVersion supportedVersion = NIC_INFO_V3;
-         NicInfoV3 *nicInfoV3 = (NicInfoV3 *)info;
-         char *reply = NULL;
-         size_t replyLen;
-         Bool status;
-
-nicinfo_fsm:
-         switch (supportedVersion) {
-         case NIC_INFO_V3:
-         case NIC_INFO_V2:
-            {
-               /*
-                * 13 = max size of string representation of an int + 3 spaces.
-                * XXX Ditch the magic numbers.
-                */
-               char request[sizeof GUEST_INFO_COMMAND + 13];
-               GuestNicProto message = {0};
-               GuestNicList *nicList = NULL;
-               NicInfoVersion fallbackVersion;
-               XDR xdrs;
-
-               if (DynXdr_Create(&xdrs) == NULL) {
-                  return FALSE;
-               }
-
-               /* Add the RPC preamble: message name, and type. */
-               Str_Sprintf(request, sizeof request, "%s  %d ",
-                           GUEST_INFO_COMMAND, INFO_IPADDRESS_V2);
-
-               /*
-                * Fill in message according to the version we'll attempt.  Also
-                * note which version we'll fall back to should the VMX reject
-                * our update.
-                */
-               if (supportedVersion == NIC_INFO_V3) {
-                  message.ver = NIC_INFO_V3;
-                  message.GuestNicProto_u.nicInfoV3 = nicInfoV3;
-
-                  fallbackVersion = NIC_INFO_V2;
-               } else {
-                  nicList = NicInfoV3ToV2(nicInfoV3);
-                  message.ver = NIC_INFO_V2;
-                  message.GuestNicProto_u.nicsV2 = nicList;
-
-                  fallbackVersion = NIC_INFO_V1;
-               }
-
-               /* Write preamble and serialized nic info to XDR stream. */
-               if (!DynXdr_AppendRaw(&xdrs, request, strlen(request)) ||
-                   !xdr_GuestNicProto(&xdrs, &message)) {
-                  g_warning("Error serializing nic info v%d data.", message.ver);
-                  DynXdr_Destroy(&xdrs, TRUE);
-                  return FALSE;
-               }
-
-               status = RpcChannel_Send(ctx->rpc,
-                                        DynXdr_Get(&xdrs),
-                                        xdr_getpos(&xdrs),
-                                        &reply,
-                                        &replyLen);
-               DynXdr_Destroy(&xdrs, TRUE);
-
-               /*
-                * Do not free/destroy contents of `message`.  The v3 nicInfo
-                * passed to us belongs to our caller.  Instead, we'll only
-                * destroy our local V2 converted data.
-                */
-               if (nicList) {
-                  VMX_XDR_FREE(xdr_GuestNicList, nicList);
-                  free(nicList);
-                  nicList = NULL;
-               }
-
-               if (!status) {
-                  g_warning("NicInfo update failed: version %u, reply \"%s\".\n",
-                            supportedVersion, reply);
-                  supportedVersion = fallbackVersion;
-                  vm_free(reply);
-                  reply = NULL;
-                  goto nicinfo_fsm;
-               }
-            }
-            break;
-
-         case NIC_INFO_V1:
-            {
-               /*
-                * Could be that we are talking to the old protocol that
-                * GuestNicInfo is still fixed size.  Another try to send the
-                * fixed sized Nic info.
-                */
-               char request[sizeof (GuestNicInfoV1) + sizeof GUEST_INFO_COMMAND +
-                         2 +                 /* 2 bytes are for digits of infotype. */
-                         3 * sizeof (char)]; /* 3 spaces */
-               GuestNicInfoV1 nicInfo;
-
-               Str_Sprintf(request,
-                           sizeof request,
-                           "%s  %d ",
-                           GUEST_INFO_COMMAND,
-                           INFO_IPADDRESS);
-
-               GuestInfoConvertNicInfoToNicInfoV1(nicInfoV3, &nicInfo);
-
-               memcpy(request + strlen(request),
-                      &nicInfo,
-                      sizeof(GuestNicInfoV1));
-
-               g_debug("Sending nic info message.\n");
-               /* Send all the information in the message. */
-               status = RpcChannel_Send(ctx->rpc,
-                                        request,
-                                        sizeof request,
-                                        &reply,
-                                        &replyLen);
-
-               g_debug("Just sent fixed sized nic info message.\n");
-
-               if (!status) {
-                  g_debug("Failed to update fixed sized nic information\n");
-                  vm_free(reply);
-                  return FALSE;
-               }
-            }
-            break;
-         default:
-            NOT_REACHED();
+         if (!GuestInfoSendNicInfo(ctx, (NicInfoV3 *) info)) {
+            g_warning("Failed to update nic information.\n");
+            return FALSE;
          }
-
-         g_debug("Updated new NIC information\n");
-         vm_free(reply);
-         reply = NULL;
+         break;
       }
-      break;
 
    case INFO_MEMORY:
-   {
-      char request[sizeof(GuestMemInfo) + sizeof GUEST_INFO_COMMAND +
-                   2 +                    /* 2 bytes are for digits of infotype. */
-                   3 * sizeof (char)];    /* 3 spaces */
-      Bool status;
-
-      g_debug("Sending GuestMemInfo message.\n");
-      Str_Sprintf(request,
-                  sizeof request,
-                  "%s  %d ",
-                  GUEST_INFO_COMMAND,
-                  INFO_MEMORY);
-      memcpy(request + strlen(request),
-             info, sizeof(GuestMemInfo));
-
-      /* Send all the information in the message. */
-      status = RpcChannel_Send(ctx->rpc, request, sizeof(request), NULL, NULL);
-      if (!status) {
-         g_warning("Error sending GuestMemInfo.\n");
-         return FALSE;
+      {
+         if (!GuestInfoSendMemoryInfo(ctx, infoSize, info)) {
+            g_warning("Unable to send GuestMemInfo\n");
+            return FALSE;
+         }
+         break;
       }
-      g_debug("GuestMemInfo sent successfully.\n");
-      break;
-   }
 
    case INFO_DISK_FREE_SPACE:
       {
@@ -647,7 +898,8 @@ nicinfo_fsm:
           * 2 accounts for the digits of infotype and 3 for the three
           * spaces.
           */
-         unsigned int requestSize = sizeof GUEST_INFO_COMMAND + 2 + 3 * sizeof (char);
+         unsigned int requestSize = sizeof GUEST_INFO_COMMAND + 2 +
+                                    3 * sizeof (char);
          uint8 partitionCount;
          size_t offset;
          char *request;
@@ -703,7 +955,8 @@ nicinfo_fsm:
          }
 
          g_debug("sizeof request is %d\n", requestSize);
-         status = RpcChannel_Send(ctx->rpc, request, requestSize, &reply, &replyLen);
+         status = RpcChannel_Send(ctx->rpc, request, requestSize, &reply,
+                                  &replyLen);
          if (status) {
             status = (*reply == '\0');
          }
@@ -725,7 +978,7 @@ nicinfo_fsm:
       break;
    }
 
-   g_debug("Returning after updating guest information\n");
+   g_debug("Returning after updating guest information: %d\n", infoType);
    return TRUE;
 }
 
@@ -746,7 +999,7 @@ SendUptime(ToolsAppCtx *ctx)
 {
    gchar *uptime = g_strdup_printf("%"FMT64"u", System_Uptime());
    g_debug("Setting guest uptime to '%s'\n", uptime);
-   GuestInfoUpdateVmdb(ctx, INFO_UPTIME, uptime);
+   GuestInfoUpdateVmdb(ctx, INFO_UPTIME, uptime, 0);
    g_free(uptime);
 }
 
@@ -933,6 +1186,8 @@ GuestInfoClearCache(void)
 
    GuestInfo_FreeNicInfo(gInfoCache.nicInfo);
    gInfoCache.nicInfo = NULL;
+
+   gInfoCache.method = NIC_INFO_V3_WITH_INFO_IPADDRESS_V3;
 }
 
 
@@ -999,44 +1254,56 @@ NicInfoV3ToV2(const NicInfoV3 *infoV3)
  ******************************************************************************
  * TweakGatherLoop --                                                    */ /**
  *
- * @brief Start, stop, reconfigure the GuestInfoGather poll loop.
+ * @brief Start, stop, reconfigure a GuestInfoGather poll loop.
  *
- * This function is responsible for creating, manipulating, and resetting the
+ * This function is responsible for creating, manipulating, and resetting a
  * GuestInfoGather loop timeout source.
  *
- * @param[in]  ctx      The app context.
- * @param[in]  enable   Whether to enable the gather loop.
- *
- * @sa CONFNAME_GUESTINFO_POLLINTERVAL
+ * @param[in]     ctx           The app context.
+ * @param[in]     enable        Whether to enable the gather loop.
+ * @param[in]     cfgKey        Config key to fetch user pref.
+ * @param[in]     defInterval   Default interval value in seconds.
+ * @param[in]     callback      Function to be called on expiry of interval.
+ * @param[in,out] currInterval  Current interval value in seconds.
+ * @param[out]    timeoutSource GSource object created.
  *
  ******************************************************************************
  */
 
 static void
 TweakGatherLoop(ToolsAppCtx *ctx,
-                gboolean enable)
+                gboolean enable,
+                gchar *cfgKey,
+                gint defInterval,
+                GSourceFunc callback,
+                gint *currInterval,
+                GSource **timeoutSource)
 {
-   GError *gError = NULL;
    gint pollInterval = 0;
 
    if (enable) {
-      pollInterval = GUESTINFO_TIME_INTERVAL_MSEC;
+      pollInterval = defInterval * 1000;
 
       /*
-       * Check the config registry for a custom poll interval, converting from
-       * seconds to milliseconds.
+       * Check the config registry for custom poll interval,
+       * converting from seconds to milliseconds.
        */
       if (g_key_file_has_key(ctx->config, CONFGROUPNAME_GUESTINFO,
-                             CONFNAME_GUESTINFO_POLLINTERVAL, NULL)) {
-         pollInterval = g_key_file_get_integer(ctx->config, CONFGROUPNAME_GUESTINFO,
-                                               CONFNAME_GUESTINFO_POLLINTERVAL, &gError);
+                             cfgKey, NULL)) {
+         GError *gError = NULL;
+
+         pollInterval = g_key_file_get_integer(ctx->config,
+                                               CONFGROUPNAME_GUESTINFO,
+                                               cfgKey, &gError);
          pollInterval *= 1000;
 
          if (pollInterval < 0 || gError) {
-            g_warning("Invalid %s.%s value.  Using default.\n",
-                      CONFGROUPNAME_GUESTINFO, CONFNAME_GUESTINFO_POLLINTERVAL);
-            pollInterval = GUESTINFO_TIME_INTERVAL_MSEC;
+            g_warning("Invalid %s.%s value. Using default %us.\n",
+                      CONFGROUPNAME_GUESTINFO, cfgKey, defInterval);
+            pollInterval = defInterval * 1000;
          }
+
+         g_clear_error(&gError);
       }
    }
 
@@ -1044,8 +1311,8 @@ TweakGatherLoop(ToolsAppCtx *ctx,
     * If the interval hasn't changed, let's not interfere with the existing
     * timeout source.
     */
-   if (guestInfoPollInterval == pollInterval) {
-      ASSERT(pollInterval || gatherTimeoutSource == NULL);
+   if (pollInterval == *currInterval) {
+      ASSERT(pollInterval || *timeoutSource == NULL);
       return;
    }
 
@@ -1054,24 +1321,85 @@ TweakGatherLoop(ToolsAppCtx *ctx,
     * exists, then create and attach a new one.
     */
 
-   if (gatherTimeoutSource != NULL) {
-      g_source_destroy(gatherTimeoutSource);
-      gatherTimeoutSource = NULL;
+   if (*timeoutSource != NULL) {
+      g_source_destroy(*timeoutSource);
+      *timeoutSource = NULL;
    }
 
-   guestInfoPollInterval = pollInterval;
+   *currInterval = pollInterval;
 
-   if (guestInfoPollInterval) {
-      g_info("New poll interval is %us.\n", guestInfoPollInterval / 1000);
+   if (*currInterval) {
+      g_info("New value for %s is %us.\n", cfgKey, *currInterval / 1000);
 
-      gatherTimeoutSource = g_timeout_source_new(guestInfoPollInterval);
-      VMTOOLSAPP_ATTACH_SOURCE(ctx, gatherTimeoutSource, GuestInfoGather, ctx, NULL);
-      g_source_unref(gatherTimeoutSource);
+      *timeoutSource = g_timeout_source_new(*currInterval);
+      VMTOOLSAPP_ATTACH_SOURCE(ctx, *timeoutSource, callback, ctx, NULL);
+      g_source_unref(*timeoutSource);
    } else {
-      g_info("Poll loop disabled.\n");
+      g_info("Poll loop for %s disabled.\n", cfgKey);
+   }
+}
+
+
+/*
+ ******************************************************************************
+ * TweakGatherLoops --                                                   */ /**
+ *
+ * @brief Start, stop, reconfigure the GuestInfoGather loops.
+ *
+ * This function is responsible for creating, manipulating, and resetting
+ * the GuestInfoGather loops (info and stats) timeout sources.
+ *
+ * @param[in]  ctx      The app context.
+ * @param[in]  enable   Whether to enable the gather loops.
+ *
+ * @sa CONFNAME_GUESTINFO_POLLINTERVAL
+ * @sa CONFNAME_GUESTINFO_STATSINTERVAL
+ *
+ ******************************************************************************
+ */
+
+static void
+TweakGatherLoops(ToolsAppCtx *ctx,
+                 gboolean enable)
+{
+   gboolean perfmonEnabled;
+
+   perfmonEnabled = !g_key_file_get_boolean(ctx->config,
+                                            CONFGROUPNAME_GUESTINFO,
+                                            CONFNAME_GUESTINFO_DISABLEPERFMON,
+                                            NULL);
+
+   if (perfmonEnabled) {
+      /*
+       * Tweak GuestStats gather loop
+       */
+      TweakGatherLoop(ctx, enable,
+                      CONFNAME_GUESTINFO_STATSINTERVAL,
+                      GUESTINFO_STATS_INTERVAL,
+                      GuestInfoStatsGather,
+                      &guestInfoStatsInterval,
+                      &gatherStatsTimeoutSource);
+   } else {
+      /*
+       * Destroy the existing timeout source, if it exists.
+       */
+      if (gatherStatsTimeoutSource != NULL) {
+         g_source_destroy(gatherStatsTimeoutSource);
+         gatherStatsTimeoutSource = NULL;
+
+         g_info("PerfMon gather loop disabled.\n");
+      }
    }
 
-   g_clear_error(&gError);
+   /*
+    * Tweak GuestInfo gather loop
+    */
+   TweakGatherLoop(ctx, enable,
+                   CONFNAME_GUESTINFO_POLLINTERVAL,
+                   GUESTINFO_POLL_INTERVAL,
+                   GuestInfoGather,
+                   &guestInfoPollInterval,
+                   &gatherInfoTimeoutSource);
 }
 
 
@@ -1099,7 +1427,7 @@ GuestInfoServerConfReload(gpointer src,
                           ToolsAppCtx *ctx,
                           gpointer data)
 {
-   TweakGatherLoop(ctx, TRUE);
+   TweakGatherLoops(ctx, TRUE);
 }
 
 
@@ -1124,7 +1452,7 @@ GuestInfoServerIOFreeze(gpointer src,
                         gboolean freeze,
                         gpointer data)
 {
-   TweakGatherLoop(ctx, !freeze);
+   TweakGatherLoops(ctx, !freeze);
 }
 
 
@@ -1148,12 +1476,18 @@ GuestInfoServerShutdown(gpointer src,
 {
    GuestInfoClearCache();
 
-   if (gatherTimeoutSource != NULL) {
-      g_source_destroy(gatherTimeoutSource);
-      gatherTimeoutSource = NULL;
+   if (gatherInfoTimeoutSource != NULL) {
+      g_source_destroy(gatherInfoTimeoutSource);
+      gatherInfoTimeoutSource = NULL;
+   }
+
+   if (gatherStatsTimeoutSource != NULL) {
+      g_source_destroy(gatherStatsTimeoutSource);
+      gatherStatsTimeoutSource = NULL;
    }
 
 #ifdef _WIN32
+   GuestInfo_StatProviderShutdown();
    NetUtil_FreeIpHlpApiDll();
 #endif
 }
@@ -1255,13 +1589,7 @@ GuestInfoServerSetOption(gpointer src,
       goto exit;
    }
 
-   ip = NetUtil_GetPrimaryIP();
-   if (ip == NULL) {
-      /*
-       * If there is any error, then return a blank string.
-       */
-      ip = Util_SafeStrdup("");
-   }
+   ip = GuestInfo_GetPrimaryIP();
 
    msg = g_strdup_printf("info-set guestinfo.ip %s", ip);
    ret = RpcChannel_Send(ctx->rpc, msg, strlen(msg) + 1, NULL, NULL);
@@ -1321,17 +1649,20 @@ ToolsOnLoad(ToolsAppCtx *ctx)
          g_warning("Failed to load iphlpapi.dll.  Cannot report networking details.\n");
          return NULL;
       }
+
+      ToolsCore_InitializeCOM(ctx);
 #endif
 
       regData.regs = VMTools_WrapArray(regs, sizeof *regs, ARRAYSIZE(regs));
 
       memset(&gInfoCache, 0, sizeof gInfoCache);
       vmResumed = FALSE;
+      gInfoCache.method = NIC_INFO_V3_WITH_INFO_IPADDRESS_V3;
 
       /*
-       * Set up the GuestInfoGather loop.
+       * Set up the GuestInfo gather loops.
        */
-      TweakGatherLoop(ctx, TRUE);
+      TweakGatherLoops(ctx, TRUE);
 
       return &regData;
    }
