@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -43,14 +43,16 @@
 #include "strutil.h"
 #include "util.h"
 #include "vmBackupSignals.h"
-#if defined(_WIN32)
-#include "vmware/guestrpc/guestQuiesce.h"
+#include "guestQuiesce.h"
+#if !defined(_WIN32)
+#include "vmware/tools/threadPool.h"
 #endif
 #include "vmware/tools/utils.h"
 #include "vmware/tools/vmbackup.h"
 #include "xdrutil.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -77,6 +79,9 @@ static VmBackupState *gBackupState = NULL;
 static Bool
 VmBackupEnableSync(void);
 
+static Bool
+VmBackupEnableSyncWait(void);
+
 
 /**
  * Returns a string representation of the given state machine state.
@@ -96,6 +101,9 @@ VmBackupGetStateName(VmBackupMState state)
    case VMBACKUP_MSTATE_SCRIPT_FREEZE:
       return "SCRIPT_FREEZE";
 
+   case VMBACKUP_MSTATE_SYNC_FREEZE_WAIT:
+      return "SYNC_FREEZE_WAIT";
+
    case VMBACKUP_MSTATE_SYNC_FREEZE:
       return "SYNC_FREEZE";
 
@@ -137,6 +145,39 @@ VmBackupKeepAliveCallback(void *clientData)
 }
 
 
+#if defined(__linux__)
+static Bool
+VmBackupPrivSendMsg(gchar *msg,
+                    char **result,
+                    size_t *resultLen)
+{
+   Bool success;
+   unsigned int oldLevel;
+
+   ASSERT(gBackupState != NULL);
+
+   g_debug("*** %s\n", __FUNCTION__);
+
+   oldLevel = Iopl_Get();
+
+   g_debug("Raising the IOPL, oldLevel=%u\n", oldLevel);
+   if (iopl(3) < 0) {
+      g_warning("Error raising the IOPL, %s\n", strerror(errno));
+   }
+
+   success = RpcChannel_Send(gBackupState->ctx->rpc, msg,
+                             strlen(msg) + 1,
+                             result, resultLen);
+
+   if (iopl(oldLevel) < 0) {
+      g_warning("Error restoring the IOPL, %s\n", strerror(errno));
+   }
+
+   return success;
+}
+#endif
+
+
 /**
  * Sends a command to the VMX asking it to update VMDB about a new backup event.
  * This will restart the keep-alive timer.
@@ -157,9 +198,6 @@ VmBackup_SendEvent(const char *event,
    char *result = NULL;
    size_t resultLen;
    gchar *msg;
-#if defined(__linux__)
-   unsigned int oldLevel;
-#endif
 
    ASSERT(gBackupState != NULL);
 
@@ -169,32 +207,52 @@ VmBackup_SendEvent(const char *event,
       g_source_unref(gBackupState->keepAlive);
    }
 
-   msg = g_strdup_printf(VMBACKUP_PROTOCOL_EVENT_SET" %s %u %s", event, code,
-                         desc);
-   g_debug("sending vmbackup event, %s\n", msg);
+   msg = g_strdup_printf(VMBACKUP_PROTOCOL_EVENT_SET" %s %u %s",
+                         event, code, desc);
+   g_debug("Sending vmbackup event: %s\n", msg);
 
 #if defined(__linux__)
-   oldLevel = Iopl_Get();
+   if (gBackupState->needsPriv) {
+      success = VmBackupPrivSendMsg(msg, &result, &resultLen);
+   } else {
+      success = RpcChannel_Send(gBackupState->ctx->rpc,
+                                msg, strlen(msg) + 1,
+                                &result, &resultLen);
+      if (!success) {
+         const char *privErr = "Guest is not privileged";
+         if (resultLen > strlen(privErr) &&
+             strncmp(result, privErr, strlen(privErr)) == 0) {
+            g_debug("Failed to send event: %s\n", result);
+            vm_free(result);
 
-   if (iopl(3) < 0) {
-      g_debug("Error raising the IOPL, %s", strerror(errno));
+            /*
+             * PR1444259:
+             * Some hosts enforce privilege elevation for sending this
+             * event, especially 5.5. This is Linux specific because
+             * version 9.4.x on Linux only triggers host side check for
+             * privilege elevation by sending iopl_elevation capability
+             * to the host.
+             */
+            gBackupState->needsPriv = TRUE;
+
+            g_debug("Sending event with priv: %s\n", msg);
+            success = VmBackupPrivSendMsg(msg, &result, &resultLen);
+         } else {
+            gBackupState->needsPriv = FALSE;
+         }
+      }
    }
-#endif
-
-   success = RpcChannel_Send(gBackupState->ctx->rpc, msg, strlen(msg) + 1,
+#else
+   success = RpcChannel_Send(gBackupState->ctx->rpc,
+                             msg, strlen(msg) + 1,
                              &result, &resultLen);
-
-#if defined(__linux__)
-   if (iopl(oldLevel) < 0) {
-      g_debug("Error restoring the IOPL, %s", strerror(errno));
-   }
 #endif
 
    if (!success) {
-      g_warning("Failed to send vmbackup event to the VMX: %s.\n", result);
+      g_warning("Failed to send vmbackup event: %s.\n", result);
    }
-
    vm_free(result);
+   g_free(msg);
 
    gBackupState->keepAlive = g_timeout_source_new(VMBACKUP_KEEP_ALIVE_PERIOD / 2);
    VMTOOLSAPP_ATTACH_SOURCE(gBackupState->ctx,
@@ -220,10 +278,13 @@ VmBackupFinalize(void)
       g_source_destroy(gBackupState->abortTimer);
       g_source_unref(gBackupState->abortTimer);
    }
+
+   g_static_mutex_lock(&gBackupState->opLock);
    if (gBackupState->currentOp != NULL) {
       VmBackup_Cancel(gBackupState->currentOp);
       VmBackup_Release(gBackupState->currentOp);
    }
+   g_static_mutex_unlock(&gBackupState->opLock);
 
    VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_DONE, VMBACKUP_SUCCESS, "");
 
@@ -238,6 +299,7 @@ VmBackupFinalize(void)
    }
 
    gBackupState->provider->release(gBackupState->provider);
+   g_static_mutex_free(&gBackupState->opLock);
    g_free(gBackupState->scriptArg);
    g_free(gBackupState->volumes);
    g_free(gBackupState->snapshots);
@@ -318,6 +380,7 @@ VmBackupOnError(void)
       }
       break;
 
+   case VMBACKUP_MSTATE_SYNC_FREEZE_WAIT:
    case VMBACKUP_MSTATE_SYNC_FREEZE:
    case VMBACKUP_MSTATE_SYNC_THAW:
       /* Next state is "sync error". */
@@ -355,11 +418,13 @@ VmBackupDoAbort(void)
    if (gBackupState->machineState != VMBACKUP_MSTATE_SCRIPT_ERROR &&
        gBackupState->machineState != VMBACKUP_MSTATE_SYNC_ERROR) {
       /* Mark the current operation as cancelled. */
+      g_static_mutex_lock(&gBackupState->opLock);
       if (gBackupState->currentOp != NULL) {
          VmBackup_Cancel(gBackupState->currentOp);
          VmBackup_Release(gBackupState->currentOp);
          gBackupState->currentOp = NULL;
       }
+      g_static_mutex_unlock(&gBackupState->opLock);
 
       VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_ABORT,
                          VMBACKUP_REMOTE_ABORT,
@@ -413,27 +478,32 @@ VmBackupAsyncCallback(void *clientData)
    g_source_unref(gBackupState->timerEvent);
    gBackupState->timerEvent = NULL;
 
+   g_static_mutex_lock(&gBackupState->opLock);
    if (gBackupState->currentOp != NULL) {
       g_debug("VmBackupAsyncCallback: checking %s\n", gBackupState->currentOpName);
       status = VmBackup_QueryStatus(gBackupState->currentOp);
    }
+   g_static_mutex_unlock(&gBackupState->opLock);
 
    switch (status) {
    case VMBACKUP_STATUS_PENDING:
       goto exit;
 
    case VMBACKUP_STATUS_FINISHED:
+      g_static_mutex_lock(&gBackupState->opLock);
       if (gBackupState->currentOpName != NULL) {
          g_debug("Async request '%s' completed\n", gBackupState->currentOpName);
          VmBackup_Release(gBackupState->currentOp);
          gBackupState->currentOpName = NULL;
       }
       gBackupState->currentOp = NULL;
+      g_static_mutex_unlock(&gBackupState->opLock);
       break;
 
    default:
       {
          gchar *msg;
+         g_static_mutex_lock(&gBackupState->opLock);
          if (gBackupState->errorMsg != NULL) {
             msg = g_strdup_printf("'%s' operation failed: %s",
                                   gBackupState->currentOpName,
@@ -449,6 +519,7 @@ VmBackupAsyncCallback(void *clientData)
 
          VmBackup_Release(gBackupState->currentOp);
          gBackupState->currentOp = NULL;
+         g_static_mutex_unlock(&gBackupState->opLock);
          VmBackupOnError();
          goto exit;
       }
@@ -463,9 +534,12 @@ VmBackupAsyncCallback(void *clientData)
       gBackupState->callback = NULL;
 
       if (cb(gBackupState)) {
+         g_static_mutex_lock(&gBackupState->opLock);
          if (gBackupState->currentOp != NULL || gBackupState->forceRequeue) {
+            g_static_mutex_unlock(&gBackupState->opLock);
             goto exit;
          }
+         g_static_mutex_unlock(&gBackupState->opLock);
       } else {
          VmBackupOnError();
          goto exit;
@@ -478,6 +552,13 @@ VmBackupAsyncCallback(void *clientData)
     */
    switch (gBackupState->machineState) {
    case VMBACKUP_MSTATE_SCRIPT_FREEZE:
+      /* Next state is "sync freeze wait". */
+      if (!VmBackupEnableSyncWait()) {
+         VmBackupOnError();
+      }
+      break;
+
+   case VMBACKUP_MSTATE_SYNC_FREEZE_WAIT:
       /* Next state is "sync freeze". */
       if (!VmBackupEnableSync()) {
          VmBackupOnError();
@@ -533,23 +614,44 @@ exit:
 
 
 /**
- * Calls the sync provider's start function.
+ * Calls the sync provider's start function and moves the state
+ * machine to next state.
  *
- * @param[in]  state    The backup state.
+ * For Windows next state is VMBACKUP_MSTATE_SYNC_FREEZE, whereas
+ * next state is VMBACKUP_MSTATE_SYNC_FREEZE_WAIT for Linux. Details
+ * below.
  *
  * @return Whether the provider's start callback was successful.
  */
 
 static Bool
-VmBackupEnableSync(void)
+VmBackupEnableSyncWait(void)
 {
    g_debug("*** %s\n", __FUNCTION__);
    g_signal_emit_by_name(gBackupState->ctx->serviceObj,
                          TOOLS_CORE_SIG_IO_FREEZE,
                          gBackupState->ctx,
                          TRUE);
+#if defined(_WIN32)
+   gBackupState->freezeStatus = VMBACKUP_FREEZE_FINISHED;
    if (!gBackupState->provider->start(gBackupState,
                                       gBackupState->provider->clientData)) {
+#else
+   /*
+    * PR 1020224:
+    * For Linux, FIFREEZE could take really long at times,
+    * especially when guest is running IO load. We have also
+    * seen slowness in performing open() on NFS mount points.
+    * So, we need to run freeze operation in a separate thread
+    * and track it with an extra state in the state machine.
+    */
+   gBackupState->freezeStatus = VMBACKUP_FREEZE_PENDING;
+   if (!ToolsCorePool_SubmitTask(gBackupState->ctx,
+                                 gBackupState->provider->start,
+                                 gBackupState,
+                                 NULL)) {
+      g_warning("Failed to submit backup start task.");
+#endif
       g_signal_emit_by_name(gBackupState->ctx->serviceObj,
                             TOOLS_CORE_SIG_IO_FREEZE,
                             gBackupState->ctx,
@@ -560,7 +662,48 @@ VmBackupEnableSync(void)
       return FALSE;
    }
 
+#if defined(_WIN32)
+   /* Move to next state */
    gBackupState->machineState = VMBACKUP_MSTATE_SYNC_FREEZE;
+#else
+   g_debug("Submitted backup start task.");
+   /* Move to next state */
+   gBackupState->machineState = VMBACKUP_MSTATE_SYNC_FREEZE_WAIT;
+#endif
+
+   return TRUE;
+}
+
+
+/**
+ * After sync provider's start function returns and moves the state
+ * machine to VMBACKUP_MSTATE_SYNC_FREEZE state.
+ *
+ * @return Whether the provider's start callback was successful.
+ */
+
+static Bool
+VmBackupEnableSync(void)
+{
+   g_debug("*** %s\n", __FUNCTION__);
+   if (gBackupState->freezeStatus == VMBACKUP_FREEZE_ERROR) {
+      g_signal_emit_by_name(gBackupState->ctx->serviceObj,
+                            TOOLS_CORE_SIG_IO_FREEZE,
+                            gBackupState->ctx,
+                            FALSE);
+      VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_ERROR,
+                         VMBACKUP_SYNC_ERROR,
+                         "Error when enabling the sync provider.");
+      return FALSE;
+
+   } else if (gBackupState->freezeStatus == VMBACKUP_FREEZE_CANCELED ||
+              gBackupState->freezeStatus == VMBACKUP_FREEZE_FINISHED) {
+      /* Move to next state */
+      gBackupState->machineState = VMBACKUP_MSTATE_SYNC_FREEZE;
+   } else {
+      ASSERT(gBackupState->freezeStatus == VMBACKUP_FREEZE_PENDING);
+   }
+
    return TRUE;
 }
 
@@ -599,15 +742,17 @@ VmBackupConfigGetBoolean(GKeyFile *config,
  * Starts the quiesce operation according to the supplied specification unless
  * some unexpected error occurs.
  *
- * @param[in]  data      RPC data.
- * @param[in]  forceVss  Only allow Vss quiescing or no quiescing.
+ * @param[in]  data          RPC data.
+ * @param[in]  forceQuiesce  Only allow Vss quiescing on Windows platform or
+ *                           SyncDriver quiescing on Linux platform ( only file
+ *                           system quiescing )
  *
  * @return TRUE on success.
  */
 
 static gboolean
 VmBackupStartCommon(RpcInData *data,
-                    gboolean forceVss)
+                    gboolean forceQuiesce)
 {
    GError *err = NULL;
    ToolsAppCtx *ctx = data->appCtx;
@@ -627,12 +772,24 @@ VmBackupStartCommon(RpcInData *data,
       { VmBackup_NewNullProvider, NULL },
    };
 
-   if (forceVss) {
+   if (forceQuiesce) {
       if (gBackupState->quiesceApps || gBackupState->quiesceFS) {
-          /* If quiescing is requested, only allow VSS provider */
+         /*
+          * If quiescing is requested on windows platform,
+          * only allow VSS provider
+          */
 #if defined(_WIN32)
-          if (VmBackupConfigGetBoolean(ctx->config, "enableVSS", TRUE)) {
-             provider = VmBackup_NewVssProvider();
+         if (VmBackupConfigGetBoolean(ctx->config, "enableVSS", TRUE)) {
+            provider = VmBackup_NewVssProvider();
+         }
+#elif defined(_LINUX) || defined(__linux__)
+         /*
+          * If quiescing is requested on linux platform,
+          * only allow SyncDriver provider
+          */
+         if (gBackupState->quiesceFS &&
+             VmBackupConfigGetBoolean(ctx->config, "enableSyncDriver", TRUE)) {
+            provider = VmBackup_NewSyncDriverOnlyProvider();
           }
 #endif
       } else {
@@ -663,13 +820,21 @@ VmBackupStartCommon(RpcInData *data,
    gBackupState->ctx = data->appCtx;
    gBackupState->pollPeriod = 1000;
    gBackupState->machineState = VMBACKUP_MSTATE_IDLE;
+   gBackupState->freezeStatus = VMBACKUP_FREEZE_FINISHED;
    gBackupState->provider = provider;
+   gBackupState->needsPriv = FALSE;
+   g_static_mutex_init(&gBackupState->opLock);
+   gBackupState->enableNullDriver = VmBackupConfigGetBoolean(ctx->config,
+                                                             "enableNullDriver",
+                                                             TRUE);
+
    g_debug("Using quiesceApps = %d, quiesceFS = %d, allowHWProvider = %d,"
-           "execScripts = %d, scriptArg = %s, timeout = %u\n",
+           " execScripts = %d, scriptArg = %s, timeout = %u,"
+           " enableNullDriver = %d, forceQuiesce = %d\n",
            gBackupState->quiesceApps, gBackupState->quiesceFS,
            gBackupState->allowHWProvider, gBackupState->execScripts,
            (gBackupState->scriptArg != NULL) ? gBackupState->scriptArg : "",
-           gBackupState->timeout);
+           gBackupState->timeout, gBackupState->enableNullDriver, forceQuiesce);
    g_debug("Quiescing volumes: %s",
            (gBackupState->volumes) ? gBackupState->volumes : "(null)");
 
@@ -787,7 +952,6 @@ VmBackupStart(RpcInData *data)
    return VmBackupStartCommon(data, FALSE);
 }
 
-#if defined(_WIN32)
 
 /**
  * Handler for the "vmbackup.startWithOpts" message. Starts processing the
@@ -854,7 +1018,6 @@ VmBackupStartWithOpts(RpcInData *data)
    return retval;
 }
 
-#endif
 
 /**
  * Aborts the current operation if one is active, and stops the backup
@@ -1000,11 +1163,9 @@ ToolsOnLoad(ToolsAppCtx *ctx)
 
    RpcChannelCallback rpcs[] = {
       { VMBACKUP_PROTOCOL_START, VmBackupStart, NULL, NULL, NULL, 0 },
-#if defined(_WIN32)
       /* START_WITH_OPTS command supported only on Windows for now */
       { VMBACKUP_PROTOCOL_START_WITH_OPTS, VmBackupStartWithOpts, NULL,
                     xdr_GuestQuiesceParams, NULL, sizeof (GuestQuiesceParams) },
-#endif
       { VMBACKUP_PROTOCOL_ABORT, VmBackupAbort, NULL, NULL, NULL, 0 },
       { VMBACKUP_PROTOCOL_SNAPSHOT_DONE, VmBackupSnapshotDone, NULL, NULL, NULL, 0 }
    };
