@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -31,6 +31,9 @@
 #include "vmware.h"
 #include "vmware/guestrpc/vmbackup.h"
 #include "vmware/tools/plugin.h"
+#if !defined(_WIN32)
+#include "vmware/tools/threadPool.h"
+#endif
 
 typedef enum {
    VMBACKUP_STATUS_PENDING,
@@ -39,6 +42,13 @@ typedef enum {
    VMBACKUP_STATUS_ERROR
 } VmBackupOpStatus;
 
+typedef enum {
+   VMBACKUP_FREEZE_PENDING,
+   VMBACKUP_FREEZE_FINISHED,
+   VMBACKUP_FREEZE_CANCELED,
+   VMBACKUP_FREEZE_ERROR
+} VmBackupFreezeStatus;
+
 typedef enum {
    VMBACKUP_SCRIPT_FREEZE,
    VMBACKUP_SCRIPT_FREEZE_FAIL,
@@ -48,6 +58,7 @@ typedef enum {
 typedef enum {
    VMBACKUP_MSTATE_IDLE,
    VMBACKUP_MSTATE_SCRIPT_FREEZE,
+   VMBACKUP_MSTATE_SYNC_FREEZE_WAIT,
    VMBACKUP_MSTATE_SYNC_FREEZE,
    VMBACKUP_MSTATE_SYNC_THAW,
    VMBACKUP_MSTATE_SCRIPT_THAW,
@@ -75,12 +86,19 @@ struct VmBackupSyncProvider;
  * Holds information about the current state of the backup operation.
  * Don't modify the fields directly - rather, use VmBackup_SetCurrentOp,
  * which does most of the handling needed by users of the state machine.
+ *
+ * NOTE: The thread for freeze operation modifies currentOp in BackupState
+ *       which is also accessed by the AsyncCallback driving the state
+ *       machine (run by main thread). Also, gcc might generate two
+ *       instructions for writing a 64-bit value. Therefore, protect the
+ *       access to currentOp and related fields using opLock mutex.
  */
 
 typedef struct VmBackupState {
    ToolsAppCtx   *ctx;
    VmBackupOp    *currentOp;
    const char    *currentOpName;
+   GStaticMutex   opLock; // See note above
    char          *volumes;
    char          *snapshots;
    guint          pollPeriod;
@@ -94,6 +112,8 @@ typedef struct VmBackupState {
    Bool           quiesceFS;
    Bool           allowHWProvider;
    Bool           execScripts;
+   Bool           enableNullDriver;
+   Bool           needsPriv;
    char          *scriptArg;
    guint          timeout;
    gpointer       clientData;
@@ -102,6 +122,7 @@ typedef struct VmBackupState {
    ssize_t        currentScript;
    gchar         *errorMsg;
    VmBackupMState machineState;
+   VmBackupFreezeStatus freezeStatus;
    struct VmBackupSyncProvider *provider;
 } VmBackupState;
 
@@ -116,7 +137,11 @@ typedef Bool (*VmBackupProviderCallback)(VmBackupState *, void *clientData);
  */
 
 typedef struct VmBackupSyncProvider {
+#if defined(_WIN32)
    VmBackupProviderCallback start;
+#else
+   ToolsCorePoolCb start;
+#endif
    VmBackupProviderCallback snapshotDone;
    void (*release)(struct VmBackupSyncProvider *);
    void *clientData;
@@ -145,10 +170,16 @@ VmBackup_SetCurrentOp(VmBackupState *state,
    ASSERT(state != NULL);
    ASSERT(state->currentOp == NULL);
    ASSERT(currentOpName != NULL);
+
+   g_static_mutex_lock(&state->opLock);
+
    state->currentOp = op;
    state->callback = callback;
    state->currentOpName = currentOpName;
-   state->forceRequeue = (callback != NULL && state->currentOp == NULL);
+   state->forceRequeue = (callback != NULL && op == NULL);
+
+   g_static_mutex_unlock(&state->opLock);
+
    return (op != NULL);
 }
 
@@ -211,6 +242,11 @@ VmBackup_NewNullProvider(void);
 VmBackupSyncProvider *
 VmBackup_NewSyncDriverProvider(void);
 
+#if defined(_LINUX) || defined(__linux__)
+VmBackupSyncProvider *
+VmBackup_NewSyncDriverOnlyProvider(void);
+#endif
+
 #if defined(G_PLATFORM_WIN32)
 VmBackupSyncProvider *
 VmBackup_NewVssProvider(void);
