@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -42,7 +42,7 @@
 #include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
 #include "vmware/tools/i18n.h"
-
+#include "vm_version.h"
 
 /**
  * Runs the given Tools RPC command, printing the result to the terminal and
@@ -215,6 +215,8 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
                            int argc,
                            char *argv[])
 {
+   int i = 0;
+   char *cmdStr = NULL;
    gboolean ret = FALSE;
    gboolean version = FALSE;
 #if defined(G_PLATFORM_WIN32)
@@ -280,6 +282,23 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
    state->ctx.blockFD = -1;
 #endif
 
+   /*
+    * Form the commandline for debug log before calling
+    * g_option_context_parse(), because it modifies argv.
+    */
+   cmdStr = Str_SafeAsprintf(NULL, "%s", argv[0]);
+   for (i = 1; i < argc; i++) {
+      char *prefix = cmdStr;
+      cmdStr = Str_SafeAsprintf(NULL, "%s %s", prefix, argv[i]);
+      free(prefix);
+      /*
+       * NOTE: We can't log the cmdStr here, we can
+       * only log it after logging gets configured.
+       * Logging it before ToolsCore_ReloadConfig call
+       * will not generate proper logs.
+       */
+   }
+
    context = g_option_context_new(NULL);
 #if GLIB_CHECK_VERSION(2, 12, 0)
    g_option_context_set_summary(context, N_("Runs the VMware Tools daemon."));
@@ -307,10 +326,14 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
          g_printerr("%s is an invalid container name.\n", state->name);
          goto exit;
       }
-      state->mainService = (strcmp(state->name, VMTOOLS_GUEST_SERVICE) == 0);
+      state->mainService = TOOLS_IS_MAIN_SERVICE(state);
    }
 
-   VMTools_ConfigLogging(state->name, NULL, TRUE, TRUE);
+   /* Configure logging system. */
+   ToolsCore_ReloadConfig(state, TRUE);
+
+   /* Log the commandline for debugging purposes. */
+   g_info("CmdLine: \"%s\"\n", cmdStr);
 
 #if defined(G_PLATFORM_WIN32)
    if (kill) {
@@ -321,7 +344,7 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
    }
 #else
    /* If not running the "vmusr" service, ignore the blockFd parameter. */
-   if (strcmp(state->name, VMTOOLS_USER_SERVICE) != 0) {
+   if (!TOOLS_IS_USER_SERVICE(state)) {
       if (state->ctx.blockFD >= 0) {
          close(state->ctx.blockFD);
       }
@@ -332,6 +355,7 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
    ret = TRUE;
 
 exit:
+   free(cmdStr);
    g_clear_error(&error);
    g_option_context_free(context);
    return ret;
