@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -37,6 +37,7 @@
 #include "util.h"
 #include "vmcheck.h"
 #include "vm_tools_version.h"
+#include "vm_version.h"
 #include "vmware/guestrpc/tclodefs.h"
 #include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
@@ -60,6 +61,7 @@ ToolsCoreCleanup(ToolsServiceState *state)
    ToolsCorePool_Shutdown(&state->ctx);
    ToolsCore_UnloadPlugins(state);
    if (state->ctx.rpc != NULL) {
+      RpcChannel_Stop(state->ctx.rpc);
       RpcChannel_Destroy(state->ctx.rpc);
       state->ctx.rpc = NULL;
    }
@@ -158,8 +160,10 @@ ToolsCoreIOFreezeCb(gpointer src,
    if (state->configCheckTask > 0 && freeze) {
       g_source_remove(state->configCheckTask);
       state->configCheckTask = 0;
+      VMTools_SuspendLogIO();
    } else if (state->configCheckTask == 0 && !freeze) {
-      state->configCheckTask = g_timeout_add(CONF_POLL_TIME * 10,
+      VMTools_ResumeLogIO();
+      state->configCheckTask = g_timeout_add(CONF_POLL_TIME * 1000,
                                              ToolsCoreConfFileCb,
                                              state);
    }
@@ -227,7 +231,7 @@ ToolsCoreRunLoop(ToolsServiceState *state)
                           state);
       }
 
-      state->configCheckTask = g_timeout_add(CONF_POLL_TIME * 10,
+      state->configCheckTask = g_timeout_add(CONF_POLL_TIME * 1000,
                                              ToolsCoreConfFileCb,
                                              state);
 
@@ -312,7 +316,7 @@ ToolsCore_GetTcloName(ToolsServiceState *state)
 {
    if (state->mainService) {
       return TOOLS_DAEMON_NAME;
-   } else if (strcmp(state->name, VMTOOLS_USER_SERVICE) == 0) {
+   } else if (TOOLS_IS_USER_SERVICE(state)) {
       return TOOLS_DND_NAME;
    } else {
       return NULL;
@@ -345,7 +349,10 @@ ToolsCore_ReloadConfig(ToolsServiceState *state,
    if (!first && loaded) {
       g_debug("Config file reloaded.\n");
 
-      /* Inform plugins of config file update. */
+      /*
+       * Inform plugins of config file update.
+       */
+      ASSERT(state->ctx.serviceObj != NULL);
       g_signal_emit_by_name(state->ctx.serviceObj,
                             TOOLS_CORE_SIG_CONF_RELOAD,
                             &state->ctx);
@@ -381,8 +388,6 @@ ToolsCore_Setup(ToolsServiceState *state)
       g_thread_init(NULL);
    }
 
-   ToolsCore_ReloadConfig(state, FALSE);
-
    /*
     * Useful for debugging purposes. Log the vesion and build information.
     */
