@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -40,6 +40,7 @@
 #include "vmware/tools/utils.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -88,6 +89,7 @@ ToolsCoreSigHandler(const siginfo_t *info,
 
 /**
  * Handles a USR1 signal; logs the current service state.
+ * Also shutdown rpc connection so we can do tools upgrade.
  *
  * @param[in]  info     Unused.
  * @param[in]  data     Unused.
@@ -100,6 +102,13 @@ ToolsCoreSigUsrHandler(const siginfo_t *info,
                        gpointer data)
 {
    ToolsCore_DumpState(&gState);
+
+   g_info("Shutting down guestrpc on signal USR1 ...\n");
+   if (TOOLS_IS_USER_SERVICE(&gState.ctx)) {
+      RpcChannel_Shutdown(gState.ctx.rpc);
+      gState.ctx.rpc = NULL;
+   }
+
    return TRUE;
 }
 
@@ -261,6 +270,8 @@ main(int argc,
    /* Ignore SIGUSR2 by default. */
    signal(SIGUSR2, SIG_IGN);
 
+   signal(SIGPIPE, SIG_IGN);
+
    /*
     * Save the original environment so that we can safely spawn other
     * applications (since we may have to modify the original environment
