@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,6 +27,10 @@
 
 #include <stdlib.h>
 #include <string.h>
+#if !defined(_WIN32)
+#include <unistd.h>
+#endif
+#include "vm_basic_defs.h"
 #include "vm_assert.h"
 #include "conf.h"
 #include "str.h"
@@ -34,13 +38,8 @@
 #include "toolsCoreInt.h"
 #include "vm_tools_version.h"
 #include "vmware/tools/utils.h"
-
-#if defined(__linux__)
-#include <sys/io.h>
-#include <errno.h>
-#include <string.h>
-#include "ioplGet.h"
-#endif
+#include "vmware/tools/log.h"
+#include "vm_version.h"
 
 /**
  * Take action after an RPC channel reset.
@@ -51,11 +50,12 @@
  */
 
 static void
-ToolsCoreCheckReset(struct RpcChannel *chan,
+ToolsCoreCheckReset(RpcChannel *chan,
                     gboolean success,
                     gpointer _state)
 {
    ToolsServiceState *state = _state;
+   static gboolean version_sent = FALSE;
 
    ASSERT(state != NULL);
 
@@ -75,13 +75,17 @@ ToolsCoreCheckReset(struct RpcChannel *chan,
       }
       g_free(msg);
 
-      /*
-       * Log the Tools build number to the VMX log file. We don't really care
-       * if sending the message fails.
-       */
-      msg = g_strdup_printf("log %s: Version: %s", app, BUILD_NUMBER);
-      RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL);
-      g_free(msg);
+      if (!version_sent) {
+         /*
+          * Log the Tools build number to the VMX log file. We don't really care
+          * if sending the message fails.
+          */
+         msg = g_strdup_printf("log %s: Version: %s", app, BUILD_NUMBER);
+         RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL);
+         g_free(msg);
+         /* send message only once to prevent log spewing: */
+         version_sent = TRUE;
+      }
 
       g_signal_emit_by_name(state->ctx.serviceObj,
                             TOOLS_CORE_SIG_RESET,
@@ -124,7 +128,7 @@ ToolsCoreRpcCapReg(RpcInData *data)
    /* Tell the host the location of the conf directory. */
    msg = g_strdup_printf("tools.capability.guest_conf_directory %s", confPath);
    if (!RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL)) {
-      g_debug("Unable to register guest conf directory capability.\n");
+      g_warning("Unable to register guest conf directory capability.\n");
    }
    g_free(msg);
    msg = NULL;
@@ -132,63 +136,63 @@ ToolsCoreRpcCapReg(RpcInData *data)
    /* Send the tools version to the VMX. */
    if (state->mainService) {
       uint32 version;
+      uint32 type = TOOLS_TYPE_UNKNOWN;
       char *result = NULL;
       size_t resultLen;
       gchar *toolsVersion;
+      gboolean disableVersion = g_key_file_get_boolean(state->ctx.config,
+                                                       "vmtools",
+                                                       CONFNAME_DISABLETOOLSVERSION,
+                                                       NULL);
 
-#if defined(OPEN_VM_TOOLS)
-      version = TOOLS_VERSION_UNMANAGED;
+#if defined(_WIN32)
+      type = TOOLS_TYPE_MSI;
 #else
-      gboolean disableVersion;
+#if defined(OPEN_VM_TOOLS)
+      type = TOOLS_TYPE_OVT;
+#else
+      {
+         static int is_osp = -1;
 
-      disableVersion = g_key_file_get_boolean(state->ctx.config,
-                                              "vmtools",
-                                              CONFNAME_DISABLETOOLSVERSION,
-                                              NULL);
-      version = disableVersion ? TOOLS_VERSION_UNMANAGED : TOOLS_VERSION_CURRENT;
+         if (is_osp == -1) {
+            is_osp = (access("/usr/lib/vmware-tools/dsp", F_OK) == 0);
+         }
+         type = is_osp ? TOOLS_TYPE_OSP : TOOLS_TYPE_TARBALL;
+      }
+#endif
 #endif
 
-      toolsVersion = g_strdup_printf("tools.set.version %u", version);
+      version = disableVersion ? TOOLS_VERSION_UNMANAGED : TOOLS_VERSION_CURRENT;
+
+      /*
+       * First try "tools.set.versiontype", if that fails because host is too
+       * old, fall back to "tools.set.version."
+       */
+      toolsVersion = g_strdup_printf("tools.set.versiontype %u %u", version, type);
 
       if (!RpcChannel_Send(state->ctx.rpc, toolsVersion, strlen(toolsVersion) + 1,
                            &result, &resultLen)) {
-         g_debug("Error setting tools version: %s.\n", result);
+         vm_free(result);
+         g_free(toolsVersion);
+
+         /*
+          * Fall back to old behavior for OSPs and OVT so that tools will be
+          * reported as guest managed.
+          */
+         if (type == TOOLS_TYPE_OSP || type == TOOLS_TYPE_OVT) {
+            version = TOOLS_VERSION_UNMANAGED;
+         }
+         toolsVersion = g_strdup_printf("tools.set.version %u", version);
+
+         if (!RpcChannel_Send(state->ctx.rpc, toolsVersion, strlen(toolsVersion) + 1,
+                              &result, &resultLen)) {
+            g_warning("Error setting tools version: %s.\n", result);
+         }
       }
       vm_free(result);
       g_free(toolsVersion);
    }
 
-#if defined (__linux__)
-   /* Send the IOPL elevation capability to VMX. */
-   if (state->mainService) {
-      unsigned int oldLevel;
-      char *result = NULL;
-      size_t resultLen;
-
-      const char ioplElev[] = "tools.capability.iopl_elevation";
-
-      oldLevel = Iopl_Get();
-      g_debug("%s: old IOPL = %u\n", __FUNCTION__, oldLevel);
-
-      if (iopl(3) < 0) {
-         g_debug("Error raising the IOPL, %s", strerror(errno));
-      }
-
-      g_debug("%s: new IOPL = %u\n", __FUNCTION__, Iopl_Get());
-
-      if (!RpcChannel_Send(state->ctx.rpc, ioplElev, sizeof ioplElev,
-                           &result, &resultLen)) {
-         g_debug("Error setting tools iopl elevation capability: %s\n",
-                 result);
-         vm_free(result);
-      }
-
-      if (iopl(oldLevel) < 0) {
-         g_debug("Error restoring the IOPL, %s", strerror(errno));
-      }
-   }
-#endif
-
    state->capsRegistered = TRUE;
    free(confPath);
    return RPCIN_SETRETVALS(data, "", TRUE);
@@ -278,11 +282,11 @@ ToolsCore_InitRpc(ToolsServiceState *state)
        * XXX: this should be relaxed when we try to bring up a VMCI or TCP channel.
        */
       if (!state->ctx.isVMware) {
-         g_warning("The %s service needs to run inside a virtual machine.\n",
-                   state->name);
+         g_info("The %s service needs to run inside a virtual machine.\n",
+                state->name);
          state->ctx.rpc = NULL;
       } else {
-         state->ctx.rpc = BackdoorChannel_New();
+         state->ctx.rpc = RpcChannel_New();
       }
       app = ToolsCore_GetTcloName(state);
       if (app == NULL) {
@@ -390,4 +394,3 @@ ToolsCore_SetCapabilities(RpcChannel *chan,
       g_free(newcaps);
    }
 }
-
