@@ -1,6 +1,6 @@
 
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -38,7 +38,10 @@
 #include "vmware/tools/i18n.h"
 #include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
+#include "vm_version.h"
+#include "vm_product_versions.h"
 
+#include "vm_version.h"
 #include "embed_version.h"
 VM_EMBED_VERSION(TOOLBOXCMD_VERSION_STRING);
 
@@ -103,6 +106,7 @@ static CmdTable commands[] = {
 #if (defined(_WIN32) || defined(linux)) && !defined(OPEN_VM_TOOLS)
    { "upgrade",   Upgrade_Command,  TRUE,    TRUE,   Upgrade_Help},
 #endif
+   { "logging",   Logging_Command,  TRUE,    TRUE,    Logging_Help},
    { "help",      HelpCommand,      FALSE,   FALSE,   ToolboxCmdHelp},
 };
 
@@ -222,7 +226,7 @@ ToolsCmd_SendRPC(const char *rpc,      // IN
                  size_t *resultLen)    // OUT
 {
    char *lrpc = (char *) rpc;
-   RpcChannel *chan = BackdoorChannel_New();
+   RpcChannel *chan = RpcChannel_New();
    gboolean ret = RpcChannel_Start(chan);
 
    if (!ret) {
@@ -291,6 +295,7 @@ ToolboxCmdHelp(const char *progName,   // IN
                           "Available commands:\n"
                           "   device\n"
                           "   disk\n"
+                          "   logging\n"
                           "   script\n"
                           "   stat\n"
                           "   timesync\n"
