@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -391,7 +391,7 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
     * Verify that wiping/shrinking are permitted before going through with the
     * wiping operation.
     */
-   if (!ShrinkGetWiperState() == WIPER_ENABLED && !Wiper_IsWipeSupported(part)) {
+   if (ShrinkGetWiperState() != WIPER_ENABLED && !Wiper_IsWipeSupported(part)) {
       g_debug("%s cannot be wiped / shrunk\n", mountPoint);
       ToolsCmd_PrintErr("%s",
                         SU_(disk.shrink.disabled, SHRINK_DISABLED_ERR));
@@ -444,7 +444,8 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
       }
 
       if (!quiet) {
-         g_print("\rProgress: %d [", progress);
+         g_print(SU_(disk.wiper.progress, "\rProgress: %d"), progress);
+         g_print(" [");
          for (i = 0; i <= progress / 10; i++) {
             putchar('=');
          }
@@ -461,6 +462,7 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
 #endif
 
    rc = EXIT_SUCCESS;
+   g_print("\n");
    if (progress >= 100 && performShrink) {
       rc = ShrinkDiskSendRPC();
    } else if (progress < 100) {
